﻿IF OBJECT_ID('TR__DeployableDependencies__CircularDependencyCheck') IS NOT NULL
DROP TRIGGER [TR__DeployableDependencies__CircularDependencyCheck]
GO

CREATE TRIGGER [TR__DeployableDependencies__CircularDependencyCheck]
ON [DeployableDependencies] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @ResolvedDependencies TABLE(
		[Base_Deployable_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[Dependant_Deployable_Id] INT NOT NULL, 
		[DependsOn_Deployable_Id] INT NOT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @ResolvedDependencies 
	  ([Base_Deployable_Id], [Resolution_Depth], [Dependant_Deployable_Id], [DependsOn_Deployable_Id])
	SELECT 
	   [Deployable_Id], 1, [Deployable_Id], [DependsOn_Deployable_Id]
	  FROM [inserted] 

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's dependecies from the actual table
		INSERT INTO @ResolvedDependencies 
		  ([Base_Deployable_Id], [Resolution_Depth], [Dependant_Deployable_Id], [DependsOn_Deployable_Id])
		SELECT
		  [Base_Deployable_Id], @depth, RD.[DependsOn_Deployable_Id], DD.[DependsOn_Deployable_Id]
		FROM @ResolvedDependencies RD
	         INNER JOIN [DeployableDependencies] DD
		             ON RD.[DependsOn_Deployable_Id] = DD.[Deployable_Id]
		WHERE RD.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if a inserted deployable depends on itself (somewhere down the line) then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @ResolvedDependencies RD
		                          ON RD.[Resolution_Depth] > 1
		                         AND I.[Deployable_Id] = RD.[Base_Deployable_Id]
		                         AND I.[Deployable_Id] = RD.[Dependant_Deployable_Id] )

		BEGIN 
				RAISERROR('This insertion will cause a circular dependecy at depth %i, which is not supported.',16,2, @depth) 
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO

GO
IF OBJECT_ID('[TR__DeployableDependencies__ValidateDeployableId]') IS NOT NULL DROP TRIGGER [TR__DeployableDependencies__ValidateDeployableId]
GO

CREATE TRIGGER [TR__DeployableDependencies__ValidateDeployableId] 
       ON [DeployableDependencies] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS(SELECT *
				FROM [inserted] i
					 INNER JOIN [Deployables] D1 ON D1.[Deployable_Id] = i.[Deployable_Id]
					 INNER JOIN [Deployables] D2 ON D2.[Deployable_Id] = i.[DependsOn_Deployable_Id]
			   WHERE D1.[Application_Id] <> D2.[Application_Id])
		BEGIN
			RAISERROR('Cross-application deployables are not supported.', 16, 1)
			ROLLBACK
		END
END
GO

GO
IF OBJECT_ID('[TR__EventOccurenceDetails__ValidateDetailValue]') IS NOT NULL DROP TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue]
GO

CREATE TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue] 
       ON [EventOccurenceDetails] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS
	(
		SELECT *
		  FROM inserted /*[EventOccurenceDetails]*/ EOD
			   INNER JOIN [EventTypeDetails] ETD
					   ON ETD.[Event_Code] = EOD.[Event_Code]
					  AND ETD.[Detail_Name] = EOD.[Detail_Name]
		 WHERE ETD.[Detail_Type] <> sql_variant_property(EOD.[Detail_Value], 'BaseType')
	)
		BEGIN
			RAISERROR('Event detail value/type mismatch.', 16, 1)
			ROLLBACK
		END
END
GO



GO
IF OBJECT_ID('TR__PlanActionGroups__ValidateDeployableId') IS NOT NULL DROP TRIGGER TR__PlanActionGroups__ValidateDeployableId
GO

CREATE TRIGGER TR__PlanActionGroups__ValidateDeployableId
       ON [PlanActionGroups] FOR UPDATE, INSERT 
AS 
BEGIN
	
	/* if there is no row being inserted/updated, ignore this trigger */
	IF ((SELECT COUNT(*) FROM [inserted]) = 0) 
	    RETURN
	
	/* If a deployable ID is found for the given application, or an imported deployabled ID is found, return without error */
	IF EXISTS(SELECT * FROM [inserted] WHERE [Deployable_Id] IS NULL)
	OR EXISTS(SELECT * FROM [Deployables] D
			INNER JOIN [inserted] i ON 
				i.[Application_Id] = D.[Application_Id]
			AND	i.[Deployable_Id] = D.[Deployable_Id])
	OR EXISTS(SELECT * FROM [DeployablesImported] D
			INNER JOIN [inserted] i ON
				i.[Application_Id] = D.[Application_Id]
			AND	i.[Deployable_Id] = D.[Deployable_Id])
	RETURN

	RAISERROR('An invalid Deployable ID was provided for the plan', 16, 1)
	ROLLBACK
		
END
GO


GO
IF OBJECT_ID('TR__PlanActionGroups__ValidateSequence') IS NOT NULL DROP TRIGGER TR__PlanActionGroups__ValidateSequence
GO

CREATE TRIGGER TR__PlanActionGroups__ValidateSequence
       ON [PlanActionGroups] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [PlanActionGroups] PAG 
				   INNER JOIN (SELECT [PlanActionGroup_Id], [PlanActionGroup_Sequence],
									  [Actual_PlanActionGroup_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Application_Id], [Environment_Id]
										   ORDER BY [Application_Id], [Environment_Id], [PlanActionGroup_Sequence])
									  FROM [PlanActionGroups]) PAGS
						   ON PAG.[PlanActionGroup_Id] = PAGS.[PlanActionGroup_Id]
			 WHERE PAG.[PlanActionGroup_Sequence] <> PAGS.[Actual_PlanActionGroup_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE PAG.[Application_Id] = [Application_ID]
			                     AND PAG.[Environment_Id] = [Environment_Id] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE PAG.[Application_Id] = [Application_ID]
			                     AND PAG.[Environment_Id] = [Environment_Id] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid PlanActionGroup_Sequence was provided for the PlanActionGroup.', 16, 1)
		ROLLBACK
	
	END
		
END
GO

GO
IF OBJECT_ID('TR__PlanActions__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__PlanActions__ValidateSequence]
GO

CREATE TRIGGER [TR__PlanActions__ValidateSequence]
       ON [PlanActions] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [PlanActions] PA
				   INNER JOIN (SELECT [Plan_Id], [Action_Sequence],
									  [Actual_Action_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Plan_Id]
										   ORDER BY [Plan_Id], [Action_Sequence])
									  FROM [PlanActions]) PA_ACTUAL
						   ON PA.[Plan_Id] = PA_ACTUAL.[Plan_Id]
						  AND PA.[Action_Sequence] = PA_ACTUAL.[Action_Sequence]
			 WHERE PA.[Action_Sequence] <> PA_ACTUAL.[Actual_Action_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE PA.[Plan_Id] = [Plan_Id])
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE PA.[Plan_Id] = [Plan_Id])
				   )
		) 
	BEGIN

		RAISERROR('An invalid PlanAction_Sequence was provided for the PlanAction.', 16, 1)
		ROLLBACK
	
	END
		
END
GO

GO

IF OBJECT_ID('[TR__ReleaseDeployables__ValidateDeployableId]') IS NOT NULL DROP TRIGGER [TR__ReleaseDeployables__ValidateDeployableId]
GO

CREATE TRIGGER [TR__ReleaseDeployables__ValidateDeployableId] 
       ON [ReleaseDeployables] FOR UPDATE,INSERT 
AS 
BEGIN

	/* If a deployable ID is found for the given application, or an imported deployabled ID is found, return without error */
	IF @@ROWCOUNT = 0
	OR EXISTS(SELECT * FROM [Deployables] D
			INNER JOIN [inserted] i ON 
				i.[Application_Id] = D.[Application_Id]
			AND	i.[Deployable_Id] = D.[Deployable_Id]
			AND i.[InclusionType_Code] = 'I')
	OR EXISTS(SELECT * FROM [DeployablesImported] D
			INNER JOIN [inserted] i ON
				i.[Application_Id] = D.[Application_Id]
			AND	i.[Deployable_Id] = D.[Deployable_Id]
			AND i.[Application_Id] <> i.[Referenced_Application_Id])
	RETURN

	RAISERROR('An invalid Deployable ID was provided for this release', 16, 1)
	ROLLBACK
	
END
GO


GO

IF OBJECT_ID('TR__Releases__ValidateReleaseNumber') IS NOT NULL DROP TRIGGER TR__Releases__ValidateReleaseNumber
GO

CREATE TRIGGER TR__Releases__ValidateReleaseNumber
ON [Releases] FOR UPDATE,INSERT
AS BEGIN
	IF EXISTS( SELECT * FROM inserted
		        WHERE 0 = dbo.IsReleaseNumberValid( [Application_Id], [Release_Number] ) )
	BEGIN
		DECLARE @Release_Number RELEASENUMBER
		SELECT @Release_Number = MAX([Release_Number]) FROM inserted

		RAISERROR('Release number ''%s'' is invalid for the selected application.', 16, 1, @Release_Number)
		ROLLBACK
	END
END
GO

GO
IF OBJECT_ID('TR__Releases__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__Releases__ValidateSequence]
GO

CREATE TRIGGER [TR__Releases__ValidateSequence]
       ON [Releases] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [Releases] R 
				   INNER JOIN (SELECT [Application_Id], 
                                      [Release_Number],
									  [Actual_Release_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Application_Id]
										        ORDER BY [Application_Id], [Release_Sequence])
									  FROM [Releases]) R_ACTUAL
						   ON R.[Application_Id] = R_ACTUAL.[Application_Id]
                          AND R.[Release_Number] = R_ACTUAL.[Release_Number]
			 WHERE R.[Release_Sequence] <> R_ACTUAL.[Actual_Release_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE R.[Application_Id] = [Application_Id]
                                 AND R.[Release_Number] = [Release_Number] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE R.[Application_Id] = [Application_Id]
                                 AND R.[Release_Number] = [Release_Number] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid Release_Sequence was provided for the Release.', 16, 1)
		ROLLBACK
	
	END
		
END
GO

GO
IF OBJECT_ID('TR__WorkflowSteps__ValidateSequence') IS NOT NULL DROP TRIGGER [TR__WorkflowSteps__ValidateSequence]
GO

CREATE TRIGGER [TR__WorkflowSteps__ValidateSequence]
       ON [WorkflowSteps] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [WorkflowSteps] WS
				   INNER JOIN (SELECT [Workflow_Id],
				                      [Step_Sequence],
									  [Actual_Step_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Workflow_Id]
										   ORDER BY [Workflow_Id], [Step_Sequence])
									  FROM [WorkflowSteps]) WS_ACTUAL
						   ON WS.[Workflow_Id] = WS_ACTUAL.[Workflow_Id]
						  AND WS.[Step_Sequence] = WS_ACTUAL.[Step_Sequence]
			 WHERE WS.[Step_Sequence] <> WS_ACTUAL.[Actual_Step_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE WS.[Workflow_Id] = [Workflow_Id])
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE WS.[Workflow_Id] = [Workflow_Id])
				   )
		) 
	BEGIN

		RAISERROR('An invalid Step_Sequence was provided for the Workflow Step.', 16, 1)
		ROLLBACK
	
	END
		
END
GO

GO
IF OBJECT_ID('ABS_MONTH_NUMBER') IS NOT NULL DROP FUNCTION ABS_MONTH_NUMBER
GO
CREATE FUNCTION ABS_MONTH_NUMBER
(
  @Date DATETIME
) RETURNS INT AS BEGIN
  
  RETURN
    /* Months Past Since Day 0 */
    ((DATEPART(yy,@Date) - 1900) * 12)
    /* Plus Months this year */
    + DATEPART(m,@Date)
END
GO

GO
IF OBJECT_ID('ABS_WEEK_NUMBER') IS NOT NULL DROP FUNCTION ABS_WEEK_NUMBER
GO
CREATE FUNCTION ABS_WEEK_NUMBER
(
  @DateTime DATETIME
) RETURNS INT AS BEGIN

  DECLARE @Dec DECIMAL(9,2)
  /* Convert to Days since 1900-01-01 */
  SET @Dec = CAST(@DateTime AS DECIMAL(9,2))
  /* Since 1900-01-01 is a Monday and Week start on Sunday, Add 1 */
  SET @Dec = @Dec + 1
  /* Floor of /7 is week # */
  RETURN FLOOR(@Dec/7)
END
GO

GO

IF OBJECT_ID('CompareBuildNumber') IS NOT NULL DROP FUNCTION CompareBuildNumber
GO

CREATE FUNCTION CompareBuildNumber
(
	@Application_Id  INT,
	@Left_Build_Number BUILDNUMBER,
	@Right_Build_Number BUILDNUMBER
)
RETURNS INT
AS

BEGIN

	DECLARE @Ret INT
	DECLARE @iLeft INT, @iRight INT
	SET @iLeft = @Left_Build_Number
	SET @iRight = @Right_Build_Number

	IF @iLeft = @iRight SET @Ret = 0
	ELSE IF @iLeft < @iRight SET @Ret = -1
	ELSE SET @Ret = 1

	RETURN @Ret

END
	
GO


GO

IF OBJECT_ID('CompareReleaseNumber') IS NOT NULL DROP FUNCTION CompareReleaseNumber
GO

CREATE FUNCTION CompareReleaseNumber
(
	@Application_Id  INT,
	@Left_Release_Number ReleaseNUMBER,
	@Right_Release_Number ReleaseNUMBER
)
RETURNS INT
AS

BEGIN

	DECLARE @Ret INT

	IF @Left_Release_Number = @Right_Release_Number SET @Ret = 0
	ELSE IF @Left_Release_Number < @Right_Release_Number SET @Ret = -1
	ELSE SET @Ret = 1

	RETURN @Ret

END
	
GO



GO

IF OBJECT_ID('CsvToTable') IS NOT NULL DROP FUNCTION CsvToTable
GO

CREATE FUNCTION CsvToTable (@CSVString VARCHAR(MAX), @Delimiter	VARCHAR(10))
  RETURNS @tbl TABLE ([Value_Sequence] INT, [Value_Text] VARCHAR(MAX))

AS BEGIN 

    DECLARE @i INT, @j INT, @ctr INT
	SET @i = 1 

	WHILE @i <= len(@CSVString)
	BEGIN
		SET @j = CHARINDEX(@Delimiter, @CSVString, @i)
        SET @ctr = COALESCE(@ctr,0) + 1
		IF @j = 0 SET @j = LEN(@CSVString) + 1
		INSERT @tbl SELECT @ctr, SUBSTRING(@CSVString, @i, @j - @i)
		SELECT @i = @j + LEN(@Delimiter)
	END
	
    RETURN

END
GO



GO
IF OBJECT_ID('[GetSortableReleaseNumberWithValidation]') IS NOT NULL
	DROP FUNCTION [GetSortableReleaseNumberWithValidation]
GO

CREATE FUNCTION [GetSortableReleaseNumberWithValidation]
(
	@Release_Number RELEASENUMBER,
    @ReleaseNumber_Scheme_Name VARCHAR(20)
)
RETURNS @tbl TABLE(
  [Release_Sequence] BIGINT, 
  [Valid_Indicator] CHAR(1), 
  [InvalidReason_Text] VARCHAR(100)) AS

BEGIN

    DECLARE @ERR VARCHAR(100), @RELNO BIGINT
    
    -- split into parts and digits
    DECLARE @Digits TABLE ([Part] INT, [Pos] INT, [Digit] CHAR(1), [Val] INT, [Seq] INT, [Pow] BIGINT)
     INSERT @Digits
     SELECT [Part] = [Value_Sequence],
            [Pos] = LEN(R.[Value_Text]) - N.[Number] + 1,
            [Digit] = SUBSTRING(R.[Value_Text], N.[Number], 1),
            [Val] = NULL, [Seq] = NULL, [Pow] = NULL
      FROM  dbo.[CsvToTable](@Release_Number, '.') R INNER JOIN [Numbers] N
            ON N.[Number] BETWEEN 1 AND LEN(R.[Value_Text])
      WHERE N.[Number] BETWEEN 1 AND LEN(@Release_Number) --optimization
                                              
    -- set vals
    UPDATE @Digits SET [Val] = CHARINDEX([Digit], '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ') - 1

    -- validate
    IF @ReleaseNumber_Scheme_Name = 'MajorMinorRevision' BEGIN
      IF (SELECT MAX([Part]) FROM @Digits) <> 3 BEGIN SET @ERR='Release Numbers must have exactly three parts' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits) > 4 BEGIN SET @ERR='The maximum length for a part is 4' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Pos] > 1) BEGIN SET @ERR='Only the last digit of a Release Number part may be a character' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Part] NOT IN (SELECT [Part] FROM @Digits WHERE [Digit] LIKE '[0-9]')) BEGIN SET @ERR='Each part must contain a number' GOTO ERR END
    END ELSE IF @ReleaseNumber_Scheme_Name = 'MajorMinor' BEGIN
      IF (SELECT MAX([Part]) FROM @Digits) <> 2 BEGIN SET @ERR='Release Number must have exactly two parts'  GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=1) > 4 BEGIN SET @ERR='The maximum length for the first part is 4.' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=2) > 8 BEGIN SET @ERR='The maximum length for the second part is 8.' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Pos] > 1 AND [Part] = 1) BEGIN SET @ERR='Only the last digit of the first part may be a character' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Part] NOT IN (SELECT [Part] FROM @Digits WHERE [Digit] LIKE '[0-9]')) BEGIN SET @ERR='Each part must contain a number' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Part]=2) > 4 BEGIN SET @ERR='The maximum length for the non-numeric portion of the second part is 4' GOTO ERR END
    END ELSE IF @ReleaseNumber_Scheme_Name = 'DateBased' BEGIN
      IF (SELECT MAX([Part]) FROM @Digits) > 2 BEGIN SET @ERR='Release Number must have at most two parts' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=1) > 8 BEGIN SET @ERR='The maximum length for the first part is 8' GOTO ERR END
      IF (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=2) > 4 BEGIN SET @ERR='The maximum length for the second part is 4' GOTO ERR END
      IF EXISTS(SELECT * FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Part] = 1) BEGIN SET @ERR='The first part may only contain numbers' GOTO ERR END
    END


    -- munge digits
    IF @ReleaseNumber_Scheme_Name = 'MajorMinorRevision' BEGIN
      -- shift all parts left one if there's no char in first position
      UPDATE @Digits SET [Pos] = [Pos] + 1
       WHERE [Part] NOT IN (SELECT [Part] FROM @Digits WHERE [Digit] LIKE '[A-Z]')

    END ELSE IF @ReleaseNumber_Scheme_Name = 'MajorMinor' BEGIN
      -- move all chars from part 2 to part 3
      UPDATE @Digits SET [Part] = 3 
       WHERE [Part] = 2
         AND [Pos] <= (SELECT MAX([Pos]) FROM @Digits WHERE [Digit] LIKE '[A-Z]' AND [Part]=2)
      -- shift part 2 right by number of chars moved
      UPDATE @Digits SET [Pos] = [Pos] - (SELECT MAX([Pos]) FROM @Digits WHERE [Part]=3)
       WHERE [Part] = 2 AND EXISTS (SELECT * FROM @Digits WHERE [Part]=3)

    END ELSE IF @ReleaseNumber_Scheme_Name = 'DateBased' BEGIN
      -- move part 1 to part 2, and part 2 to part 3
      UPDATE @Digits SET [Part] = [Part] + 1 WHERE [Part] IN (1,2)

    END


    /* 
    -- insert missing digits (debug)
    INSERT @Digits SELECT NPART.[Number], NPOS.[Number], '0', 0, NULL, NULL
     FROM [Numbers] NPART CROSS JOIN [Numbers] NPOS
          LEFT JOIN @Digits D ON NPOS.[Number] = D.[Pos] AND NPART.[Number] = D.[Part]
    WHERE NPART.[Number] BETWEEN 1 AND 3 AND NPOS.[Number] BETWEEN 1 AND 4
      AND D.[Part] IS NULL
    */

    -- set seqs, pow
    UPDATE @Digits SET [Seq] = (3 - [Part]) * 4  + [Pos]
    UPDATE @Digits SET [Pow] = POWER(CAST(36 AS BIGINT),[Seq]-1)

    -- SELECT * FROM @Digits ORDER BY [Part], [Pos] DESC -- (debug)
    
    SET @RELNO = (SELECT SUM([Pow] * [Val]) FROM @Digits)
    
    INSERT INTO @tbl VALUES(@RELNO, 'Y', NULL)
    GOTO RET

ERR:
    -- PRINT @ERR
    INSERT INTO @tbl VALUES(NULL, 'N', @ERR)

RET:
    RETURN

END
GO

GO
IF OBJECT_ID('IsAuthorizedForTask') IS NOT NULL DROP FUNCTION [IsAuthorizedForTask]
GO

CREATE FUNCTION [IsAuthorizedForTask]
(
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@Task_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT
) RETURNS BIT AS
BEGIN

	DECLARE @RETVAL BIT
	SET @RETVAL = 0

	SET @ApplicationGroup_Id = NULLIF(@ApplicationGroup_Id, 0)
	SET @Application_Id = NULLIF(@Application_Id, 0)
	SET @Environment_Id = NULLIF(@Environment_Id, 0)

	/* If application group is not specified but the application is, look up the group */
	IF @ApplicationGroup_Id IS NULL AND @Application_Id IS NOT NULL
		SELECT @ApplicationGroup_Id = [ApplicationGroup_Id] FROM [Applications] WHERE [Application_Id] = @Application_Id

	SET @RETVAL = COALESCE(
	(
		SELECT TOP 1 CASE [PrivilegeType_Code]
                     WHEN 'G' THEN 1 WHEN 'D' THEN 0 END
		  FROM [Privileges] P
			   INNER JOIN [RoleTasks] RT
					   ON RT.[Role_Id] = P.[Role_Id]
			   INNER JOIN [Tasks] T
					   ON T.[Task_Id] = RT.[Task_Id]
		 WHERE (( /* User */
				   P.[PrincipalType_Code] = 'U'
				   AND P.[Principal_Name] = @User_Name
				 ) OR (
				   /* Group */ 
				   P.[PrincipalType_Code] = 'G'
				   AND P.[Principal_Name] IN 
					 (SELECT [Value_Text]
						FROM [dbo].[CsvToTable](@GroupNames_Csv, ',') ) 
				))
			AND (
				/* Filter out denys that are not scopeable */
				P.[PrivilegeType_Code] <> 'D'
				OR
				(T.[Application_Scopeable_Indicator] = 'Y' AND P.[Application_Id] IS NOT NULL)
				OR
				(T.[Environment_Scopeable_Indicator] = 'Y' AND P.[Environment_Id] IS NOT NULL)
                OR
                (P.[Application_Id] IS NULL AND P.[Environment_Id] IS NULL)
				OR
				(T.Application_Scopeable_Indicator = 'N' AND T.[Environment_Scopeable_Indicator] = 'N')
			) AND ((
			      /* If not app scopeable, include regardless of input */			
                  T.[Application_Scopeable_Indicator] = 'N'
                  OR
			      /* If no app group specified, only system-scope is acceptable */			
			      (@ApplicationGroup_Id IS NULL AND P.[ApplicationGroup_Id] IS NULL)
			      OR
			      /* If app group is specified, system or app group scopes are acceptable */
			      (@ApplicationGroup_Id IS NOT NULL AND
			         (P.[ApplicationGroup_Id] IS NULL
			          OR P.[ApplicationGroup_Id] = @ApplicationGroup_Id)
			      )
				) AND (
			      /* If not app scopeable, include regardless of input */			
                  T.[Application_Scopeable_Indicator] = 'N'
                  OR
			      /* If no app specified, only system-scope is acceptable */			
			      (@Application_Id IS NULL AND P.[Application_Id] IS NULL)
			      OR
			      /* If app is specified, system or app scopes are acceptable */
			      (@Application_Id IS NOT NULL AND
			         (P.[Application_Id] IS NULL
			          OR P.[Application_Id] = @Application_Id)
			      )
				) AND (
			      /* If not env scopeable, include regardless of input */			
                  T.[Environment_Scopeable_Indicator] = 'N'
                  OR
			      /* If no env specified, only system-scope is acceptable */			
			      (@Environment_Id IS NULL AND P.[Environment_Id] IS NULL)
			      OR
			      /* If env is specified, system or env scopes are acceptable */
			      (@Environment_Id IS NOT NULL AND
			         (P.[Environment_Id] IS NULL
			          OR P.[Environment_Id] = @Environment_Id)
			      )
				))
			AND T.[Task_Id] = @Task_Id
			AND P.[Directory_Provider_Id] = @Directory_Provider_Id
		 ORDER BY CHARINDEX([PrincipalType_Code], 'UG'), /* Users (Explicit) always override Groups (Implicit) */
                  CASE WHEN P.[Application_Id] IS NOT NULL THEN 0 ELSE 1 END, /* Scopes always override system */
                  CASE WHEN P.[Environment_Id] IS NOT NULL THEN 0 ELSE 1 END, 
                  CHARINDEX([PrivilegeType_Code], 'DG') /* Deny Always overrides Grant */
	),0)

	RETURN @RETVAL
END
GO

GO

IF OBJECT_ID('IsBuildNumberValid') IS NOT NULL DROP FUNCTION IsBuildNumberValid
GO

CREATE FUNCTION IsBuildNumberValid
(
	@Application_Id INT,
	@Build_Number BUILDNUMBER
) RETURNS BIT AS
BEGIN

	DECLARE @RETVAL BIT  SET @RETVAL = 0

	/* BuildNumber is always numeric-only */
	IF @Build_Number NOT LIKE '%[^0-9]%' SET @RETVAL = 1
	
	RETURN @RETVAL
END
GO

GO
IF OBJECT_ID('[IsPromotionApproved]') IS NOT NULL
	DROP FUNCTION [IsPromotionApproved]
GO

CREATE FUNCTION [IsPromotionApproved]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
) RETURNS CHAR(1) AS
BEGIN

	/* Variables used for validation/cursor */
	DECLARE @LastGroup_Indicator YNINDICATOR
		   ,@CurrentGroup_Indicator YNINDICATOR
		   ,@IsPromotionApproved_Indicator CHAR(1)
		   ,@User_Name VARCHAR(255)
		   ,@ApprovalType_Code CHAR(1)
		   ,@Or_Indicator YNINDICATOR
		   ,@Workflow_Id INT
	SET @LastGroup_Indicator = 'N'
	SET @CurrentGroup_Indicator = NULL
	SET @IsPromotionApproved_Indicator = 'X' /* No approvals required (default) */

	IF NOT EXISTS ( SELECT *
					  FROM [Builds]
					 WHERE [Application_Id] = @Application_Id
					   AND [Release_Number] = @Release_Number
					   AND [Build_Number] = @Build_Number
					   AND [BuildStatus_Name] = 'Active' )
	BEGIN
		/* Inactive build; assume no approvals required */
		RETURN @IsPromotionApproved_Indicator
	END

	SET @Workflow_Id = (SELECT [Workflow_Id] FROM [Releases]
	                     WHERE [Application_Id] = @Application_Id
	                       AND [Release_Number] = @Release_Number)


	/* Prepare to step through in order of approval sequence */
	DECLARE [PromotionApprovalRequirements_Cursor] CURSOR FOR
	SELECT [User_Name]
		  ,[ApprovalType_Code]
		  ,[Or_Indicator]
	  FROM [PromotionApprovalRequirements]
	 WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id
	ORDER BY [Approval_Sequence]

	OPEN [PromotionApprovalRequirements_Cursor]
	FETCH NEXT FROM [PromotionApprovalRequirements_Cursor]
		INTO @User_Name, @ApprovalType_Code, @Or_Indicator
	WHILE @@FETCH_STATUS = 0
	BEGIN

		SET @IsPromotionApproved_Indicator = NULL /* At least one approval is required */

		IF @Or_Indicator = 'Y'
		BEGIN

			SET @LastGroup_Indicator = CASE WHEN @LastGroup_Indicator = 'Y'
												 THEN 'Y'
											 WHEN @CurrentGroup_Indicator = 'Y'
												 THEN 'Y'
											 ELSE 'N' END
			SET @CurrentGroup_Indicator = 'Y'

		END

		/* Workaround for a bug if the first approval requirement is OR */
		SET @CurrentGroup_Indicator = COALESCE(@CurrentGroup_Indicator, 'Y')

		/* Check PromotionApprovalRequirements against BuildApprovals table */
		SET @CurrentGroup_Indicator =
			CASE WHEN @CurrentGroup_Indicator = 'Y'
				  AND (
						( /* User approvals */
							@ApprovalType_Code = 'U'
						AND EXISTS ( SELECT *
									   FROM [BuildApprovals]
									  WHERE [Application_Id] = @Application_Id
										AND [Release_Number] = @Release_Number
										AND [Build_Number] = @Build_Number
										AND [Environment_Id] = @Environment_Id
										AND [User_Name] = @User_Name ) )
				   OR ( /* Group approvals */
							@ApprovalType_Code = 'G'
						AND ( SELECT COUNT(*)
								FROM [BuildApprovals]
							   WHERE [Application_Id] = @Application_Id
								 AND [Release_Number] = @Release_Number
								 AND [Build_Number] = @Build_Number
								 AND [Environment_Id] = @Environment_Id
								 AND [Group_Name] = @User_Name )
						 >= ( SELECT MAX([RequiredInGroup_Count])
								FROM [PromotionApprovalRequirements]
							   WHERE [Workflow_Id] = @Workflow_Id
								 AND [Environment_Id] = @Environment_Id
								 AND [ApprovalType_Code] = 'G'
								 AND [User_Name] = @User_Name )
					  )
					) THEN 'Y'
				ELSE 'N' END

		FETCH NEXT FROM [PromotionApprovalRequirements_Cursor]
			INTO @User_Name, @ApprovalType_Code, @Or_Indicator

	END

	CLOSE [PromotionApprovalRequirements_Cursor]
	DEALLOCATE [PromotionApprovalRequirements_Cursor]

	SET @IsPromotionApproved_Indicator = COALESCE(@IsPromotionApproved_Indicator,
												  CASE WHEN @LastGroup_Indicator = 'Y'
													THEN 'Y'
												  WHEN @CurrentGroup_Indicator = 'Y'
													THEN 'Y'
												  ELSE 'N' END)

	RETURN @IsPromotionApproved_Indicator

END

GO

IF OBJECT_ID('IsReleaseNumberValid') IS NOT NULL DROP FUNCTION IsReleaseNumberValid
GO

CREATE FUNCTION IsReleaseNumberValid
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) RETURNS BIT AS
BEGIN

	DECLARE @RETVAL BIT, @ReleaseNumber_Scheme_Name VARCHAR(20)
	SET @RETVAL = 0

	SET @ReleaseNumber_Scheme_Name = 
		(SELECT [ReleaseNumber_Scheme_Name] FROM [Applications]
	      WHERE [Application_Id] = @Application_Id)

	DECLARE @MAJ VARCHAR(MAX), @MIN VARCHAR(MAX), @REV VARCHAR(MAX)
	IF @Release_Number IS NULL OR @ReleaseNumber_Scheme_Name IS NULL
		BEGIN
			SET @RETVAL = 0
		END 
	ELSE IF @ReleaseNumber_Scheme_Name = 'MajorMinor' /* 1.0[A] */
		BEGIN
			/* Verify a single dot */
			IF LEN(@Release_Number) <> LEN(REPLACE(@Release_Number,'.','')) + 1
				SET @RETVAL = 0
			ELSE BEGIN

				SET @MAJ = LEFT( @Release_Number, CHARINDEX('.', @Release_Number) - 1 )
				SET @MIN = RIGHT( @Release_Number, LEN(@Release_Number) - CHARINDEX('.', @Release_Number) )
				
				IF 
					/* Major doesn't have any non-numeric characters */
					@MAJ NOT LIKE '%[^0-9]%' 
					
					/* Major actually has a digit */
					AND LEN(@MAJ) > 0

					/* And Minor doesn't contain any num-numeric/non-alphabetic characters */
					AND @MIN NOT LIKE '%[^0-9A-Z]%'

					/* And Minor starts with a numeric */
					AND @MIN LIKE '[0-9]%'

					/* And .... */
					AND
					(
						/* Minor has Alpha... */
						(@MIN LIKE '%[A-Z]%' 
							/* ... and Minor Ends with Alpha */
							AND @MIN LIKE '%[A-Z]'
							/* ... and Minor Ends does not have Alpha "squished" in middle */
							AND @MIN NOT LIKE '%[A-Z]%[0-9]%'
							)
						/* Or Minor has no Alpha  */
						OR @MIN NOT LIKE '%[A-Z]%'
					)
				SET @RETVAL = 1 ELSE SET @RETVAL = 0
			END 
		END
	ELSE IF @ReleaseNumber_Scheme_Name = 'MajorMinorRevision' /* 1.0[A].0[A] */
		BEGIN
			/* Verify two dots */
			IF LEN(@Release_Number) <> LEN(REPLACE(@Release_Number,'.','')) + 2
				SET @RETVAL = 0
			ELSE BEGIN

				DECLARE @TMP_Release_Number RELEASENUMBER
				SET @TMP_Release_Number = @Release_Number
				SET @MAJ = LEFT( @TMP_Release_Number, CHARINDEX('.', @TMP_Release_Number) - 1 )
				SET @TMP_Release_Number = SUBSTRING(@TMP_Release_Number, LEN(@MAJ) + 2, LEN(@TMP_Release_Number) - LEN(@MAJ))
				SET @MIN = LEFT( @TMP_Release_Number, CHARINDEX('.', @TMP_Release_Number) - 1 )
				SET @TMP_Release_Number = SUBSTRING(@TMP_Release_Number, LEN(@MIN) + 2, LEN(@TMP_Release_Number) - LEN(@MIN))
				SET @REV = @TMP_Release_Number
				
				IF 
					/* Major doesn't have any non-numeric characters */
					@MAJ NOT LIKE '%[^0-9]%'
					
					/* Major actually has a digit */
					AND LEN(@MAJ) > 0

					/* And Minor doesn't contain any num-numeric/non-alphabetic characters */
					AND @MIN NOT LIKE '%[^0-9A-Z]%'

					/* And Minor starts with a numeric */
					AND @MIN LIKE '[0-9]%'

					/* And .... */
					AND
					(
						/* Minor has Alpha... */
						(@MIN LIKE '%[A-Z]%' 
							/* ... and Minor Ends with Alpha */
							AND @MIN LIKE '%[A-Z]'
							/* ... and Minor Ends does not have Alpha "squished" in middle */
							AND @MIN NOT LIKE '%[A-Z]%[0-9]%'
							)
						/* Or Minor has no Alpha  */
						OR @MIN NOT LIKE '%[A-Z]%'
					)

					/* And Revision doesn't contain any num-numeric/non-alphabetic characters */
					AND @REV NOT LIKE '%[^0-9A-Z]%'

					/* And Revision starts with a numeric */
					AND @REV LIKE '[0-9]%'

					/* And .... */
					AND
					(
						/* Revision has Alpha... */
						(@REV LIKE '%[A-Z]%' 
							/* ... and Revision Ends with Alpha */
							AND @REV LIKE '%[A-Z]'
							/* ... and Revision Ends does not have Alpha "squished" in middle */
							AND @REV NOT LIKE '%[A-Z]%[0-9]%'
							)
						/* Or Revision has no Alpha  */
						OR @REV NOT LIKE '%[A-Z]%'
					)

				SET @RETVAL = 1 ELSE SET @RETVAL = 0
			END 
		END
	ELSE IF @ReleaseNumber_Scheme_Name = 'DateBased' /* Must work with max of 99999999[.9999] */
		BEGIN
			IF LEN(@Release_Number) = LEN(REPLACE(@Release_Number, '.', '')) + 1
			BEGIN
				/* 99999999.9999 */
				SET @MAJ = LEFT(@Release_Number, CHARINDEX('.', @Release_Number) - 1)
				SET @MIN = RIGHT(@Release_Number, LEN(@Release_Number) - CHARINDEX('.', @Release_Number))
			END
			ELSE
			BEGIN
				/* 99999999 */
				SET @MAJ = @Release_Number
				SET @MIN = '0'
			END

			IF		@MAJ NOT LIKE '%[^0-9]%'
				AND @MIN NOT LIKE '%[^0-9]%'
				AND LEN(@MAJ) <= 8
				AND LEN(@MIN) <= 4
					SET @RETVAL = 1
			ELSE
				SET @RETVAL = 0
		END

	RETURN @RETVAL

END
GO

GO
IF OBJECT_ID('MAXOF') IS NOT NULL DROP FUNCTION MAXOF
GO
CREATE FUNCTION MAXOF
( 
  @a SQL_VARIANT, 
  @b SQL_VARIANT
) RETURNS SQL_VARIANT AS BEGIN 

  IF @a > @b RETURN @a ELSE RETURN @b
  RETURN NULL 
END
GO

GO

IF OBJECT_ID('Session_GetUserName') IS NOT NULL DROP FUNCTION Session_GetUserName
GO

CREATE FUNCTION Session_GetUserName () RETURNS USERNAME
AS BEGIN

	DECLARE @Context_Info VARCHAR(MAX)
	SET @Context_Info = CAST(CONTEXT_INFO() AS VARCHAR(MAX))

	IF CHARINDEX(CHAR(0), @Context_Info) > 0
		SET @Context_Info = SUBSTRING(@Context_Info, 0, CHARINDEX(CHAR(0), @Context_Info))
	RETURN @Context_Info

END
GO

GO

IF OBJECT_ID('SortableBuildNumber') IS NOT NULL DROP FUNCTION SortableBuildNumber
GO

CREATE FUNCTION SortableBuildNumber
(
	@Application_Id INT,
	@Build_Number BUILDNUMBER
) RETURNS VARCHAR(MAX) AS
BEGIN
	/* Dec/Set Return Value */
	DECLARE @RETVAL VARCHAR(MAX)
	SET @RETVAL = NULL

	/* Dec/set BuildNumber Type Length */
	DECLARE @BuildNumber_Length INT
	SET @BuildNumber_Length = 10

	/* Dec/set Scheme */
	DECLARE @BuildNumber_Scheme_Name VARCHAR(20)
	SET @BuildNumber_Scheme_Name = 
		(SELECT [BuildNumber_Scheme_Name] FROM [Applications]
	      WHERE [Application_Id] = @Application_Id)

	/*******************
     * ERROR CONDITION 
     ******************/
	IF 0=dbo.IsBuildNumberValid(@Application_Id, @Build_Number) OR @BuildNumber_Length IS NULL
		BEGIN
			SET @RETVAL = NULL
		END 

	/*******************
     * Scheme Sequential or DateTimeBased
     ******************/
	ELSE IF @BuildNumber_Scheme_Name IN ('Sequential','DateTimeBased','Unique')
		BEGIN
			/* Simply return a padded string */
			SET @RETVAL = RIGHT( SPACE(@BuildNumber_Length) + @Build_Number , @BuildNumber_Length)
		END

	RETURN @RETVAL

END
GO

GO
IF OBJECT_ID('Variables_CascadedValuesByApplication') IS NOT NULL DROP VIEW Variables_CascadedValuesByApplication
GO
IF OBJECT_ID('Variables_CascadedValuesByExecution') IS NOT NULL DROP VIEW Variables_CascadedValuesByExecution
GO
IF OBJECT_ID('Variables_GetValuesForExecution') IS NOT NULL DROP FUNCTION Variables_GetValuesForExecution
GO


CREATE FUNCTION [Variables_GetValuesForExecution]
(
	@ExecutionId INT,
	@Server_Id INT,
	@Deployable_Id INT
) RETURNS @vvals TABLE ([Variable_Name] VARCHAR(50), [Value_Text] VARCHAR(MAX), [Scope_Code] CHAR(1))
BEGIN


	DECLARE @VDEC_Ids TABLE ([VariableDeclaration_Id] INT)
	DECLARE @VDEF_Ids TABLE ([VariableDefinition_Id] INT)
	--DECLARE @vvals TABLE ([Variable_Name] VARCHAR(50), [Value_Text] VARCHAR(MAX), [Scope_Code] CHAR(1))

	INSERT @VDEF_Ids
	SELECT VDEF.[VariableDefinition_Id]
	  FROM [VariableDefinitions] VDEF
		   JOIN [BuildExecutions] BE
			 ON (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = BE.[Application_Id])
			AND (VDEF.[Release_Number] IS NULL OR VDEF.[Release_Number] = BE.[Release_Number])
			AND (VDEF.[Build_Number]   IS NULL OR VDEF.[Build_Number]   = BE.[Build_Number])
			AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = BE.[Environment_Id])
			AND (VDEF.[Execution_Id]   IS NULL OR VDEF.[Execution_Id]   = BE.[Execution_Id])
			AND (VDEF.[Server_Id]      IS NULL OR VDEF.[Server_Id]      = @Server_Id)
			AND (VDEF.[Deployable_Id]  IS NULL OR VDEF.[Deployable_Id]  = @Deployable_Id)
	 WHERE BE.[Execution_Id] IS NULL OR BE.[Execution_Id] = @ExecutionId

	INSERT @VDEC_Ids
	SELECT VDEC.[VariableDeclaration_Id]
	  FROM [VariableDeclarations] VDEC
		   JOIN [BuildExecutions] BE
			 ON (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = BE.[Application_Id])
			AND (VDEC.[Scoped_Environment_Id] IS NULL OR VDEC.[Scoped_Environment_Id] = BE.[Environment_Id])
			AND ((@Server_Id IS NULL AND VDEC.[Scope_Code] <> 'V') OR @Server_Id IS NOT NULL)
			AND ((@Deployable_Id IS NULL AND VDEC.[Scope_Code] <> 'D') OR @Deployable_Id IS NOT NULL)
	 WHERE BE.[Execution_Id] IS NULL OR BE.[Execution_Id] = @ExecutionId
	   AND NOT EXISTS(SELECT * FROM [VariableDefinitions] _VDEF
					   WHERE _VDEF.[VariableDefinition_Id] IN (SELECT [VariableDefinition_Id] FROM @VDEF_Ids)
						 AND _VDEF.[Variable_Name] = VDEC.[Variable_Name]
						 AND _VDEF.[Scope_Code] = VDEC.[Scope_Code])

	INSERT INTO @vvals ([Variable_Name], [Value_Text], [Scope_Code])
	SELECT [Variable_Name], [Value_Text], [Scope_Code]
	  FROM (SELECT *, [Rank] = RANK() OVER (PARTITION BY [Variable_Name] ORDER BY CHARINDEX([Scope_Code], 'SEVGADRBPX') DESC)
			  FROM (SELECT [Variable_Name], [Value_Text], [Scope_Code], NULL AS [Scoped_Environment_Id]
					  FROM [VariableDefinitions]
					 WHERE [VariableDefinition_Id] IN (SELECT [VariableDefinition_Id] FROM @VDEF_Ids)
						   UNION ALL
					SELECT [Variable_Name], [DefaultValue_Text], [Scope_Code], [Scoped_Environment_Id]
					  FROM [VariableDeclarations]
					 WHERE [VariableDeclaration_Id] IN (SELECT [VariableDeclaration_Id] FROM @VDEC_Ids))_)_
	 WHERE [Rank] = 1
  
  RETURN
END
GO

GO

IF OBJECT_ID('GetNextBuildNumber') IS NOT NULL DROP FUNCTION GetNextBuildNumber
GO

CREATE FUNCTION GetNextBuildNumber
(
	@Application_Id  INT,
	@Build_Number BUILDNUMBER
)
RETURNS BUILDNUMBER
AS

BEGIN

	
	DECLARE @Ret BUILDNUMBER SET @Ret = NULL
	DECLARE @Now DATETIME SET @Now = GETDATE()

	DECLARE @BuildNumber_Scheme_Name VARCHAR(15)
	SELECT @BuildNumber_Scheme_Name = [BuildNumber_Scheme_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	IF @BuildNumber_Scheme_Name = 'DateTimeBased'
	BEGIN

		DECLARE @DTB_Build_Number [BUILDNUMBER]
		SET @DTB_Build_Number = /* YYMMDDBBB (where B is build sequence) */
			RIGHT(CAST(DATEPART(YEAR,@Now) AS VARCHAR(4)),2) +
			RIGHT('0' + CAST(DATEPART(MONTH,@Now) AS VARCHAR(2)),2) +
			RIGHT('0' + CAST(DATEPART(DAY,@Now) AS VARCHAR(2)),2)

		/* Find the next build sequence */
		DECLARE @Build_Sequence INT
		SELECT @Build_Sequence = CAST(MAX(RIGHT([Build_Number], 3)) AS INT)
		  FROM [Builds]
		 WHERE [Application_Id] = @Application_Id
		   AND [Build_Number] LIKE @DTB_Build_Number + '%'

		SET @Build_Sequence = COALESCE(@Build_Sequence, 0) + 1

		SET @Ret = @DTB_Build_Number + RIGHT('00' + CAST(@Build_Sequence AS VARCHAR(3)), 3)

	END

	ELSE IF @BuildNumber_Scheme_Name = 'Sequential'
	BEGIN
		SET @Ret = COALESCE(CAST(@Build_Number AS INT), 0) + 1
	END
	ELSE IF @BuildNumber_Scheme_Name = 'Unique'
	BEGIN
	    
	    SELECT @Ret = COALESCE(MAX(CAST([Build_Number] AS INT)), 0) + 1
	     FROM [Builds]
	    WHERE [Application_Id] = @Application_Id
	    
	END

	RETURN @Ret

END
	
GO


GO

IF OBJECT_ID('GetNextReleaseNumber') IS NOT NULL DROP FUNCTION GetNextReleaseNumber
GO

CREATE FUNCTION GetNextReleaseNumber
(
	@Application_Id  INT,
    @Release_Number RELEASENUMBER
)
RETURNS RELEASENUMBER
AS

BEGIN

	DECLARE @New_Release_Number RELEASENUMBER  SET @New_Release_Number = NULL

    DECLARE @ReleaseNumber_Scheme_Name VARCHAR(20)
	 SELECT @ReleaseNumber_Scheme_Name = [ReleaseNumber_Scheme_Name]
	   FROM [Applications]
	  WHERE [Application_Id] = @Application_Id

	IF @ReleaseNumber_Scheme_Name = 'DateBased'
	BEGIN
		/* Try to autoincrement a freeform release number */
		SET @New_Release_Number = CHARINDEX('.', @Release_Number)
		IF CHARINDEX('.', @Release_Number) <> 0
			SET @New_Release_Number = LEFT(@Release_Number, CHARINDEX('.', @Release_Number) - 1)
		ELSE
			SET @New_Release_Number = @Release_Number
		IF ISNUMERIC(@New_Release_Number) = 1
			SET @New_Release_Number = CAST(@New_Release_Number + 1 AS VARCHAR(10))
		ELSE
			SET @New_Release_Number = NULL
	END
	ELSE
	BEGIN
		/* Try to autoincrement a major-minor/major-minor-rev release number */
		DECLARE @LastDotIndex INT
		SET @LastDotIndex = CHARINDEX('.', REVERSE(@Release_Number)) - 1
		SET @New_Release_Number = RIGHT(@Release_Number, @LastDotIndex)
		IF ISNUMERIC(@New_Release_Number) = 1
			SET @New_Release_Number = LEFT(@Release_Number, LEN(@Release_Number) - @LastDotIndex) + CAST(@New_Release_Number + 1 AS VARCHAR(10))
		ELSE
			SET @New_Release_Number = NULL
	END

	RETURN @New_Release_Number

END
	
GO


GO
IF OBJECT_ID('[GetSortableReleaseNumber]') IS NOT NULL
	DROP FUNCTION [GetSortableReleaseNumber]
GO

CREATE FUNCTION [GetSortableReleaseNumber]
(
	@Release_Number RELEASENUMBER,
    @ReleaseNumber_Scheme_Name VARCHAR(20)
)
RETURNS BIGINT AS

BEGIN

    RETURN (SELECT [Release_Sequence]
              FROM dbo.[GetSortableReleaseNumberWithValidation](@Release_Number,@ReleaseNumber_Scheme_Name))

END
GO

GO
IF OBJECT_ID('[ApplicationEnvironments_Merged]') IS NOT NULL
	DROP VIEW [ApplicationEnvironments_Merged]
GO

CREATE VIEW [ApplicationEnvironments_Merged]
AS

	SELECT A.[Application_Id]
		  ,E.[Environment_Id]
		  ,[Workflow_Step_Sequence] = dWS.[Step_Sequence]
	  FROM [Applications] A
		   JOIN (SELECT _W.[Application_Id], _WS.[Environment_Id], MAX(_WS.[Step_Sequence]) [Step_Sequence]
				   FROM [WorkflowSteps] _WS
						JOIN [Workflows] _W
						  ON _W.[Workflow_Id] = _WS.[Workflow_Id]
						 AND _W.[Active_Indicator] = 'Y'
				  GROUP BY _W.[Application_Id], _WS.[Environment_Id]) dWS
			 ON A.[Application_Id] = dWS.[Application_Id]
		   JOIN [Environments] E
			 ON dWS.[Environment_Id] = E.[Environment_Id]


GO
IF OBJECT_ID('[Applications_Extended]') IS NOT NULL
	DROP VIEW [Applications_Extended]
GO

CREATE VIEW [Applications_Extended]
AS
	SELECT A.[Application_Id],
		   A.[Application_Name],
		   A.[Application_Description],
		   A.[ReleaseNumber_Scheme_Name],
		   A.[BuildNumber_Scheme_Name],
		   A.[AllowMultipleActiveReleases_Indicator],
		   A.[AllowMultipleActiveBuilds_Indicator],
		   A.[IssueTracking_Provider_Id],
		   A.[IssueTracking_CategoryIdList_Text],
		   A.[CreatedBy_User_Name],
		   A.[CreatedOn_Date],
		   A.[ModifiedBy_User_Name],
		   A.[ModifiedOn_Date],
		   A.[Dashboard_Text],
		   A.[ApplicationGroup_Id],
		   AG.[ApplicationGroup_Name],
		   AG.[ApplicationGroup_Description],
		   [Active_Releases_Count] = (SELECT COUNT(*) 
		                                FROM [Releases] 
		                               WHERE [Application_Id] = A.[Application_Id] 
		                                 AND [ReleaseStatus_Name] = 'Active'),
		   A.[VariableSupport_Code]
	  FROM [Applications] A
		   LEFT JOIN [ApplicationGroups] AG
				  ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]

GO
IF OBJECT_ID('[BuildApprovals_Extended]') IS NOT NULL DROP VIEW [BuildApprovals_Extended]
GO

CREATE VIEW [BuildApprovals_Extended] AS
SELECT BA.[Application_Id]
      ,BA.[Release_Number]
      ,BA.[Build_Number]
      ,BA.[Environment_Id]
      ,BA.[User_Name]
      ,BA.[Comments_Text]
      ,BA.[Group_Name]
      ,BA.[Approval_Id]
      ,BA.[Received_Date]
      ,BA.[Approval_Description]

      ,E.[Environment_Name]
  FROM [BuildApprovals] BA
       INNER JOIN [Environments] E
               ON E.[Environment_Id] = BA.[Environment_Id]

GO

IF OBJECT_ID('[BuildArtifacts_Extended]') IS NOT NULL DROP VIEW [BuildArtifacts_Extended]
GO

CREATE VIEW [BuildArtifacts_Extended] AS

	SELECT BA.[Application_Id]
		  ,BA.[Release_Number]
		  ,BA.[Build_Number]
		  ,BA.[Deployable_Id]
		  ,D.[Deployable_Name]
		  ,BA.[Artifact_Name]
		  ,BA.[Execution_Id]
		  ,BA.[Created_Date]
		  ,BE.[Environment_Id]
		  ,ENV.[Environment_Name]

	  FROM [BuildArtifacts] BA
		   LEFT  JOIN [BuildExecutions] BE
				   ON BA.[Execution_Id] = BE.[Execution_Id]
		   LEFT  JOIN [Environments] ENV
				   ON BE.[Environment_Id] = ENV.[Environment_Id]
		    LEFT JOIN [Deployables] D
				   ON BA.[Deployable_Id] = D.[Deployable_Id]

GO


GO
IF OBJECT_ID('[BuildArtifacts_Slim]') IS NOT NULL DROP VIEW [BuildArtifacts_Slim]
GO

CREATE VIEW [BuildArtifacts_Slim] AS

	SELECT BA.[Application_Id]
		  ,BA.[Release_Number]
		  ,BA.[Build_Number]
		  ,BA.[Deployable_Id]
		  ,BA.[Artifact_Name]
		  ,BA.[Execution_Id]
		  ,BA.[Created_Date]
		  ,B.[BuildStatus_Name]

	  FROM [BuildArtifacts] BA
		   INNER JOIN [Builds] B
				   ON BA.[Application_Id] = B.[Application_Id] AND BA.[Release_Number] = B.[Release_Number] AND BA.[Build_Number] = B.[Build_Number]

GO


GO

IF OBJECT_ID('[BuildExecutions_ActionsInProgress]') IS NOT NULL DROP VIEW [BuildExecutions_ActionsInProgress]
GO

CREATE VIEW [BuildExecutions_ActionsInProgress] AS

SELECT 
       BE.[Execution_Id],
	   [BuildExecutionStatus_Name] = BE.[ExecutionStatus_Name],
	   BE.[Environment_Id],

	   ENV.[Environment_Name],

       B.[Application_Id],
       B.[Release_Number],
       B.[Build_Number],

       PA.[Action_Sequence],
       PA.[ActionType_Name],
       PA.[Action_Description],
       PA.[ResumeNextOnFailure_Indicator],
	   PA.[Action_Configuration],

	   SVR.[Server_Id],
	   SVR.[Server_Name],

       BEP.[PlanExecution_Sequence],
	   BEP.[ActionGroup_Name],

       BEPA.[BuildExecution_PlanAction_Id],
       BEPA.[Original_Action_Sequence],
       BEPA.[ExecutionStarted_Date],
       BEPA.[ExecutionEnded_Date],
       [ExecutionStatus_Name] = COALESCE(BEPA.[ExecutionStatus_Name],'Pending'),

       P.[Plan_Id],
       PAG.[PlanActionGroup_Id],
       PAG.[PlanActionGroup_Sequence],

       DPL.[Deployable_Id],
       DPL.[Deployable_Name]

	  ,BEPA.[WarningLogEntry_Indicator]
	  ,A.[VariableSupport_Code]

  FROM [BuildExecutions] BE
	   INNER JOIN [Environments] ENV
			   ON BE.[Environment_Id] = ENV.[Environment_Id]
       INNER JOIN [BuildExecution_Plans] BEP
               ON BEP.[Execution_Id] = BE.[Execution_Id]
       INNER JOIN [PlanActionGroups] PAG
               ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
       INNER JOIN [Plans] P
               ON P.[Plan_Id] = PAG.[ActionGroup_Id]
       INNER JOIN [PlanActions] PA
               ON PA.[Plan_Id] = P.[Plan_Id]
       INNER JOIN [Builds] B
               ON B.[Application_Id] = BE.[Application_Id]
              AND B.[Release_Number] = BE.[Release_Number]
              AND B.[Build_Number] = BE.[Build_Number]
	   INNER JOIN [Releases] R
			   ON B.[Application_Id] = R.[Application_Id]
			  AND B.[Release_Number] = R.[Release_Number]
	   INNER JOIN [Applications] A
	           ON A.[Application_Id] = BE.[Application_Id]

        LEFT JOIN [BuildExecution_PlanActions] BEPA
               ON BEPA.[Original_PlanActionGroup_Id] = PAG.[PlanActionGroup_Id]
              AND BEPA.[Original_Action_Sequence] = PA.[Action_Sequence]
              AND BEPA.[Execution_Id] = BE.[Execution_Id]

        LEFT JOIN [Deployables] DPL
               ON DPL.[Deployable_Id] = PAG.[Deployable_Id]
			  AND ( PAG.[Deployable_Id] IS NULL OR
					PAG.[Deployable_Id] IN (SELECT _RD.[Deployable_Id]
										  FROM [ReleaseDeployables] _RD
										 WHERE _RD.[Application_Id] = B.[Application_Id]
										   AND _RD.[Release_Number] = B.[Release_Number]) )
	    LEFT JOIN [Servers] SVR
			   ON PA.[Server_Id] = SVR.[Server_Id]

GO


GO

IF OBJECT_ID('[dbo].[BuildExecutions_Extended]') IS NOT NULL
	DROP VIEW [dbo].[BuildExecutions_Extended]
GO

CREATE VIEW [dbo].[BuildExecutions_Extended]
AS

	SELECT BE.[Execution_Id]
		  ,BE.[Application_Id]
		  ,BE.[Release_Number]
		  ,BE.[Build_Number]
		  ,B.[BuildStatus_Name]
		  ,R.[Workflow_Id]
		  ,WF.[Workflow_Name]
		  ,BE.[Environment_Id]
		  ,ENV.[Environment_Name]
		  ,BE.[ExecutionStart_Date]
		  ,BE.[ExecutionEnded_Date]
		  ,BE.[ExecutionStatus_Name]
		  ,BE.[CreatedBy_User_Name]
		  ,BE.[CreatedOn_Date]
		  ,BE.[CanceledBy_User_Name]
		  ,BE.[CanceledOn_Date]
		  ,BE.[LastActionReexecutionOf_Execution_Id]
		  ,A.[Application_Name]
		  ,BP.[PromotionStatus_Name]
		  ,BP.[Promoted_Date]
		  ,BP.[PromotedBy_Name]
		  ,BE.[WarningLogEntry_Indicator]
          ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
		  ,R.[ReleaseStatus_Name]
		
	  FROM [dbo].[BuildExecutions] BE
		   INNER JOIN [dbo].[Releases] R
				  ON R.[Application_Id] = BE.[Application_Id]
				 AND R.[Release_Number] = BE.[Release_Number]
		   LEFT JOIN [dbo].[Workflows] WF
				  ON R.[Workflow_Id] = WF.[Workflow_Id]
		   LEFT JOIN [dbo].[Environments] ENV
				  ON BE.[Environment_Id] = ENV.[Environment_Id]
		   INNER JOIN [dbo].[Builds] B
				  ON BE.[Application_Id] = B.[Application_Id]
				 AND BE.[Release_Number] = B.[Release_Number]
				 AND BE.[Build_Number] = B.[Build_Number]
		   INNER JOIN [dbo].[Applications] A
				  ON A.[Application_Id] = BE.[Application_Id]
		   INNER JOIN [BuildPromotions] BP
				  ON BE.[Application_Id] = BP.[Application_Id]
				 AND BE.[Release_Number] = BP.[Release_Number]
				 AND BE.[Build_Number] = BP.[Build_Number]
				 AND BE.[Environment_Id] = BP.[Environment_Id]

GO


GO
IF OBJECT_ID('[BuildExecution_PlanActionLogEntries_Extended]') IS NOT NULL
	DROP VIEW [BuildExecution_PlanActionLogEntries_Extended]
GO

CREATE VIEW [BuildExecution_PlanActionLogEntries_Extended]
AS

	SELECT LOGENT.*

		  ,PLANACT.[ExecutionStarted_Date]
		  ,PLANACT.[ExecutionEnded_Date]
		  ,PLANACT.[ExecutionStatus_Name]
		  ,PLANACT.[ExecutionAction_Description]
		  ,PLANACT.[Execution_Id]
		  ,PLANACT.[Original_PlanActionGroup_Id]
		  ,PLANACT.[Original_Action_Sequence]

		  ,XPLAN.[PlanExecution_Sequence]
		  ,XPLAN.[Deployable_Name]
		  ,XPLAN.[ActionGroup_Name]
		  ,PLANACT.[Server_Id]

	  FROM [BuildExecution_PlanActionLogEntries] LOGENT

		   INNER JOIN [BuildExecution_PlanActions] PLANACT
				   ON LOGENT.[BuildExecution_PlanAction_Id] = PLANACT.[BuildExecution_PlanAction_Id]

		   INNER JOIN [BuildExecution_Plans] XPLAN
				   ON PLANACT.[Execution_Id] = XPLAN.[Execution_Id]
				  AND PLANACT.[Original_PlanActionGroup_Id] = XPLAN.[Original_PlanActionGroup_Id]

GO


GO

IF OBJECT_ID('[BuildExecution_PlanActions_Extended]') IS NOT NULL DROP VIEW [BuildExecution_PlanActions_Extended]
GO

CREATE VIEW [BuildExecution_PlanActions_Extended] AS

SELECT BEPA.[BuildExecution_PlanAction_Id]
      ,BEPA.[Execution_Id]
      ,BEPA.[Original_PlanActionGroup_Id]
      ,BEPA.[Original_Action_Sequence]
      ,BEPA.[ExecutionStarted_Date]
      ,BEPA.[ExecutionEnded_Date]
      ,BEPA.[ExecutionStatus_Name]
      ,BEPA.[ExecutionAction_Description]

      ,BEP.[PlanExecution_Sequence]
	  ,BEP.[Deployable_Name]
	  ,BEP.[ActionGroup_Name]
	  ,[PlanSkipped_Indicator] = BEP.[Skipped_Indicator]

      ,PAG.[PlanActionGroup_Sequence]
      ,P.[Plan_Id]
      ,PA.[Action_Sequence]
      ,PA.[Action_Description]
      ,PA.[Action_Configuration]
      ,PA.[ResumeNextOnFailure_Indicator]
      ,PA.[Active_Indicator]
      ,PA.[Retry_Count]
      ,PA.[LogFailureAsWarning_Indicator]
	  
	  ,BEPA.[Attempt_Sequence]

      ,[Server_Id] = COALESCE(BEPA.[Server_Id], VAR_SRV.[Server_Id])
	  ,[Server_Name] = COALESCE(SRV.[Server_Name], VAR_SRV.[Server_Name])
	  ,BEPA.[Original_Server_Name]
	  
	  ,[Target_Server_Id] = COALESCE(PA.[Target_Server_Id], TARGET_VAR_SRV.[Server_Id])
	  ,[Target_Server_Name] = COALESCE(TARGET_SRV.[Server_Name], TARGET_VAR_SRV.[Server_Name])

      ,A.[ApplicationGroup_Id]
      ,BE.[Application_Id]
      ,BE.[Release_Number]
      ,BE.[Build_Number]
      ,[Execution_CreatedBy_User_Name] = BE.[CreatedBy_User_Name]
      ,[Execution_CreatedOn_Date] = BE.[CreatedOn_Date]
	  ,R.[Workflow_Id]
	  
      ,PAG.[Deployable_Id]
      
      ,[Numeric_Release_Number] = R.[Release_Sequence]

	  ,[HasTestResults_Indicator] =
			CASE WHEN EXISTS (
				SELECT *
				  FROM [BuildTestResults] _BTR
				 WHERE _BTR.[BuildExecution_PlanAction_Id] = BEPA.[BuildExecution_PlanAction_Id]
				 )
				THEN 'Y'
				ELSE 'N' END

      ,[WarningLogEntry_Indicator] = BEPA.[WarningLogEntry_Indicator]
      ,PA.[ExtensionConfiguration_Id]
      ,A.[VariableSupport_Code]
      ,PAG.[Environment_Id]

  FROM [BuildExecution_PlanActions] BEPA
       INNER JOIN [BuildExecution_Plans] BEP
               ON BEP.[Execution_Id] = BEPA.[Execution_Id]
              AND BEP.[Original_PlanActionGroup_Id] = BEPA.[Original_PlanActionGroup_Id]
       INNER JOIN [BuildExecutions] BE
               ON BE.[Execution_Id] = BEPA.[Execution_Id]
       INNER JOIN [Applications] A
               ON A.[Application_Id] = BE.[Application_Id]
	   INNER JOIN [Releases] R
	           ON R.[Application_Id] = BE.[Application_Id]
	          AND R.[Release_Number] = BE.[Release_Number]
        LEFT JOIN [PlanActionGroups] PAG
	           ON PAG.[PlanActionGroup_Id] = BEPA.[Original_PlanActionGroup_Id]
        LEFT JOIN [Plans] P
               ON P.[Plan_Id] = PAG.[ActionGroup_Id]
			  AND BE.[ExecutionStatus_Name] IN ('Pending', 'Executing')
	    LEFT JOIN [Deployables] DPL
			   ON PAG.[Deployable_Id] = DPL.[Deployable_Id]
        LEFT JOIN [PlanActions] PA
               ON PA.[Plan_Id] = P.[Plan_Id]
              AND PA.[Action_Sequence] = BEPA.[Original_Action_Sequence]

		LEFT JOIN [Servers] SRV
			   ON SRV.[Server_Id] = BEPA.[Server_Id]
		LEFT JOIN [BuildExecution_PlanActionVariableValues] BEPAVV
		       ON BEPAVV.[BuildExecution_PlanAction_Id] = BEPA.[BuildExecution_PlanAction_Id]
		      AND BEPAVV.[Variable_Name] = COALESCE(PA.[Server_Variable_Name], PAG.[Server_Variable_Name])
		LEFT JOIN [Servers] VAR_SRV
		       ON VAR_SRV.[Server_Name] = BEPAVV.[Value_Text]
		
		LEFT JOIN [Servers] TARGET_SRV
		       ON TARGET_SRV.[Server_Id] = PA.[Server_Id]
		LEFT JOIN [BuildExecution_PlanActionVariableValues] TARGET_BEPAVV
		       ON TARGET_BEPAVV.[BuildExecution_PlanAction_Id] = BEPA.[BuildExecution_PlanAction_Id]
		      AND TARGET_BEPAVV.[Variable_Name] = PA.[Target_Server_Variable_Name]
		LEFT JOIN [Servers] TARGET_VAR_SRV
		       ON TARGET_VAR_SRV.[Server_Name] = TARGET_BEPAVV.[Value_Text]
GO


GO

IF OBJECT_ID('[BuildExecution_Plans_Extended]') IS NOT NULL DROP VIEW [BuildExecution_Plans_Extended]
GO

CREATE VIEW [BuildExecution_Plans_Extended] AS

SELECT 
       BEP.[Execution_Id]
      ,BEP.[Original_PlanActionGroup_Id]
      ,BEP.[PlanExecution_Sequence]
	  ,BEP.[ActionGroup_Name]
	  ,BEP.[Skipped_Indicator]

	  ,[AllPlanActions_Succeeded_Indicator] =
		CASE WHEN EXISTS ( SELECT *
							 FROM [BuildExecution_PlanActions] _BEPA
							WHERE _BEPA.[Execution_Id] = BEP.[Execution_Id]
							  AND _BEPA.[Original_PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
							  AND _BEPA.[ExecutionStatus_Name] IN ('Failed', 'Canceled') )
			THEN 'N'
			ELSE 'Y' END
							

      ,BE.[Application_Id]
      ,BE.[Release_Number]
      ,BE.[Build_Number]
      ,R.[Workflow_Id]
      ,WS.[Step_Sequence]
      ,BE.[ExecutionStart_Date]
      ,BE.[ExecutionStatus_Name]
      ,PAG.[PlanActionGroup_Sequence]

      ,D.[Deployable_Id]
      ,D.[Deployable_Name]

      ,E.[Environment_Id]
      ,E.[Environment_Name]

      ,[Executed_Actions_Count] =
           (SELECT COUNT(*) FROM [BuildExecution_PlanActions] _BEPA
             WHERE _BEPA.[Execution_Id] = BEP.[Execution_Id]
               AND _BEPA.[Original_PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id])

      ,[Last_ExecutionAction_Description] = BEPA_LST.[ExecutionAction_Description]

  FROM [BuildExecution_Plans] BEP
        LEFT JOIN [PlanActionGroups] PAG
               ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
        LEFT JOIN [Plans] P
               ON P.[Plan_Id] = PAG.[ActionGroup_Id]
       INNER JOIN [BuildExecutions] BE
               ON BE.[Execution_Id] = BEP.[Execution_Id] 
	   INNER JOIN [dbo].[Releases] R
	           ON R.[Application_Id] = BE.[Application_Id]
	          AND R.[Release_Number] = BE.[Release_Number]
       INNER JOIN [WorkflowSteps] WS
               ON WS.[Workflow_Id] = R.[Workflow_Id]
              AND WS.[Environment_Id] = BE.[Environment_Id]
        LEFT JOIN [Deployables] D
               ON D.[Deployable_Id] = PAG.[Deployable_Id]
       INNER JOIN [Environments] E
               ON E.[Environment_Id] = WS.[Environment_Id]
        LEFT JOIN [BuildExecution_PlanActions] BEPA_LST
               ON BEPA_LST.[Execution_Id] = BEP.[Execution_Id] 
              AND BEPA_LST.[Original_PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
              AND BEPA_LST.[Original_Action_Sequence] =
                  (
                    SELECT MAX([Original_Action_Sequence])
                      FROM [BuildExecution_PlanActions] 
                     WHERE [Execution_Id] = BEP.[Execution_Id]
                       AND [Original_PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
                       AND [PlanExecution_Sequence] = BEP.[PlanExecution_Sequence]
                  )
   
GO


GO

IF OBJECT_ID('[BuildOutputs_Extended]') IS NOT NULL DROP VIEW [BuildOutputs_Extended]
GO

CREATE VIEW [BuildOutputs_Extended] AS


    SELECT  BO.[Execution_Id]
           ,BO.[Output_Sequence]
           ,BO.[Output_Name]
           ,BO.[BuildExecution_PlanAction_Id]
           ,BO.[Output_Bytes]
           ,BO.[OutputType_Code]
           
           ,E.[Application_Id]
           ,E.[Release_Number]
           ,E.[Build_Number]
           ,E.[Environment_Id]
           ,E.[ExecutionStart_Date]
           ,E.[ExecutionStatus_Name]
       
      FROM [BuildOutputs] BO
INNER JOIN [BuildExecutions] E
        ON E.[Execution_Id] = BO.[Execution_Id]


GO


GO

IF OBJECT_ID('[BuildPromotions_Extended]') IS NOT NULL DROP VIEW [BuildPromotions_Extended]
GO

CREATE VIEW [BuildPromotions_Extended] AS

SELECT BP.[Application_Id]
      ,BP.[Release_Number]
      ,BP.[Build_Number]
      ,W.[Workflow_Id]
      ,BP.[Environment_Id]
      ,BP.[Promoted_Date]
      ,BP.[PromotedBy_Name]
      ,BP.[Comments_Text]
      ,BP.[PromotionStatus_Name]
	  ,BP.[StatusOverriddenBy_User_Name]
	  ,BP.[StatusOverriddenReason_Text]
	  ,BP.[StatusOverridden_Date]
	  ,BP.[Forced_Indicator]

      ,A.[Application_Name]

      ,WS.[Step_Sequence]

      ,B.[BuildStatus_Name]

      ,W.[Workflow_Name]

      ,E.[Environment_Name]

	  ,BE.[Execution_Id] AS [Last_Execution_Id]
	  ,BE.[ExecutionStart_Date] AS [Last_ExecutionStart_Date]
	  ,BE.[ExecutionStatus_Name] AS [Last_ExecutionStatus_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])

  FROM [BuildPromotions] BP
       INNER JOIN [Applications] A
               ON A.[Application_Id] = BP.[Application_Id]
       INNER JOIN [Builds] B
               ON B.[Application_Id] = BP.[Application_Id]
              AND B.[Release_Number] = BP.[Release_Number]
              AND B.[Build_Number] = BP.[Build_Number]
       INNER JOIN [Releases] R
               ON R.[Application_Id] = BP.[Application_Id]
              AND R.[Release_Number] = BP.[Release_Number]
       INNER JOIN [Workflows] W
               ON W.[Workflow_Id] = R.[Workflow_Id]
       INNER JOIN [WorkflowSteps] WS
               ON WS.[Workflow_Id] = W.[Workflow_Id]
              AND WS.[Environment_Id] = BP.[Environment_Id]
       INNER JOIN [Environments] E        
               ON E.[Environment_Id] = WS.[Environment_Id]
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = 
	             (
	              SELECT TOP 1 [Execution_Id]
	                FROM [BuildExecutions] _BE
	               WHERE _BE.[Application_Id] = BP.[Application_Id]
	                 AND _BE.[Release_Number] = BP.[Release_Number]
	                 AND _BE.[Build_Number] = BP.[Build_Number]
	                 AND _BE.[Environment_Id] = BP.[Environment_Id]
	               ORDER BY [ExecutionStart_Date] DESC
	             )

GO


GO
IF OBJECT_ID('[BuildPromotions_RequiredApprovals]') IS NOT NULL DROP VIEW [BuildPromotions_RequiredApprovals]
GO

CREATE VIEW [BuildPromotions_RequiredApprovals] AS

SELECT B.[Application_Id]
      ,B.[Release_Number]
      ,B.[Build_Number]

      ,PAR.[Environment_Id]
      ,R.[Workflow_Id]
      ,[Principal_Name] = PAR.[User_Name]
      ,PAR.[Approval_Sequence]
      ,PAR.[ApprovalType_Code]
      ,PAR.[Or_Indicator]
      ,PAR.[RequiredInGroup_Count]
      ,PAR.[Approval_Description]
      ,[ApprovalsReceived_Count] = 
         (
             SELECT COUNT(*)
               FROM [BuildApprovals] _BA
                    INNER JOIN [Releases] _R
                            ON _R.[Application_Id] = _BA.[Application_Id]
                           AND _R.[Release_Number] = _BA.[Release_Number]
              WHERE PAR.[Workflow_Id] = _R.[Workflow_Id]
               AND PAR.[Environment_Id] = _BA.[Environment_Id]
               AND PAR.[Approval_Description] = _BA.[Approval_Description]
               AND B.[Build_Number] = _BA.[Build_Number]
               AND B.[Release_Number] = _BA.[Release_Number]
         )
      ,[ApprovalComplete_Indicator] = CASE 
           WHEN COALESCE([RequiredInGroup_Count],1) <=
           (
             SELECT COUNT(*)
               FROM [BuildApprovals] _BA
                    INNER JOIN [Releases] _R
                            ON _R.[Application_Id] = _BA.[Application_Id]
                           AND _R.[Release_Number] = _BA.[Release_Number]
              WHERE PAR.[Workflow_Id] = _R.[Workflow_Id]
               AND PAR.[Environment_Id] = _BA.[Environment_Id]
               AND PAR.[Approval_Description] = _BA.[Approval_Description]
               AND B.[Build_Number] = _BA.[Build_Number]
               AND B.[Release_Number] = _BA.[Release_Number]
            ) THEN 'Y' ELSE 'N' END

  FROM [Builds] B
       INNER JOIN [Releases] R
               ON R.[Application_Id] = B.[Application_Id]
              AND R.[Release_Number] = B.[Release_Number]
       INNER JOIN [PromotionApprovalRequirements] PAR
               ON PAR.[Workflow_Id] = R.[Workflow_Id]
 WHERE [BuildStatus_Name] = 'Active'

GO
IF OBJECT_ID('[BuildPromotions_RequiredApprovalsByUser]') IS NOT NULL DROP VIEW [BuildPromotions_RequiredApprovalsByUser]
GO

CREATE VIEW [BuildPromotions_RequiredApprovalsByUser] AS

SELECT B.[Application_Id]
      ,A.[Application_Name]
      ,B.[Release_Number]
      ,B.[Build_Number]

      ,PAR.[Environment_Id]
      ,E.[Environment_Name]
      ,R.[Workflow_Id]
      ,[Principal_Name] = PAR.[User_Name]
      ,PAR.[Approval_Sequence]
      ,PAR.[ApprovalType_Code]
      ,PAR.[Or_Indicator]
      ,PAR.[RequiredInGroup_Count]
      ,PAR.[Approval_Description]

      ,BA.[Approval_Id]
      ,[ApprovedBy_User_Name] = BA.[User_Name]
      ,[Approval_Comments_Text] = BA.[Comments_Text]
      ,[ApprovedBy_Group_Name] = BA.[Group_Name]
      ,BA.[Received_Date]

      ,BA.[ReceivedInGroup_Sequence] 
      ,[RequiredInGroup_Sequence] = CASE WHEN PAR.[RequiredInGroup_Count] IS NULL THEN NULL ELSE N.[Number] END

  FROM [Builds] B
       INNER JOIN [Applications] A
               ON A.[Application_Id] = B.[Application_Id]
       INNER JOIN [Releases] R
               ON R.[Application_Id] = B.[Application_Id]
              AND R.[Release_Number] = B.[Release_Number]
       INNER JOIN [PromotionApprovalRequirements] PAR
               ON PAR.[Workflow_Id] = R.[Workflow_Id]
       INNER JOIN [Environments] E
              ON E.[Environment_Id] = PAR.[Environment_Id]
       INNER JOIN [Numbers] N
               ON N.[Number] BETWEEN 1 AND COALESCE(PAR.[RequiredInGroup_Count],1)
       LEFT  JOIN (
                    SELECT *, [ReceivedInGroup_Sequence] = RANK() OVER 
                              (
                               PARTITION BY 
                                _BA.[Application_Id]
                                ,_BA.[Release_Number]
                                ,_BA.[Build_Number]
                                ,_BA.[Environment_Id]
                                ,_BA.[Group_Name]
                                ,_BA.[Approval_Description]
                               ORDER BY _BA.[Received_Date]
                              )
                      FROM [BuildApprovals] _BA
                  ) BA
              ON PAR.[Environment_Id] = BA.[Environment_Id]
             AND PAR.[Approval_Description] = BA.[Approval_Description]
             AND B.[Application_Id] = BA.[Application_Id]
             AND B.[Build_Number] = BA.[Build_Number]
             AND B.[Release_Number] = BA.[Release_Number]
             AND BA.[ReceivedInGroup_Sequence] = N.[Number]
               
 WHERE [BuildStatus_Name] = 'Active'

GO

IF OBJECT_ID('[BuildSchedules_Extended]') IS NOT NULL DROP VIEW [BuildSchedules_Extended]
GO

CREATE VIEW [BuildSchedules_Extended] AS

    SELECT
           BS.[Schedule_Id]
          ,BS.[Schedule_Name]
          ,BS.[Application_Id]
          ,A.[Application_Name]
          ,BS.[Active_Indicator]
          ,BS.[CreatedBy_User_Name]
          ,BS.[CreatedOn_Date]
          ,BS.[ModifiedBy_User_Name]
          ,BS.[ModifiedOn_Date]
          ,BS.[Schedule_Type]
          ,BS.[StartTime_Minutes]
          ,BS.[EndTime_Minutes]
          ,BS.[DailyInterval_Minutes]
          ,BS.[Recurrence_Type]
          ,BS.[RecurrenceFrequency_Value]
          ,BS.[RecurrenceWeeklyDay_Mask]
          ,BS.[RecurrenceMonthlyDay_Value]
          ,BS.[RecurrenceMonthlyRelativity_Type]
          ,BS.[RecurrenceMonthlyRelativityDay_Type]
          ,BS.[RecurrenceMonthlyRelativityWeek_Type]
          ,BS.[RecurrenceMonthlyRelativityWeekDay_Value]
          ,BS.[Url_AllowedIP_CSV]
          ,BS.[Url_Username_Text]
          ,BS.[Url_Password_Text]
          ,BS.[SourceControl_Path_Name]
          ,BS.[SourceControl_Provider_Id]
          ,[SourceControl_Provider_Name] = P.[Provider_Name]
          ,BS.[SourceControl_QuietPeriod_Minutes]
          ,BS.ReleaseFilter_ReleaseType_Code
          ,BS.Releasefilter_Workflow_Id


      FROM [BuildSchedules] BS

INNER JOIN [Applications] A
        ON A.[Application_Id] = BS.[Application_Id]

LEFT JOIN [Providers] P
        ON P.[Provider_Id] = BS.[SourceControl_Provider_Id]

GO


GO
IF OBJECT_ID('[BuildSchedules_Scheduled]') IS NOT NULL DROP VIEW [BuildSchedules_Scheduled]
GO

CREATE VIEW BuildSchedules_Scheduled AS

SELECT
  [Schedule_Id],

  A.[Application_Id],
  A.[Application_Name],

  [Schedule_Name],

  [RunAt_Date] = CASE WHEN [Schedule_Type] = 'Recurring' THEN BS.[CreatedOn_Date] ELSE NULL END,
  [RunAt_Minutes] = [StartTime_Minutes],
  
  [Url_AllowedIP_CSV],
  [Url_Username_Text],
  [Url_Password_Text],
  [SourceControl_Path_Name],
  [SourceControl_Provider_Id],
  
  [Schedule_Type],
  [SourceControl_QuietPeriod_Minutes],
  [ReleaseFilter_ReleaseType_Code],
  [Releasefilter_Workflow_Id]
FROM 
  [BuildSchedules] BS
  JOIN [Applications] A ON A.[Application_Id] = BS.[Application_Id]

 WHERE [Schedule_Type] <> 'Recurring'
	AND [Active_Indicator] = 'Y'

UNION ALL


SELECT
  [Schedule_Id],

  A.[Application_Id],
  A.[Application_Name],

  [Schedule_Name],

  [RunAt_Date] = [Date],
  [RunAt_Minutes] =[Number],
  [Url_AllowedIP_CSV],
  [Url_Username_Text],
  [Url_Password_Text],
  [SourceControl_Path_Name],
  [SourceControl_Provider_Id],
  [Schedule_Type],
  [SourceControl_QuietPeriod_Minutes],
  [ReleaseFilter_ReleaseType_Code],
  [Releasefilter_Workflow_Id]
FROM 
  [BuildSchedules] BS
  JOIN [Applications] A ON A.[Application_Id] = BS.[Application_Id]
  CROSS JOIN [Calendar] C
  CROSS JOIN [Numbers] N
WHERE 
  [Active_Indicator] = 'Y' AND
  [Number] BETWEEN [StartTime_Minutes] AND COALESCE([EndTime_Minutes],[StartTime_Minutes]) AND
  ([StartTime_Minutes] + [Number]) % COALESCE([DailyInterval_Minutes],1) = 0 AND
  [Schedule_Type] = 'Recurring' AND
  ( (
      [Recurrence_Type] = 'Monthly' AND
      ( (
          [RecurrenceMonthlyDay_Value] IS NOT NULL AND
            [RecurrenceMonthlyDay_Value] = DATEPART(d,[Date])
        ) OR (
          [RecurrenceMonthlyDay_Value] IS NULL AND
          ( (
               [RecurrenceMonthlyRelativity_Type] = 'Day' AND 
               ( /* [RecurrenceMonthlyRelativityDay_Type] can only be 'Last' */
                  DATEPART(m,[Date]+1) <> DATEPART(m,[Date])
               )
            ) OR (
               [RecurrenceMonthlyRelativity_Type] = 'Week' AND 
               (  [RecurrenceMonthlyRelativityWeekDay_Value] = 
                    DATEPART(weekday,[Date]) AND
                  CEILING(DATEPART(day,[Date])/7.0) =
                    CASE [RecurrenceMonthlyRelativityWeek_Type]
                      WHEN 'First'	THEN 1
                      WHEN 'Second' THEN 2
                      WHEN 'Third'	THEN 3
                      WHEN 'Fourth' THEN 4
                      WHEN 'Last' THEN CASE 
                        WHEN DATEPART(month,[Date]+7) <> DATEPART(month,[Date]) 
                             THEN CEILING(DATEPART(day,[Date])/7.0)
                        ELSE -1 END
                    END  
               )
            )
          )
        )
      ) AND
      0 = (dbo.ABS_MONTH_NUMBER([Date]) - dbo.ABS_MONTH_NUMBER(BS.[CreatedOn_Date])) % [RecurrenceFrequency_Value]
    ) OR (
      [Recurrence_Type] = 'Weekly' AND
      'Y' = SUBSTRING([RecurrenceWeeklyDay_Mask],DATEPART(dw,[Date]),1) AND
      0 = (dbo.ABS_WEEK_NUMBER([Date]) - dbo.ABS_WEEK_NUMBER(BS.[CreatedOn_Date])) % [RecurrenceFrequency_Value]
    ) OR (
      [Recurrence_Type] = 'Daily' AND
      0 = (CAST([Date] AS INT) - CAST(BS.[CreatedOn_Date] AS INT)) % [RecurrenceFrequency_Value]
    )
  )
GO


GO
IF OBJECT_ID('[BuildScheduleVariableDefinitions_Extended]') IS NOT NULL DROP VIEW [BuildScheduleVariableDefinitions_Extended]
GO

CREATE VIEW [BuildScheduleVariableDefinitions_Extended] AS

SELECT BSVD.[Schedule_Id],
       BSVD.[VariableDeclaration_Id],
       VDEC.[Variable_Name],
       VDEC.[Scope_Code],
       VDEC.[Required_Indicator],
       VDEC.[DefaultValue_Text],
       VDEC.[Variable_Configuration],
       BSVD.[Value_Text],
       [Application_Id] = VDEC.[Scoped_Application_Id]
  FROM [BuildScheduleVariableDefinitions] BSVD
       INNER JOIN [VariableDeclarations] VDEC
               ON BSVD.[VariableDeclaration_Id] = VDEC.[VariableDeclaration_Id]
GO

GO

IF OBJECT_ID('[Builds_Extended]') IS NOT NULL DROP VIEW [Builds_Extended]
GO

CREATE VIEW [Builds_Extended] AS


SELECT B.[Release_Number]
      ,B.[Build_Number]
      ,B.[BuildStatus_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
      ,R.[Target_Date]
      ,[Sortable_Build_Number] = dbo.SortableBuildNumber(R.[Application_Id], B.[Build_Number])

      ,R.[ReleaseStatus_Name]
      ,[Numeric_Release_Number] = R.[Release_Sequence]
      ,[Release_Notes_Text] = R.[Notes_Text]
      ,B.[Rejection_Notes]
      ,B.[RejectedBy_User_Name]
      ,B.[Rejected_Date]
      ,B.[CreatedBy_User_Name]
      ,B.[CreatedOn_Date]
      ,B.[ModifiedBy_User_Name]
      ,B.[ModifiedOn_Date]

	  ,A.[Application_Id]
	  ,A.[Application_Name]

	  ,W.[Workflow_Id]
	  ,W.[Workflow_Name]
      ,W.[AutoDeployRelease_Indicator]
      ,W.[AllowOutOfSequencePromotions_Indicator]
      ,W.[AutoCancelReleases_Indicator]

	  ,WS_1ST.[Step_Sequence] AS [First_Step_Sequence]
	  ,E_1ST.[Environment_Id] AS [First_Environment_Id]
	  ,E_1ST.[Environment_Name] AS [First_Environment_Name]

	  ,WS_CUR.[Step_Sequence] AS [Current_Step_Sequence]
	  ,E_CUR.[Environment_Id] AS [Current_Environment_Id]
	  ,E_CUR.[Environment_Name] AS [Current_Environment_Name]
	  ,BP_CUR.[PromotedBy_Name] AS [Current_PromotedBy_Name]
	  ,BP_CUR.[Promoted_Date] AS [Current_Promoted_Date]
	  ,BP_CUR.[PromotionStatus_Name] AS [Current_PromotionStatus_Name]
	  ,BP_CUR.[Environment_Id] AS [Current_Promotion_Environment_Id]
      ,BP_CUR.[Comments_Text] AS [Current_Promotion_Comments_Text]

	  ,[TotalExecutions_Count] =
	       (SELECT COUNT([Execution_Id])
			  FROM [dbo].[BuildExecutions] _BE
	         WHERE _BE.[Application_Id] = B.[Application_Id]
			   AND _BE.[Release_Number] = B.[Release_Number]
			   AND _BE.[Build_Number] = B.[Build_Number])

	  ,BE.[Execution_Id] AS [Current_Execution_Id]
	  ,BE_E.[Environment_Id] AS [Current_Execution_Environment_Id]
	  ,BE_E.[Environment_Name] AS [Current_Execution_Environment_Name]
	  ,BE.[ExecutionStart_Date] AS [Current_ExecutionStart_Date]
	  ,BE.[ExecutionStatus_Name] AS [Current_ExecutionStatus_Name]

	  ,WS_NXT.[Step_Sequence] AS [Next_Step_Sequence]
	  ,E_NXT.[Environment_Id] AS [Next_Environment_Id]
	  ,E_NXT.[Environment_Name] AS [Next_Environment_Name]

	  ,WS_FNL.[Step_Sequence] AS [Final_Step_Sequence]
	  ,E_FNL.[Environment_Id] AS [Final_Environment_Id]
	  ,E_FNL.[Environment_Name] AS [Final_Environment_Name]

	  ,[CurrentExecution_WarningLogEntry_Indicator] = BE.[WarningLogEntry_Indicator]

  FROM [Builds] B
       INNER JOIN [Releases] R
               ON B.[Application_Id] = R.[Application_Id]
              AND B.[Release_Number] = R.[Release_Number]
       INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	   INNER JOIN [Workflows] W
               ON R.[Workflow_Id] = W.[Workflow_Id]

       LEFT JOIN [WorkflowSteps] WS_1ST 
              ON WS_1ST.[Workflow_Id] = W.[Workflow_Id]
             AND WS_1ST.[Step_Sequence] =
                  (SELECT MIN(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id])
       LEFT JOIN [Environments] E_1ST
              ON E_1ST.[Environment_Id] = WS_1ST.[Environment_Id]

       LEFT JOIN [WorkflowSteps] WS_CUR
              ON WS_CUR.[Workflow_Id] = W.[Workflow_Id]
             AND WS_CUR.[Step_Sequence] =
                  (
					SELECT MAX(_WS.[Step_Sequence])
					  FROM [BuildPromotions] _BP
					  JOIN [WorkflowSteps] _WS
						ON R.[Workflow_Id] = _WS.[Workflow_Id]
					   AND _BP.[Environment_Id] = _WS.[Environment_Id]
					 WHERE _BP.[Application_Id] = B.[Application_Id]
					   AND _BP.[Release_Number] = B.[Release_Number]
					   AND _BP.[Build_Number] = B.[Build_Number]
				  )
	   LEFT JOIN [BuildPromotions] BP_CUR
			  ON B.[Application_Id] = BP_CUR.[Application_Id]
			 AND B.[Release_Number] = BP_CUR.[Release_Number]
			 AND B.[Build_Number] = BP_CUR.[Build_Number]
			 AND BP_CUR.[Promoted_Date] =
				 ( SELECT TOP 1 [Promoted_Date]
					 FROM [BuildPromotions] _BP_CUR
					WHERE _BP_CUR.[Application_Id] = B.[Application_Id]
					  AND _BP_CUR.[Release_Number] = B.[Release_Number]
					  AND _BP_CUR.[Build_Number] = B.[Build_Number]
				   ORDER BY [Promoted_Date] DESC )
       LEFT JOIN [Environments] E_CUR
              ON BP_CUR.[Environment_Id] = E_CUR.[Environment_Id]

	   /* Last (most recent) promotion */
	   LEFT JOIN [BuildPromotions] BP_LAST
			  ON B.[Application_Id] = BP_LAST.[Application_Id]
			 AND B.[Release_Number] = BP_LAST.[Release_Number]
			 AND B.[Build_Number] = BP_LAST.[Build_Number]
			 AND BP_LAST.[Promoted_Date] = 
				( SELECT TOP 1 [Promoted_Date]
					FROM [BuildPromotions] _BP_LAST
				   WHERE _BP_LAST.[Application_Id] = B.[Application_Id]
					 AND _BP_LAST.[Release_Number] = B.[Release_Number]
					 AND _BP_LAST.[Build_Number] = B.[Build_Number]
				  ORDER BY [Promoted_Date] DESC )

       LEFT JOIN [WorkflowSteps] WS_NXT
              ON WS_NXT.[Workflow_Id] = W.[Workflow_Id]
             AND B.[BuildStatus_Name] <> 'Rejected'
             AND WS_NXT.[Step_Sequence] =
                  (SELECT MIN(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id]
                      AND WS.[Step_Sequence] > COALESCE(WS_CUR.[Step_Sequence],0) )
       LEFT JOIN [Environments] E_NXT
              ON E_NXT.[Environment_Id] = WS_NXT.[Environment_Id]

	   LEFT JOIN [WorkflowSteps] WS_FNL
              ON WS_FNL.[Workflow_Id] = W.[Workflow_Id]
             AND WS_FNL.[Step_Sequence] =
                  (SELECT MAX(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id])
	   LEFT JOIN [Environments] E_FNL
              ON E_FNL.[Environment_Id] = WS_FNL.[Environment_Id]
	
	   LEFT JOIN [BuildExecutions] BE
	          ON BE.[Execution_Id] = 
	             (
	              SELECT TOP 1 [Execution_Id]
	                FROM [BuildExecutions] _BE
	               WHERE _BE.[Application_Id] = BP_CUR.[Application_Id]
	                 AND _BE.[Release_Number] = BP_CUR.[Release_Number]
	                 AND _BE.[Build_Number] = BP_CUR.[Build_Number]
	               ORDER BY [ExecutionStart_Date] DESC
	             )
	   LEFT JOIN [Environments] BE_E
	          ON BE_E.[Environment_Id] = BE.[Environment_Id]
GO


GO
IF OBJECT_ID('[Builds_RequiredPromotions]') IS NOT NULL DROP VIEW [Builds_RequiredPromotions]
GO

CREATE VIEW [Builds_RequiredPromotions] AS

SELECT B.[Application_Id]
      ,B.[Release_Number]
      ,B.[Build_Number]
      ,B.[BuildStatus_Name]

      ,R.[Workflow_Id]
      ,R.[ReleaseStatus_Name]

      ,E.[Environment_Id]
      ,[Step_Sequence] = COALESCE(WS.[Step_Sequence], ROW_NUMBER() OVER(ORDER BY [Promoted_Date]))

      ,E.[Environment_Name]

      ,BP.[Promoted_Date]
      ,BP.[PromotedBy_Name]
      ,BP.[Comments_Text]
      ,BP.[PromotionStatus_Name]
      ,BP.[StatusOverriddenBy_User_Name]
      ,BP.[StatusOverriddenReason_Text]
      ,BP.[StatusOverridden_Date]

  FROM [Builds] B
       INNER JOIN [Releases] R
               ON R.[Application_Id] = B.[Application_Id]
              AND R.[Release_Number] = B.[Release_Number]
       LEFT  JOIN [WorkflowSteps] WS
               ON WS.[Workflow_Id] = R.[Workflow_Id]
              AND R.[ReleaseStatus_Name] = 'Active'
       LEFT  JOIN [BuildPromotions] BP
               ON BP.[Application_Id] = B.[Application_Id]
              AND BP.[Release_Number] = B.[Release_Number]
              AND BP.[Build_Number] = B.[Build_Number]
              AND BP.[Environment_Id] = COALESCE(WS.[Environment_Id],BP.[Environment_Id])
       INNER JOIN [Environments] E
               ON E.[Environment_Id] = COALESCE(WS.[Environment_Id],BP.[Environment_Id])


GO
IF OBJECT_ID('[BuildTestResults_AllGroups]') IS NOT NULL DROP VIEW [BuildTestResults_AllGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE VIEW [BuildTestResults_AllGroups] WITH SCHEMABINDING AS

SELECT BTR.[Group_Name],
       [Group_Count] = COUNT_BIG(*)
  FROM dbo.[BuildTestResults] BTR
 GROUP BY BTR.[Group_Name]

GO

CREATE UNIQUE CLUSTERED INDEX [IX__BuildTestResults_AllGroups]
	ON [BuildTestResults_AllGroups] ([Group_Name])

GO

GO

IF OBJECT_ID('[BuildTestResults_Extended]') IS NOT NULL DROP VIEW [BuildTestResults_Extended]
GO

CREATE VIEW [BuildTestResults_Extended] AS

SELECT BTR.[Execution_Id]
      ,BTR.[Group_Name]
      ,BTR.[Test_Sequence]
      ,BTR.[Test_Name]
      ,BTR.[TestStatus_Code]
      ,[TestPassed_Indicator] = CASE BTR.[TestStatus_Code] WHEN 'P' THEN 'Y' ELSE 'N' END
      ,BTR.[TestResult_Text]
      ,BTR.[TestStarted_Date]
      ,BTR.[TestEnded_Date]
      ,BTR.[BuildExecution_PlanAction_Id]
      ,BEPA.[Original_PlanActionGroup_Id]
      ,BEPA.[Original_Action_Sequence],
      A.[Application_Id],
      A.[Application_Name],
      R.[Release_Number],
      [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])

  FROM [BuildTestResults] BTR
	   INNER JOIN [BuildExecution_PlanActions] BEPA
			   ON BTR.[BuildExecution_PlanAction_Id] = BEPA.[BuildExecution_PlanAction_Id]
	   INNER JOIN [BuildExecutions] BE
	           ON BTR.[Execution_Id] = BE.[Execution_Id]
	   INNER JOIN [Releases] R
	           ON BE.[Application_Id] = R.[Application_Id]
	          AND BE.[Release_Number] = R.[Release_Number]
	   INNER JOIN [Applications] A
	           ON BE.[Application_Id] = A.[Application_Id]
GO


GO
IF OBJECT_ID('[BuildTestResults_TestGroupResults]') IS NOT NULL DROP VIEW [BuildTestResults_TestGroupResults]
GO

CREATE VIEW [BuildTestResults_TestGroupResults] AS

SELECT TR.[Execution_Id],
       TR.[Group_Name],
       BE.[Application_Id],
       BE.[Release_Number],
       BE.[Build_Number],
       COUNT(*) AS [Total_Tests], 
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'F'), 'I')) AS [Passed_Tests],
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'P'), 'I')) AS [Failed_Tests],
       COUNT(NULLIF(NULLIF(TR.[TestStatus_Code], 'P'), 'F')) AS [Inconclusive_Tests]
  FROM [BuildTestResults] TR
  INNER JOIN [BuildExecutions] BE
          ON BE.[Execution_Id] = TR.[Execution_Id]
 GROUP BY 
    TR.[Execution_Id], 
    TR.[Group_Name],
    BE.[Application_Id],
    BE.[Release_Number],
    BE.[Build_Number]

GO
IF OBJECT_ID('[ChangeControlEnvironments_Extended]') IS NOT NULL DROP VIEW [ChangeControlEnvironments_Extended]
GO

CREATE VIEW [ChangeControlEnvironments_Extended] AS

	SELECT CCE.[ChangeControl_Id],
	       CCE.[Environment_Id],
	       CC.[ChangeControl_Name],
	       E.[Environment_Name]
	  FROM [ChangeControlEnvironments] CCE
INNER JOIN [Environments] E
	    ON E.[Environment_Id] = CCE.[Environment_Id]
INNER JOIN [ChangeControls] CC
        ON CC.[ChangeControl_Id] = CCE.[ChangeControl_Id]

GO

GO
IF OBJECT_ID('[ChangeControlPerformances_Extended]') IS NOT NULL DROP VIEW [ChangeControlPerformances_Extended]
GO

CREATE VIEW [ChangeControlPerformances_Extended] AS

	SELECT CCP.[ChangeControl_Id],
	       CCP.[Environment_Id],
	       CCP.[PerformedBy_User_Name],
	       CCP.[Performed_Date],
	       CCP.[Notes_Text],
	       CCP.[Execution_Id],
	       E.[Environment_Name],
	       CC.[Application_Id],
	       CC.[Release_Number],
	       CC.[Group_Name],
	       CC.[Description_Text],
	       CC.[ChangeControl_Name],
	       CC.[Principal_Name],
	       CC.[PrincipalType_Code],
	       CC.[CreatedBy_User_Name],
	       CC.[ModifiedBy_User_Name],
	       CC.[CreatedOn_Date],
	       CC.[ModifiedOn_Date],
	       CC.[PerformanceType_Code],
	       CC.[Active_Indicator],
           [Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
	  FROM [ChangeControlPerformances] CCP
INNER JOIN [Environments] E
	    ON  CCP.[Environment_Id] = E.[Environment_Id]
INNER JOIN [ChangeControls] CC
        ON CCP.[ChangeControl_Id] = CC.[ChangeControl_Id]
           INNER JOIN [Releases] R
                   ON R.[Application_Id] = CC.[Application_Id]
                  AND R.[Release_Number] = CC.[Release_Number]
GO

GO
IF OBJECT_ID('[ChangeControlPerformances_ReleaseSummary]') IS NOT NULL DROP VIEW [ChangeControlPerformances_ReleaseSummary]
GO

CREATE VIEW [ChangeControlPerformances_ReleaseSummary] AS

	    SELECT  CC.[ChangeControl_Id],
	     E.[Environment_Name],
	     E.[Environment_Id],
       CCP.[ChangeControlPerformance_Id],
       CCP.[Notes_Text],
       CCP.[PerformedBy_User_Name],
       CCP.[Performed_Date],
           [ChangeControlPerformed_Indicator] = CASE 
                WHEN [ChangeControlPerformance_Id] IS NULL 
                    THEN 'N' 
                    ELSE 'Y' 
                END
          FROM [ChangeControlEnvironments] CCE
    INNER JOIN [ChangeControls] CC
            ON CC.[ChangeControl_Id] = CCE.[ChangeControl_Id]
    LEFT JOIN  [ChangeControlPerformances] CCP
            ON CCP.[ChangeControl_Id] = CC.[ChangeControl_Id]
            AND CCP.[Environment_Id] = CCE.[Environment_Id]
    INNER JOIN [Environments] E
            ON E.[Environment_Id] = CCE.[Environment_Id]

GO

GO

IF OBJECT_ID('[ConfigurationFileDeployments_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileDeployments_Extended]
GO

CREATE VIEW [ConfigurationFileDeployments_Extended] AS

SELECT CFD.[ConfigurationFileDeployment_Id]
      ,CFD.[Execution_Id]
      ,CFD.[ConfigurationFile_Id]
      ,CFD.[Instance_Name]
      ,CFD.[Version_Number]
      ,CFV.[Release_Number]
      ,CFD.[Server_Id]
      ,CFD.[DeployedTo_Path]
      ,CFD.[DeployedBy_User_Name]
      ,CFD.[DeployedOn_Date]

      ,CFI.[Environment_Id]

      ,CF.[Deployable_Id]
      ,CF.[FilePath_Text]
      ,[ConfigurationFile_Active_Indicator] = CF.[Active_Indicator]

      ,D.[Deployable_Name]
      ,D.[Application_Id]

      ,Sv.[Server_Name]
      ,E.[Environment_Name]

 FROM [ConfigurationFileDeployments] CFD
      INNER JOIN [ConfigurationFileInstances] CFI
              ON CFI.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
             AND CFI.[Instance_Name] = CFD.[Instance_Name]
      INNER JOIN [Environments] E
              ON E.[Environment_Id] = CFI.[Environment_Id]

      INNER JOIN [ConfigurationFiles] CF
              ON CF.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
      INNER JOIN [ConfigurationFileVersions] CFV
              ON CFV.[ConfigurationFile_Id] = CFD.[ConfigurationFile_Id]
             AND CFV.[Instance_Name] = CFD.[Instance_Name]
             AND CFV.[Version_Number] = CFD.[Version_Number]
      INNER JOIN [Deployables] D
              ON D.[Deployable_Id] = CF.[Deployable_Id]
      INNER JOIN [Servers] Sv
              ON Sv.[Server_Id] = CFD.[Server_Id]


GO



GO
IF OBJECT_ID('[ConfigurationFileInstances_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileInstances_Extended]
GO

CREATE VIEW [ConfigurationFileInstances_Extended] AS

SELECT CFI.[ConfigurationFile_Id]
      ,CFI.[Instance_Name]
      ,CFI.[Active_Indicator]
      ,CFI.[Environment_Id]
      ,E.[Environment_Name]

      ,CF.[Deployable_Id]
      ,CF.[FilePath_Text]
      ,[ConfigurationFile_Active_Indicator] = CF.[Active_Indicator]

      ,D.[Application_Id]

      ,[LatestVersion_Version_Number] = CFV.[Version_Number]
      ,[LatestVersion_VersionNotes_Text] = CFV.[VersionNotes_Text]
      ,[LatestVersion_File_Date] = CFV.[File_Date]
      ,[LatestVersion_File_Bytes] = CFV.[File_Bytes]
      ,[LatestVersion_Release_Number] = CFV.[Release_Number]
      ,[LatestVersion_CreatedOn_Date] = CFV.[CreatedOn_Date]
      ,[LatestVersion_CreatedByUser_Name] = CFV.[CreatedBy_User_Name]


      ,CFI.[Template_Indicator]
      ,CFI.[Template_Instance_Name]
      ,[Template_Instance_Count] = (    
                             SELECT CASE WHEN CFI.[Template_Indicator] = 'Y' THEN COUNT(*) ELSE NULL END
                             FROM [ConfigurationFileInstances] 
                             WHERE [ConfigurationFile_Id] = CFI.[ConfigurationFile_Id] 
                               AND [Active_Indicator] = 'Y'
                               AND [Template_Instance_Name] = CFI.[Instance_Name]
                               )

  FROM [ConfigurationFileInstances] CFI
       INNER JOIN [ConfigurationFiles] CF
               ON CFI.[ConfigurationFile_Id] = CF.[ConfigurationFile_Id]
       INNER JOIN [Deployables] D
               ON D.[Deployable_Id] = CF.[Deployable_Id]
       LEFT  JOIN (
                   SELECT [ConfigurationFile_Id]
                         ,[Instance_Name]
                         ,[Version_Number] = MAX([Version_Number])
                     FROM [ConfigurationFileVersions]
                    GROUP BY [ConfigurationFile_Id],[Instance_Name]
                  ) CFV_MAX
               ON CFV_MAX.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
              AND CFV_MAX.[Instance_Name] = CFI.[Instance_Name]
       LEFT  JOIN [ConfigurationFileVersions] CFV
               ON CFV.[ConfigurationFile_Id] = CFV_MAX.[ConfigurationFile_Id]
              AND CFV.[Instance_Name] = CFV_MAX.[Instance_Name]
              AND CFV.[Version_Number] = CFV_MAX.[Version_Number]
       INNER JOIN [Environments] E
               ON E.[Environment_Id] = CFI.[Environment_Id]
GO


GO

IF OBJECT_ID('[ConfigurationFiles_Extended]') IS NOT NULL DROP VIEW [ConfigurationFiles_Extended]
GO

CREATE VIEW [ConfigurationFiles_Extended] AS

SELECT C.[ConfigurationFile_Id]

      ,C.[Deployable_Id]
      ,C.[FilePath_Text]
      ,D.[Application_Id]
      ,C.[Active_Indicator]
      ,C.[CreatedBy_User_Name]
      ,C.[CreatedOn_Date]
      ,C.[ModifiedBy_User_Name]
      ,C.[ModifiedOn_Date]

      ,D.[Deployable_Name]

      ,[LatestDeployment_ConfigurationFileDeployment_Id] = CFD.[ConfigurationFileDeployment_Id]
      ,[LatestDeployment_Execution_Id] = CFD.[Execution_Id]
      ,[LatestDeployment_Instance_Name] = CFD.[Instance_Name]
      ,[LatestDeployment_Version_Number] = CFD.[Version_Number]
      ,[LatestDeployment_Server_Id] = CFD.[Server_Id]
      ,[LatestDeployment_DeployedTo_Path]= CFD.[DeployedTo_Path]
      ,[LatestDeployment_DeployedBy_User_Name] = CFD.[DeployedBy_User_Name]
      ,[LatestDeployment_DeployedOn_Date] = CFD.[DeployedOn_Date]
      ,[LatestDeployment_Server_Name] = S_CFD.[Server_Name]
  
      ,CFV.[Instance_Name]  [LatestVersion_Instance_Name]
      ,CFV.[Version_Number]  [LatestVersion_Version_Number]
      ,CFV.[VersionNotes_Text]  [LatestVersion_VersionNotes_Text]
      ,CFV.[File_Date]  [LatestVersion_File_Date]
      ,CFV.[CreatedBy_User_Name]  [LatestVersionCreatedBy_User_Name]
      ,CFV.[CreatedOn_Date]  [LatestVersionCreatedOn_Date]
      ,CFV.[Application_Id]  [LatestVersion_Application_Id]
      ,CFV.[Release_Number] [LatestVersion_Release_Number]

 FROM [ConfigurationFiles] C
       INNER JOIN [Deployables] D
               ON C.[Deployable_Id] = D.[Deployable_Id]
       LEFT  JOIN (SELECT [ConfigurationFile_Id]
                         ,MAX([ConfigurationFileDeployment_Id]) [ConfigurationFileDeployment_Id]
                     FROM [ConfigurationFileDeployments] _CFD
                    GROUP BY [ConfigurationFile_Id]) CFD_MAX
               ON CFD_MAX.[ConfigurationFile_Id]  = C.[ConfigurationFile_Id] 
       LEFT  JOIN [ConfigurationFileDeployments] CFD
               ON CFD.[ConfigurationFileDeployment_Id]= CFD_MAX.[ConfigurationFileDeployment_Id]
       LEFT  JOIN [Servers] S_CFD
               ON S_CFD.[Server_Id] = CFD.[Server_Id]
  
       LEFT  JOIN (SELECT ROW_NUMBER() OVER (PARTITION BY [ConfigurationFile_Id] 
                                                 ORDER BY [File_Date] DESC) [Row_Number],
                          [ConfigurationFile_Id], [Instance_Name], [Version_Number]
                     FROM [ConfigurationFileVersions]) CFV_MAX
               ON CFV_MAX.[ConfigurationFile_Id] = C.[ConfigurationFile_Id]
              AND CFV_MAX.[Row_Number] = 1
       LEFT  JOIN [ConfigurationFileVersions] CFV
               ON CFV.[ConfigurationFile_Id] = CFV_MAX.[ConfigurationFile_Id]
              AND CFV.[Instance_Name] = CFV_MAX.[Instance_Name]
              AND CFV.[Version_Number] = CFV_MAX.[Version_Number]
GO


GO

IF OBJECT_ID('[ConfigurationFileVersions_Extended]') IS NOT NULL DROP VIEW [ConfigurationFileVersions_Extended]
GO

CREATE VIEW [ConfigurationFileVersions_Extended] AS

        SELECT	V.[ConfigurationFile_Id],
				V.[Version_Number],
				V.[CreatedOn_Date],
				V.[CreatedBy_User_Name],
				V2.[Version_Number] AS [LatestVersion_Number],
				V.[Instance_Name],
                I.[Environment_Id],
                E.[Environment_Name],
				I.[Active_Indicator],
				V.[VersionNotes_Text],
				V.[File_Date],
				V.[File_Bytes],
				V.[Application_Id],
                A.[ApplicationGroup_Id],
				V.[Release_Number],
				R.[Release_Sequence],
				C.[Deployable_Id],
				D.[Deployable_Name],
				C.[FilePath_Text],
				[ConfigurationFile_Active_Indicator] = C.[Active_Indicator],

 				I.[Template_Indicator],
				I.[Template_Instance_Name]

		  FROM	[dbo].[ConfigurationFileVersions] V
    
    INNER JOIN [Releases] R
            ON R.[Application_Id] = V.[Application_Id]
           AND R.[Release_Number] = V.[Release_Number]
    
    INNER JOIN	[dbo].[ConfigurationFiles] C
		    ON	V.[ConfigurationFile_Id] = C.[ConfigurationFile_Id]
	
    INNER JOIN  [dbo].[ConfigurationFileInstances] I
			ON  V.[ConfigurationFile_Id] = I.[ConfigurationFile_Id]
		   AND  V.[Instance_Name] = I.[Instance_Name]
	
    INNER JOIN	[dbo].[Deployables] D
			ON	C.[Deployable_Id] = D.[Deployable_Id]
    
    INNER JOIN [dbo].[Environments] E
            ON I.[Environment_Id] = E.[Environment_Id]
    
    INNER JOIN [Applications] A
            ON V.[Application_Id] = A.[Application_Id]

     LEFT JOIN  [dbo].[ConfigurationFileVersions] V2
			ON  V.[ConfigurationFile_Id] = V2.[ConfigurationFile_Id]
		   AND  V.[Instance_Name] = V2.[Instance_Name]
		   AND  V2.[Version_Number] = (SELECT MAX([Version_Number])
										 FROM [dbo].[ConfigurationFileVersions] _V
										WHERE _V.[ConfigurationFile_Id] = V.[ConfigurationFile_Id]
										  AND _V.[Instance_Name] = V.[Instance_Name])

GO


GO

IF OBJECT_ID('[DashboardGadgets_Extended]') IS NOT NULL DROP VIEW [DashboardGadgets_Extended]
GO

CREATE VIEW [DashboardGadgets_Extended] AS

SELECT G.[Gadget_Id],
       G.[Dashboard_Id],
       D.[Application_Id],
       A.[Application_Name],
       D.[User_Name],
       D.[DashboardScope_Code],
       G.[Dashboard_Zone],
       G.[Display_Sequence],
       G.[DashboardGadget_Name],
       G.[DashboardGadget_Description],
       G.[DashboardGadget_Configuration],
       G.[CreatedBy_User_Name],
       G.[CreatedOn_Date],
       G.[ModifiedBy_User_Name],
       G.[ModifiedOn_Date]
  FROM [DashboardGadgets] G
       INNER JOIN [Dashboards] D
               ON D.[Dashboard_Id] = G.[Dashboard_Id]
       LEFT JOIN [Applications] A
              ON A.[Application_Id] = D.[Application_Id]
GO


GO

IF OBJECT_ID('[DatabaseChangeScriptExecutions_Extended]') IS NOT NULL
	DROP VIEW [DatabaseChangeScriptExecutions_Extended]
GO

CREATE VIEW [DatabaseChangeScriptExecutions_Extended]
AS


SELECT SX.[ScriptExecution_Id]
      ,SX.[Script_Id]
      ,SX.[ExecutedOn_Date]
      ,SX.[ExecutedBy_User_Name]
      ,SX.[Success_Indicator]
      ,SX.[Log_Text]
      ,SX.[Execution_Id]
      ,SX.[Provider_Id]

      ,S.[Application_Id]
      ,S.[Release_Number]
      ,S.[Deployable_Id]
      ,S.[Script_Name]
      ,[Script_Active_Indicator] = S.[Active_Indicator]
      ,D.[Deployable_Name]

      ,P.[Provider_Name]

      ,E.[Environment_Id]
      ,E.[Environment_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])

  FROM [DatabaseChangeScriptExecutions] SX
       INNER JOIN [DatabaseChangeScripts] S
               ON SX.[Script_Id] = S.[Script_Id]
       INNER JOIN [Deployables] D
               ON S.[Deployable_Id] = D.[Deployable_Id]
       INNER JOIN [Releases] R
               ON S.[Release_Number] = R.[Release_Number]
              AND S.[Application_Id] = R.[Application_Id]
       LEFT  JOIN [Providers] P
               ON SX.[Provider_Id] = P.[Provider_Id]
       LEFT  JOIN [DatabaseChangeScriptProviders] DBP
               ON DBP.[Provider_Id] = P.[Provider_Id]
       LEFT  JOIN [Environments] E
               ON DBP.[Environment_Id] = E.[Environment_Id]


GO

IF OBJECT_ID('[DatabaseChangeScriptProviders_Extended]') IS NOT NULL DROP VIEW [DatabaseChangeScriptProviders_Extended]
GO

CREATE VIEW [DatabaseChangeScriptProviders_Extended] AS

SELECT DBP.[Provider_Id]

      ,P.[Provider_Name]
      ,P.[Provider_Description]
      ,P.[Provider_Configuration]
      ,P.[ExecuteOn_Server_Id]
      ,P.[CreatedBy_User_Name]
      ,P.[CreatedOn_Date]
      ,P.[ModifiedBy_User_Name]
      ,P.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,E.[Environment_Id]
      ,E.[Environment_Name]

  FROM [DatabaseChangeScriptProviders] DBP
       INNER JOIN [Providers] P
               ON DBP.[Provider_Id] = P.[Provider_Id]
       INNER JOIN [Applications] A
               ON DBP.[Application_Id] = A.[Application_Id]

       INNER JOIN [Environments] E
               ON DBP.[Environment_Id] = E.[Environment_Id]
GO


GO

IF OBJECT_ID('[DatabaseChangeScripts_Extended]') IS NOT NULL DROP VIEW [DatabaseChangeScripts_Extended]
GO

CREATE VIEW [DatabaseChangeScripts_Extended] AS

	SELECT dS.[Script_Id]
		  ,dS.[Application_Id]
		  ,dS.[Release_Number]
		  ,dS.[Deployable_Id]
		  ,dS.[Script_Name]
		  ,dS.[Script_Text]
		  ,dS.[Active_Indicator]
		  ,dS.[CreatedBy_User_Name]
		  ,dS.[CreatedOn_Date]
		  ,dS.[ModifiedBy_User_Name]
		  ,dS.[ModifiedOn_Date]

          ,A.[Application_Name]
          ,[Numeric_Release_Number] = R.[Release_Sequence]

		  ,[LastExecuted_ScriptExecution_Id] = SX.[ScriptExecution_Id]
		  ,[LastExecutedOn_Date] = SX.[ExecutedOn_Date]
		  ,[LastExecutedBy_User_Name] = SX.[ExecutedBy_User_Name]
		  ,[LastExecuted_Success_Indicator] = SX.[Success_Indicator]
		  ,[LastExecuted_Log_Text] = SX.[Log_Text]
		  ,[LastExecuted_Execution_Id] = SX.[Execution_Id]
		  ,[LastExecuted_Provider_Id] = SX.[Provider_Id]

		  ,[LastExecuted_Provider_Name] = P.[Provider_Name]

		  ,D.[Deployable_Name]
          ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])

	  FROM [DatabaseChangeScripts] dS
	       INNER JOIN [Applications] A
	               ON dS.[Application_Id] = A.[Application_Id]
           INNER JOIN [Releases] R
                   ON R.[Application_Id] = dS.[Application_Id]
                  AND R.[Release_Number] = dS.[Release_Number]
	       INNER JOIN [Deployables] D
	               ON dS.[Deployable_Id] = D.[Deployable_Id]
	       LEFT  JOIN (
	                    SELECT [Script_Id], MAX([ScriptExecution_Id]) [ScriptExecution_Id]
	                      FROM [DatabaseChangeScriptExecutions] GROUP BY [Script_Id]
	                  ) SX_MAX
	               ON SX_MAX.[Script_Id] = dS.[Script_Id]
	       LEFT  JOIN [DatabaseChangeScriptExecutions] SX
	               ON SX.[ScriptExecution_Id] = SX_MAX.[ScriptExecution_Id]
		   LEFT  JOIN [Providers] P
				   ON SX.[Provider_Id] = P.[Provider_Id]
GO


GO

IF OBJECT_ID('[DeployablesImported_Extended]') IS NOT NULL DROP VIEW [DeployablesImported_Extended]
GO

CREATE VIEW [DeployablesImported_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]
	
      ,A.[Application_Id]
      ,A.[Application_Name]
      
      ,[ImportedFrom_Application_Id] = DA.[Application_Id]
      ,[ImportedFrom_Application_Name] = DA.[Application_Name]

FROM [DeployablesImported] DI
INNER JOIN [Applications] A
  ON A.[Application_Id] = DI.[Application_Id]
INNER JOIN [Deployables] D
  ON D.[Deployable_Id] = DI.[Deployable_Id]
INNER JOIN [Applications] DA
  ON DA.[Application_Id] = D.[Application_Id]
  
GO


GO

IF OBJECT_ID('[Deployables_Extended]') IS NOT NULL DROP VIEW [Deployables_Extended]
GO

CREATE VIEW [Deployables_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[DeployableType_Code]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,[Dependencies_Count] =
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[Deployable_Id] = D.[Deployable_Id])
      ,[Dependants_Count] = 
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[DependsOn_Deployable_Id] = D.[Deployable_Id])

FROM [Deployables] D
JOIN [Applications] A
  ON A.[Application_Id] = D.[Application_Id]
GO


GO

IF OBJECT_ID('[EnvironmentServers_Extended]') IS NOT NULL DROP VIEW [EnvironmentServers_Extended]
GO

CREATE VIEW [EnvironmentServers_Extended] AS

	SELECT  ES.[Environment_Id],
            E.[Environment_Name],
            [Environment_Active_Indicator] = E.[Active_Indicator],
			S.[Server_Id],
			S.[Server_Name],
			S.[Agent_Configuration],
			S.[ServerGroup_Indicator],
			[Server_Active_Indicator] = S.[Active_Indicator]
      FROM	[Servers] S
 LEFT JOIN	[EnvironmentServers] ES
        ON	S.[Server_Id] = ES.[Server_Id]
 LEFT JOIN	[Environments] E
        ON	ES.[Environment_Id] = E.[Environment_Id]

GO


GO
IF OBJECT_ID('[Environments_LatestDeployedReleases]') IS NOT NULL DROP VIEW [Environments_LatestDeployedReleases]
GO

CREATE VIEW [Environments_LatestDeployedReleases] AS

SELECT A.[Application_Id]
	  ,A.[ApplicationGroup_Id]
	  ,A.[Application_Name]
	  ,AG.[ApplicationGroup_Name]
	  ,E.[Environment_Id]
      ,E.[Environment_Name]
      ,E.[Display_Sequence]
	  ,BE.[Release_Number]
	  ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
	  ,BE.[Build_Number]
	  ,BE.[Execution_Id]
      ,BE.[ExecutionStart_Date]
	  ,BE.[ExecutionStatus_Name]
      ,[PromotionStatus_Name] = CASE BE.[ExecutionStatus_Name] WHEN 'Executing' THEN 'Executing' ELSE
                                (SELECT [PromotionStatus_Name] 
                                   FROM [BuildPromotions] 
                                  WHERE [Application_Id] = A.[Application_Id]
                                    AND [Release_Number] = BE.[Release_Number]
                                    AND [Build_Number] = BE.[Build_Number]
                                    AND [Environment_Id] = BE.[Environment_Id]) END
	  ,[Workflow_Step_Sequence] = dWS.[Step_Sequence]
  FROM [Applications] A
       JOIN (SELECT _W.[Application_Id], _WS.[Environment_Id], MAX(_WS.[Step_Sequence]) [Step_Sequence]
               FROM [WorkflowSteps] _WS
                    JOIN [Workflows] _W
                      ON _W.[Workflow_Id] = _WS.[Workflow_Id]
                     AND _W.[Active_Indicator] = 'Y'
              GROUP BY _W.[Application_Id], _WS.[Environment_Id]) dWS
         ON dWS.[Application_Id] = A.[Application_Id]
       JOIN [Environments] E
         ON E.[Environment_Id] = dWS.[Environment_Id]
       LEFT JOIN [BuildExecutions] BE
         ON BE.[Execution_Id] = (SELECT TOP 1 [Execution_Id]
                                   FROM [BuildExecutions] _BE
                                  WHERE _BE.[Application_Id] = A.[Application_Id]
                                     AND _BE.[Environment_Id] = E.[Environment_Id]
                                  ORDER BY [ExecutionStart_Date] DESC)
       LEFT JOIN [Releases] R
         ON R.[Application_Id] = A.[Application_Id]
        AND R.[Release_Number] = BE.[Release_Number]
         
       LEFT JOIN [ApplicationGroups] AG
         ON AG.[ApplicationGroup_Id] = A.[ApplicationGroup_Id]

GO
IF OBJECT_ID('[EventNotifiers]') IS NOT NULL DROP VIEW [EventNotifiers]
GO

CREATE VIEW [EventNotifiers] AS

SELECT N.[Notifier_Id]
      ,N.[Notifier_Name]
      ,N.[Notifier_Configuration]
      ,N.[Owner_User_Name]

      ,EO.[EventOccurence_Id]
      ,EO.[Event_Code]
      ,EO.[User_Name]
      ,EO.[Occurence_Date]
      ,EO.[Application_Id]

  FROM [EventOccurences] EO
       INNER JOIN [NotifierEventCodes] NEC
               ON EO.[Event_Code] = NEC.[Event_Code]
       INNER JOIN [Notifiers] N
               ON N.[Notifier_Id] = NEC.[Notifier_Id]
 WHERE N.[Active_Indicator] = 'Y'

GO
IF OBJECT_ID('[EventOccurences_Extended]') IS NOT NULL
	DROP VIEW [EventOccurences_Extended]
GO

CREATE VIEW [EventOccurences_Extended]
AS

	SELECT EO.*
	      ,ET.[Event_Description]
	      ,A.[Application_Name]
	  FROM [EventOccurences] EO
	       INNER JOIN [EventTypes] ET	              
	               ON ET.[Event_Code] = EO.[Event_Code]
	       LEFT  JOIN [Applications] A
	               ON EO.[Application_Id] = A.[Application_Id]


GO
IF OBJECT_ID('[FeatureLevels_Extended]') IS NOT NULL  DROP VIEW [FeatureLevels_Extended]
GO

CREATE VIEW [FeatureLevels_Extended] AS
SELECT FL.[Feature_Code]
      ,FL.[Application_Id]
      ,FL.[Feature_Level]
      ,F.[Feature_Description]
      ,A.[Application_Name]
  FROM [FeatureLevels] FL
       LEFT  JOIN [Features] F
               ON FL.[Feature_Code] = F.[Feature_Code] 
       LEFT  JOIN [Applications] A
               ON A.[Application_Id] = FL.[Application_Id]


GO
IF OBJECT_ID('LogMessages_Extended') IS NOT NULL DROP VIEW LogMessages_Extended
GO

CREATE VIEW LogMessages_Extended AS
SELECT L.[LogMessage_Id]
      ,L.[Read_Indicator]
      ,L.[Logged_Date]
      ,L.[Message_Text]
      ,L.[ExceptionStackTrace_Text]
      ,L.[Server_Id]
      ,S.[Server_Name]
  FROM [LogMessages] L
       LEFT JOIN [Servers] S
               ON L.[Server_Id] = S.[Server_Id]       
GO




GO

IF OBJECT_ID('ManualActionsInProgress_Extended') IS NOT NULL DROP VIEW ManualActionsInProgress_Extended
GO

CREATE VIEW [ManualActionsInProgress_Extended] AS
	SELECT A.[Application_Id],
		   A.[Application_Name],
 	       R.[Release_Number],
 	       [Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number]),
 	       BE.[Build_Number],
 	       BEPA.[Execution_Id],
	       BEPA.[Original_PlanActionGroup_Id],
	       BEPA.[Original_Action_Sequence],
	       MAIP.[ActionComplete_Indicator],
		   MAIP.[AssignedTo_Principal_Name],
		   MAIP.[CompletedBy_User_Name],
		   PA.[Action_Description],
		   MAIP.[Task_Name],
		   MAIP.[Task_Description],
	       [Action_ExecutionStarted_Date] = BEPA.[ExecutionStarted_Date],
	       MAIP.[BuildExecution_PlanAction_Id]
	  FROM [ManualActionsInProgress] MAIP
	       INNER JOIN [BuildExecution_PlanActions] BEPA
	               ON MAIP.[BuildExecution_PlanAction_Id] = BEPA.[BuildExecution_PlanAction_Id]
		   INNER JOIN [PlanActionGroups] PAG
				   ON BEPA.[Original_PlanActionGroup_Id] = PAG.[PlanActionGroup_Id]
		   INNER JOIN [Plans] P
				   ON PAG.[ActionGroup_Id] = P.[Plan_Id]
		   INNER JOIN [Applications] A
				   ON PAG.[Application_Id] = A.[Application_Id]
		   INNER JOIN [PlanActions] PA
				   ON PAG.[ActionGroup_Id] = PA.[Plan_Id]
				  AND BEPA.[Original_Action_Sequence] = PA.[Action_Sequence]
	       INNER JOIN [BuildExecutions] BE
	               ON BE.[Execution_Id] = BEPA.[Execution_Id]
		   INNER JOIN [Releases] R
		           ON R.[Release_Number] = BE.[Release_Number]
		          AND R.[Application_Id] = BE.[Application_Id]


GO
IF OBJECT_ID('[PlanActionGroups_Extended]') IS NOT NULL DROP VIEW [PlanActionGroups_Extended]
GO

CREATE VIEW [PlanActionGroups_Extended] AS

	SELECT PAG.[PlanActionGroup_Sequence]
	      ,[Active_Indicator] = PAG.[Active_Indicator]
	      ,[ActionGroup_Name] = COALESCE(PAG.[ActionGroup_Name], P.[ActionGroup_Name])
	      ,[ActionGroup_Description] = COALESCE(PAG.[ActionGroup_Description], P.[ActionGroup_Description])
	      ,[Predicate_Configuration] = COALESCE(PAG.[Predicate_Configuration], P.[Predicate_Configuration])
	      ,[Server_Id] = CASE WHEN PAG.[Server_Variable_Name] IS NULL THEN COALESCE(PAG.[Server_Id], P.[Server_Id]) ELSE NULL END
	      ,[Server_Variable_Name] = CASE WHEN PAG.[Server_Id] IS NULL THEN COALESCE(PAG.[Server_Variable_Name], P.[Server_Variable_Name]) ELSE NULL END
	      ,[ServerGroup_Indicator] = CASE WHEN PAG.[Server_Variable_Name] IS NULL THEN S.[ServerGroup_Indicator] ELSE NULL END
	      ,[Server_Name] = CASE WHEN PAG.[Server_Variable_Name] IS NULL THEN S.[Server_Name] ELSE NULL END

	      ,PAG.[PlanActionGroup_Id]
	      ,[PlanActionGroup_ActionGroup_Name] = PAG.[ActionGroup_Name]
	      ,[PlanActionGroup_ActionGroup_Description] = PAG.[ActionGroup_Description]
	      ,[PlanActionGroup_Predicate_Configuration] = PAG.[Predicate_Configuration]
	      ,[PlanActionGroup_ModifiedBy_User_Name] = PAG.[ModifiedBy_User_Name]
	      ,[PlanActionGroup_ModifiedOn_Date] = PAG.[ModifiedOn_Date]
	      ,[PlanActionGroup_Server_Id] = PAG.[Server_Id]
	      ,[PlanActionGroup_Server_Variable_Name] = PAG.[Server_Variable_Name]

	      ,[Plan_Id] = P.[Plan_Id]
	      ,[Plan_CreatedBy_User_Name] = P.[CreatedBy_User_Name]
	      ,[Plan_CreatedOn_Date] = P.[CreatedOn_Date]
	      ,[Plan_ModifiedBy_User_Name] = P.[ModifiedBy_User_Name]
	      ,[Plan_ModifiedOn_Date] = P.[ModifiedOn_Date]
	      ,[Plan_ActionGroup_Name] = P.[ActionGroup_Name]
	      ,[Plan_Predicate_Configuration] = P.[Predicate_Configuration]
	      ,[Plan_ActionGroup_Description] = P.[ActionGroup_Description]
	      ,[Plan_Server_Id] = P.[Server_Id]
	      ,[Plan_Server_Variable_Name] = P.[Server_Variable_Name]

	      ,A.[Application_Id]
	      ,A.[Application_Name]

	      ,E.[Environment_Id]
	      ,E.[Environment_Name]

	      ,D.[Deployable_Id]
	      ,D.[Deployable_Name]
	      ,[InUse_Indicator] =
	          (CASE WHEN EXISTS(
	              SELECT *
	                FROM [BuildExecution_Plans] _BEP
	                     INNER JOIN [BuildExecutions] _BE
	                             ON _BEP.[Execution_Id] = _BE.[Execution_Id]
	                          WHERE _BEP.[Original_PlanActionGroup_Id] = PAG.[PlanActionGroup_Id]
	                            AND _BE.[ExecutionStatus_Name] IN ('Pending', 'Executing')) 
	           THEN 'Y' ELSE 'N' END)
	      ,[Action_Count] = (SELECT COUNT(*) FROM [PlanActions] WHERE [Plan_Id] = P.[Plan_Id])
	      ,[Execution_Count] = (SELECT COUNT(*) FROM [BuildExecution_Plans] WHERE [Plan_Id] = P.[Plan_Id])
	      ,[Plan_ActionGroup_Count] = (SELECT COUNT(*)
	                                     FROM [PlanActionGroups] PAG2
	                                    WHERE PAG2.[ActionGroup_Id] = P.[Plan_Id])
	      ,A.[VariableSupport_Code]
	      ,[Referenced_Application_Id] = R_A.[Application_Id]
	      ,[Referenced_Application_Name] = R_A.[Application_Name]

	  FROM	[PlanActionGroups] PAG
            INNER JOIN [Plans] P
                    ON PAG.[ActionGroup_Id] = P.[Plan_Id]
            INNER JOIN [Applications] A
                    ON A.[Application_Id] = PAG.[Application_Id]
            INNER JOIN [Environments] E
                    ON E.[Environment_Id] = PAG.[Environment_Id]
             LEFT JOIN [Deployables] D
                    ON D.[Deployable_Id] = PAG.[Deployable_Id]
             LEFT JOIN [Servers] S
                    ON S.[Server_Id] = COALESCE(PAG.[Server_Id], P.[Server_Id])
             LEFT JOIN [DeployablesImported] DI
                    ON DI.[Deployable_Id] = PAG.[Deployable_Id]
                   AND DI.[Application_Id] = A.[Application_Id]
             LEFT JOIN [Deployables] R_D
                    ON R_D.[Deployable_Id] = DI.[Deployable_Id]
             LEFT JOIN [Applications] R_A
                    ON R_A.[Application_Id] = R_D.[Application_Id]

GO


GO
IF OBJECT_ID('[PlanActions_Extended]') IS NOT NULL DROP VIEW [PlanActions_Extended]
GO

CREATE VIEW [PlanActions_Extended] AS

    SELECT PA.[Plan_Id]
          ,PA.[Action_Sequence]
          ,PA.[Server_Id]
          ,PA.[Action_Description]
          ,PA.[ResumeNextOnFailure_Indicator]
          ,PA.[Action_Configuration]
          ,PA.[ActionType_Name]
          ,PA.[Active_Indicator]
          ,PA.[Retry_Count]
          ,PA.[LogFailureAsWarning_Indicator]

          ,[Shared_Indicator] = (CASE WHEN 
	                                ((SELECT COUNT([PlanActionGroup_Id])
                                         FROM [PlanActionGroups] PAG 
                                        WHERE PAG.[ActionGroup_Id] = PA.[Plan_Id]) > 1)
	                                  THEN 'Y'
	                                  ELSE 'N'
	                             END)
	      ,PA.[Server_Variable_Name]
	      ,PA.[Target_Server_Id]
	      ,PA.[Target_Server_Variable_Name]
	      ,PA.[ExtensionConfiguration_Id]
          ,[Configuration_Profile_Name] = E.[Profile_Name]
          ,SRC_S.[Server_Name]
          ,[Target_Server_Name] = TRG_S.[Server_Name]
          ,SRC_S.[ServerGroup_Indicator]
          ,[Target_ServerGroup_Indicator] = TRG_S.[ServerGroup_Indicator]

      FROM [PlanActions] PA
INNER JOIN [Plans] P
        ON P.[Plan_Id] = PA.[Plan_Id]
 LEFT JOIN [ExtensionConfigurations] E
        ON E.[ExtensionConfiguration_Id] = PA.[ExtensionConfiguration_Id]
 LEFT JOIN [Servers] SRC_S
        ON SRC_S.[Server_Id] = PA.[Server_Id]
 LEFT JOIN [Servers] TRG_S
        ON TRG_S.[Server_Id] = PA.[Target_Server_Id]

GO


GO

IF OBJECT_ID('[Plans_Extended]') IS NOT NULL DROP VIEW [Plans_Extended]
GO


GO
IF OBJECT_ID('[Plans_InApplications]') IS NOT NULL DROP VIEW [Plans_InApplications]
GO

GO
IF OBJECT_ID('[Plans_MostCommonActions]') IS NOT NULL DROP VIEW [Plans_MostCommonActions]
GO

CREATE VIEW [Plans_MostCommonActions] AS
    
      SELECT [ActionType_Name], 
             [Action_Count] = COUNT([ActionType_Name])
        FROM [PlanActions]
    GROUP BY [ActionType_Name]
		   
GO


GO
IF OBJECT_ID('[PlanUsage_Slim]') IS NOT NULL DROP VIEW [PlanUsage_Slim]
GO

CREATE VIEW [PlanUsage_Slim]
AS
SELECT [Plan_Id] = PAG.[ActionGroup_Id],
       PAG.[PlanActionGroup_Id],
       A.[Application_Id],
       A.[Application_Name],
       AG.[ApplicationGroup_Id],
       AG.[ApplicationGroup_Name],
       E.[Environment_Id],
       E.[Environment_Name]

  FROM [PlanActionGroups] PAG
       INNER JOIN [Applications] A
               ON A.[Application_Id] = PAG.[Application_Id]
        LEFT JOIN [ApplicationGroups] AG
               ON A.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       INNER JOIN [Environments] E
               ON E.[Environment_Id] = PAG.[Environment_Id]

GO

GO
IF OBJECT_ID('[Privileges_Extended]') IS NOT NULL
	DROP VIEW [Privileges_Extended]
GO

CREATE VIEW [Privileges_Extended]
AS

	SELECT P.[Privilege_Id]
		  ,P.[Directory_Provider_Id]
		  ,P.[Principal_Name]
		  ,P.[PrincipalType_Code]
		  ,P.[PrivilegeType_Code]

		  ,R.[Role_Id]
		  ,R.[Role_Name]

		  ,A.[Application_Id]
		  ,A.[Application_Name]

		  ,E.[Environment_Id]
		  ,E.[Environment_Name]

		  ,AG.[ApplicationGroup_Id]
		  ,AG.[ApplicationGroup_Name]

	  FROM [Privileges] P
		   INNER JOIN [Roles] R
				   ON P.[Role_Id] = R.[Role_Id]

		   LEFT JOIN [Applications] A
				  ON P.[Application_Id] = A.[Application_Id]

		   LEFT JOIN [Environments] E
				  ON P.[Environment_Id] = E.[Environment_Id]
				  
		   LEFT JOIN [ApplicationGroups] AG
				  ON P.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]

GO


GO

IF OBJECT_ID('[ReleaseDeployables_Extended]') IS NOT NULL DROP VIEW [ReleaseDeployables_Extended]
GO

CREATE VIEW [ReleaseDeployables_Extended] AS

SELECT D.[Deployable_Id]
      ,D.[Deployable_Name]
      ,D.[CreatedBy_User_Name]
      ,D.[CreatedOn_Date]
      ,D.[ModifiedBy_User_Name]
      ,D.[ModifiedOn_Date]

      ,A.[Application_Id]
      ,A.[Application_Name]

      ,[Dependencies_Count] =
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[Deployable_Id] = D.[Deployable_Id])
      ,[Dependants_Count] = 
                             (SELECT COUNT(*) FROM [DeployableDependencies] _DD
                               WHERE _DD.[DependsOn_Deployable_Id] = D.[Deployable_Id])
      ,RD.[Release_Number]
      ,RD.[InclusionType_Code]
      ,RD.[Referenced_Release_Number]
      ,RD.[Referenced_Application_Id]
      ,[Referenced_Release_Name] = COALESCE(RREF.[Release_Name],RREF.[Release_Number])
      
      ,[Referenced_Application_Name] = AREF.[Application_Name]

FROM [ReleaseDeployables] RD
     INNER JOIN [Deployables] D
             ON D.[Deployable_Id] = RD.[Deployable_Id]
     INNER JOIN [Applications] A
             ON A.[Application_Id] = RD.[Application_Id]
     LEFT JOIN [Applications] AREF
			 ON RD.[Referenced_Application_Id] = AREF.[Application_Id]
     LEFT JOIN [Releases] RREF
			 ON RD.[Referenced_Application_Id] = RREF.[Application_Id]
            AND RD.[Referenced_Release_Number] = RREF.[Release_Number]
GO




GO
IF OBJECT_ID('[ReleaseNotes_Extended]') IS NOT NULL DROP VIEW [ReleaseNotes_Extended]
GO

CREATE VIEW [ReleaseNotes_Extended] AS

SELECT RN.*
      ,E.[Environment_Name]
  FROM [ReleaseNotes] RN
       LEFT  JOIN [Environments] E
               ON E.[Environment_Id] = RN.[Environment_Id]
GO

GO

IF OBJECT_ID('[Releases_Extended]') IS NOT NULL DROP VIEW [Releases_Extended]
GO

CREATE VIEW [Releases_Extended] AS

SELECT R.[Application_Id]
	  ,R.[Release_Number]
      ,R.[ReleaseStatus_Name]
      ,R.[Notes_Text]
	  ,R.[CreatedBy_User_Name]
	  ,R.[CreatedOn_Date]
	  ,R.[ModifiedBy_User_Name]
	  ,R.[ModifiedOn_Date]
	  ,R.[CancelledBy_User_Name]
	  ,R.[CancelledReason_Text]
	  ,R.[CancelledOn_Date]
      ,R.[Target_Date]
      ,[Release_Name] = COALESCE(R.[Release_Name],R.[Release_Number])
      ,R.[Release_Sequence]

	  ,A.[Application_Name]

	  ,W.[Workflow_Id]
	  ,W.[Workflow_Name]
      ,W.[AutoDeployRelease_Indicator]
      ,W.[AllowOutOfSequencePromotions_Indicator]
      ,W.[AutoCancelReleases_Indicator]
      ,W.[AutoIncrementRelease_Indicator]

	  ,[Latest_Step_Sequence] = WS_LAT.[Step_Sequence]
	  ,[Latest_Environment_Id] = E_LAT.[Environment_Id]
	  ,[Latest_Environment_Name] = E_LAT.[Environment_Name]
	  ,[Latest_Build_Number] = B_LAT.[Build_Number]
	  ,[Latest_Promoted_Date] = BP_LAT.[Promoted_Date]
	  ,[Latest_Promoted_Build_Number] = BP_LAT.[Build_Number]
	  ,[Latest_Promoted_Environment_Id] = BP_LAT.[Environment_Id]
      ,[Latest_Promotion_Status] = BP_LAT.[PromotionStatus_Name]
	  ,[Latest_PromotedBy_Name] = BP_LAT.[PromotedBy_Name]
	  ,[Latest_PromotionComments_Text] = BP_LAT.[Comments_Text]

	  ,[NextLatest_Step_Sequence] = WS_NXTL.[Step_Sequence]
	  ,[NextLatest_Environment_Id] = E_NXTL.[Environment_Id]
	  ,[NextLatest_Environment_Name] = E_NXTL.[Environment_Name]
	  ,[NextLatest_IsApproved_Indicator] = [dbo].[IsPromotionApproved]
                                           (
                                                R.[Application_Id], 
                                                R.[Release_Number], 
                                                B_LAT.[Build_Number], 
                                                WS_NXTL.[Environment_Id]
                                           )

	  ,[Furthest_Step_Sequence] = WS_FUR.[Step_Sequence]
	  ,[Furthest_Environment_Id] = E_FUR.[Environment_Id]
	  ,[Furthest_Environment_Name] = E_FUR.[Environment_Name]
	  ,[Furthest_Build_Number] = B_FUR.[Build_Number]
	  ,[Furthest_Promoted_Date] = BP_FUR.[Promoted_Date]
	  ,[Furthest_PromotedBy_Name] = BP_FUR.[PromotedBy_Name]

	  ,[NextFurthest_Step_Sequence] = WS_NXTF.[Step_Sequence]
	  ,[NextFurthest_Environment_Id] = E_NXTF.[Environment_Id]
	  ,[NextFurthest_Environment_Name] = E_NXTF.[Environment_Name]


	  ,[First_Step_Sequence] = WS_1ST.[Step_Sequence]
	  ,[First_Environment_Id] = E_1ST.[Environment_Id]
	  ,[First_Environment_Name] = E_1ST.[Environment_Name]
	  ,[First_ApprovalsRequired_Count] = 
	       (SELECT COUNT(*) FROM [PromotionApprovalRequirements]
	         WHERE [Workflow_Id] = WS_1ST.[Workflow_Id]
	           AND [Environment_Id] = WS_1ST.[Environment_Id])
	  ,[First_RequirePromotionRequirementsMetBefore_Indicator] = CASE WHEN 
	        EXISTS(
	            SELECT * FROM PromotionRequirements PR 
	            WHERE PR.[Workflow_Id] = R.[Workflow_Id]
	              AND PR.[Environment_Id] = WS_1ST.[Environment_Id]
	        ) 
	        THEN 'Y' ELSE 'N' 
	   END

	  ,[Final_Step_Sequence] = WS_FNL.[Step_Sequence]
	  ,[Final_Environment_Id] = E_FNL.[Environment_Id]
	  ,[Final_Environment_Name] = E_FNL.[Environment_Name]

	  ,[Builds_Count] = (SELECT COUNT(*)
						   FROM [Builds] _B
						  WHERE R.[Application_Id] = _B.[Application_Id]
							AND R.[Release_Number] = _B.[Release_Number])

  FROM [Releases] R
	   INNER JOIN [Applications] A
               ON R.[Application_Id] = A.[Application_Id]
	   INNER JOIN [Workflows] W
               ON R.[Workflow_Id] = W.[Workflow_Id]


        /* Latest (i.e. most recently created) Build */
        LEFT JOIN [Builds] B_LAT
               ON B_LAT.[Application_Id] = R.[Application_Id]
              AND B_LAT.[Release_Number] = R.[Release_Number]
              AND B_LAT.[Build_Number] = 
                     (SELECT MAX([Build_Number])
                        FROM [Builds]
                       WHERE [Application_Id] = R.[Application_Id]
                         AND [Release_Number] = R.[Release_Number]
                         AND [BuildStatus_Name] <> 'Rejected')
        LEFT JOIN [WorkflowSteps] WS_LAT
               ON WS_LAT.[Workflow_Id] = W.[Workflow_Id]
              AND WS_LAT.[Step_Sequence] =
              (
					SELECT TOP 1 _WS.[Step_Sequence]
					  FROM [BuildPromotions] _BP
					  JOIN [WorkflowSteps] _WS
						ON _BP.[Environment_Id] = _WS.[Environment_Id]
					 WHERE _BP.[Application_Id] = R.[Application_Id]
					   AND _BP.[Release_Number] = R.[Release_Number]
					   AND _BP.[Build_Number] = B_LAT.[Build_Number]
					ORDER BY [Promoted_Date] DESC	
			  )

		/* Latest (most recent) Promotion */
		LEFT JOIN [BuildPromotions] BP_LAT
			   ON BP_LAT.[Application_Id] = R.[Application_Id]
			  AND BP_LAT.[Release_Number] = R.[Release_Number]
			  AND BP_LAT.[Promoted_Date] =
				( SELECT TOP 1 [Promoted_Date]
					FROM [BuildPromotions] _BP_LAT
				   WHERE _BP_LAT.[Application_Id] = R.[Application_Id]
					 AND _BP_LAT.[Release_Number] = R.[Release_Number]
				  ORDER BY [Promoted_Date] DESC )
        LEFT JOIN [Environments] E_LAT
               ON BP_LAT.[Environment_Id] = E_LAT.[Environment_Id]

      /* Next LAtest Workflow Step */
        LEFT JOIN [WorkflowSteps] WS_NXTL
               ON WS_NXTL.[Workflow_Id] = R.[Workflow_Id]
              AND WS_NXTL.[Step_Sequence] =
                  (SELECT MIN(_WS.[Step_Sequence])
                     FROM [WorkflowSteps] _WS
                    WHERE _WS.[Workflow_Id] = WS_LAT.[Workflow_Id]
                      AND _WS.[Step_Sequence] > COALESCE(WS_LAT.[Step_Sequence],0) )
        LEFT JOIN [Environments] E_NXTL
               ON E_NXTL.[Environment_Id] = WS_NXTL.[Environment_Id]

        /* Furthest (i.e. longest down the workflow line) Build */
        LEFT JOIN [WorkflowSteps] WS_FUR
               ON WS_FUR.[Workflow_Id] = W.[Workflow_Id]
              AND WS_FUR.[Step_Sequence] =
              (
					SELECT MAX(_WS.[Step_Sequence])
					  FROM [BuildPromotions] _BP
						  INNER JOIN [Builds] _B 
								  ON _B.[Application_Id] = _BP.[Application_Id]
								 AND _B.[Release_Number] = _BP.[Release_Number]
								 AND _B.[Build_Number] = _BP.[Build_Number]
								 AND _B.[BuildStatus_Name] <> 'Rejected'
							JOIN [WorkflowSteps] _WS
							  ON R.[Workflow_Id] = _WS.[Workflow_Id]
							 AND _BP.[Environment_Id] = _WS.[Environment_Id]
					 WHERE _BP.[Application_Id] = R.[Application_Id]
					   AND _BP.[Release_Number] = R.[Release_Number]
              )
        LEFT JOIN [Builds] B_FUR
               ON B_FUR.[Application_Id] = R.[Application_Id]
              AND B_FUR.[Release_Number] = R.[Release_Number]
              AND B_FUR.[BuildStatus_Name] <> 'Rejected' /* optimization */
              AND 
              (
                  ( B_FUR.[Build_Number] = 
                         (SELECT MAX(_BP.[Build_Number])
                            FROM [BuildPromotions] _BP
                                 INNER JOIN [Builds] _B 
                                         ON _B.[Application_Id] = _BP.[Application_Id]
                                        AND _B.[Release_Number] = _BP.[Release_Number]
                                        AND _B.[Build_Number] = _BP.[Build_Number]
                                        AND _B.[BuildStatus_Name] = 'Active'
                           WHERE _BP.[Application_Id] = R.[Application_Id]
                             AND _BP.[Release_Number] = R.[Release_Number]
                             AND R.[Workflow_Id] = WS_FUR.[Workflow_Id]
                             AND _BP.[Environment_Id] = WS_FUR.[Environment_Id]))
               OR
                  ( B_FUR.[BuildStatus_Name] = 'Deployed')
              )   
        LEFT JOIN [BuildPromotions] BP_FUR
               ON BP_FUR.[Application_Id] = B_FUR.[Application_Id]
              AND BP_FUR.[Release_Number] = B_FUR.[Release_Number]
              AND BP_FUR.[Build_Number] = B_FUR.[Build_Number]
              AND BP_FUR.[Environment_Id] = (SELECT TOP 1 [Environment_Id]
                                               FROM [BuildPromotions] _BP
                                              WHERE BP_FUR.[Application_Id] = _BP.[Application_Id]
                                                AND BP_FUR.[Release_Number] = _BP.[Release_Number]
                                                AND BP_FUR.[Build_Number] = _BP.[Build_Number]
                                              ORDER BY [Promoted_Date] DESC)
        LEFT JOIN [Environments] E_FUR
               ON BP_FUR.[Environment_Id] = E_FUR.[Environment_Id]

      /* Next Furthest Workflow Step */
        LEFT JOIN [WorkflowSteps] WS_NXTF
               ON WS_NXTF.[Workflow_Id] = R.[Workflow_Id]
              AND WS_NXTF.[Step_Sequence] =
                  (SELECT MIN(_WS.[Step_Sequence])
                     FROM [WorkflowSteps] _WS
                    WHERE _WS.[Workflow_Id] = WS_FUR.[Workflow_Id]
                      AND _WS.[Step_Sequence] > COALESCE(WS_FUR.[Step_Sequence],0) )
        LEFT JOIN [Environments] E_NXTF
               ON E_NXTF.[Environment_Id] = WS_NXTF.[Environment_Id]

        /* First Workflow Step */
        LEFT JOIN [WorkflowSteps] WS_1ST 
               ON WS_1ST.[Workflow_Id] = W.[Workflow_Id]
              AND WS_1ST.[Step_Sequence] =
                  (SELECT MIN(WS.[Step_Sequence])
                     FROM [WorkflowSteps] WS
                    WHERE WS.[Workflow_Id] = R.[Workflow_Id])
        LEFT JOIN [Environments] E_1ST
               ON E_1ST.[Environment_Id] = WS_1ST.[Environment_Id]

  

        /* Final Step */
        LEFT JOIN [WorkflowSteps] WS_FNL
               ON WS_FNL.[Workflow_Id] = R.[Workflow_Id]
              AND WS_FNL.[Step_Sequence] =
                  (SELECT MAX(_WS.[Step_Sequence])
                     FROM [WorkflowSteps] _WS
                    WHERE _WS.[Workflow_Id] = R.[Workflow_Id])
        LEFT JOIN [Environments] E_FNL
               ON E_FNL.[Environment_Id] = WS_FNL.[Environment_Id]

GO

GO
IF OBJECT_ID('Releases_RequiredPromotions') IS NOT NULL DROP VIEW Releases_RequiredPromotions
GO

CREATE VIEW [Releases_RequiredPromotions] AS

SELECT R.[Application_Id]
      ,R.[Release_Number]
      ,R.[ReleaseStatus_Name]

      ,R.[Workflow_Id]
      ,[Step_Sequence] = COALESCE(WS.[Step_Sequence],BP_IDX.[Actual_Step_Sequence])

      ,E.[Environment_Id]
      ,E.[Environment_Name]
      
      ,BP.[PromotionStatus_Name]
      ,BP.[Promoted_Date]

      ,B.[Build_Number]
      ,B.[BuildStatus_Name]

  FROM [Releases] R
       LEFT  JOIN [WorkflowSteps] WS
               ON WS.[Workflow_Id] = R.[Workflow_Id]
              AND R.[ReleaseStatus_Name] = 'Active'

       LEFT  JOIN (SELECT _BP.[Application_Id]
                         ,_BP.[Release_Number]
                         ,_BP.[Environment_Id]
                         ,[Build_Number] = LTRIM(RTRIM(MAX([dbo].SortableBuildNumber(_BP.[Application_Id], _BP.[Build_Number]))))
                         ,[Actual_Step_Sequence] = ROW_NUMBER() OVER (ORDER BY MAX([Promoted_Date]))
                     FROM [BuildPromotions] _BP
                          INNER JOIN [Releases] _R
                                  ON _R.[Application_Id] = _BP.[Application_Id]
                                 AND _R.[Release_Number] = _BP.[Release_Number]
                          INNER JOIN [Builds] _B
                                  ON _B.[Application_Id] = _BP.[Application_Id]
                                 AND _B.[Release_Number] = _BP.[Release_Number]
                                 AND _B.[Build_Number] = _BP.[Build_Number]
                    WHERE _R.[ReleaseStatus_Name] = 'Cancelled'
                       OR _B.[BuildStatus_Name] <> 'Rejected'
                    GROUP BY _BP.[Application_Id]
                            ,_BP.[Release_Number]
                            ,_BP.[Environment_Id]) BP_IDX
               ON BP_IDX.[Application_Id] = R.[Application_Id]
              AND BP_IDX.[Release_Number] = R.[Release_Number]
              AND BP_IDX.[Environment_Id] = COALESCE(WS.[Environment_Id],BP_IDX.[Environment_Id])

       INNER JOIN [Environments] E
               ON E.[Environment_Id] = COALESCE(WS.[Environment_Id],BP_IDX.[Environment_Id])

       LEFT  JOIN [BuildPromotions] BP
               ON BP.[Application_Id] = BP_IDX.[Application_Id]
              AND BP.[Release_Number] = BP_IDX.[Release_Number]
              AND BP.[Build_Number] = BP_IDX.[Build_Number]
              AND BP.[Environment_Id] = BP_IDX.[Environment_Id]
       LEFT  JOIN [Builds] B
               ON B.[Application_Id] = BP.[Application_Id]
              AND B.[Release_Number] = BP.[Release_Number]
              AND B.[Build_Number] = BP.[Build_Number]


GO
IF OBJECT_ID('[dbo].[RoleTasks_Extended]') IS NOT NULL
	DROP VIEW [dbo].[RoleTasks_Extended]
GO

CREATE VIEW [dbo].[RoleTasks_Extended]
AS

	SELECT RT.[Role_Id]

		  ,T.[Task_Id]
		  ,T.[Task_Name]
          ,T.[Application_Scopeable_Indicator]
          ,T.[Environment_Scopeable_Indicator]

	  FROM [dbo].[RoleTasks] RT
		   INNER JOIN [dbo].[Tasks] T
				   ON RT.[Task_Id] = T.[Task_Id]

GO


GO
IF OBJECT_ID('[Scripts_Extended]') IS NOT NULL DROP VIEW [Scripts_Extended]
GO

CREATE VIEW [Scripts_Extended] AS

SELECT S.[Script_Id],
       S.[Script_Name],
       S.[Description_Text],
       S.[Active_Indicator],
       S.[AutoSync_Indicator],
       S.[ScriptType_Code],
       S.[CreatedBy_User_Name],
       S.[CreatedOn_Date],
       S.[ModifiedBy_User_Name],
       S.[ModifiedOn_Date],
       [LatestVersion_Number] = SV.[Version_Number],
       [LatestVersion_Bytes] = SV.[Script_Bytes]

  FROM [Scripts] S
        LEFT JOIN (SELECT _SV.[Script_Id],
                          [Version_Number] = MAX(_SV.[Version_Number])
                     FROM [ScriptVersions] _SV
                    GROUP BY _SV.[Script_Id]) _Latest_SV
               ON S.[Script_Id] = _Latest_SV.[Script_Id]
        LEFT JOIN [ScriptVersions] SV
               ON _Latest_SV.[Script_Id] = SV.[Script_Id]
              AND _Latest_SV.[Version_Number] = SV.[Version_Number]
GO


GO
IF OBJECT_ID('[ScriptTypes_Usage]') IS NOT NULL DROP VIEW [ScriptTypes_Usage]
GO

CREATE VIEW [ScriptTypes_Usage] AS

	SELECT S.[ScriptType_Code],
	       [Usage_Count] = COUNT(S.[Script_Id])
	  FROM [Scripts] S
	 WHERE S.[Active_Indicator] = 'Y'
	 GROUP BY S.[ScriptType_Code]

GO


GO
IF OBJECT_ID('[VariableDefinitions_Extended]') IS NOT NULL DROP VIEW [VariableDefinitions_Extended]
GO

CREATE VIEW [VariableDefinitions_Extended] AS

SELECT VDEF.[VariableDefinition_Id]
      ,VDEF.[Variable_Name]
      ,VDEF.[Environment_Id]
      ,VDEF.[Server_Id]
      ,VDEF.[ApplicationGroup_Id]
      ,VDEF.[Application_Id]
      ,VDEF.[Deployable_Id]
      ,VDEF.[Release_Number]
      ,VDEF.[Build_Number]
      ,VDEF.[Value_Text]
      ,VDEF.[Execution_Id]
	  ,VDEF.[Scope_Code]

      ,VDEC.[VariableDeclaration_Id]
      ,VDEC.[Scoped_Application_Id]
      ,VDEC.[Required_Indicator]
      ,VDEC.[Variable_Configuration]
      ,VDEC.[DefaultValue_Text]
      ,VDEC.[Scoped_Environment_Id]

      ,E.[Environment_Name]
      ,S.[Server_Name]
      ,AG.[ApplicationGroup_Name]
      ,A.[Application_Name]
      ,D.[Deployable_Name]
      ,VDEF.[Sensitive_Indicator]

  FROM [VariableDefinitions] VDEF
       LEFT JOIN [VariableDeclarations] VDEC
              ON VDEF.[Variable_Name] = VDEC.[Variable_Name]
             AND (VDEC.[Scope_Code] NOT IN ('R','B','P','X') OR
                  VDEF.[Application_Id] = VDEC.[Scoped_Application_Id])
             AND (VDEC.[Scope_Code] NOT IN ('P','X') OR
                  VDEF.[Environment_Id] = VDEC.[Scoped_Environment_Id])
             AND VDEC.[Scope_Code] = VDEF.[Scope_Code]
       LEFT JOIN [Applications] A
                 ON VDEF.[Application_Id] = A.[Application_Id]
       LEFT JOIN [ApplicationGroups] AG
                 ON VDEF.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       LEFT JOIN [Environments] E
                 ON VDEF.[Environment_Id] = E.[Environment_Id]
       LEFT JOIN [Deployables] D
                 ON VDEF.[Deployable_Id] = D.[Deployable_Id]
       LEFT JOIN [Servers] S
                 ON VDEF.[Server_Id] = S.[Server_Id]
GO

GO
IF OBJECT_ID('[VariableDefinitions_System]') IS NOT NULL DROP VIEW [VariableDefinitions_System]
GO

CREATE VIEW [VariableDefinitions_System] AS

SELECT VDEF.*
      ,E.[Environment_Name]
      ,S.[Server_Name]
      ,AG.[ApplicationGroup_Name]
      ,A.[Application_Name]
      ,D.[Deployable_Name]
  FROM [VariableDefinitions] VDEF
       LEFT JOIN [Applications] A
              ON VDEF.[Application_Id] = A.[Application_Id]
       LEFT JOIN [ApplicationGroups] AG
              ON VDEF.[ApplicationGroup_Id] = AG.[ApplicationGroup_Id]
       LEFT JOIN [Environments] E
              ON VDEF.[Environment_Id] = E.[Environment_Id]
       LEFT JOIN [Deployables] D
              ON VDEF.[Deployable_Id] = D.[Deployable_Id]
       LEFT JOIN [Servers] S
              ON VDEF.[Server_Id] = S.[Server_Id]
  
  WHERE [Scope_Code] IN ('S','E','V','G','A','D')

GO

GO
IF OBJECT_ID('[Variable_Values]') IS NOT NULL DROP VIEW [Variable_Values]
GO

CREATE VIEW [Variable_Values] AS

SELECT DISTINCT
	   VDEF.[Variable_Name]
      ,VDEF.[Value_Text]
	  ,VDEF.[Scope_Code]
	  ,VDEF.[Sensitive_Indicator]

  FROM [VariableDefinitions] VDEF
GO

GO

IF OBJECT_ID('[WorkflowSteps_Extended]') IS NOT NULL DROP VIEW [WorkflowSteps_Extended]
GO

CREATE VIEW [WorkflowSteps_Extended] AS

SELECT WS.[Workflow_Id]
	  ,W.[Workflow_Name]
      ,WS.[Step_Sequence]
	  ,[ApprovalsRequired_Count] =
		  ( SELECT COUNT(_PAR.[Approval_Sequence])
			  FROM [PromotionApprovalRequirements] _PAR
			 WHERE _PAR.[Workflow_Id] = WS.[Workflow_Id]
			   AND _PAR.[Environment_Id] = WS.[Environment_Id] )

      ,WS.[Environment_Id]
      ,E.[Environment_Name]

	  ,[Previous_Step_Sequence] = WS_PREV.[Step_Sequence]
	  ,[Previous_Environment_Id] = WS_PREV.[Environment_Id]
	  ,[Previous_Environment_Name] = E_PREV.[Environment_Name]

	  ,[Next_Step_Sequence] = WS_NEXT.[Step_Sequence]
	  ,[Next_Environment_Id] = WS_NEXT.[Environment_Id]
	  ,[Next_Environment_Name] = E_NEXT.[Environment_Name]

FROM [WorkflowSteps] WS
	 INNER JOIN [Workflows] W
			 ON WS.[Workflow_Id] = W.[Workflow_Id]

     JOIN [Environments] E
       ON WS.[Environment_Id] = E.[Environment_Id]

	/* Previous Step */
	LEFT JOIN [WorkflowSteps] WS_PREV
		   ON WS_PREV.[Workflow_Id] = WS.[Workflow_Id]
		  AND WS_PREV.[Environment_Id] =
			( SELECT TOP 1 _PREV.[Environment_Id]
				FROM [WorkflowSteps] _PREV
			   WHERE _PREV.[Workflow_Id] = WS.[Workflow_Id]
				 AND _PREV.[Step_Sequence] < WS.[Step_Sequence]
			  ORDER BY [Step_Sequence] DESC )
	LEFT JOIN [Environments] E_PREV
		   ON WS_PREV.[Environment_Id] = E_PREV.[Environment_Id]

	/* Next Step */
	LEFT JOIN [WorkflowSteps] WS_NEXT
		   ON WS_NEXT.[Workflow_Id] = WS.[Workflow_Id]
		  AND WS_NEXT.[Environment_Id] =
			( SELECT TOP 1 _NEXT.[Environment_Id]
				FROM [WorkflowSteps] _NEXT
			   WHERE _NEXT.[Workflow_Id] = WS.[Workflow_Id]
				 AND _NEXT.[Step_Sequence] > WS.[Step_Sequence]
			  ORDER BY [Step_Sequence] )
	LEFT JOIN [Environments] E_NEXT
		   ON WS_NEXT.[Environment_Id] = E_NEXT.[Environment_Id]

GO


GO
IF OBJECT_ID('[Workflows_Extended]') IS NOT NULL DROP VIEW [Workflows_Extended]
GO

CREATE VIEW [Workflows_Extended] AS

	SELECT 
           [Workflow_Id]
          ,[Application_Id]
          ,[Workflow_Name]
          ,[Active_Indicator]
          ,[CreatedBy_User_Name]
          ,[CreatedOn_Date]
          ,[ModifiedBy_User_Name]
          ,[ModifiedOn_Date]
          ,[AutoIncrementRelease_Indicator]
          ,[AutoDeployRelease_Indicator]
          ,[AllowOutOfSequencePromotions_Indicator]
          ,[AutoCancelReleases_Indicator]

		  ,[Releases_Count] = (SELECT COUNT(*) FROM [Releases]
	                            WHERE [Workflow_Id] = W.[Workflow_Id])
		  
          ,[InUse_Indicator] =
				(CASE WHEN EXISTS(SELECT * 
									FROM [Releases]
								   WHERE [Workflow_Id] = W.[Workflow_Id]
									 AND [ReleaseStatus_Name] = 'Active') 
					  THEN 'Y' ELSE 'N' END)

	  FROM [Workflows] W

GO

GO
IF OBJECT_ID('Executions_GetNextAction') IS NOT NULL DROP FUNCTION Executions_GetNextAction
GO

CREATE FUNCTION Executions_GetNextAction
(
	@Execution_Id INT
) RETURNS @tbl TABLE (
	[Next_PlanActionGroup_Id] INT,
	[Next_PlanExecution_Sequence] INT,
	[Next_Action_Sequence] INT,
	[Next_Server_Id] INT,
	[Next_Attempt_Sequence] INT
) AS BEGIN

	-- veryify execution status before even trying
	IF NOT EXISTS( SELECT * FROM [BuildExecutions] 
	                WHERE [ExecutionStatus_Name] IN ('Pending', 'Executing', 'Canceled')
					  AND [Execution_Id] = @Execution_Id ) RETURN

	-- check for a failure in a non-retry w/ no resume next
	IF EXISTS( SELECT * FROM [BuildExecution_PlanActions] BEPA 
			 INNER JOIN [PlanActionGroups] PAG
					 ON PAG.[PlanActionGroup_Id] = BEPA.[Original_PlanActionGroup_Id]
			 INNER JOIN [PlanActions] PA
					 ON PA.[Plan_Id] = PAG.[ActionGroup_Id]
					AND PA.[Action_Sequence] = BEPA.[Original_Action_Sequence]
				  WHERE BEPA.[Execution_Id] = @Execution_Id
					AND BEPA.[ExecutionStatus_Name] = 'Failed'
					AND PA.[ResumeNextOnFailure_Indicator] = 'N'
					AND PA.[Retry_Count] = 0) RETURN

	-- compile the full plan, less variable servers
	DECLARE @Actions TABLE (
	  [PlanActionGroup_Id] INT, 
	  [PlanExecution_Sequence] INT, [Action_Sequence] INT, [Attempt_Sequence] INT,
	  [Server_Id] INT, [Server_Variable_Name] VARCHAR(50),
	  [Deployable_Id] INT)
	INSERT @Actions
	SELECT PAG.[PlanActionGroup_Id], 
		   BEP.[PlanExecution_Sequence], PA.[Action_Sequence], [Attempt_Sequence] = N.[Number], 
		   [Server_Id] = S.[Server_Id],
		   [Server_Variable_Name] = COALESCE(PA.[Server_Variable_Name], PAG.[Server_Variable_Name], P.[Server_Variable_Name]),
		   PAG.[Deployable_Id]
	  FROM [PlanActions] PA
		   INNER JOIN [Plans] P
				   ON P.[Plan_Id] = PA.[Plan_Id]
		   INNER JOIN [PlanActionGroups] PAG
				   ON PAG.[ActionGroup_Id] = P.[Plan_Id]
		   INNER JOIN [BuildExecution_Plans] BEP
				   ON BEP.[Original_PlanActionGroup_Id] = PAG.[PlanActionGroup_Id]
			LEFT JOIN [Numbers] N
				   ON N.[Number] BETWEEN 1 AND COALESCE(PA.[Retry_Count], 0) + 1
			LEFT JOIN [Servers] SID
				   ON SID.[Server_Id] = CASE
					  WHEN  PA.[Server_Variable_Name] IS NOT NULL THEN NULL
					  WHEN  PA.[Server_Id] IS NOT NULL THEN PA.[Server_Id]
					  WHEN PAG.[Server_Variable_Name] IS NOT NULL THEN NULL
					  WHEN PAG.[Server_Id] IS NOT NULL THEN PAG.[Server_Id]
					  WHEN   P.[Server_Variable_Name] IS NOT NULL THEN NULL
					  WHEN   P.[Server_Id] IS NOT NULL THEN P.[Server_Id]
					  ELSE 1 END
			LEFT JOIN [ServerGroups] SG
				   ON SG.[ServerGroup_Server_Id] = SID.[Server_Id]
			LEFT JOIN [Servers] S
				   ON S.[Server_Id] = COALESCE(SG.[Server_Id], SID.[Server_Id])
	 WHERE BEP.[Execution_Id] = @Execution_Id
	   AND PA.[Active_Indicator] = 'Y'
	   AND BEP.[Skipped_Indicator] = 'N'

	-- grab variable values scoped to deployables
	DECLARE @Variables TABLE([Variable_Name] VARCHAR(50), [Value_Text] VARCHAR(MAX), [Deployable_Id] INT)
	INSERT @Variables
	SELECT VV.[Variable_Name], VV.[Value_Text], DIDS.[Deployable_Id]
	  FROM (SELECT DISTINCT [Deployable_Id] FROM @Actions) DIDS
		   CROSS APPLY dbo.[Variables_GetValuesForExecution](@Execution_Id, NULL, DIDS.[Deployable_Id]) VV
       
	-- add new rows for matching variable servers
	INSERT @Actions
	SELECT A.[PlanActionGroup_Id], 
		   A.[PlanExecution_Sequence], A.[Action_Sequence], A.[Attempt_Sequence],
		   S.[Server_Id], V.[Variable_Name],
		   A.[Deployable_Id]
	  FROM @Actions A
			LEFT JOIN @Variables V
				   ON V.[Variable_Name] = A.[Server_Variable_Name]
				  AND (V.[Deployable_Id] = A.[Deployable_Id] 
					   OR (V.[Deployable_Id] IS NULL AND 
						   A.[Deployable_Id] IS NULL))
			LEFT JOIN [Servers] SVAR 
				  ON SVAR.[Server_Name] = V.[Value_Text]
			LEFT JOIN [ServerGroups] SG
				   ON SG.[ServerGroup_Server_Id] = SVAR.[Server_Id]
			LEFT JOIN [Servers] S
				   ON S.[Server_Id] = COALESCE(SG.[Server_Id], SVAR.[Server_Id])
	 WHERE V.Variable_Name IS NOT NULL

	 -- delete variable names with matched servers
	DELETE @Actions FROM @Actions A
	 WHERE A.[Server_Variable_Name] IS NOT NULL
	   AND A.[Server_Id] IS NULL
	   AND EXISTS (SELECT * FROM @Actions _A 
					WHERE _A.[Server_Variable_Name] = A.[Server_Variable_Name]
					  AND _A.[Server_Id] IS NOT NULL)

	-- finally do our return
	INSERT @tbl
	SELECT TOP 1 A.[PlanActionGroup_Id], A.[PlanExecution_Sequence], A.[Action_Sequence], A.[Server_Id], A.[Attempt_Sequence]
	  FROM @Actions A
		   CROSS JOIN [BuildExecutions] BE       
		   INNER JOIN [BuildExecution_Plans] BEP
				  ON BEP.Execution_Id = BE.[Execution_Id]
				 AND BEP.[Original_PlanActionGroup_Id] = A.[PlanActionGroup_Id]
			LEFT JOIN [BuildExecution_PlanActions] BEPA
				   ON BEPA.[Execution_Id] = BEP.[Execution_Id]
				  AND BEPA.[Original_PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
				  AND BEPA.[Original_Action_Sequence] = A.[Action_Sequence]
				  AND BEPA.[Attempt_Sequence] = A.[Attempt_Sequence]
				  AND (  (BEPA.[Server_Id] = A.[Server_Id]
				      OR (BEPA.[Server_Id] IS NULL AND A.[Server_Id] IS NULL)))
	 WHERE BE.[Execution_Id] = @Execution_Id

	   AND BEPA.[BuildExecution_PlanAction_Id] IS NULL -- not already executed

	   AND NOT EXISTS (SELECT * FROM [BuildExecution_PlanActions] _BEPA -- success within a retry set
						WHERE _BEPA.[ExecutionStatus_Name] = 'Succeeded'
						  AND _BEPA.[Execution_Id] = BE.[Execution_Id]
						  AND _BEPA.[Original_PlanActionGroup_Id] = A.[PlanActionGroup_Id]
						  AND _BEPA.[Original_Action_Sequence] = A.[Action_Sequence]
						  AND _BEPA.[Server_Id] = A.[Server_Id])

	 ORDER BY A.[PlanExecution_Sequence], A.[Action_Sequence], A.[Attempt_Sequence]


	RETURN
END
GO

GO
IF OBJECT_ID('[__BuildMaster_StoredProcInfo]') IS NOT NULL
	DROP TABLE [__BuildMaster_StoredProcInfo]
GO

CREATE TABLE [__BuildMaster_StoredProcInfo]
(
    [StoredProc_Name] SYSNAME,
    CONSTRAINT [PK__BuildMaster_StoredProcInfo]
        PRIMARY KEY ([StoredProc_Name]),
    [Internal_Indicator] YNINDICATOR,
    [ReturnType_Name] VARCHAR(100) NULL,
    CONSTRAINT [CK__BuildMaster_StoredProcInfo__ReturnType_Name_Domain]
        CHECK ([ReturnType_Name] IS NULL OR [ReturnType_Name] IN ('void', 'DataRow', 'DataTable', 'DataSet')),
    [DataTableNames_Csv] VARCHAR(255) NULL,
    [OutputPropertyNames_Csv] VARCHAR(255) NULL,
    [Description_Text] VARCHAR(MAX),
    [Remarks_Text] VARCHAR(MAX)
)
GO

IF OBJECT_ID('[__BuildMaster_AddStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_AddStoredProcInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_AddStoredProcInfo]
(
	@StoredProc_Name SYSNAME,
	@Internal_Indicator YNINDICATOR,
	@ReturnType_Name VARCHAR(100),
	@DataTableNames_Csv VARCHAR(255),
	@OutputPropertyNames_Csv VARCHAR(100),
	@Description_Text VARCHAR(MAX),
	@Remarks_TEXT VARCHAR(MAX) = NULL
)
AS
BEGIN
	
	IF EXISTS(SELECT * FROM [__BuildMaster_StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name)
		DELETE [__BuildMaster_StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name

    INSERT INTO [__BuildMaster_StoredProcInfo]
    (
         [StoredProc_Name]
        ,[Internal_Indicator]
        ,[ReturnType_Name]
        ,[DataTableNames_Csv]
        ,[OutputPropertyNames_Csv]
        ,[Description_Text]
        ,[Remarks_Text]
    )
    VALUES
    (
         @StoredProc_Name
	    ,@Internal_Indicator
	    ,@ReturnType_Name
	    ,@DataTableNames_Csv
	    ,@OutputPropertyNames_Csv
	    ,@Description_Text
	    ,@Remarks_Text
    )

END
GO

GRANT EXECUTE ON [__BuildMaster_AddStoredProcInfo] TO [BuildMasterUser_Role]
GO

IF OBJECT_ID('[__BuildMaster_GetStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetStoredProcInfo]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetStoredProcInfo]
(
	@StoredProc_Name SYSNAME = NULL
)
AS
BEGIN
    
    SET NOCOUNT ON
    
    IF @StoredProc_Name IS NULL
	    SELECT * FROM [__BuildMaster_StoredProcInfo]
	ElSE
		SELECT * FROM [__BuildMaster_StoredProcInfo] WHERE [StoredProc_Name] = @StoredProc_Name

END
GO

GRANT EXECUTE ON [__BuildMaster_GetStoredProcInfo] TO [BuildMasterUser_Role]
GO

GO
IF OBJECT_ID('[__BuildMaster_GetEventReference]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetEventReference]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetEventReference]
AS
BEGIN
	
	SELECT [Event_Code], [Event_Description], [StoredProc_Name] = ROUTINE_NAME
	  FROM INFORMATION_SCHEMA.ROUTINES 
		   INNER JOIN [EventTypes]
				   ON ROUTINE_DEFINITION LIKE '%' + [Event_Code] + '%' 
				  AND ROUTINE_TYPE='PROCEDURE'
	 ORDER BY 1

END
GO

GRANT EXECUTE ON [__BuildMaster_GetEventReference] TO [BuildMasterUser_Role]
GO

GO
IF OBJECT_ID('[__BuildMaster_GetSecurityTasks]') IS NOT NULL
	DROP PROCEDURE [__BuildMaster_GetSecurityTasks]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [__BuildMaster_GetSecurityTasks]
AS
BEGIN
	
	SELECT * FROM [Tasks]

END
GO

GRANT EXECUTE ON [__BuildMaster_GetSecurityTasks] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_RaiseEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Events_RaiseEvent]') IS NOT NULL
	DROP PROCEDURE [Events_RaiseEvent]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_RaiseEvent]
(
	@Event_Code VARCHAR(8),
	@Application_Id INT = NULL,
	@detail1 SQL_VARIANT = NULL,
	@detail2 SQL_VARIANT = NULL,
	@detail3 SQL_VARIANT = NULL,
	@detail4 SQL_VARIANT = NULL,
	@detail5 SQL_VARIANT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @User_Name USERNAME
		SET @User_Name = dbo.Session_GetUserName()

	INSERT INTO [EventOccurences]
	 ([Event_Code],[User_Name],[Occurence_Date],[Application_Id])
	 VALUES
	 (@Event_Code, @User_Name, GETUTCDATE(), @Application_Id)

	DECLARE @EventOccurence_Id INT
		SET @EventOccurence_Id = SCOPE_IDENTITY()

	INSERT INTO [EventOccurenceDetails]
	 ([EventOccurence_Id],[Event_Code],[Detail_Name],[Detail_Value])
	SELECT @EventOccurence_Id
		  ,[Event_Code]
		  ,[Detail_Name]
		  ,CASE [Detail_Sequence]
				WHEN 1 THEN @detail1
				WHEN 2 THEN @detail2
				WHEN 3 THEN @detail3
				WHEN 4 THEN @detail4
				WHEN 5 THEN @detail5
				ELSE NULL END
	  FROM [EventTypeDetails]
	 WHERE [Event_Code] = @Event_Code

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Events_RaiseEvent] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'HandleError',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[HandleError]') IS NOT NULL DROP PROCEDURE [HandleError]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [HandleError] AS BEGIN
   -- Return if there is no error information to retrieve.
    IF ERROR_NUMBER() IS NULL
        RETURN;

    DECLARE 
        @ErrorMessage    NVARCHAR(4000),
        @ErrorNumber     INT,
        @ErrorSeverity   INT,
        @ErrorState      INT,
        @ErrorLine       INT,
        @ErrorProcedure  NVARCHAR(200);

    -- Assign variables to error-handling functions that 
    -- capture information for RAISERROR.
    SELECT 
        @ErrorNumber = ERROR_NUMBER(),
        @ErrorSeverity = ERROR_SEVERITY(),
        @ErrorState = ERROR_STATE(),
        @ErrorLine = ERROR_LINE(),
        @ErrorProcedure = REPLACE(ISNULL(ERROR_PROCEDURE(), '-'),'`','''');

    -- Building the message string that will contain original
    -- error information.
    SELECT @ErrorMessage = N'%d`%d`%d`%s`%d`'+ ERROR_MESSAGE();

    -- Raise an error: msg_str parameter of RAISERROR will contain
    -- the original error information.
    RAISERROR 
        (
        @ErrorMessage, 
        @ErrorSeverity, 
        42,               
        @ErrorNumber,    -- parameter: original error number.
        @ErrorSeverity,  -- parameter: original error severity.
        @ErrorState,     -- parameter: original error state.
        @ErrorProcedure, -- parameter: original error procedure name.
        @ErrorLine       -- parameter: original error line number.
        );
END
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'InvalidateLastActionReexecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[InvalidateLastActionReexecution]') IS NOT NULL DROP PROCEDURE [InvalidateLastActionReexecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [InvalidateLastActionReexecution] 
(
	@Application_Id INT,
	@Environment_Id INT = NULL
)
AS BEGIN
				
	UPDATE [BuildExecutions]
	   SET [PlanModifiedSinceLastExecution_Indicator] = 'Y'
	 WHERE [Application_Id] = @Application_Id
	   AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])

END
GO



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_CreateOrUpdateApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApplicationGroup_Id',
    /* Description_Text        */ 'Creates or updates a new application group'
GO

IF OBJECT_ID('[ApplicationGroups_CreateOrUpdateApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_CreateOrUpdateApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_CreateOrUpdateApplicationGroup]
(
	@ApplicationGroup_Id INT = NULL OUT,
	@ApplicationGroup_Name VARCHAR(50),
	@ApplicationGroup_Description VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @ApplicationGroup_Id IS NULL
	BEGIN
		INSERT INTO [ApplicationGroups]
		(
			[ApplicationGroup_Name],
			[ApplicationGroup_Description],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@ApplicationGroup_Name,
			@ApplicationGroup_Description,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @ApplicationGroup_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [ApplicationGroups]
		   SET [ApplicationGroup_Name] = @ApplicationGroup_Name,
			   [ApplicationGroup_Description] = @ApplicationGroup_Description,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_CreateOrUpdateApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_DeleteApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an application group'
GO

IF OBJECT_ID('[ApplicationGroups_DeleteApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_DeleteApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_DeleteApplicationGroup]
(
	@ApplicationGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION
	
        UPDATE [Applications]
           SET [ApplicationGroup_Id] = NULL
         WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

		DELETE [ApplicationGroups]
		 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_DeleteApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ApplicationGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an application group by ID.'
GO

IF OBJECT_ID('[ApplicationGroups_GetApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetApplicationGroup]
(
	@ApplicationGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON
	SELECT *
	  FROM [ApplicationGroups]
	 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetApplicationGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ApplicationGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the application groups'
GO

IF OBJECT_ID('[ApplicationGroups_GetApplicationGroups]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetApplicationGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetApplicationGroups]
AS
BEGIN

SET NOCOUNT ON
	SELECT * FROM [ApplicationGroups] ORDER BY [ApplicationGroup_Name]

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetApplicationGroups] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ApplicationGroups_GetOrCreateApplicationGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ApplicationGroup_Id',
    /* Description_Text        */ 'Gets or creates a new application group using the name'
GO

IF OBJECT_ID('[ApplicationGroups_GetOrCreateApplicationGroup]') IS NOT NULL DROP PROCEDURE [ApplicationGroups_GetOrCreateApplicationGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ApplicationGroups_GetOrCreateApplicationGroup]
(
	@ApplicationGroup_Name VARCHAR(50),
    @ApplicationGroup_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Existing_ApplicationGroup_Id INT
    DECLARE @Existing_ApplicationGroup_Name VARCHAR(50)

    SELECT @Existing_ApplicationGroup_Name = [ApplicationGroup_Name],
           @Existing_ApplicationGroup_Id = [ApplicationGroup_Id]
      FROM [ApplicationGroups]
     WHERE [ApplicationGroup_Name] = @ApplicationGroup_Name
    
    IF (@ApplicationGroup_Name IS NULL OR LTRIM(RTRIM(@ApplicationGroup_Name)) = '')
    BEGIN
        
        SET @ApplicationGroup_Id = NULL

    END
	ELSE IF (@Existing_ApplicationGroup_Id IS NULL)
	BEGIN

		INSERT INTO [ApplicationGroups]
		(
			[ApplicationGroup_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@ApplicationGroup_Name,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		
		SET @ApplicationGroup_Id = SCOPE_IDENTITY()

	END
    ELSE
    BEGIN

        SET @ApplicationGroup_Id = @Existing_ApplicationGroup_Id

    END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ApplicationGroups_GetOrCreateApplicationGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_AddImportedDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds an imported deployable to an application'
GO

IF OBJECT_ID('[Applications_AddImportedDeployable]') IS NOT NULL DROP PROCEDURE [Applications_AddImportedDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_AddImportedDeployable]
(
	@Deployable_Id INT,
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
		
		IF NOT EXISTS(
			SELECT * FROM DeployablesImported
			WHERE [Deployable_Id] = @Deployable_Id
			  AND [Application_Id] = @Application_Id
		)
		BEGIN
			INSERT INTO [DeployablesImported]
			(
				[Deployable_Id],
				[Application_Id]
			)
			VALUES
			(
				@Deployable_Id,
				@Application_Id
			)
		END

		/* Raise Application Modified Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_AddImportedDeployable] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CloneApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'New_Application_Id',
    /* Description_Text        */ 'Clones an application, but does not copy any history'
GO

IF OBJECT_ID('[Applications_CloneApplication]') IS NOT NULL
	DROP PROCEDURE [Applications_CloneApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CloneApplication]
(
	@Application_Id INT,
	@LinkAllActionGroups_Indicator YNINDICATOR = 'N',
	@New_Application_Name VARCHAR(50),
	@New_Application_Id INT = NULL OUT
) AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)
		
	DECLARE @Application_Name VARCHAR(50)
	SET @Application_Name = (SELECT [Application_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)

	/* Application */		
	INSERT INTO [Applications] ([Application_Name], [ReleaseNumber_Scheme_Name],[BuildNumber_Scheme_Name],[IssueTracking_Provider_Id],[AllowMultipleActiveReleases_Indicator],[AllowMultipleActiveBuilds_Indicator],[ApplicationGroup_Id],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[VariableSupport_Code])
	SELECT @New_Application_Name, [ReleaseNumber_Scheme_Name],[BuildNumber_Scheme_Name],[IssueTracking_Provider_Id],[AllowMultipleActiveReleases_Indicator],[AllowMultipleActiveBuilds_Indicator],[ApplicationGroup_Id],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[VariableSupport_Code]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id
	SET @New_Application_Id = SCOPE_IDENTITY()

	/* Deployables */
	INSERT INTO [Deployables] ([Application_Id],[Deployable_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[DeployableType_Code])
	SELECT @New_Application_Id,[Deployable_Name],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[DeployableType_Code]
	 FROM [Deployables]
	WHERE [Application_Id] = @Application_Id
	

	INSERT INTO [DeployableDependencies] ([Deployable_Id],[DependsOn_Deployable_Id])
	SELECT D_NEW.[Deployable_Id],D2_NEW.[Deployable_Id]
	  FROM [DeployableDependencies] DD
	       INNER JOIN [Deployables] D
	               ON D.[Deployable_Id] = DD.[Deployable_Id]
	       INNER JOIN [Deployables] D2
	               ON D2.[Deployable_Id] = DD.[DependsOn_Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id
	       INNER JOIN [Deployables] D2_NEW
	               ON D2_NEW.[Deployable_Name] = D2.[Deployable_Name]
	              AND D2_NEW.[Application_Id] = @New_Application_Id
	 WHERE D.[Application_Id] = @Application_Id

	INSERT INTO [DeployablesImported]([Deployable_Id],[Application_Id])
	SELECT [Deployable_Id], @New_Application_Id
	  FROM [DeployablesImported]
	 WHERE [Application_Id] = @Application_Id
	  
	/* Workflows */
	INSERT INTO [Workflows] ([Application_Id],[Workflow_Name],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[AutoDeployRelease_Indicator],[AllowOutOfSequencePromotions_Indicator],[AutoCancelReleases_Indicator],[AutoIncrementRelease_Indicator])
	SELECT @New_Application_Id,[Workflow_Name],[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[AutoDeployRelease_Indicator],[AllowOutOfSequencePromotions_Indicator],[AutoCancelReleases_Indicator],[AutoIncrementRelease_Indicator]
	  FROM [Workflows]
	 WHERE [Application_Id] = @Application_Id
	   AND [Active_Indicator] = 'Y'

	INSERT [WorkflowSteps]([Workflow_Id],[Environment_Id],[Step_Sequence])
	SELECT W_NEW.[Workflow_Id],[Environment_Id],[Step_Sequence]
	  FROM [WorkflowSteps] WS
	       INNER JOIN [Workflows] W
	               ON W.[Workflow_Id] = WS.[Workflow_Id]
	       INNER JOIN [Workflows] W_NEW
	               ON W_NEW.[Workflow_Name] = W.[Workflow_Name]
	              AND W_NEW.[Application_Id] = @New_Application_Id
	WHERE W.[Application_Id] = @Application_Id

	INSERT INTO [PromotionApprovalRequirements]([Workflow_Id],[Environment_Id],[User_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[Approval_Sequence],[ApprovalType_Code],[Or_Indicator],[RequiredInGroup_Count],[Approval_Description])
	SELECT W_NEW.[Workflow_Id],[Environment_Id],[User_Name],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[Approval_Sequence],[ApprovalType_Code],[Or_Indicator],[RequiredInGroup_Count],[Approval_Description]
	 FROM [PromotionApprovalRequirements] PAR
	      INNER JOIN [Workflows] W
	              ON W.[Workflow_Id] = PAR.[Workflow_Id]
	      INNER JOIN [Workflows] W_NEW
	              ON W_NEW.[Workflow_Name] = W.[Workflow_Name]
	             AND W_NEW.[Application_Id] = @New_Application_Id
	WHERE W.[Application_Id] = @Application_Id

	INSERT INTO [PromotionRequirements]([Workflow_Id],[Environment_Id],[PromotionRequirement_Sequence],[PromotionRequirement_Name],[PromotionRequirement_Description],[PromotionRequirement_Configuration],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
	SELECT W_NEW.[Workflow_Id],[Environment_Id],[PromotionRequirement_Sequence],[PromotionRequirement_Name],[PromotionRequirement_Description],[PromotionRequirement_Configuration],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE()
	  FROM [PromotionRequirements] PR
	       INNER JOIN [Workflows] W
	               ON W.[Workflow_Id] = PR.[Workflow_Id]
	       INNER JOIN [Workflows] W_NEW
	              ON W_NEW.[Workflow_Name] = W.[Workflow_Name]
	             AND W_NEW.[Application_Id] = @New_Application_Id
	WHERE W.[Application_Id] = @Application_Id	   
	
	/* Set-up Release */
	DECLARE @New_Workflow_Id INT
	   SET  @New_Workflow_Id = (SELECT TOP 1 [Workflow_Id] FROM [Workflows] WHERE [Application_Id] = @New_Application_Id)
	
	DECLARE @New_Release_Number RELEASENUMBER
	SELECT @New_Release_Number = CASE [ReleaseNumber_Scheme_Name]
	         WHEN 'MajorMinorRevision' THEN '0.0.0'
	         WHEN 'MajorMinor' THEN '0.0'
	         WHEN 'DateBased' THEN '0' END
	  FROM [Applications]
	 WHERE [Application_Id] = @New_Application_Id
	 
	IF @New_Workflow_Id IS NOT NULL
		INSERT INTO [Releases]([Application_Id],[Release_Number],[Workflow_Id],[ReleaseStatus_Name],[Notes_Text],[Target_Date],[Release_Name],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[CancelledBy_User_Name],[CancelledReason_Text],[CancelledOn_Date],[Release_Sequence])
		VALUES (@New_Application_Id,@New_Release_Number,@New_Workflow_Id,'Active',NULL,NULL,'Setup',dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),NULL,NULL,NULL, 1)

    IF @New_Workflow_Id IS NOT NULL
        INSERT [ReleaseDeployables] ([Application_Id], [Release_Number], [Deployable_Id], [InclusionType_Code])
        SELECT @New_Application_Id, @New_Release_Number, [Deployable_Id], 'I' FROM [Deployables] WHERE [Application_Id] = @New_Application_Id
    

	/* Variables */
    INSERT INTO [VariableDeclarations] 
    (
		[Scope_Code],
		[Variable_Name],
		[Scoped_Application_Id],
		[Scoped_Environment_Id],
		[Required_Indicator],
		[Variable_Configuration],
		[DefaultValue_Text],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[Sensitive_Indicator]
	)
    SELECT [Scope_Code],
		   [Variable_Name],
		   @New_Application_Id,
		   [Scoped_Environment_Id],
		   [Required_Indicator],
		   [Variable_Configuration],
		   [DefaultValue_Text],
		   dbo.Session_GetUserName(),
		   GETUTCDATE(),
		   [Sensitive_Indicator]
     FROM [VariableDeclarations] 
	WHERE [Scoped_Application_Id] = @Application_Id

	/* Build Schedules */
	INSERT INTO [BuildSchedules] ([Schedule_Name],[Application_Id],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],[Schedule_Type],[StartTime_Minutes],[EndTime_Minutes],[DailyInterval_Minutes],[Recurrence_Type],[RecurrenceFrequency_Value],[RecurrenceWeeklyDay_Mask],[RecurrenceMonthlyDay_Value],[RecurrenceMonthlyRelativity_Type],[RecurrenceMonthlyRelativityDay_Type],[RecurrenceMonthlyRelativityWeek_Type],[RecurrenceMonthlyRelativityWeekDay_Value],[Url_AllowedIP_CSV],[Url_Username_Text],[Url_Password_Text],[SourceControl_Path_Name],[SourceControl_Provider_Id],[SourceControl_QuietPeriod_Minutes],[ReleaseFilter_ReleaseType_Code])
	SELECT [Schedule_Name],@New_Application_Id,[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),[Schedule_Type],[StartTime_Minutes],[EndTime_Minutes],[DailyInterval_Minutes],[Recurrence_Type],[RecurrenceFrequency_Value],[RecurrenceWeeklyDay_Mask],[RecurrenceMonthlyDay_Value],[RecurrenceMonthlyRelativity_Type],[RecurrenceMonthlyRelativityDay_Type],[RecurrenceMonthlyRelativityWeek_Type],[RecurrenceMonthlyRelativityWeekDay_Value],[Url_AllowedIP_CSV],[Url_Username_Text],[Url_Password_Text],[SourceControl_Path_Name],[SourceControl_Provider_Id],[SourceControl_QuietPeriod_Minutes],[ReleaseFilter_ReleaseType_Code]
	  FROM [BuildSchedules]
	 WHERE [Application_Id] = @Application_Id
   
	INSERT INTO [BuildScheduleVariableDefinitions]([Schedule_Id],[VariableDeclaration_Id],[Value_Text])
	 SELECT BS_NEW.[Schedule_Id], VD_NEW.[Variable_Name],BSVD.[Value_Text]
	   FROM [BuildSchedules] BS
	        INNER JOIN [BuildScheduleVariableDefinitions] BSVD
	                ON BSVD.[Schedule_Id] = BS.[Schedule_Id]
	        INNER JOIN [VariableDeclarations] VD
	                ON BSVD.[VariableDeclaration_Id] = VD.[VariableDeclaration_Id]
	        INNER JOIN [BuildSchedules] BS_NEW
	                ON BS_NEW.[Schedule_Name] = BS.[Schedule_Name]
	               AND BS_NEW.[Application_Id] = @New_Application_Id
            INNER JOIN [VariableDeclarations] VD_NEW
                    ON VD_NEW.[Scoped_Application_Id] = VD.[Scoped_Application_Id]
                   AND VD_NEW.[Scope_Code] = VD.[Scope_Code]
                   AND VD_NEW.[Variable_Name] = VD.[Variable_Name]
	  WHERE BS.[Application_Id] = @Application_Id

	/* Privileges */
	INSERT INTO [Privileges]([Principal_Name],[PrincipalType_Code],[Role_Id],[Application_Id],[Environment_Id],[PrivilegeType_Code],[Directory_Provider_Id],[ApplicationGroup_Id])
	SELECT [Principal_Name],[PrincipalType_Code],[Role_Id],@New_Application_Id,[Environment_Id],[PrivilegeType_Code],[Directory_Provider_Id],[ApplicationGroup_Id]
	  FROM [Privileges]
	 WHERE [Application_Id] = @Application_Id




	/* Plans (Plans) */
	DECLARE @PLANIDS TABLE ([Plan_Id] INT, [New_Plan_Id] INT)
	DECLARE @PLANCUR_Plan_Id INT, @PLANCUR CURSOR
	    SET @PLANCUR = CURSOR FOR SELECT [Plan_Id]
	                                FROM [Plans]
	                               WHERE [Plan_Id] IN (SELECT [ActionGroup_Id] 
	                                                     FROM [PlanActionGroups]
	                                                    WHERE [Application_Id] = @Application_Id)
	OPEN @PLANCUR
	FETCH NEXT FROM @PLANCUR INTO @PLANCUR_Plan_Id
	WHILE @@FETCH_STATUS <> -1 BEGIN
		
		IF (@LinkAllActionGroups_Indicator = 'N' AND 1 = (SELECT COUNT(*) FROM [PlanActionGroups] WHERE [ActionGroup_Id] = @PLANCUR_Plan_Id)) BEGIN
			
			INSERT INTO [Plans] 
			  ([CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date],
			   [ActionGroup_Name],[Predicate_Configuration],[ActionGroup_Description])
			SELECT
			   dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE(),
			   [ActionGroup_Name],[Predicate_Configuration],[ActionGroup_Description]
			FROM [Plans] WHERE [Plan_Id] = @PLANCUR_Plan_Id

			INSERT INTO @PLANIDS ([Plan_Id],[New_Plan_Id])
			VALUES (@PLANCUR_Plan_Id, SCOPE_IDENTITY())


		END ELSE BEGIN

			INSERT INTO @PLANIDS ([Plan_Id],[New_Plan_Id])
			VALUES (@PLANCUR_Plan_Id, @PLANCUR_Plan_Id)

		END
		FETCH NEXT FROM @PLANCUR INTO @PLANCUR_Plan_Id
	END CLOSE @PLANCUR


	/* Plans (PlanActionGroups) */
	DECLARE @PAGIDS TABLE ([PlanActionGroup_Id] INT, [New_PlanActionGroup_Id] INT)
	DECLARE @PAGCUR_PlanActionGroup_Id INT, @PAGCUR CURSOR
	    SET @PAGCUR = CURSOR FOR SELECT [PlanActionGroup_Id] 
	                               FROM [PlanActionGroups] 
	                              WHERE [Application_Id] = @Application_Id
								 ORDER BY [PlanActionGroup_Sequence]
	OPEN @PAGCUR
	FETCH NEXT FROM @PAGCUR INTO @PAGCUR_PlanActionGroup_Id
	WHILE @@FETCH_STATUS <> -1 BEGIN
	
		INSERT INTO [PlanActionGroups]
           ([ActionGroup_Id],[Application_Id],[Environment_Id],[PlanActionGroup_Sequence],[Deployable_Id],
		    [Active_Indicator],[ActionGroup_Name],[ActionGroup_Description],
			[Predicate_Configuration],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[Server_Id])
		SELECT 
			PIDS.[New_Plan_Id], @New_Application_Id, [Environment_Id], [PlanActionGroup_Sequence], COALESCE(D_NEW.[Deployable_Id],DI.[Deployable_Id]),
			[Active_Indicator],[ActionGroup_Name],[ActionGroup_Description],
			[Predicate_Configuration],
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			[Server_Id]

		  FROM [PlanActionGroups] PAG
		       INNER JOIN @PLANIDS PIDS
			           ON PAG.[ActionGroup_Id] = PIDS.[Plan_Id]
			   LEFT  JOIN [Deployables] D
					   ON D.[Deployable_Id] = PAG.[Deployable_Id]
					  AND D.[Application_Id] = PAG.[Application_Id]
					  AND PAG.[Deployable_Id] IS NOT NULL
			   LEFT  JOIN [Deployables] D_NEW
					   ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
					  AND D_NEW.[Application_Id] = @New_Application_Id
			   LEFT  JOIN [DeployablesImported] DI
					   ON DI.[Deployable_Id] = PAG.[Deployable_Id]
					  AND DI.[Application_Id] = @New_Application_Id
					  AND PAG.[Deployable_Id] IS NOT NULL
		 WHERE PAG.[PlanActionGroup_Id] = @PAGCUR_PlanActionGroup_Id
		
		INSERT INTO @PAGIDS ([PlanActionGroup_Id],[New_PlanActionGroup_Id])
		VALUES (@PAGCUR_PlanActionGroup_Id, SCOPE_IDENTITY())
		FETCH NEXT FROM @PAGCUR INTO @PAGCUR_PlanActionGroup_Id

	END CLOSE @PAGCUR

	INSERT INTO [PlanActions]
	(
		[Plan_Id],
		[Action_Sequence],
		[Server_Id],
		[Action_Description],
		[ResumeNextOnFailure_Indicator],
		[Action_Configuration],
		[ActionType_Name],
		[Active_Indicator],
		[Retry_Count],
		[LogFailureAsWarning_Indicator],
		[Server_Variable_Name],
		[Target_Server_Id],
		[Target_Server_Variable_Name],
		[ExtensionConfiguration_Id]
	)
	SELECT PLANIDS.[New_Plan_Id],
	       [Action_Sequence],
	       [Server_Id],
	       [Action_Description],
	       [ResumeNextOnFailure_Indicator],
		   [Action_Configuration],
		   [ActionType_Name],
		   [Active_Indicator],
		   [Retry_Count],
		   [LogFailureAsWarning_Indicator],
		   [Server_Variable_Name],
		   [Target_Server_Id],
		   [Target_Server_Variable_Name],
		   [ExtensionConfiguration_Id]
	  FROM [PlanActions] PA
			   INNER JOIN @PLANIDS PLANIDS
					   ON PLANIDS.[Plan_Id] = PA.[Plan_Id]
     WHERE PLANIDS.[New_Plan_Id] <> PLANIDS.[Plan_Id]


	INSERT INTO [ConfigurationFiles]([Deployable_Id],[FilePath_Text],[Active_Indicator],[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
	SELECT D_NEW.[Deployable_Id],[FilePath_Text],[Active_Indicator],dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE()
	  FROM [ConfigurationFiles] CF
	       INNER JOIN [Deployables] D
			       ON D.[Deployable_Id] = CF.[Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id
	 WHERE CF.[Active_Indicator] = 'Y'
	   AND D.[Application_Id] = @Application_Id
	   AND @New_Release_Number IS NOT NULL

	INSERT INTO [ConfigurationFileInstances]([ConfigurationFile_Id],[Instance_Name],[Active_Indicator],[Environment_Id],[Template_Indicator],[Template_Instance_Name])
	SELECT CF_NEW.[ConfigurationFile_Id],[Instance_Name],'Y',[Environment_Id],[Template_Indicator],[Template_Instance_Name]
	  FROM [ConfigurationFileInstances] CFI
	       INNER JOIN [ConfigurationFiles] CF
	               ON CF.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
	       INNER JOIN [Deployables] D
			       ON D.[Deployable_Id] = CF.[Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id			       
	       INNER JOIN [ConfigurationFiles] CF_NEW
	               ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
	              AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]
	 WHERE CFI.[Active_Indicator] = 'Y'
	   AND D.[Application_Id] = @Application_Id


	INSERT INTO [ConfigurationFileVersions] ([ConfigurationFile_Id],[Instance_Name],[Version_Number],[VersionNotes_Text],[File_Date],[File_Bytes],[CreatedBy_User_Name],[CreatedOn_Date],[Application_Id],[Release_Number])
	SELECT CF_NEW.[ConfigurationFile_Id],CFV.[Instance_Name],1,'Cloned from ' + @Application_Name,GETUTCDATE(),[File_Bytes],dbo.Session_GetUserName(),GETUTCDATE(),@New_Application_Id,@New_Release_Number
	  FROM [ConfigurationFileVersions] CFV
		   INNER JOIN (SELECT [ConfigurationFile_Id],[Instance_Name],[Max_Version_Number] = MAX([Version_Number])
						 FROM [ConfigurationFileVersions]
						GROUP BY [ConfigurationFile_Id],[Instance_Name]) CFVMAX
				   ON CFVMAX.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
				  AND CFVMAX.[Instance_Name] = CFV.[Instance_Name]
				  AND CFVMAX.[Max_Version_Number] = CFV.[Version_Number]
			   INNER JOIN [ConfigurationFileInstances] CFI
					   ON CFI.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
					  AND CFI.[Instance_Name] = CFV.[Instance_Name]
			   INNER JOIN [ConfigurationFiles] CF
					   ON CF.[ConfigurationFile_Id] = CFI.[ConfigurationFile_Id]
			   INNER JOIN [Deployables] D
					   ON D.[Deployable_Id] = CF.[Deployable_Id]
			   INNER JOIN [Deployables] D_NEW
					   ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
					  AND D_NEW.[Application_Id] = @New_Application_Id			       
			   INNER JOIN [ConfigurationFiles] CF_NEW
					   ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
					  AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]
			   INNER JOIN [ConfigurationFileInstances] CFI_NEW
					   ON CFI_NEW.[ConfigurationFile_Id] = CF_NEW.[ConfigurationFile_Id]
					  AND CFI_NEW.[Instance_Name] = CFI.[Instance_Name]
	 WHERE D.[Application_Id] = @Application_Id
	 
	/* Update Action Ids */
	SELECT PA.[Plan_Id], PA.[Action_Sequence], [Action_Configuration] = CAST(
	       REPLACE(CAST(PA.[Action_Configuration] AS NVARCHAR(MAX)),
	         'ConfigurationFileId="' + CAST(CF.[ConfigurationFile_Id] AS NVARCHAR(MAX)) + '"',
	         'ConfigurationFileId="' + CAST(CF_NEW.[ConfigurationFile_Id] AS NVARCHAR(MAX)) + '"') AS XML)
	  INTO #ActionUpdates
	  FROM [ConfigurationFiles] CF
	       INNER JOIN [Deployables] D
			       ON D.[Deployable_Id] = CF.[Deployable_Id]
	       INNER JOIN [Deployables] D_NEW
	               ON D_NEW.[Deployable_Name] = D.[Deployable_Name]
	              AND D_NEW.[Application_Id] = @New_Application_Id			       
	       INNER JOIN [ConfigurationFiles] CF_NEW
	               ON CF_NEW.[FilePath_Text] = CF.[FilePath_Text]
	              AND CF_NEW.[Deployable_Id] = D_NEW.[Deployable_Id]       
	       INNER JOIN [PlanActionGroups] PAG_NEW
		           ON PAG_NEW.[Application_Id] = @New_Application_Id
		   INNER JOIN [Plans] P
		           ON PAG_NEW.[ActionGroup_Id] = P.[Plan_Id]
				  AND 1 = (SELECT COUNT(*) FROM [PlanActionGroups] _PAG WHERE _PAG.[ActionGroup_Id] = P.[Plan_Id])
		   INNER JOIN [PlanActions] PA
		           ON PA.[Plan_Id] = P.[Plan_Id]
		          AND CAST(PA.[Action_Configuration] AS NVARCHAR(MAX)) LIKE '%ConfigurationFileId="' + CAST(CF.[ConfigurationFile_Id] AS NVARCHAR(MAX)) + '"%'
	 WHERE D.[Application_Id] = @Application_Id

	UPDATE [PlanActions]
	   SET [Action_Configuration] = PAU.[Action_Configuration]
	  FROM [PlanActions] PA
	        INNER JOIN #ActionUpdates PAU
	                ON PAU.[Plan_Id] = PA.[Plan_Id]
	               AND PAU.[Action_Sequence] = PA.[Action_Sequence]

	/* Providers aren't cloned */
	UPDATE [PlanActions]
	   SET [Active_Indicator] = 'N'
	  FROM [PlanActions] PA
	       INNER JOIN @PLANIDS PIDS
		           ON PA.[Plan_Id] = PIDS.[New_Plan_Id]
	 WHERE CAST([Action_Configuration].query('local-name(/*[1])') AS NVARCHAR(MAX)) LIKE 'Inedo.BuildMaster.Extensibility.Actions.Database%'		           
	   AND PIDS.[Plan_Id] <> PIDS.[New_Plan_Id]


	INSERT INTO [FeatureLevels]([Feature_Code],[Application_Id],[Feature_Level])
	SELECT [Feature_Code],@New_Application_Id,[Feature_Level]
	  FROM [FeatureLevels]
	 WHERE [Application_Id] = @Application_Id


	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPNEW', @New_Application_Id


	COMMIT
END TRY BEGIN CATCH
	ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CloneApplication] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Application_Id',
    /* Description_Text        */ 'Creates a new application'
GO

IF OBJECT_ID('[Applications_CreateApplication]') IS NOT NULL DROP PROCEDURE [Applications_CreateApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateApplication]
(
	@Application_Name VARCHAR(50),
	@ReleaseNumber_Scheme_Name VARCHAR(20) = NULL,
	@IssueTracking_Provider_Id INT = NULL,
	@BuildNumber_Scheme_Name VARCHAR(15) = NULL,
	@AllowMultipleActiveReleases_Indicator YNINDICATOR  = NULL,
	@AllowMultipleActiveBuilds_Indicator YNINDICATOR  = NULL,
	@ApplicationGroup_Id INT = NULL,
	@Application_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON 

BEGIN TRY
	BEGIN TRANSACTION

	/* Set defaults */
	SET @ReleaseNumber_Scheme_Name = ISNULL(@ReleaseNumber_Scheme_Name, 'MajorMinor')
	SET @BuildNumber_Scheme_Name = ISNULL(@BuildNumber_Scheme_Name, 'Sequential')
	SET @AllowMultipleActiveReleases_Indicator = ISNULL(@AllowMultipleActiveReleases_Indicator, 'N')
	SET @AllowMultipleActiveBuilds_Indicator = ISNULL(@AllowMultipleActiveBuilds_Indicator, 'N')

	INSERT INTO [Applications] (
		[Application_Name], 
		[ReleaseNumber_Scheme_Name],
		[BuildNumber_Scheme_Name],
		[IssueTracking_Provider_Id],
		[AllowMultipleActiveReleases_Indicator],
		[AllowMultipleActiveBuilds_Indicator],
		[ApplicationGroup_Id],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date],
		[VariableSupport_Code]
	)
	VALUES (
		@Application_Name,
		@ReleaseNumber_Scheme_Name,
		@BuildNumber_Scheme_Name,
		@IssueTracking_Provider_Id,
		@AllowMultipleActiveReleases_Indicator,
		@AllowMultipleActiveBuilds_Indicator,
		@ApplicationGroup_Id,
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		'N'
	)
	SET @Application_Id = SCOPE_IDENTITY()
	
	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPNEW',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateOrUpdateDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Deployable_Id',
    /* Description_Text        */ 'Creates a deployable for a specified application, or updates an existing one'
GO


IF OBJECT_ID('[Applications_CreateOrUpdateDeployable]') IS NOT NULL DROP PROCEDURE [Applications_CreateOrUpdateDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateOrUpdateDeployable]
(
	@Deployable_Id INT OUT,
	@Application_Id INT,
	@Deployable_Name VARCHAR(50),
	@DeployableType_Code CHAR(4)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Deployable_Id IS NOT NULL
	BEGIN
		UPDATE [Deployables]
		   SET [Deployable_Name] = @Deployable_Name,
		       [DeployableType_Code] = @DeployableType_Code,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Deployable_Id] = @Deployable_Id
		IF @@ROWCOUNT=0 RAISERROR('Deployable %i does not exist.',16,1,@Deployable_Id)

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	END
	ELSE
	BEGIN
		INSERT INTO [Deployables]
		(
			[Application_Id],
			[Deployable_Name],
			[DeployableType_Code],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
        (
			@Application_Id,
			@Deployable_Name,
			@DeployableType_Code,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)
		SET @Deployable_Id = SCOPE_IDENTITY()

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPNWD',@Application_Id,@Deployable_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateOrUpdateDeployable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_DeleteImportedDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an existing imported deployable relation, but does not delete the deployable from the host application'
GO

IF OBJECT_ID('[Applications_DeleteImportedDeployable]') IS NOT NULL DROP PROCEDURE [Applications_DeleteImportedDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_DeleteImportedDeployable]
(
	@Deployable_Id INT,
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [DeployablesImported]
		WHERE [Deployable_Id] = @Deployable_Id
		  AND [Application_Id] = @Application_Id

		/* Raise Application Modified Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_DeleteImportedDeployable] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_EditApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Edits an existing application.'
GO


IF OBJECT_ID('[Applications_EditApplication]') IS NOT NULL DROP PROCEDURE [Applications_EditApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_EditApplication]
(
	@Application_Id INT,
	@Application_Name VARCHAR(50) = NULL,
	@Application_Description VARCHAR(MAX) = NULL,
	@ApplicationGroup_Id INT,
	@AllowMultipleActiveReleases_Indicator YNINDICATOR = NULL,
	@AllowMultipleActiveBuilds_Indicator YNINDICATOR = NULL,
	@VariableSupport_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
       SET [Application_Name] = @Application_Name
		  ,[Application_Description] = @Application_Description
		  ,[ApplicationGroup_Id] = @ApplicationGroup_Id
		  ,[AllowMultipleActiveReleases_Indicator] = @AllowMultipleActiveReleases_Indicator
		  ,[AllowMultipleActiveBuilds_Indicator] = @AllowMultipleActiveBuilds_Indicator
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
		  ,[VariableSupport_Code] = @VariableSupport_Code
     WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_EditApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplication',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an application along with the associated deployables'
GO


IF OBJECT_ID('[Applications_GetApplication]') IS NOT NULL DROP PROCEDURE [Applications_GetApplication]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplication]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Applications_Extended]
     WHERE [Application_Id] = @Application_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Application_Id] = @Application_Id
	ORDER BY [Deployable_Name]

END
GO

GRANT EXECUTE ON [Applications_GetApplication] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified number of applications in the system, or all if null is passed in'
GO


IF OBJECT_ID('[Applications_GetApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplications]
(
    @Application_Count INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Application_Count IS NOT NULL
        SET ROWCOUNT @Application_Count

	SELECT * FROM [Applications_Extended]
	ORDER BY [ApplicationGroup_Name], [Application_Name]

END
GO

GRANT EXECUTE ON [Applications_GetApplications] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetApplicationsVisibleToUser',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the applications which the specified user is allowed to view.'
GO


IF OBJECT_ID('[Applications_GetApplicationsVisibleToUser]') IS NOT NULL DROP PROCEDURE [Applications_GetApplicationsVisibleToUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetApplicationsVisibleToUser]
(
    @User_Name USERNAME,
    @GroupNames_Csv VARCHAR(MAX),
    @Directory_Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

-- Applications_ViewOverview = 40

SELECT AE.*
  FROM [Applications_Extended] AE
 WHERE dbo.IsAuthorizedForTask(@Directory_Provider_Id, @User_Name, @GroupNames_Csv, 40, AE.[ApplicationGroup_Id], AE.[Application_Id], NULL) = 1

END
GO

GRANT EXECUTE ON [Applications_GetApplicationsVisibleToUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDependentApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Applications_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all applications with imported deployables from a specified application'
GO

IF OBJECT_ID('[Applications_GetDependentApplications]') IS NOT NULL DROP PROCEDURE [Applications_GetDependentApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDependentApplications]
(
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT DISTINCT Applications_Extended.* 
		  FROM Applications_Extended, DeployablesImported_Extended
		WHERE [DeployablesImported_Extended].[ImportedFrom_Application_Id] = @Application_Id
			  AND [DeployablesImported_Extended].[Application_Id] = Applications_Extended.[Application_Id]

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetDependentApplications] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified deployable'
GO


IF OBJECT_ID('[Applications_GetDeployable]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployable]
(
	@Deployable_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Deployable_Id] = @Deployable_Id

END
GO

GRANT EXECUTE ON [Applications_GetDeployable] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployableDependencies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeployableDependencies',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of dependencies for the deployables in a given application or a specific deployable if one is specified'
GO

IF OBJECT_ID('[Applications_GetDeployableDependencies]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployableDependencies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployableDependencies]
(
	@Application_Id INT,
	@Deployable_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT * FROM [DeployableDependencies]
		 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM [Deployables] 
		                            WHERE [Application_Id] = @Application_Id)
		   AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetDeployableDependencies] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Deployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the deployables for a specific application'
GO

IF OBJECT_ID('[Applications_GetDeployables]') IS NOT NULL DROP PROCEDURE [Applications_GetDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetDeployables]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Deployables_Extended]
     WHERE [Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Applications_GetDeployables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_GetImportedDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DeployablesImported_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set a imported deployables for a specified application'
GO

IF OBJECT_ID('[Applications_GetImportedDeployables]') IS NOT NULL DROP PROCEDURE [Applications_GetImportedDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_GetImportedDeployables]
(
	@Application_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SELECT * FROM DeployablesImported_Extended
		WHERE @Application_Id IS NULL 
		   OR [Application_Id] = @Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_GetImportedDeployables] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_PurgeApplicationData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire application and all associated data from the system'
GO

IF OBJECT_ID('[Applications_PurgeApplicationData]') IS NOT NULL
	DROP PROCEDURE [Applications_PurgeApplicationData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_PurgeApplicationData]
(
	@Application_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)

	/* 1) Delete data without sub-application dependencies */
	DELETE
	  FROM [BuildScheduleVariableDefinitions]
	 WHERE [Schedule_Id] IN (SELECT [Schedule_Id] 
		                            FROM [BuildSchedules] WHERE [Application_Id] = @Application_Id)
	 
	DELETE
	  FROM [BuildSchedules]
	 WHERE [Application_Id] = @Application_Id	 

	DELETE
	  FROM [Privileges]
	 WHERE [Application_Id] = @Application_Id


	/* Find dependent data: */
	DECLARE @Workflows TABLE ([Workflow_Id] INT NOT NULL)
	INSERT INTO @Workflows ([Workflow_Id])
	SELECT [Workflow_Id]
	  FROM [Workflows]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Deployables TABLE ([Deployable_Id] INT NOT NULL)
	INSERT INTO @Deployables ([Deployable_Id])
	SELECT [Deployable_Id]
	  FROM [Deployables]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @PlanActionGroups TABLE ([PlanActionGroup_Id] INT NOT NULL)
	INSERT INTO @PlanActionGroups ([PlanActionGroup_Id])
	SELECT [PlanActionGroup_Id]
	  FROM [PlanActionGroups]
	 WHERE [Application_Id] = @Application_Id
	    OR [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

	DECLARE @Releases TABLE ([Release_Number] RELEASENUMBER NOT NULL)
	INSERT INTO @Releases ([Release_Number])
	SELECT [Release_Number]
	  FROM [Releases]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Builds TABLE ([Release_Number] RELEASENUMBER NOT NULL, [Build_Number] BUILDNUMBER)
	INSERT INTO @Builds ([Release_Number], [Build_Number])
	SELECT [Release_Number], [Build_Number]
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id
	 
	DECLARE @BuildExecutionPlanActions TABLE ([BuildExecution_PlanAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecutionPlanActions
	SELECT [BuildExecution_PlanAction_Id]
	  FROM [BuildExecution_PlanActions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DECLARE @ConfigurationFiles TABLE ([ConfigurationFile_Id] INT NOT NULL)
	INSERT INTO @ConfigurationFiles ([ConfigurationFile_Id])
	SELECT [ConfigurationFile_Id]
	  FROM [ConfigurationFiles]
	 WHERE [Deployable_Id] IN ( SELECT [Deployable_Id] FROM @Deployables )

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
	INSERT INTO @DatabaseChangeScripts ([Script_Id])
	SELECT [Script_Id]
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id
	 
	DECLARE @ChangeControls TABLE ([ChangeControl_Id] INT NOT NULL)
	INSERT INTO @ChangeControls ([ChangeControl_Id])
	 SELECT [ChangeControl_Id]
	   FROM [ChangeControls]
	  WHERE [Application_Id] = @Application_Id
	  
	DECLARE @Dashboards TABLE ([Dashboard_Id] INT NOT NULL)
	INSERT INTO @Dashboards ([Dashboard_Id])
	 SELECT [Dashboard_Id]
	   FROM [Dashboards]
	  WHERE [Application_Id] = @Application_Id

	/* 2) Delete data with sub-application dependencies */
	/* 2a) Variables */

	DELETE
	  FROM [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [VariableDeclarations]
	 WHERE [Scoped_Application_Id] = @Application_Id

	DELETE
	  FROM [ReleaseNotes]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [BuildPromotionContingencies]
	 WHERE [Application_Id] = @Application_Id
	    OR [Contingent_Application_Id] = @Application_Id
	    
	DELETE
	  FROM [BuildExecution_PlanActionVariableValues]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions)


	/* 2b) Configuration */
	DELETE
	  FROM [ConfigurationFileDeployments]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
	  FROM [ConfigurationFileVersions]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
	  FROM [ConfigurationFileInstances]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
	  FROM [ConfigurationFiles]
	 WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	/* 2c) Builds */
	DELETE
	  FROM [BuildApprovals]
	 WHERE [Application_Id] = @Application_Id

	DELETE
	  FROM [PromotionRequirementStatuses]
	 WHERE [Application_Id] = @Application_Id	 


	DELETE
	  FROM [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id


	DELETE
	  FROM [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )
	 
	 DELETE
	  FROM [BuildOutputs]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_PlanActionVariableValues]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )

	DELETE
	  FROM [BuildExecution_PlanActionLogEntries]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )


	DELETE
	  FROM [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )


	DELETE
	  FROM [BuildExecution_PlanActions]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )


	DELETE
	  FROM [BuildExecution_Plans]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	
	/* Have to delete change script executions before Build Executions... */
	DELETE
	  FROM [DatabaseChangeScriptExecutions]
	 WHERE [Script_Id] IN ( SELECT [Script_Id] FROM @DatabaseChangeScripts )
	 
	DELETE
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )


	DELETE
	  FROM [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id


	DELETE
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id


	/* 2d) Change Scripts */
	DELETE
	  FROM [DatabaseChangeScriptProviders]
	 WHERE [Application_Id] = @Application_Id


	DELETE
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id
	 

	/* 2d2) Change Controls */
	DELETE 
      FROM [ChangeControlEnvironments]
     WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)
    
    DELETE
      FROM [ChangeControlPerformances]
     WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)
    
    DELETE 
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)
	 


	/* 2e) Releases */
	DELETE [ReleaseDeployables]
	 WHERE [Application_Id] = @Application_Id
	    OR [Referenced_Application_Id] = @Application_Id


	DELETE [Releases]
	 WHERE [Application_Id] = @Application_Id


	/* 2f) Plans */
	DECLARE @TmpPlanActionGroups TABLE ([PlanActionGroup_Id] INT)
	 INSERT @TmpPlanActionGroups
	 SELECT [PlanActionGroup_Id] 
	 FROM @PlanActionGroups
	 
	 WHILE ((SELECT COUNT(*) FROM @TmpPlanActionGroups) > 0 )
	 BEGIN
		DECLARE @PlanActionGroup_Id INT
		SET @PlanActionGroup_Id = (SELECT TOP 1 [PlanActionGroup_Id] FROM @TmpPlanActionGroups)
		
		EXEC [Plans_DeletePlanActionGroup] @PlanActionGroup_Id

		DELETE @TmpPlanActionGroups WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	 END
	    
	DECLARE @Plans TABLE ([Plan_Id] INT NOT NULL)	 
	INSERT INTO @Plans ([Plan_Id])
	SELECT [Plan_Id]
	  FROM [Plans] P
	 WHERE NOT EXISTS (SELECT * FROM [PlanActionGroups] _PAG
	                     WHERE _PAG.[ActionGroup_Id] = P.[Plan_Id]) 
	
    DELETE [PlanActions]
     WHERE [Plan_Id] IN (SELECT [Plan_Id] FROM @Plans)

	DELETE [Plans] FROM [Plans] P
	 WHERE [Plan_Id] IN (SELECT [Plan_Id] FROM @Plans)


	/* 2g) Workflows */
	DELETE
	  FROM [PromotionApprovalRequirements]
	 WHERE [Workflow_Id] IN ( SELECT [Workflow_Id] FROM @Workflows )

	DELETE
	  FROM [PromotionRequirements]
	 WHERE [Workflow_Id] IN ( SELECT [Workflow_Id] FROM @Workflows )
	 
	DELETE
	  FROM [WorkflowSteps]
	 WHERE [Workflow_Id] IN ( SELECT [Workflow_Id] FROM @Workflows )


	DELETE
	  FROM [Workflows]
	 WHERE [Application_Id] = @Application_Id


	/* 2h) Deployables */
	DELETE [DeployableDependencies]
	 WHERE [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)
	    OR [DependsOn_Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)

	DELETE [DeployablesImported]
	 WHERE [Application_Id] = @Application_Id
	    OR [Deployable_Id] IN (SELECT [Deployable_Id] FROM @Deployables)
 	
	DELETE
	  FROM [Deployables]
	 WHERE [Application_Id] = @Application_Id
	 

	/* Feature level */
	DELETE FeatureLevels
	 WHERE [Application_Id] = @Application_Id
	 
 DECLARE @Application_Name SQL_VARIANT
	SET @Application_Name = (SELECT [Application_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)

	/* Dashboards */
	
	DELETE [DashboardGadgets]
	 WHERE [Dashboard_Id] IN (SELECT [Dashboard_Id] FROM @Dashboards)
	
	DELETE [Dashboards]
	 WHERE [Dashboard_Id] IN (SELECT [Dashboard_Id] FROM @Dashboards)
	
	/* Event Occurrences */

	 DELETE [EventOccurenceDetails]
	  FROM [EventOccurenceDetails] EOD
	       INNER JOIN [EventOccurences] ED
	               ON ED.[EventOccurence_Id] = EOD.[EventOccurence_Id]
	 WHERE ED.[Application_Id] = @Application_Id

	DELETE [EventOccurences]
	 WHERE [Application_Id] = @Application_Id

	/* 3) Application. *phew* */
	DELETE
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPDEL',NULL,@Application_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_PurgeApplicationData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_PurgeDeployableData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges a deployable and all its associations from the system'
GO

IF OBJECT_ID('[Applications_PurgeDeployableData]') IS NOT NULL
	DROP PROCEDURE [Applications_PurgeDeployableData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_PurgeDeployableData]
(
	@Deployable_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Deployables] WHERE [Deployable_Id] = @Deployable_Id)
		RAISERROR('Deployable %i does not exist.', 16, 1, @Deployable_Id)

	DECLARE @ConfigurationFiles TABLE ([ConfigurationFile_Id] INT NOT NULL)
    INSERT INTO @ConfigurationFiles ([ConfigurationFile_Id])
    SELECT [ConfigurationFile_Id]
      FROM [ConfigurationFiles]
     WHERE [Deployable_Id] = @Deployable_Id

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
    INSERT INTO @DatabaseChangeScripts ([Script_Id])
    SELECT [Script_Id]
      FROM [DatabaseChangeScripts]
     WHERE [Deployable_Id] = @Deployable_Id

    DECLARE @PlanActionGroups TABLE ([PlanActionGroup_Id] INT NOT NULL)
	INSERT INTO @PlanActionGroups ([PlanActionGroup_Id])
	SELECT [PlanActionGroup_Id]
	  FROM [PlanActionGroups]
	 WHERE [Deployable_Id] = @Deployable_Id

     DELETE [DeployableDependencies]
     WHERE [Deployable_Id] = @Deployable_Id
	    OR [DependsOn_Deployable_Id] = @Deployable_Id

	DELETE [DeployablesImported]
	 WHERE [Deployable_Id] = @Deployable_Id

	DELETE
      FROM [BuildArtifacts]
     WHERE [Deployable_Id] = @Deployable_Id

	DELETE
      FROM [ConfigurationFileDeployments]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFileVersions]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFileInstances]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [ConfigurationFiles]
     WHERE [ConfigurationFile_Id] IN ( SELECT [ConfigurationFile_Id] FROM @ConfigurationFiles )


	DELETE
      FROM [DatabaseChangeScriptExecutions]
     WHERE [Script_Id] IN ( SELECT [Script_Id] FROM @DatabaseChangeScripts )


	DELETE
      FROM [DatabaseChangeScripts]
     WHERE [Deployable_Id] = @Deployable_Id

	DELETE 
      FROM [ReleaseDeployables]
     WHERE [Deployable_Id] = @Deployable_Id 

    DELETE
      FROM [VariableDefinitions]
     WHERE [Deployable_Id] = @Deployable_Id 


	 DECLARE @TmpPlanActionGroups TABLE ([PlanActionGroup_Id] INT)
	 INSERT @TmpPlanActionGroups
	 SELECT [PlanActionGroup_Id] 
	 FROM @PlanActionGroups
	 
	 WHILE ((SELECT COUNT(*) FROM @TmpPlanActionGroups) > 0 )
	 BEGIN
		DECLARE @PlanActionGroup_Id INT
		SET @PlanActionGroup_Id = (SELECT TOP 1 [PlanActionGroup_Id] FROM @TmpPlanActionGroups)

		EXEC [Plans_DeletePlanActionGroup] @PlanActionGroup_Id

		DELETE @TmpPlanActionGroups WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	 END


   DECLARE @Plans TABLE ([Plan_Id] INT NOT NULL)
	INSERT INTO @Plans ([Plan_Id])
	SELECT [Plan_Id]
	  FROM [Plans] P
	 WHERE NOT EXISTS (SELECT * FROM [PlanActionGroups] _PAG
	                     WHERE _PAG.[ActionGroup_Id] = P.[Plan_Id])

	DELETE [PlanActions]
     WHERE [Plan_Id] IN (SELECT [Plan_Id] FROM @Plans)

	DELETE [Plans] FROM [Plans] P
	 WHERE [Plan_Id] IN (SELECT [Plan_Id] FROM @Plans)

	DECLARE @Deployable_Name SQL_VARIANT, @Application_Id INT
	 SELECT @Deployable_Name = [Deployable_Name] 
	       ,@Application_Id = [Application_Id] 
       FROM [Deployables] 
      WHERE [Deployable_Id] = @Deployable_Id

	DELETE 
      FROM [Deployables]
     WHERE [Deployable_Id] = @Deployable_Id 


	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPDLD',@Application_Id,@Deployable_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_PurgeDeployableData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the dashboard text for an application'
GO


IF OBJECT_ID('[Applications_SetDashboard]') IS NOT NULL DROP PROCEDURE [Applications_SetDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetDashboard]
(
	@Application_Id INT,
	@Dashboard_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
       SET [Dashboard_Text] = @Dashboard_Text
     WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetDashboard] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetDeployableDependencies',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the dependency relationships between the specified deployable and its dependents'
GO

IF OBJECT_ID('[Applications_SetDeployableDependencies]') IS NOT NULL DROP PROCEDURE [Applications_SetDeployableDependencies]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetDeployableDependencies]
(
	@Deployable_Id INT,
	@DependsOn_Deployables_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [DeployableDependencies]
		 WHERE [Deployable_Id] = @Deployable_Id

		INSERT INTO [DeployableDependencies]
			(
				[Deployable_Id],
				[DependsOn_Deployable_Id]
			)
		SELECT
				@Deployable_Id,
				CAST(CSV.[Value_Text] AS INT)
		  FROM [dbo].[CsvToTable](@DependsOn_Deployables_Csv, ',') CSV

		DECLARE @Application_Id INT 
		    SET @Application_Id = (SELECT [Application_Id] 
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetDeployableDependencies] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_SetIssueTracker',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the issue tracking provider of a specified application'
GO

IF OBJECT_ID('[Applications_SetIssueTracker]') IS NOT NULL
	DROP PROCEDURE [Applications_SetIssueTracker]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_SetIssueTracker]
(
	@Application_Id INT,
	@IssueTracking_Provider_Id INT,
	@IssueTracking_CategoryIdList_Text VARCHAR(MAX)
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Applications]
	   SET [IssueTracking_Provider_Id] = @IssueTracking_Provider_Id
		  ,[IssueTracking_CategoryIdList_Text] = @IssueTracking_CategoryIdList_Text
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
	 WHERE [Application_Id] = @Application_Id
	IF @@ROWCOUNT=0 RAISERROR('Application %i does not exist.',16,1, @Application_Id)

	/* Raise Event */
	EXEC [Events_RaiseEvent] 'APPMOD',@Application_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_SetIssueTracker] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_ValidateHealth',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'HasDeployable_Indicator,HasActiveWorkflow_Indicator,HasRelease_Indicator,HasDeploymentPlans_Indicator',
    /* Description_Text        */ 'Indicates whether an application should redirect to the Application Health page'
GO


IF OBJECT_ID('[Applications_ValidateHealth]') IS NOT NULL DROP PROCEDURE [Applications_ValidateHealth]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_ValidateHealth]
(
	@Application_Id INT,
    @HasDeployable_Indicator CHAR(1) OUT,
    @HasActiveWorkflow_Indicator CHAR(1) OUT,
    @HasRelease_Indicator CHAR(1) OUT,
    @HasDeploymentPlans_Indicator CHAR(1) OUT
)
AS
BEGIN

	SET NOCOUNT ON

    SET @HasDeployable_Indicator = 'N'
    SET @HasActiveWorkflow_Indicator = 'N'
    SET @HasRelease_Indicator = 'N'
    SET @HasDeploymentPlans_Indicator = 'N'

    IF EXISTS(SELECT * FROM [Deployables] WHERE [Application_Id] = @Application_Id)
        SET @HasDeployable_Indicator = 'Y'

    IF EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id)
        SET @HasRelease_Indicator = 'Y'

    IF EXISTS(SELECT * FROM [Workflows] WHERE [Application_Id] = @Application_Id AND [Active_Indicator] = 'Y')
        SET @HasActiveWorkflow_Indicator = 'Y'

    IF EXISTS(SELECT * FROM [PlanActionGroups] WHERE [Application_Id] = @Application_Id)
        SET @HasDeploymentPlans_Indicator = 'Y'
	
END
GO

GRANT EXECUTE ON [Applications_ValidateHealth] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_CreateOrReplaceArtifact',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a reference to an artifact in the artifact library'
GO


IF OBJECT_ID('[Artifacts_CreateOrReplaceArtifact]') IS NOT NULL DROP PROCEDURE [Artifacts_CreateOrReplaceArtifact]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_CreateOrReplaceArtifact]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Deployable_Id INT,
	@Artifact_Name VARCHAR(50),

	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
       AND (
             (@Deployable_Id IS NULL AND [Deployable_Id] IS NULL)
             OR
             ([Deployable_Id] = @Deployable_Id)
           )
	   AND [Artifact_Name] = @Artifact_Name

	INSERT INTO [BuildArtifacts]
		([Application_Id], [Release_Number], [Build_Number], [Deployable_Id], [Artifact_Name], [Execution_Id], [Created_Date])
	VALUES
		(@Application_Id, @Release_Number, @Build_Number, @Deployable_Id, @Artifact_Name, @Execution_Id, GETUTCDATE())

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Artifacts_CreateOrReplaceArtifact] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifacts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of artifacts based on the specified criteria'
GO


IF OBJECT_ID('[Artifacts_GetArtifacts]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifacts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifacts]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@Build_Number BUILDNUMBER = NULL,
	@Deployable_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildArtifacts_Extended]
	 WHERE [Application_Id] = COALESCE(@Application_Id, [Application_Id])
	   AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
	   AND [Build_Number] = COALESCE(@Build_Number, [Build_Number])
	   AND ([Deployable_Id] =  COALESCE(@Deployable_Id, Deployable_Id) OR [Deployable_Id] IS NULL)

END
GO

GRANT EXECUTE ON [Artifacts_GetArtifacts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Artifacts_GetArtifactsByBuildStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildArtifacts_Slim',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of artifacts that are associated with a specified build status, optionally filtered by application and release.'
GO


IF OBJECT_ID('[Artifacts_GetArtifactsByBuildStatus]') IS NOT NULL DROP PROCEDURE [Artifacts_GetArtifactsByBuildStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Artifacts_GetArtifactsByBuildStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@BuildStatus_Name VARCHAR(8)
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Application_Id IS NULL
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [BuildStatus_Name] = @BuildStatus_Name
	END
	ELSE IF @Release_Number IS NULL
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [Application_Id] = @Application_Id
		   AND [BuildStatus_Name] = @BuildStatus_Name
	END
	ELSE
	BEGIN
		SELECT *
		  FROM [BuildArtifacts_Slim]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [BuildStatus_Name] = @BuildStatus_Name
	END
END
GO

GRANT EXECUTE ON [Artifacts_GetArtifactsByBuildStatus] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildExecutionPlanAction_GetVariableValues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_PlanActionVariableValues',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets execution-time variable values'
GO

IF OBJECT_ID('[BuildExecutionPlanAction_GetVariableValues]') IS NOT NULL DROP PROCEDURE [BuildExecutionPlanAction_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildExecutionPlanAction_GetVariableValues]
(
	@BuildExecution_PlanAction_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecution_PlanActionVariableValues]
	 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

END
GO

GRANT EXECUTE ON [BuildExecutionPlanAction_GetVariableValues] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildExecutionPlanAction_SetVariableValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates, updates, or deletes an execution-time variable value'
GO

IF OBJECT_ID('[BuildExecutionPlanAction_SetVariableValue]') IS NOT NULL DROP PROCEDURE [BuildExecutionPlanAction_SetVariableValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildExecutionPlanAction_SetVariableValue]
(
	@BuildExecution_PlanAction_Id INT,
	@Variable_Name VARCHAR(20),
	@Value_Text VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON
	
BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @ExecutionStatus_Name VARCHAR(10)
	
	SELECT @ExecutionStatus_Name = BE.[ExecutionStatus_Name]
	  FROM [BuildExecution_PlanActions] BEPA
	       INNER JOIN [BuildExecutions] BE
	               ON BEPA.[Execution_Id] = BE.[Execution_Id]
	 WHERE BEPA.[BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	 
	IF @ExecutionStatus_Name <> 'Pending' AND @ExecutionStatus_Name <> 'Executing'
		RAISERROR('Variables cannot be set on a %s execution.', 16, 1, @ExecutionStatus_Name)
	
	IF EXISTS(SELECT *
	            FROM [BuildExecution_PlanActionVariableValues]
	           WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	             AND [Variable_Name] = @Variable_Name)
	BEGIN
		IF @Value_Text IS NULL
		BEGIN
			DELETE [BuildExecution_PlanActionVariableValues]
             WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
               AND [Variable_Name] = @Variable_Name
		END
		ELSE
		BEGIN
			UPDATE [BuildExecution_PlanActionVariableValues]
			   SET [Value_Text] = @Value_Text
	         WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	           AND [Variable_Name] = @Variable_Name
	    END
	END
	ELSE IF @Value_Text IS NOT NULL
	BEGIN
		INSERT INTO [BuildExecution_PlanActionVariableValues]
		(
			[BuildExecution_PlanAction_Id],
			[Variable_Name],
			[Value_Text]
		)
		VALUES
		(
			@BuildExecution_PlanAction_Id,
			@Variable_Name,
			@Value_Text
		)
	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildExecutionPlanAction_SetVariableValue] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetAlerts',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Alerts_Xml',
    /* Description_Text        */ 'Gets the alerts.'
GO

IF OBJECT_ID('[BuildMaster_GetAlerts]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetAlerts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetAlerts]
(
	@Current_User_Name VARCHAR(255),
	@Current_User_Groups_Csv VARCHAR(MAX),
	@Alerts_Xml XML OUT
)
AS
BEGIN

	SET NOCOUNT ON


    DECLARE @DirectoryProvider_Id INT
    SET @DirectoryProvider_Id = (SELECT [Value_Text]
                                   FROM [Configuration]
                                  WHERE [Scope_Name] = 'CoreEx'
                                    AND [Key_Name] = 'DirectoryProvider')

SET @Alerts_Xml =
    (SELECT
           [@IgnoreServiceWarning] = COALESCE((SELECT [Value_Text]
                                        FROM [Configuration]
                                       WHERE [Scope_Name] = 'Web'
                                         AND [Key_Name] = 'IgnoreServiceWarning'), 'False'),

           [@ErrorsInLog] = (CASE WHEN EXISTS(SELECT *
                                                FROM [LogMessages]
                                               WHERE [Read_Indicator] = 'N')
                                         THEN 'True'
                                         ELSE 'False' END),

           [@BmUpdateReminder] = (SELECT [Value_Text]
                                    FROM [Configuration]
                                   WHERE [Scope_Name] = 'CoreEx'
                                     AND [Key_Name] = 'BuildMasterUpdateCheck_ShowReminder'),

           [@ExtUpdateReminder] = (SELECT [Value_Text]
                                     FROM [Configuration]
                                    WHERE [Scope_Name] = 'CoreEx'
                                      AND [Key_Name] = 'ExtensionUpdateCheck_ShowReminder'),
       
           [@ManualAction] = (SELECT CASE WHEN COUNT(*) = 0 THEN 'False' ELSE 'True' END
                                FROM [ManualActionsInProgress]
                               WHERE [ActionComplete_Indicator] = 'N'
                                 AND [AssignedTo_Principal_Name] IN
                                     (SELECT [Value_Text] FROM dbo.CsvToTable(@Current_User_Groups_Csv, ',')
                                      UNION ALL
                                      SELECT @Current_User_Name)),
       
           [@CanConfigure] = CASE WHEN dbo.IsAuthorizedForTask(@DirectoryProvider_Id, @Current_User_Name, @Current_User_Groups_Csv, 62, NULL, NULL, NULL) = 0 THEN 'False' ELSE 'True' END,

           [@CanManageExtensions] = CASE WHEN dbo.IsAuthorizedForTask(@DirectoryProvider_Id, @Current_User_Name, @Current_User_Groups_Csv, 39, NULL, NULL, NULL) = 0 THEN 'False' ELSE 'True' END

   FOR XML PATH('Alerts'))

END
GO

GRANT EXECUTE ON [BuildMaster_GetAlerts] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetCurrentDateTime',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Current_Date',
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[BuildMaster_GetCurrentDateTime]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetCurrentDateTime]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetCurrentDateTime]
(
	@Current_Date DATETIME OUT
)
AS
BEGIN

	SET NOCOUNT ON
    
    SET @Current_Date = GETUTCDATE()

END
GO

GRANT EXECUTE ON [BuildMaster_GetCurrentDateTime] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildMaster_GetUsageData',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'UsageData_Xml',
    /* Description_Text        */ 'Gets the usage data.'
GO


IF OBJECT_ID('[BuildMaster_GetUsageData]') IS NOT NULL DROP PROCEDURE [BuildMaster_GetUsageData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildMaster_GetUsageData]
(
	@UsageData_Xml XML OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SET @UsageData_Xml = (SELECT 

	--Host
	(SELECT [@SQL] = @@VERSION,
	        [@LastUsed] = (SELECT TOP 1 [Occurence_Date]
	                         FROM [EventOccurences]
	                        WHERE [User_Name] <> 'SYSTEM'
	                        ORDER BY [EventOccurence_Id] DESC)
	    FOR XML PATH('Host'), TYPE),

	--Applications
	(SELECT [@Count] = COUNT(*),
            [@Grouped] = COUNT(A.[ApplicationGroup_Id]),
            [@HasIssueTracker] = COUNT(A.[IssueTracking_Provider_Id])
       FROM [Applications] A
        FOR XML PATH('Applications'), TYPE),

	--ApplicationGroups
	(SELECT [@Count] = COUNT(*)
	   FROM [ApplicationGroups]
	    FOR XML PATH('ApplicationGroups'), TYPE),

	--Deployables
	(SELECT [@Count] = COUNT(*),
	        [@Imported] = COUNT(DI.[Deployable_Id])
	   FROM [Deployables] D
	        LEFT JOIN [DeployablesImported] DI
	                  ON D.[Deployable_Id] = DI.[Deployable_Id]
	    FOR XML PATH('Deployables'), TYPE),

	--Environments
	(SELECT [@Count] = COUNT(*),
	        [@Active] = COUNT(NULLIF([Active_Indicator], 'N'))
	   FROM [Environments]
	    FOR XML PATH('Environments'), TYPE),

	--Servers
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _S.[AgentType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [AgentType] = CAST(S.[Agent_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [Servers] S
 			          WHERE [ServerGroup_Indicator] = 'N') _S
		      GROUP BY _S.[AgentType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [Servers]
	    FOR XML PATH('Servers'), TYPE),

	--Scripts
	(SELECT [@Count] = COUNT(*),
	        (SELECT [@Name] = S.[ScriptType_Code],
	                [@Count] = COUNT(*)
	           FROM [Scripts] S
	          GROUP BY S.[ScriptType_Code]
	            FOR XML PATH('Type'), TYPE)
	   FROM [Scripts]
	    FOR XML PATH('Scripts'), TYPE),
	   
	--IssueTrackingProviders
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _P.[ProviderType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [ProviderType] = CAST(P.[Provider_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [Providers] P
				      WHERE P.[ProviderType_Code] = 'I') _P
		      GROUP BY _P.[ProviderType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [Providers]
	  WHERE [ProviderType_Code] = 'I'
	    FOR XML PATH('IssueTrackingProviders'), TYPE),

	--SourceControlProviders
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _P.[ProviderType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [ProviderType] = CAST(P.[Provider_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [Providers] P
				      WHERE P.[ProviderType_Code] = 'S') _P
		      GROUP BY _P.[ProviderType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [Providers]
	  WHERE [ProviderType_Code] = 'S'
	    FOR XML PATH('SourceControlProviders'), TYPE),

	--DatabaseProviders
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _P.[ProviderType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [ProviderType] = CAST(P.[Provider_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [Providers] P
				      WHERE P.[ProviderType_Code] = 'B') _P
		      GROUP BY _P.[ProviderType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [Providers]
	  WHERE [ProviderType_Code] = 'B'
	    FOR XML PATH('DatabaseProviders'), TYPE),

	--Notifiers
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _N.[NotifierType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [NotifierType] = CAST(N.[Notifier_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [Notifiers] N
				      WHERE N.[Owner_User_Name] IS NOT NULL) _N
		      GROUP BY _N.[NotifierType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [Notifiers]
	  WHERE [Owner_User_Name] IS NOT NULL
	    FOR XML PATH('Notifiers'), TYPE),

	--Triggers
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _N.[NotifierType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [NotifierType] = CAST(N.[Notifier_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [Notifiers] N
				      WHERE N.[Owner_User_Name] IS NULL) _N
		      GROUP BY _N.[NotifierType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [Notifiers]
	  WHERE [Owner_User_Name] IS NULL
	    FOR XML PATH('Triggers'), TYPE),

	--PromotionRequirements
	(SELECT [@Count] = COUNT(*),
			(SELECT [@Name] = _PR.[PromotionRequirementType],
			        [@Count] = COUNT(*)
			   FROM (SELECT [PromotionRequirementType] = CAST(PR.[PromotionRequirement_Configuration].query('local-name(/*[1])') AS VARCHAR(MAX))
				       FROM [PromotionRequirements] PR) _PR
		      GROUP BY _PR.[PromotionRequirementType]
		        FOR XML PATH('Type'), TYPE)
	   FROM [PromotionRequirements]
	    FOR XML PATH('PromotionRequirements'), TYPE),

	--ConfigFiles	    
	(SELECT [@Count] = COUNT(*),
	        [@Active] = COUNT(NULLIF([Active_Indicator], 'N')),
	        (SELECT [@Count] = COUNT(*),
	                [@Active] = COUNT(NULLIF([Active_Indicator], 'N')),
	                [@Templates] = COUNT(NULLIF([Template_Indicator], 'N'))
	           FROM [ConfigurationFileInstances]
	            FOR XML PATH('Instances'), TYPE)
	   FROM [ConfigurationFiles]
	    FOR XML PATH('ConfigFiles'), TYPE),

	--ChangeScripts
	(SELECT [@Count] = COUNT(*),
	        [@Active] = COUNT(NULLIF([Active_Indicator], 'N'))
	   FROM [DatabaseChangeScripts]
	    FOR XML PATH('ChangeScripts'), TYPE),

	--Workflows
	(SELECT [@Count] = COUNT(*),
	        [@Active] = COUNT(NULLIF([Active_Indicator], 'N')),
	        [@AutoDeploy] = COUNT(NULLIF([AutoDeployRelease_Indicator], 'N')),
	        [@AllowOutOfSequence] = COUNT(NULLIF([AllowOutOfSequencePromotions_Indicator], 'N')),
	        [@AutoCancel] = COUNT(NULLIF([AutoCancelReleases_Indicator], 'N')),
	        [@AutoIncrement] = COUNT(NULLIF([AutoIncrementRelease_Indicator], 'N'))
	   FROM [Workflows]
	    FOR XML PATH('Workflows'), TYPE),

	--ChangeControls
	(SELECT [@Count] = COUNT(*),
	        [@Active] = COUNT(NULLIF([Active_Indicator], 'N'))
	   FROM [ChangeControls]
	    FOR XML PATH('ChangeControls'), TYPE),
	    
	--Approvals
	(SELECT [@Count] = COUNT(*),
	        [@User] = COUNT(NULLIF([ApprovalType_Code], 'G')),
	        [@Group] = COUNT(NULLIF([ApprovalType_Code], 'U')),
	        [@Or] = COUNT(NULLIF([Or_Indicator], 'N'))
	   FROM [PromotionApprovalRequirements]
	    FOR XML PATH('Approvals'), TYPE),

	--ScheduledBuilds
	(SELECT [@Count] = COUNT(*),
	        [@Active] = COUNT(NULLIF([Active_Indicator], 'N')),
	        [@UrlTriggered] = COUNT(CASE WHEN [Schedule_Type] = 'URL Triggered' THEN [Schedule_Type] ELSE NULL END),
	        [@SourceControlTriggered] = COUNT(CASE WHEN [Schedule_Type] = 'SCM Triggered' THEN [Schedule_Type] ELSE NULL END),
	        [@Recurring] = COUNT(CASE WHEN [Schedule_Type] = 'Recurring' THEN [Schedule_Type] ELSE NULL END),
	        [@Once] = COUNT(CASE WHEN [Schedule_Type] = 'Once' THEN [Schedule_Type] ELSE NULL END)
	   FROM [BuildSchedules]
	    FOR XML PATH('ScheduledBuilds'), TYPE)

   FOR XML PATH('BuildMaster'))

END
GO

GRANT EXECUTE ON [BuildMaster_GetUsageData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_AddOutput',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a build output report to a specified build execution'
GO

IF OBJECT_ID('[BuildOutputs_AddOutput]') IS NOT NULL DROP PROCEDURE [BuildOutputs_AddOutput]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_AddOutput]
(
	@BuildExecution_PlanAction_Id INT,

	@Output_Name VARCHAR(50),
	@Output_Bytes VARBINARY(MAX),
    @OutputType_Code CHAR(1)
)
AS
BEGIN

	SET NOCOUNT ON
	
	INSERT INTO [BuildOutputs]
			   ([Execution_Id]
			   ,[Output_Sequence]
			   ,[Output_Name]
			   ,[Output_Bytes]
			   ,[OutputType_Code]
			   ,[BuildExecution_PlanAction_Id])
		 SELECT
			    (SELECT [Execution_Id] FROM [BuildExecution_PlanActions]
	                               WHERE @BuildExecution_PlanAction_Id = [BuildExecution_PlanAction_Id])
			   ,ISNULL(MAX([Output_Sequence]), 0) + 1
			   ,@Output_Name
			   ,@Output_Bytes
			   ,@OutputType_Code
			   ,@BuildExecution_PlanAction_Id
		   FROM [BuildOutputs]
		  WHERE [Execution_Id] = (SELECT [Execution_Id] FROM [BuildExecution_PlanActions]
	                               WHERE @BuildExecution_PlanAction_Id = [BuildExecution_PlanAction_Id])

END
GO

GRANT EXECUTE ON [BuildOutputs_AddOutput] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_GetOutput',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildOutputs_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build output report for a specified build and sequence number'
GO


IF OBJECT_ID('[BuildOutputs_GetOutput]') IS NOT NULL DROP PROCEDURE [BuildOutputs_GetOutput]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_GetOutput]
(
	@Execution_Id INT,
	@Output_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildOutputs_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Output_Sequence] = @Output_Sequence

END
GO

GRANT EXECUTE ON [BuildOutputs_GetOutput] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildOutputs_GetOutputs',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildOutputs_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build output reports for a specified build'
GO


IF OBJECT_ID('[BuildOutputs_GetOutputs]') IS NOT NULL DROP PROCEDURE [BuildOutputs_GetOutputs]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildOutputs_GetOutputs]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Application_Id INT
	DECLARE @Release_Number RELEASENUMBER
	DECLARE @Build_Number BUILDNUMBER
	
	SELECT @Application_Id = [Application_Id],
	       @Release_Number = [Release_Number],
	       @Build_Number = [Build_Number]
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] = @Execution_Id
	 

	SELECT *
	  FROM [BuildOutputs_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
  ORDER BY [Output_Sequence]

END
GO

GRANT EXECUTE ON [BuildOutputs_GetOutputs] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateRecurring',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a recurring build based on the specified parameters, or updates an existing schedule if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateRecurring]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateRecurring]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateRecurring]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@StartTime_Minutes INT,
	@EndTime_Minutes SMALLINT,
	@DailyInterval_Minutes SMALLINT,
	@Recurrence_Type VARCHAR(10),
	@RecurrenceFrequency_Value INT,
	@RecurrenceWeeklyDay_Mask CHAR(7),
	@RecurrenceMonthlyDay_Value TINYINT,
	@RecurrenceMonthlyRelativity_Type VARCHAR(4),
    @RecurrenceMonthlyRelativityDay_Type VARCHAR(5),
    @RecurrenceMonthlyRelativityWeek_Type VARCHAR(7),
    @RecurrenceMonthlyRelativityWeekDay_Value TINYINT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT,
    @Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[StartTime_Minutes]
			,[EndTime_Minutes]
			,[DailyInterval_Minutes]
			,[Recurrence_Type]
			,[RecurrenceFrequency_Value]
			,[RecurrenceWeeklyDay_Mask]
			,[RecurrenceMonthlyDay_Value]
			,[RecurrenceMonthlyRelativity_Type]
			,[RecurrenceMonthlyRelativityDay_Type]
			,[RecurrenceMonthlyRelativityWeek_Type]
			,[RecurrenceMonthlyRelativityWeekDay_Value]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'Recurring'
			,@StartTime_Minutes
			,@EndTime_Minutes
			,@DailyInterval_Minutes
			,@Recurrence_Type
			,@RecurrenceFrequency_Value
			,@RecurrenceWeeklyDay_Mask
			,@RecurrenceMonthlyDay_Value
			,@RecurrenceMonthlyRelativity_Type
			,@RecurrenceMonthlyRelativityDay_Type
			,@RecurrenceMonthlyRelativityWeek_Type
			,@RecurrenceMonthlyRelativityWeekDay_Value
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'Recurring' RAISERROR('Existing BuildSchedule #%i is not a Recurring schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[StartTime_Minutes] = @StartTime_Minutes
			  ,[EndTime_Minutes] = @EndTime_Minutes
			  ,[DailyInterval_Minutes] = @DailyInterval_Minutes
			  ,[Recurrence_Type] = @Recurrence_Type
			  ,[RecurrenceFrequency_Value] = @RecurrenceFrequency_Value
			  ,[RecurrenceWeeklyDay_Mask] = @RecurrenceWeeklyDay_Mask
			  ,[RecurrenceMonthlyDay_Value] = @RecurrenceMonthlyDay_Value
			  ,[RecurrenceMonthlyRelativity_Type] = @RecurrenceMonthlyRelativity_Type
			  ,[RecurrenceMonthlyRelativityDay_Type] = @RecurrenceMonthlyRelativityDay_Type
			  ,[RecurrenceMonthlyRelativityWeek_Type] = @RecurrenceMonthlyRelativityWeek_Type
			  ,[RecurrenceMonthlyRelativityWeekDay_Value] = @RecurrenceMonthlyRelativityWeekDay_Value
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
		 WHERE [Schedule_Id] = @Schedule_Id

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateRecurring] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a scheduled build based on the specified parameters, or updates an existing scheduled build if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateSchedule]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateSchedule]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
    @Active_Indicator YNINDICATOR,
    @Schedule_Type VARCHAR(15),
	@StartTime_Minutes INT,
	@EndTime_Minutes SMALLINT,
	@DailyInterval_Minutes SMALLINT,
	@Recurrence_Type VARCHAR(10),
	@RecurrenceFrequency_Value INT,
	@RecurrenceWeeklyDay_Mask CHAR(7),
	@RecurrenceMonthlyDay_Value TINYINT,
	@RecurrenceMonthlyRelativity_Type VARCHAR(4),
    @RecurrenceMonthlyRelativityDay_Type VARCHAR(5),
    @RecurrenceMonthlyRelativityWeek_Type VARCHAR(7),
    @RecurrenceMonthlyRelativityWeekDay_Value TINYINT,
	@Url_AllowedIP_CSV varchar(MAX),
	@Url_Username_Text varchar(50),
	@Url_Password_Text varchar(50),
	@SourceControl_Path_Name varchar(250),
	@SourceControl_Provider_Id int,
	@SourceControl_QuietPeriod_Minutes INT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[StartTime_Minutes]
			,[EndTime_Minutes]
			,[DailyInterval_Minutes]
			,[Recurrence_Type]
			,[RecurrenceFrequency_Value]
			,[RecurrenceWeeklyDay_Mask]
			,[RecurrenceMonthlyDay_Value]
			,[RecurrenceMonthlyRelativity_Type]
			,[RecurrenceMonthlyRelativityDay_Type]
			,[RecurrenceMonthlyRelativityWeek_Type]
			,[RecurrenceMonthlyRelativityWeekDay_Value]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Url_AllowedIP_CSV]
			,[Url_Username_Text]
			,[Url_Password_Text]
			,[SourceControl_Path_Name]
			,[SourceControl_Provider_Id]
			,[SourceControl_QuietPeriod_Minutes]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,@Schedule_Type
			,@StartTime_Minutes
			,@EndTime_Minutes
			,@DailyInterval_Minutes
			,@Recurrence_Type
			,@RecurrenceFrequency_Value
			,@RecurrenceWeeklyDay_Mask
			,@RecurrenceMonthlyDay_Value
			,@RecurrenceMonthlyRelativity_Type
			,@RecurrenceMonthlyRelativityDay_Type
			,@RecurrenceMonthlyRelativityWeek_Type
			,@RecurrenceMonthlyRelativityWeekDay_Value
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Url_AllowedIP_CSV
			,@Url_Username_Text
			,@Url_Password_Text
			,@SourceControl_Path_Name
			,@SourceControl_Provider_Id
			,@SourceControl_QuietPeriod_Minutes
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[Schedule_Type] = @Schedule_Type
			  ,[StartTime_Minutes] = @StartTime_Minutes
			  ,[EndTime_Minutes] = @EndTime_Minutes
			  ,[DailyInterval_Minutes] = @DailyInterval_Minutes
			  ,[Recurrence_Type] = @Recurrence_Type
			  ,[RecurrenceFrequency_Value] = @RecurrenceFrequency_Value
			  ,[RecurrenceWeeklyDay_Mask] = @RecurrenceWeeklyDay_Mask
			  ,[RecurrenceMonthlyDay_Value] = @RecurrenceMonthlyDay_Value
			  ,[RecurrenceMonthlyRelativity_Type] = @RecurrenceMonthlyRelativity_Type
			  ,[RecurrenceMonthlyRelativityDay_Type] = @RecurrenceMonthlyRelativityDay_Type
			  ,[RecurrenceMonthlyRelativityWeek_Type] = @RecurrenceMonthlyRelativityWeek_Type
			  ,[RecurrenceMonthlyRelativityWeekDay_Value] = @RecurrenceMonthlyRelativityWeekDay_Value
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Url_AllowedIP_CSV] = @Url_AllowedIP_CSV
			  ,[Url_Username_Text] = @Url_Username_Text
			  ,[Url_Password_Text] = @Url_Password_Text
			  ,[SourceControl_Path_Name] = @SourceControl_Path_Name
			  ,[SourceControl_Provider_Id] = @SourceControl_Provider_Id
			  ,[SourceControl_QuietPeriod_Minutes] = @SourceControl_QuietPeriod_Minutes
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
		 WHERE [Schedule_Id] = @Schedule_Id
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id)

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateSchedule] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateSCMTriggered',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a source control triggered automated build based on the specified parameters, or updates an existing one if one with a given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateSCMTriggered]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateSCMTriggered]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateSCMTriggered]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@SourceControl_Path_Name VARCHAR(250),
	@SourceControl_Provider_Id INT,
	@SourceControl_QuietPeriod_Minutes INT,
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT,
	@Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[SourceControl_Path_Name]
			,[SourceControl_Provider_Id]
			,[SourceControl_QuietPeriod_Minutes]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'SCM Triggered'
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@SourceControl_Path_Name
			,@SourceControl_Provider_Id
			,@SourceControl_QuietPeriod_Minutes
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'SCM Triggered' RAISERROR('Existing BuildSchedule #%i is not an SCM Triggered schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Active_Indicator] = @Active_Indicator
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[SourceControl_Path_Name] = @SourceControl_Path_Name
			  ,[SourceControl_Provider_Id] = @SourceControl_Provider_Id
			  ,[SourceControl_QuietPeriod_Minutes] = @SourceControl_QuietPeriod_Minutes
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
		 WHERE [Schedule_Id] = @Schedule_Id
		

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateSCMTriggered] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_CreateOrUpdateURLTriggered',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Schedule_Id',
    /* Description_Text        */ 'Creates a URL-triggered build based on the specified parameters, or updates an existing schedule if one with the given ID already exists.',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_CreateOrUpdateURLTriggered]') IS NOT NULL DROP PROCEDURE [BuildSchedules_CreateOrUpdateURLTriggered]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_CreateOrUpdateURLTriggered]
(
	@Schedule_Id INT = NULL OUT,
	@Schedule_Name VARCHAR(100),
	@Application_Id INT,
	@Url_AllowedIP_CSV VARCHAR(MAX),
	@Url_Username_Text VARCHAR(50),
	@Url_Password_Text VARCHAR(50),
	@ReleaseFilter_ReleaseType_Code CHAR(1),
	@Releasefilter_Workflow_Id INT,
	@Active_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Schedule_Id IS NULL
	BEGIN
		INSERT INTO [dbo].[BuildSchedules]
			([Schedule_Name]
			,[Application_Id]
			,[Active_Indicator]
			,[Schedule_Type]
			,[CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Url_AllowedIP_CSV]
			,[Url_Username_Text]
			,[Url_Password_Text]
			,[ReleaseFilter_ReleaseType_Code]
			,[Releasefilter_Workflow_Id])
		VALUES
			(@Schedule_Name
			,@Application_Id
			,@Active_Indicator
			,'URL Triggered'
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Url_AllowedIP_CSV
			,@Url_Username_Text
			,@Url_Password_Text
			,@ReleaseFilter_ReleaseType_Code
			,@Releasefilter_Workflow_Id)

		SET @Schedule_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'BSHNEW',@Application_Id,@Schedule_Id

	END
	ELSE
	BEGIN
		
		DECLARE @Schedule_Type VARCHAR(15)
		SELECT @Schedule_Type = [Schedule_Type] 
		  FROM [BuildSchedules]
		 WHERE [Schedule_Id] = @Schedule_Id
		
		IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id) 
		IF @Schedule_Type <> 'URL Triggered' RAISERROR('Existing BuildSchedule #%i is not a URL Triggered schedule, it is %s.', 16, 1, @Schedule_Id, @Schedule_Type)
	
		UPDATE [dbo].[BuildSchedules]
		   SET [Schedule_Name] = @Schedule_Name
			  ,[Application_Id] = @Application_Id
			  ,[Active_Indicator] = @Active_Indicator
			  ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			  ,[ModifiedOn_Date] = GETUTCDATE()
			  ,[Url_AllowedIP_CSV] = @Url_AllowedIP_CSV
			  ,[Url_Username_Text] = @Url_Username_Text
			  ,[Url_Password_Text] = @Url_Password_Text
			  ,[ReleaseFilter_ReleaseType_Code] = @ReleaseFilter_ReleaseType_Code
			  ,[Releasefilter_Workflow_Id] = @Releasefilter_Workflow_Id
		 WHERE [Schedule_Id] = @Schedule_Id

		EXEC [Events_RaiseEvent] 'BSHMOD',@Application_Id,@Schedule_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_CreateOrUpdateURLTriggered] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_DeleteBuildSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified build schedule'
GO

IF OBJECT_ID('[dbo].[BuildSchedules_DeleteBuildSchedule]') IS NOT NULL
	DROP PROCEDURE [dbo].[BuildSchedules_DeleteBuildSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[BuildSchedules_DeleteBuildSchedule]
(
	@Schedule_Id INT
) AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Schedule_Name SQL_VARIANT
	 SELECT @Application_Id = [Application_Id], @Schedule_Name = [Schedule_Name]
	   FROM [BuildSchedules]
	  WHERE [Schedule_Id] = @Schedule_Id

	DELETE
	  FROM [BuildScheduleVariableDefinitions]
	 WHERE [Schedule_Id] = @Schedule_Id

	DELETE
	  FROM [BuildSchedules]
	 WHERE [Schedule_Id] = @Schedule_Id

	IF @@ROWCOUNT <> 0 EXEC [Events_RaiseEvent] 'BSHDEL', @Application_Id, @Schedule_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[BuildSchedules_DeleteBuildSchedule] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetBuildSchedule',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildSchedules_Extended,BuildScheduleVariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified build schedule'
GO

IF OBJECT_ID('[dbo].[BuildSchedules_GetBuildSchedule]') IS NOT NULL
	DROP PROCEDURE [dbo].[BuildSchedules_GetBuildSchedule]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[BuildSchedules_GetBuildSchedule]
(
	@Schedule_Id INT
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildSchedules_Extended]
	 WHERE [Schedule_Id] = @Schedule_Id
	IF @@ROWCOUNT=0 RAISERROR('BuildSchedule #%i does not exist.',16,1,@Schedule_Id)

    SELECT *
      FROM [BuildScheduleVariableDefinitions_Extended]
     WHERE [Schedule_Id] = @Schedule_Id

END
GO

GRANT EXECUTE ON [dbo].[BuildSchedules_GetBuildSchedule] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetBuildSchedules',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildSchedules_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of builds schedules for the specified application and active indicator, or all if null is supplied'
GO

IF OBJECT_ID('[BuildSchedules_GetBuildSchedules]') IS NOT NULL
	DROP PROCEDURE [BuildSchedules_GetBuildSchedules]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_GetBuildSchedules]
(
	@Application_Id INT = NULL,
	@Active_Indicator YNINDICATOR = NULL
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildSchedules_Extended]
	 WHERE ( @Application_Id IS NULL OR [Application_Id] = @Application_Id )
	   AND [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])

END
GO

GRANT EXECUTE ON [BuildSchedules_GetBuildSchedules] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_GetScheduled',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildSchedules_Scheduled',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of build schedules that are scheduled to be executed between the specified dates'
GO

IF OBJECT_ID('BuildSchedules_GetScheduled') IS NOT NULL DROP PROCEDURE BuildSchedules_GetScheduled
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_GetScheduled]
(
	@From_Date DATETIME,
	@To_Date DATETIME 
) AS BEGIN

	SET NOCOUNT ON

	DECLARE @fDat DATETIME, @fMin INT, @tDat DATETIME, @tMin INT

	SET @fDat = DATEADD(dd, DATEDIFF(dd,0,@From_Date), 0)
	SET @fMin = DATEPART(MINUTE, @From_Date)
	          + DATEPART(HOUR, @From_Date) * 60
	SET @tDat = DATEADD(dd, DATEDIFF(dd,0,@To_Date), 0)
	SET @tMin = DATEPART(MINUTE, @To_Date)
	          + DATEPART(HOUR, @To_Date) * 60

	SELECT * FROM [BuildSchedules_Scheduled]
	 WHERE [RunAt_Date] BETWEEN @fDat AND @tDat
	   AND [RunAt_Minutes] BETWEEN @fMin AND @tMin
	    OR ([RunAt_Date] IS NULL AND [RunAt_Minutes] IS NULL)
      

END
GO

GRANT EXECUTE ON [BuildSchedules_GetScheduled] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildSchedules_SetVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates a variable definition to use with a scheduled build',
    /* Remarks_Text            */ NULL
GO

IF OBJECT_ID('[BuildSchedules_SetVariableDefinition]') IS NOT NULL DROP PROCEDURE [BuildSchedules_SetVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildSchedules_SetVariableDefinition]
(
	@Schedule_Id INT,
	@VariableDeclaration_Id INT,
	@Value_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @Value_Text IS NULL OR @Value_Text = ''
	BEGIN

		DELETE [BuildScheduleVariableDefinitions]
		 WHERE [Schedule_Id] = @Schedule_Id
		   AND [VariableDeclaration_Id] = @VariableDeclaration_Id

	END
	ELSE IF EXISTS(SELECT *
		            FROM [BuildScheduleVariableDefinitions]
				   WHERE [Schedule_Id] = @Schedule_Id
				     AND [VariableDeclaration_Id] = @VariableDeclaration_Id)
	BEGIN
		
		UPDATE [BuildScheduleVariableDefinitions]
		   SET [Value_Text] = @Value_Text
		 WHERE [Schedule_Id] = @Schedule_Id
		   AND [VariableDeclaration_Id] = @VariableDeclaration_Id
          
	END
	ELSE
	BEGIN

		INSERT INTO [BuildScheduleVariableDefinitions]
		([Schedule_Id], [VariableDeclaration_Id], [Value_Text])
		VALUES
		(@Schedule_Id, @VariableDeclaration_Id, @Value_Text)

	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [BuildSchedules_SetVariableDefinition] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_AddApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Approves a build for execution based on the specified parameters'
GO

IF OBJECT_ID('[Builds_AddApproval]') IS NOT NULL DROP PROCEDURE [Builds_AddApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_AddApproval]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Approval_Sequence INT,

	@User_Name USERNAME,
	@GroupNames_List VARCHAR(MAX),
	@Comments_Text VARCHAR(255)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [BuildApprovals]
			   ([Application_Id]
			   ,[Release_Number]
			   ,[Build_Number]
			   ,[Environment_Id]
			   ,[User_Name]
			   ,[Comments_Text]
			   ,[Group_Name]
			   ,[Received_Date]
			   ,[Approval_Description])
		SELECT TOP 1 [Application_Id]
			   ,[Release_Number]
			   ,[Build_Number]
			   ,[Environment_Id]
			   ,@User_Name
			   ,@Comments_Text
			   ,[Group_Name] = CASE WHEN [ApprovalType_Code]='G' THEN [Principal_Name] ELSE NULL END
			   ,GETUTCDATE()
			   ,[Approval_Description]
		  FROM [BuildPromotions_RequiredApprovalsByUser]
		 WHERE [Environment_Id] = @Environment_Id
		   AND [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND(
				 ([ApprovalType_Code] = 'U' AND [Principal_Name] = @User_Name)
				 OR
				 ([ApprovalType_Code] = 'G' AND [Principal_Name] IN 
				  (SELECT [Value_Text] 
					 FROM dbo.CsvToTable(@GroupNames_List, CHAR(10)) Groups))
			  )
		   AND [Approval_Sequence] = @Approval_Sequence
		   AND [Approval_Id] IS NULL

	IF @@ROWCOUNT <> 1  RAISERROR(
			'"%s" cannot approve build %s because the build is not active, or the approval has already been made or is invalid.', 
			16, 1, 
			@User_Name, 
			@Build_Number)

	/* Raise the BLDAPPRV event */
	IF [dbo].[IsPromotionApproved](
		@Application_Id, @Release_Number, @Build_Number,@Environment_Id) = 'Y'
	BEGIN

		EXEC [Events_RaiseEvent] 'BLDAPR', @Application_Id, @Release_Number, @Build_Number, @Environment_Id, @Approval_Sequence, @User_Name

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_AddApproval] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_DeployBuild',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ '',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Marks a build and its associated release as deployed based on the settings in its workflow.'
GO

IF OBJECT_ID('[Builds_DeployBuild]') IS NOT NULL DROP PROCEDURE [Builds_DeployBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_DeployBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
    @Workflow_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @AutoIncrementRelease_Indicator YNINDICATOR
        DECLARE @AutoCancelReleases_Indicator YNINDICATOR

        IF (SELECT [ReleaseStatus_Name] FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number) <> 'Active'
			RAISERROR('Can only deploy an active release', 16, 1)

		IF (SELECT [BuildStatus_Name] FROM [Builds] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [Build_Number] = @Build_Number) <> 'Active'
			RAISERROR('Can only deploy an active build', 16, 1)

        SELECT @AutoIncrementRelease_Indicator = [AutoIncrementRelease_Indicator],
               @AutoCancelReleases_Indicator = [AutoCancelReleases_Indicator]
          FROM [Workflows]
         WHERE [Workflow_Id] = @Workflow_Id

        IF @AutoIncrementRelease_Indicator IS NULL
			RAISERROR('Invalid workflow', 16, 1)

	    /* Update the Release */
	    UPDATE [Releases]
		    SET [ReleaseStatus_Name] = 'Deployed'
		    WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number

        DECLARE @Release_Sequence BIGINT

        SELECT @Release_Sequence = [Release_Sequence]
            FROM [Releases]
        WHERE [Application_Id] = @Application_Id 
            AND [Release_Number] = @Release_Number

        IF (@AutoCancelReleases_Indicator = 'Y')
        BEGIN

            /* Cancel existing releases that are older */
		    DECLARE @CNCL_Release_Number RELEASENUMBER
		    DECLARE CNCL_Release_Cursor CURSOR FOR
		    SELECT [Release_Number]
			    FROM [Releases]
			    WHERE [Application_Id] = @Application_Id
			    AND [ReleaseStatus_Name] = 'Active'
			    AND [Release_Sequence] < @Release_Sequence

		    OPEN CNCL_Release_Cursor
		    FETCH NEXT FROM CNCL_Release_Cursor INTO @CNCL_Release_Number
		    WHILE @@FETCH_STATUS <> -1
		    BEGIN
			    DECLARE @CancelledReason_Text VARCHAR(100) SET @CancelledReason_Text = 'Superceded by ' + @Release_Number
			    EXEC [Releases_CancelRelease]
				    @Application_Id = @Application_Id,
				    @Release_Number = @CNCL_Release_Number,
				    @CancelledReason_Text = @CancelledReason_Text

			    FETCH NEXT FROM CNCL_Release_Cursor INTO @CNCL_Release_Number
		    END

		    CLOSE CNCL_Release_Cursor
		    DEALLOCATE CNCL_Release_Cursor

        END

		/* Update the Current Build */
		UPDATE [Builds]
		    SET [BuildStatus_Name] = 'Deployed'
		    WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number
		    AND [Build_Number] = @Build_Number

		/* Update the Other Builds */
		UPDATE [Builds]
		    SET [BuildStatus_Name] = 'Rejected'
			    ,[RejectedBy_User_Name] = dbo.Session_GetUserName()
			    ,[Rejected_Date] = GETUTCDATE()
		    WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number
		    AND [BuildStatus_Name] = 'Active'
		
		/* If there are no other active releases, check for autoincrement setting */
		IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [ReleaseStatus_Name] = 'Active')
		BEGIN
			
		    /* Check if auto-increment is enabled */
		    IF @AutoIncrementRelease_Indicator = 'Y'
		    BEGIN
				    
                DECLARE @New_Release_Number RELEASENUMBER
			    SET @New_Release_Number = dbo.GetNextReleaseNumber(@Application_Id, @Release_Number)
				
			    /* If the autoincrement was successful */
			    IF NOT @New_Release_Number IS NULL
			    BEGIN
				    /* Make sure there is no identical existing release number */
				    IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @New_Release_Number)
				    BEGIN
					    DECLARE @ReleaseDeployables_Xml XML
					    SET @ReleaseDeployables_Xml =
						    (SELECT [Deployable_Id] '@Deployable_Id',
								    [InclusionType_Code] '@InclusionType_Code',
								    [Referenced_Release_Number] '@Referenced_Release_Number',
								    [Referenced_Application_Id] '@Referenced_Application_Id'
							    FROM [ReleaseDeployables]
							    WHERE [Application_Id] = @Application_Id
							    AND [Release_Number] = @Release_Number
							    FOR XML PATH('ReleaseDeployable'), ROOT('ReleaseDeployables'))

					    /* Create the release with the same deployables */
					    EXEC [Releases_CreateOrUpdateRelease]
							    @Application_Id,
							    @New_Release_Number,
							    @Workflow_Id,
							    NULL,
							    NULL,
							    NULL,
							    @ReleaseDeployables_Xml
					           
				    END
			    END
		    END
		END


	    EXEC [Events_RaiseEvent] 'RELDEP',@Application_Id,@Release_Number

	COMMIT
    END TRY BEGIN CATCH
	    IF XACT_STATE()<>0 ROLLBACK
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [Builds_DeployBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetAllPromotionRequiredApprovalsByUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_RequiredApprovalsByUser',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of approvals required from a specific user or group member for a build in a specified environment'
GO

IF OBJECT_ID('[Builds_GetAllPromotionRequiredApprovalsByUser]') IS NOT NULL DROP PROCEDURE [Builds_GetAllPromotionRequiredApprovalsByUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetAllPromotionRequiredApprovalsByUser]
(
	@User_Name USERNAME,
	@GroupNames_List VARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON


    SELECT * INTO #requiredApprovals
	  FROM [BuildPromotions_RequiredApprovalsByUser]
	 WHERE ('U' = [ApprovalType_Code] AND [Principal_Name] = @User_Name)
        OR ('G' = [ApprovalType_Code]
	           AND [Principal_Name] IN (
		             SELECT [Value_Text] 
		             FROM dbo.CsvToTable(@GroupNames_List, CHAR(10)) Groups)
	           AND [ApprovedBy_User_Name] IS NULL OR [ApprovedBy_User_Name] = @User_Name)

    /* somewhat tricky filter ahead:
	    filter down to one group row per approval req, include approvals assigned to the user as well
	 */

DELETE #requiredApprovals
	  FROM #requiredApprovals RA
	       INNER JOIN
	       (
	         SELECT [Application_Id]
	               ,[Release_Number]
	               ,[Build_Number]
	               ,[Environment_Id]
	               ,[Approval_Sequence]
	               ,[RequiredInGroup_Sequence]
	               ,[ApprovalRequired_Rank] = RANK() OVER (
	                  PARTITION BY [Application_Id],[Release_Number],[Build_Number],[Environment_Id],[Approval_Sequence]
                      ORDER BY CASE WHEN [ApprovedBy_User_Name] IS NULL THEN 0 ELSE 1 END DESC
                              ,[RequiredInGroup_Sequence])
	           FROM #requiredApprovals
	       ) RA2 ON RA.[Application_Id] = RA2.[Application_Id]
	            AND RA.[Release_Number] = RA2.[Release_Number]
	            AND RA.[Build_Number] = RA2.[Build_Number]
	            AND RA.[Environment_Id] = RA2.[Environment_Id]
	            AND RA.[Approval_Sequence] = RA2.[Approval_Sequence]
	            AND RA.[RequiredInGroup_Sequence] = RA2.[RequiredInGroup_Sequence]       
	            AND [ApprovalRequired_Rank] <> 1
	            
DELETE #requiredApprovals
 WHERE [Approval_Id] IS NOT NULL
 
SELECT * FROM #requiredApprovals
	   
END
GO

GRANT EXECUTE ON [Builds_GetAllPromotionRequiredApprovalsByUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build'
GO


IF OBJECT_ID('[Builds_GetBuild]') IS NOT NULL DROP PROCEDURE [Builds_GetBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %s does not exist.', 16, 1, @Application_Id, @Release_Number, @Build_Number)

END
GO

GRANT EXECUTE ON [Builds_GetBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetBuilds',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for all builds that match the specified criteria'
GO


IF OBJECT_ID('[Builds_GetBuilds]') IS NOT NULL DROP PROCEDURE [Builds_GetBuilds]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetBuilds]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@BuildStatus_Name VARCHAR(8) = NULL,
	@Build_Count INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	--Use RowCount to limit results 
	IF (@Build_Count IS NOT NULL AND @Build_Count > 0)
	BEGIN
		SET ROWCOUNT @Build_Count
	END

	SELECT *
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
	   AND [BuildStatus_Name] = COALESCE(@BuildStatus_Name, [BuildStatus_Name])
	  ORDER BY [Numeric_Release_Number] DESC, [Sortable_Build_Number] DESC, [CreatedOn_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetBuilds] TO [BuildMasterUser_Role]

GO

 EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetDependentPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all promotions that are contingent upon the specified promotion.'
GO

IF OBJECT_ID('[Builds_GetDependentPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetDependentPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetDependentPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

 
	SELECT BP.* 
	  FROM [BuildPromotionContingencies] BPC
		   INNER JOIN [BuildPromotions_Extended] BP
				   ON BPC.[Application_Id] = BP.[Application_Id]
				  AND BPC.[Release_Number] = BP.[Release_Number]
				  AND BPC.[Build_Number] = BP.[Build_Number]
				  AND BPC.[Environment_Id] = BP.[Environment_Id]
	 WHERE BPC.[Contingent_Application_Id] = @Application_Id
	   AND BPC.[Contingent_Release_Number] = @Release_Number
	   AND BPC.[Contingent_Build_Number] = @Build_Number
	   AND BPC.[Contingent_Environment_Id] = @Environment_Id
       AND BP.[BuildStatus_Name] = 'Active'
 

END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_GetDependentPromotions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all details regarding a specific build execution'
GO

IF OBJECT_ID('[Builds_GetExecution]') IS NOT NULL DROP PROCEDURE [Builds_GetExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecution]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id


END
GO

GRANT EXECUTE ON [Builds_GetExecution] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionActionsInProgress',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ 'BuildExecutions_ActionsInProgress',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Builds_GetExecutionActionsInProgress]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionActionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionActionsInProgress]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

	SELECT * 
      FROM [BuildExecutions_ActionsInProgress] 
	 WHERE [Execution_Id] = @Execution_Id
     ORDER BY [PlanExecution_Sequence], [Deployable_Name], [Plan_Id], [Action_Sequence], [Original_Action_Sequence]

END
GO

GRANT EXECUTE ON [Builds_GetExecutionActionsInProgress] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionExtended',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended,BuildExecution_Plans_Extended,BuildExecution_PlanActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all details regarding a specific build execution'
GO

IF OBJECT_ID('[Builds_GetExecutionExtended]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionExtended]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionExtended]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Execution %i does not exist.', 16, 1, @Execution_Id)

	SELECT DISTINCT *
	  FROM [BuildExecution_Plans_Extended] BEP
	 WHERE [Execution_Id] = @Execution_Id
	   AND EXISTS ( SELECT *
					  FROM [BuildExecution_PlanActions] _BEPA
					 WHERE _BEPA.[Execution_Id] = BEP.[Execution_Id]
					   AND _BEPA.[Original_PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id] )
	ORDER BY [PlanExecution_Sequence]

	SELECT *
	  FROM [BuildExecution_PlanActions_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	ORDER BY [ExecutionStarted_Date]

END
GO

GRANT EXECUTE ON [Builds_GetExecutionExtended] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_PlanActionLogEntries_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for a specified build execution, and plan or action if specified'
GO

IF OBJECT_ID('[Builds_GetExecutionLog]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionLog]
(
	@Execution_Id INT,
	@BuildExecution_PlanAction_Id INT = NULL,
    @Original_PlanActionGroup_Id INT = NULL,
	@IncludeDebugLogs_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

        SELECT *
		  FROM [BuildExecution_PlanActionLogEntries_Extended]
		 WHERE (@BuildExecution_PlanAction_Id IS NULL OR [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id)
           AND [Execution_Id] = @Execution_Id
		   AND (@Original_PlanActionGroup_Id IS NULL OR [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id)
		   AND ([LogEntry_Level] >= CASE WHEN @IncludeDebugLogs_Indicator = 'Y'
		                                 THEN 0
		                                 ELSE 10
		                            END)
	  ORDER BY [BuildExecution_PlanAction_Id], [LogEntry_Sequence]

END
GO

GRANT EXECUTE ON [Builds_GetExecutionLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the build executions for the specified build'
GO


IF OBJECT_ID('[Builds_GetExecutions]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT = NULL,
	@Execution_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF NULLIF(@Execution_Count, 0) IS NOT NULL
		SET ROWCOUNT @Execution_Count

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND (@Release_Number IS NULL OR @Release_Number = [Release_Number])
	   AND (@Build_Number IS NULL OR @Build_Number = [Build_Number])
	   AND (@Environment_Id IS NULL OR @Environment_Id = [Environment_Id])
	 ORDER BY [ExecutionStart_Date] DESC

END
GO

GRANT EXECUTE ON [Builds_GetExecutions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetExecutionsByServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets recent build executions for the specified server'
GO


IF OBJECT_ID('[Builds_GetExecutionsByServer]') IS NOT NULL DROP PROCEDURE [Builds_GetExecutionsByServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetExecutionsByServer]
(
	@Server_Id INT,
	@Execution_Count INT = 50
)
AS
BEGIN

	SET NOCOUNT ON

	SET @Execution_Count = COALESCE(@Execution_Count,50)

	SELECT * FROM [BuildExecutions_Extended]
	 WHERE [Execution_Id] IN (
		SELECT DISTINCT TOP(@Execution_Count) [Execution_Id]
		  FROM [BuildExecution_PlanActions]
		 WHERE [Server_Id] = @Server_Id 
		 ORDER BY [Execution_Id] DESC )

END
GO

GRANT EXECUTE ON [Builds_GetExecutionsByServer] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPartialExecutionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecution_PlanActionLogEntries_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the latest log entries for a specified build execution'
GO

IF OBJECT_ID('[Builds_GetPartialExecutionLog]') IS NOT NULL DROP PROCEDURE [Builds_GetPartialExecutionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPartialExecutionLog]
(
	@Execution_Id INT,
    @After_BuildExecution_PlanAction_Id INT,
	@After_LogEntry_Sequence INT,
    @IncludeDebug_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

		SELECT *
		  FROM [BuildExecution_PlanActionLogEntries_Extended]
		 WHERE [Execution_Id] = @Execution_Id
           AND (@IncludeDebug_Indicator = 'Y' OR [LogEntry_Level] >= 10)
           AND (
                  -- no BEPA ID is specified, return all log entries
                  @After_BuildExecution_PlanAction_Id IS NULL 
                  OR (
                            -- any logs with the same BEPA ID greater than the specified sequence
                            [BuildExecution_PlanAction_Id] = @After_BuildExecution_PlanAction_Id 
                        AND (@After_LogEntry_Sequence IS NULL OR [LogEntry_Sequence] > @After_LogEntry_Sequence)
                     )
                     -- any logs from a later BEPA ID
                  OR [BuildExecution_PlanAction_Id] > @After_BuildExecution_PlanAction_Id
               )
	  ORDER BY [BuildExecution_PlanAction_Id], [Server_Id], [LogEntry_Sequence]

END
GO

GRANT EXECUTE ON [Builds_GetPartialExecutionLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of promotions that have occurred to the specified environment'
GO


IF OBJECT_ID('[Builds_GetPromotion]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotion]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotions_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @Environment_Id
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %i to {Env#%i} does not exist.', 16, 1, @Application_ID,@Release_Number,@Build_Number,@Environment_Id)

END
GO

GRANT EXECUTE ON [Builds_GetPromotion] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotionApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'BuildApprovals_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified approval'
GO

IF OBJECT_ID('[Builds_GetPromotionApproval]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotionApproval]
(
	@Approval_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [BuildApprovals_Extended]
	 WHERE [Approval_Id] = @Approval_Id

	IF (@@ROWCOUNT = 0) RAISERROR('Approval #%i does not exist.', 16, 1, @Approval_Id)

END
GO

GRANT EXECUTE ON [Builds_GetPromotionApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotionApprovals',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildApprovals_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of approvals that have occurred for the specified build and optional environment'
GO

IF OBJECT_ID('[Builds_GetPromotionApprovals]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionApprovals]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotionApprovals]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON
		SELECT *
		  FROM [BuildApprovals_Extended]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])
END
GO

GRANT EXECUTE ON [Builds_GetPromotionApprovals] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotionRequiredApprovals',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_RequiredApprovals',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a set of approvals for the specified environment of a given build'
GO

IF OBJECT_ID('[Builds_GetPromotionRequiredApprovals]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionRequiredApprovals]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotionRequiredApprovals]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT * FROM [BuildPromotions_RequiredApprovals]
	 WHERE (@Environment_Id IS NULL OR [Environment_Id] = @Environment_Id)
	   AND [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
END
GO

GRANT EXECUTE ON [Builds_GetPromotionRequiredApprovals] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotionRequiredApprovalsByUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_RequiredApprovalsByUser',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of approvals required from a specific user or group member for a build in a specified environment'
GO

IF OBJECT_ID('[Builds_GetPromotionRequiredApprovalsByUser]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotionRequiredApprovalsByUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotionRequiredApprovalsByUser]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,

	@User_Name USERNAME = NULL,
	@GroupNames_List VARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON


	SELECT * INTO #requiredApprovals
	  FROM [BuildPromotions_RequiredApprovalsByUser]
	 WHERE (@Environment_Id IS NULL OR [Environment_Id] = @Environment_Id)
	   AND [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND (
	           @User_Name IS NULL
            OR 
	          ('U' = [ApprovalType_Code]
	           AND [Principal_Name] = @User_Name)
            OR ('G' = [ApprovalType_Code]
	           AND [Principal_Name] IN (
		             SELECT [Value_Text] 
		             FROM dbo.CsvToTable(@GroupNames_List, CHAR(10)) Groups)
	           AND [ApprovedBy_User_Name] IS NULL OR [ApprovedBy_User_Name] = @User_Name)
	       )

	/* somewhat tricky filter ahead:
	    when a user is passed in, filter down to one group row per approval req
	    the group row is either the user's approval, or a blank spot
	 */
	DELETE #requiredApprovals
	  FROM #requiredApprovals RA
	       INNER JOIN
	       (
	         SELECT [Approval_Sequence]
	               ,[RequiredInGroup_Sequence]
	               ,[ApprovalRequired_Rank] = RANK() OVER (
	                  PARTITION BY [Approval_Sequence]
                      ORDER BY CASE WHEN [ApprovedBy_User_Name] IS NULL THEN 0 ELSE 1 END DESC
                              ,[RequiredInGroup_Sequence])
	           FROM #requiredApprovals
	       ) RA2 ON RA.[Approval_Sequence] = RA2.[Approval_Sequence]
	            AND RA.[RequiredInGroup_Sequence] = RA2.[RequiredInGroup_Sequence]       
	            AND [ApprovalRequired_Rank] <> 1
	 WHERE @User_Name IS NOT NULL



	SELECT * FROM #requiredApprovals
	   
END
GO

GRANT EXECUTE ON [Builds_GetPromotionRequiredApprovalsByUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildPromotions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the promotions that have occurred for a specified build'
GO


IF OBJECT_ID('[Builds_GetPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetPromotions]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildPromotions_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

END
GO

GRANT EXECUTE ON [Builds_GetPromotions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_GetRequiredPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Builds_RequiredPromotions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all the promotions that must occur for a build from start to finish'
GO

IF OBJECT_ID('[Builds_GetRequiredPromotions]') IS NOT NULL DROP PROCEDURE [Builds_GetRequiredPromotions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_GetRequiredPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT * FROM [Builds_RequiredPromotions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	 ORDER BY [Step_Sequence]
END
GO

GRANT EXECUTE ON [Builds_GetRequiredPromotions] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_RejectBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Rejects the specified build'
GO


IF OBJECT_ID('[Builds_RejectBuild]') IS NOT NULL DROP PROCEDURE [Builds_RejectBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_RejectBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Rejection_Notes VARCHAR(255) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	UPDATE [Builds]
	   SET [BuildStatus_Name] = 'Rejected'
		  ,[Rejection_Notes] = @Rejection_Notes
		  ,[Rejected_Date] = GETUTCDATE()
		  ,[RejectedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [BuildStatus_Name] <> 'Deployed'
	IF (@@ROWCOUNT = 0) RAISERROR('{App#%i} %s Build %s does not exist.', 16, 1, @Application_Id, @Release_Number, @Build_Number)
	
	-- Cancel all pending and executing build executions when rejecting this build
	UPDATE [BuildExecutions]
	   SET [ExecutionStatus_Name] = 'Canceled'
		  ,[CanceledBy_User_Name] = dbo.Session_GetUserName()
		  ,[CanceledOn_Date] = GETUTCDATE()
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [ExecutionStatus_Name] IN ('Pending', 'Executing')
	
	EXEC [Events_RaiseEvent] 'BLDRJC', @Application_Id, @Release_Number, @Build_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_RejectBuild] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_RemoveApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Removes the specified build approval from a build'
GO

IF OBJECT_ID('[Builds_RemoveApproval]') IS NOT NULL DROP PROCEDURE [Builds_RemoveApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_RemoveApproval]
(
	@Approval_Id INT
)
AS
BEGIN
SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @Application_Id INT
			   ,@Release_Number RELEASENUMBER 
			   ,@Build_Number BUILDNUMBER
			   ,@Environment_Id INT
			   ,@Approval_Description SQL_VARIANT
		SELECT @Application_Id = [Application_Id]
			  ,@Release_Number = [Release_Number]
			  ,@Build_Number = [Build_Number]
			  ,@Environment_Id = [Environment_Id]
			  ,@Approval_Description = [Approval_Description]
		  FROM [BuildApprovals]
		 WHERE [Approval_Id] = @Approval_Id
		IF (@@ROWCOUNT = 0) RAISERROR('Approval #%i does not exist.', 16, 1, @Approval_Id)


		DELETE [BuildApprovals]
		 WHERE [Approval_Id] = @Approval_Id

		EXEC [Events_RaiseEvent] 'BLDUNA', @Application_Id, @Release_Number, @Build_Number, @Environment_Id, @Approval_Description

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_RemoveApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_ValidateForPromotion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Eligible_Indicator,IneligibleReason_Text',
    /* Description_Text        */ 'Indicates whether a build is eligible for a promotion to the next environment in its workflow'
GO


IF OBJECT_ID('[Builds_ValidateForPromotion]') IS NOT NULL DROP PROCEDURE [Builds_ValidateForPromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_ValidateForPromotion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Eligible_Indicator CHAR(1) OUT,
	@IneligibleReason_Text VARCHAR(MAX) OUT,
	@ForcePromotion_Indicator YNINDICATOR = 'N',
	@DependencyContingent_Indicator YNINDICATOR = 'N',
	@PromoteTo_Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @ForcePromotion_Indicator IS NULL
	BEGIN
		SET @ForcePromotion_Indicator = 'N'
	END

	DECLARE @ReleaseStatus_Name VARCHAR(8)
	DECLARE @BuildStatus_Name VARCHAR(8)
	DECLARE @Workflow_Id INT
	DECLARE @Current_Environment_Id INT
    DECLARE @Current_Environment_Name VARCHAR(50)
    DECLARE @Next_Step_Sequence INT
	DECLARE @Next_Environment_Id INT
    DECLARE @Next_Environment_Name VARCHAR(50)
    DECLARE @AllowOutOfSequencePromotions_Indicator YNINDICATOR
    DECLARE @AutoDeployRelease_Indicator YNINDICATOR

    SELECT @Current_Environment_Name = [Environment_Name]
      FROM [Environments]
     WHERE [Environment_Id] = @PromoteTo_Environment_Id

	SELECT @ReleaseStatus_Name = [ReleaseStatus_Name],
	       @BuildStatus_Name = [BuildStatus_Name],
	       @Next_Step_Sequence = [Next_Step_Sequence],
	       @Workflow_Id = [Workflow_Id],
           @Current_Environment_Id = [Current_Environment_Id],
		   @Next_Environment_Id = [Next_Environment_Id],
           @Next_Environment_Name = [Next_Environment_Name],
           @AllowOutOfSequencePromotions_Indicator = [AllowOutOfSequencePromotions_Indicator],
           @AutoDeployRelease_Indicator = [AutoDeployRelease_Indicator]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	DECLARE @Last_Promotion_Status VARCHAR(20)

	SELECT @Last_Promotion_Status = [PromotionStatus_Name]
      FROM [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
       AND [Environment_Id] = @Current_Environment_Id
       
	/* Promotions to any environment AllowOutOfSequencePromotions_Indicator */
	IF ((@PromoteTo_Environment_Id <> @Next_Environment_Id) AND @ForcePromotion_Indicator = 'N' AND @AllowOutOfSequencePromotions_Indicator = 'N')
   		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 
			  'Cannot promote to environment "' + @Current_Environment_Name + '" '
			  + 'because it''s not the next environment in the workflow "' + @Next_Environment_Name + '" '
              + 'and the workflow is not set up to allow promotions to arbitrary environments.' 
			RETURN
		END

	/* Release must be active */
	IF @ReleaseStatus_Name <> 'Active'
   		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 
			  'The release status is '
			  + @ReleaseStatus_Name 
			  + '; only builds for active releases may be promoted.'
			RETURN
		END

	/* Build must be active */
	IF @BuildStatus_Name <> 'Active'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 
			  'The build status is '
			  + @BuildStatus_Name
			  + '; only Active builds may be promoted.'
			RETURN
		END

	/* last promotion status needs to have succeeded */
	IF @Last_Promotion_Status IS NOT NULL AND @Last_Promotion_Status IN ('Failed','Executing','AwaitingExecution') 
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text =
				'The previous promotion status is '
				+ @Last_Promotion_Status
				+ ' and must be completed or overridden before the build can be promoted again.'
			RETURN
		END

	/* there is a next workflow step */
	IF @Next_Step_Sequence IS NULL AND @AllowOutOfSequencePromotions_Indicator = 'N' AND @AutoDeployRelease_Indicator = 'Y'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 'There is no step that the build can be promoted to.'
			RETURN
		END

	/* approvals checked off */
    IF (SELECT [dbo].[IsPromotionApproved](@Application_Id, @Release_Number, @Build_Number, @PromoteTo_Environment_Id)) = 'N'
			   AND @ForcePromotion_Indicator = 'N'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 'Not all required approvals are present.'
			RETURN
		END
	
	/* Cannot promote to their final environment until all referenced dependencies are promoted to their respective final environments */	
	IF EXISTS(SELECT * FROM Builds_Extended BE
	           WHERE [Next_Environment_Id] = [Final_Environment_Id]
	             AND BE.[Build_Number] = @Build_Number				
	             AND BE.[Release_Number] = @Release_Number
	             AND BE.[Application_Id] = @Application_Id
	             AND (
	                   EXISTS 
	                   (
	                     SELECT * 
	                       FROM ReleaseDeployables _RD
	                            INNER JOIN Releases _R 
	                                    ON _RD.Application_Id = BE.[Application_Id]
	                                   AND _RD.Release_Number = BE.[Release_Number]
	                                   AND _RD.Referenced_Release_Number = _R.Release_Number 
	                                   AND _RD.Referenced_Application_Id = _R.Application_Id
	                                   AND _R.[ReleaseStatus_Name] <> 'Deployed'
	                   ) AND (
	                     @DependencyContingent_Indicator <> 'Y'
	                     OR NOT EXISTS 
	                     ( /* Ensure all dependencies have a non-reject build that has been promoted to final environment */
	                       SELECT * 
	                         FROM ReleaseDeployables _RD
	                               INNER JOIN Builds_Extended _B
	                                       ON _RD.Referenced_Release_Number = _B.Release_Number 
	                                      AND _RD.Referenced_Application_Id = _B.Application_Id
	                        WHERE _RD.Application_Id = BE.[Application_Id]
	                         AND _RD.Release_Number = BE.[Release_Number]
	                         AND _B.[Current_Environment_Id] = _B.[Final_Environment_Id]
	                         AND _B.[BuildStatus_Name] <> 'Rejected'
	                     )
	                   )
				)
		) AND @ForcePromotion_Indicator = 'N'
		BEGIN
			SET @Eligible_Indicator = 'N'
			SET @IneligibleReason_Text = 'Deployables with dependencies (including imported deployables) cannot be promoted to their final environment until all referenced dependencies are promoted to their respective final environments. If forced, this may cause deployment failures or invalid dependencies.'
			RETURN
		END	
		
	/* Validates */
	SET @Eligible_Indicator = 'Y'
	SET @IneligibleReason_Text = ''
	RETURN
	
END
GO

GRANT EXECUTE ON [Builds_ValidateForPromotion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetAllGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_AllGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the test result groups in the system.'
GO


IF OBJECT_ID('[BuildTestResults_GetAllGroups]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetAllGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetAllGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildTestResults_AllGroups]
	 ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [BuildTestResults_GetAllGroups] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestGroupResults',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_TestGroupResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the unit test results for a specified build execution'
GO


IF OBJECT_ID('[BuildTestResults_GetTestGroupResults]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestGroupResults]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestGroupResults]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * 
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Execution_Id] = @Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestGroupResults] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestGroupResultsForBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_TestGroupResults',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the unit test results for a specified build execution'
GO


IF OBJECT_ID('[BuildTestResults_GetTestGroupResultsForBuild]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestGroupResultsForBuild]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestGroupResultsForBuild]
(
	@Application_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

    DECLARE @Max_Execution_Id INT

    SELECT TOP 1 @Max_Execution_Id = [Execution_Id]
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
       AND [Build_Number] = @Build_Number
     ORDER BY [Execution_Id] DESC

    SELECT *
      FROM [BuildTestResults_TestGroupResults]
	 WHERE [Execution_Id] = @Max_Execution_Id

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestGroupResultsForBuild] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestHistory',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets this history of a test by its name, and optionally filtered by test group and application.'
GO


IF OBJECT_ID('[BuildTestResults_GetTestHistory]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestHistory]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestHistory]
(
	@Test_Name VARCHAR(255),
	@Group_Name VARCHAR(50),
	@Application_Id INT,
	@TestStatus_Code CHAR(1),
	@Row_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Row_Count IS NOT NULL
		SET ROWCOUNT @Row_Count

	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE (@Group_Name IS NULL OR [Group_Name] = @Group_Name)
	   AND (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@TestStatus_Code IS NULL OR [TestStatus_Code] = @TestStatus_Code)
	   AND (@Test_Name = [Test_Name])
	 ORDER BY [Application_Name], [Group_Name], [Execution_Id] DESC

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestHistory] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestLog',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the results of a specific unit test'
GO


IF OBJECT_ID('[BuildTestResults_GetTestLog]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestLog]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50),
	@Test_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Group_Name] = @Group_Name
	   AND [Test_Sequence] = @Test_Sequence

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_GetTestResults',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildTestResults_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the build test results based on the specified criteria'
GO


IF OBJECT_ID('[BuildTestResults_GetTestResults]') IS NOT NULL DROP PROCEDURE [BuildTestResults_GetTestResults]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_GetTestResults]
(
	@Execution_Id INT,
	@Group_Name VARCHAR(50) = NULL,
	@Original_PlanActionGroup_Id INT = NULL,
	@Original_Action_Sequence INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON


	SELECT *
	  FROM [BuildTestResults_Extended]
	 WHERE [Execution_Id] = @Execution_Id
       AND [Group_Name] = ISNULL(@Group_Name, [Group_Name])
	   AND [Original_PlanActionGroup_Id] = ISNULL(@Original_PlanActionGroup_Id, [Original_PlanActionGroup_Id])
	   AND [Original_Action_Sequence] = ISNULL(@Original_Action_Sequence, [Original_Action_Sequence])
	ORDER BY [Group_Name], [Test_Sequence]

END
GO

GRANT EXECUTE ON [BuildTestResults_GetTestResults] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'BuildTestResults_RecordTestResult',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Logs the result of an individual unit test.'
GO

IF OBJECT_ID('[BuildTestResults_RecordTestResult]') IS NOT NULL DROP PROCEDURE [BuildTestResults_RecordTestResult]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [BuildTestResults_RecordTestResult]
(
	@BuildExecution_PlanAction_Id INT,
	@Group_Name VARCHAR(50),
	@Test_Name VARCHAR(255),
	@TestPassed_Indicator YNINDICATOR,
	@TestResult_Text VARCHAR(MAX),
	@TestStarted_Date DATETIME,
	@TestEnded_Date DATETIME
)
AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Execution_Id INT, @Original_Action_Sequence INT, @Original_PlanActionGroup_Id INT
	SELECT @Execution_Id = BEPA.[Execution_Id]
		  ,@Original_PlanActionGroup_Id = BEPA.[Original_PlanActionGroup_Id]
	      ,@Original_Action_Sequence = BEPA.[Original_Action_Sequence]
	 FROM [BuildExecution_PlanActions] BEPA
	WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	INSERT INTO [BuildTestResults]
			   ([Execution_Id]
			   ,[Test_Sequence]
			   ,[Group_Name]
			   ,[Test_Name]
			   ,[TestStatus_Code]
			   ,[TestResult_Text]
			   ,[TestStarted_Date]
			   ,[TestEnded_Date]
			   ,[BuildExecution_PlanAction_Id])
		 SELECT
				@Execution_Id,
				ISNULL(MAX([Test_Sequence]), 0) + 1,
				@Group_Name,
				@Test_Name,
				CASE @TestPassed_Indicator WHEN 'Y' THEN 'P' WHEN 'N' THEN 'F' ELSE 'I' END,
				@TestResult_Text,
				@TestStarted_Date,
				@TestEnded_Date,
				(SELECT TOP 1 [BuildExecution_PlanAction_Id]
				   FROM [BuildExecution_PlanActions]
				  WHERE [Execution_Id] = @Execution_Id
				    AND [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id
				    AND [Original_Action_Sequence] = @Original_Action_Sequence)
		   FROM [BuildTestResults]
		  WHERE [BuildExecution_PlanAction_Id] = (SELECT TOP 1 [BuildExecution_PlanAction_Id]
												   FROM [BuildExecution_PlanActions]
												  WHERE [Execution_Id] = @Execution_Id
												    AND [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id
												    AND [Original_Action_Sequence] = @Original_Action_Sequence)

END
GO

GRANT EXECUTE ON [BuildTestResults_RecordTestResult] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControlPerformances_GetReleaseSummary',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ChangeControlPerformances_ReleaseSummary',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a summary of the change control performances for a specified change control'
GO

IF OBJECT_ID('[ChangeControlPerformances_GetReleaseSummary]') IS NOT NULL DROP PROCEDURE [ChangeControlPerformances_GetReleaseSummary]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControlPerformances_GetReleaseSummary]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON
    
    SELECT * FROM [ChangeControlPerformances_ReleaseSummary]
    WHERE [ChangeControl_Id] = @ChangeControl_Id

END
GO

GRANT EXECUTE ON [ChangeControlPerformances_GetReleaseSummary] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_CreateOrUpdateChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ChangeControl_Id',
    /* Description_Text        */ 'Creates a new change control, or updates an existing one'
GO

IF OBJECT_ID('[ChangeControls_CreateOrUpdateChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_CreateOrUpdateChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_CreateOrUpdateChangeControl]
(
	@ChangeControl_Id INT = NULL OUT,
	@ChangeControl_Name VARCHAR(100),
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Group_Name VARCHAR(100),
	@Description_Text VARCHAR(MAX),
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@PerformanceType_Code CHAR(1),
	@Active_Indicator YNINDICATOR,
	@Environments_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
BEGIN TRANSACTION

    IF @ChangeControl_Id IS NULL
    BEGIN
    
        INSERT INTO [ChangeControls]
        (
            [Application_Id]
	       ,[Release_Number]
	       ,[ChangeControl_Name]
	       ,[Group_Name]
	       ,[Description_Text]
	       ,[Principal_Name]
	       ,[PrincipalType_Code]
	       ,[PerformanceType_Code]
	       ,[CreatedBy_User_Name]
	       ,[CreatedOn_Date]
	       ,[ModifiedBy_User_Name]
	       ,[ModifiedOn_Date]
	       ,[Active_Indicator]
        )
        VALUES
        (
	         @Application_Id
	        ,@Release_Number
	        ,@ChangeControl_Name
	        ,@Group_Name
	        ,@Description_Text
	        ,@Principal_Name
	        ,@PrincipalType_Code
	        ,@PerformanceType_Code
	        ,dbo.Session_GetUserName()
	        ,GETUTCDATE()
	        ,dbo.Session_GetUserName()
	        ,GETUTCDATE()
	        ,@Active_Indicator
        )
        
        SET @ChangeControl_Id = SCOPE_IDENTITY()
        
        INSERT INTO [ChangeControlEnvironments]
			(
				[ChangeControl_Id],
				[Environment_Id]
			)
		SELECT
				@ChangeControl_Id,
				CAST(CSV.[Value_Text] AS INT)
		  FROM [dbo].[CsvToTable](@Environments_Csv, ',') CSV
        
        EXEC [Events_RaiseEvent] 'CCTNEW', @Application_Id, @ChangeControl_Id
    
    END
    ELSE
    BEGIN
    
        UPDATE [ChangeControls]
           SET  [ChangeControl_Name] = @ChangeControl_Name
	           ,[Group_Name] = @Group_Name
	           ,[Description_Text] = @Description_Text
	           ,[Principal_Name] = @Principal_Name
	           ,[PrincipalType_Code] = @PrincipalType_Code
	           ,[PerformanceType_Code] = @PerformanceType_Code
	           ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
	           ,[ModifiedOn_Date] = GETUTCDATE()
	           ,[Active_Indicator] = @Active_Indicator
	     WHERE [ChangeControl_Id] = @ChangeControl_Id
	     
	     DELETE [ChangeControlEnvironments]
		 WHERE [ChangeControl_Id] = @ChangeControl_Id
        
         INSERT INTO [ChangeControlEnvironments] ([ChangeControl_Id], [Environment_Id])
	     SELECT
			    @ChangeControl_Id,
			    CAST(CSV.[Value_Text] AS INT)
	      FROM [dbo].[CsvToTable](@Environments_Csv, ',') CSV
	     
	     EXEC [Events_RaiseEvent] 'CCTMOD', @Application_Id, @ChangeControl_Id
    
    END

COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_CreateOrUpdateChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_DeleteChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified change control'
GO

IF OBJECT_ID('[ChangeControls_DeleteChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_DeleteChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_DeleteChangeControl]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Application_Id INT
    
    SELECT @Application_Id = [Application_Id]
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] = @ChangeControl_Id

    DELETE 
      FROM [ChangeControlEnvironments]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    DELETE
      FROM [ChangeControlPerformances]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    DELETE 
      FROM [ChangeControls]
     WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    EXEC [Events_RaiseEvent] 'CCTDEL', @Application_Id
    
COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_DeleteChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ChangeControls,ChangeControlEnvironments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified change control'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControl]
(
	@ChangeControl_Id INT
)
AS
BEGIN

SET NOCOUNT ON
    
    SELECT * FROM [ChangeControls]
    WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    SELECT * FROM [ChangeControlEnvironments_Extended]
    WHERE [ChangeControl_Id] = @ChangeControl_Id

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControlPerformances',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ChangeControlPerformances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of change control performances for a specified change control or all change controls if none is specified'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControlPerformances]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControlPerformances]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControlPerformances]
(
	@Application_Id INT,
	@ChangeControl_Id INT,
	@Release_Number RELEASENUMBER,
	@IncludeInactive_Indicator YNINDICATOR,
	@Environment_Id INT,
	@Row_Count INT
)
AS
BEGIN

SET NOCOUNT ON

    IF @Row_Count IS NOT NULL
        SET ROWCOUNT @Row_Count
    
    SELECT *
      FROM [ChangeControlPerformances_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND [ChangeControl_Id] = COALESCE(@ChangeControl_Id, [ChangeControl_Id])
       AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
       AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControlPerformances] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_GetChangeControls',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ChangeControls,ChangeControlEnvironments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of change controls based on the specified criteria'
GO

IF OBJECT_ID('[ChangeControls_GetChangeControls]') IS NOT NULL DROP PROCEDURE [ChangeControls_GetChangeControls]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_GetChangeControls]
(
    @Application_Id INT,
    @Release_Number RELEASENUMBER,
    @IncludeInactive_Indicator YNINDICATOR,
    @ChangeControl_Count INT
)
AS
BEGIN

SET NOCOUNT ON
    
    IF @ChangeControl_Count IS NOT NULL
        SET ROWCOUNT @ChangeControl_Count
    
    SELECT * FROM [ChangeControls]
     WHERE [Application_Id] = COALESCE(@Application_Id, [Application_Id])
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND ([Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
       
    SELECT * FROM [ChangeControlEnvironments_Extended]

END
GO

GRANT EXECUTE ON [ChangeControls_GetChangeControls] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ChangeControls_PerformChangeControl',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a change control performance for a specified environment'
GO

IF OBJECT_ID('[ChangeControls_PerformChangeControl]') IS NOT NULL DROP PROCEDURE [ChangeControls_PerformChangeControl]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ChangeControls_PerformChangeControl]
(
	@ChangeControl_Id INT,
	@Environment_Id INT,
	@Notes_Text VARCHAR(MAX),
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON
    
BEGIN TRY
	BEGIN TRANSACTION
    
    DECLARE @Application_Id INT
    DECLARE @ChangeControlPerformance_Id INT
    
    SELECT @Application_Id = [Application_Id]
    FROM [ChangeControls]
    WHERE [ChangeControl_Id] = @ChangeControl_Id
    
    INSERT INTO [ChangeControlPerformances]
    (
        [ChangeControl_Id]
       ,[Environment_Id]
       ,[PerformedBy_User_Name]
       ,[Performed_Date]
       ,[Notes_Text]
       ,[Execution_Id]
    )
    VALUES
    (
        @ChangeControl_Id,
        @Environment_Id,
        dbo.Session_GetUserName(),
        GETUTCDATE(),
        @Notes_Text,
        @Execution_Id
    )
    
    SET @ChangeControlPerformance_Id = SCOPE_IDENTITY()
    
    EXEC [Events_RaiseEvent] 'CCTPRF', @Application_Id, @ChangeControl_Id, @ChangeControlPerformance_Id
    
COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ChangeControls_PerformChangeControl] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ConfigurationFile_Id',
    /* Description_Text        */ 'Creates a configuration file in the system'
GO


IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFile]
(
	@ConfigurationFile_Id INT = NULL OUT,
	@Deployable_Id INT,
	@FilePath_Text VARCHAR(500)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	

	INSERT INTO [ConfigurationFiles] 
	(
		[Deployable_Id],
		[FilePath_Text],
		[Active_Indicator],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date]
	)
	VALUES 
	(
		@Deployable_Id, 
		@FilePath_Text, 
		'Y',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE()
	)

	SET @ConfigurationFile_Id = SCOPE_IDENTITY()
	
	DECLARE @Application_Id INT
	 SELECT @Application_Id = [Application_Id]
	   FROM [Deployables]
	  WHERE [Deployable_Id] = @Deployable_Id

	EXEC [Events_RaiseEvent] 'CFGNEW', @Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH



END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFile] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new instance of a configuration file'
GO


IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@Environment_Id INT,
	@Template_Indicator YNINDICATOR,
	@Template_Instance_Name VARCHAR(100)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT *
                    FROM [ConfigurationFileInstances]
				   WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
				     AND [Instance_Name] = @Instance_Name)
	BEGIN

		INSERT INTO [ConfigurationFileInstances]
	           ([ConfigurationFile_Id],[Instance_Name],[Environment_Id],[Active_Indicator],[Template_Indicator],[Template_Instance_Name])
	    VALUES
	           (@ConfigurationFile_Id,@Instance_Name,@Environment_Id,'Y',@Template_Indicator,@Template_Instance_Name)

		DECLARE @Application_Id INT
		 SELECT @Application_Id = D.[Application_Id]
		   FROM [ConfigurationFiles] CF
		        INNER JOIN [Deployables] D
			            ON CF.[Deployable_Id] = D.[Deployable_Id]
		  WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

		EXEC [Events_RaiseEvent] 'CFGNWI', @Application_Id, @ConfigurationFile_Id,@Instance_Name
		
    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileInstance] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_CreateConfigurationFileVersions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates new configuration file versions for each instance specified in the ConfigurationFiles_Xml for the given configuration file'
GO

IF OBJECT_ID('[ConfigurationFiles_CreateConfigurationFileVersions]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_CreateConfigurationFileVersions]
(
	@ConfigurationFile_Id INT,
	@ConfigurationFiles_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Find the AppID */
	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id]
	                             FROM [Deployables] D
	                                  INNER JOIN [ConfigurationFiles] CF
	                                          ON D.[Deployable_Id] = CF.[Deployable_Id]
	                            WHERE CF.[ConfigurationFile_Id] = @ConfigurationFile_Id)

	/* XML format example
		<?xml version="1.0" encoding="utf-16"?>
		<ConfigFiles>
			<Version
				Instance_Name="Default"
				VersionNotes_Text="Notes..."
				File_Bytes="::base64::"
			/>
		</ConfigFiles>
	*/
	 SELECT [ConfigurationFile_Id] = @ConfigurationFile_Id
			/* Get next version number in sequence (or 1, if no previous version exists) */
			,[Version_Number] = COALESCE(
				(SELECT TOP 1 [Version_Number] + 1
				   FROM [ConfigurationFileVersions] _CFV
				  WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
					AND _CFV.[Instance_Name] = CFVX.[Version].[value]('@Instance_Name', 'VARCHAR(100)')
				 ORDER BY [Version_Number] DESC),
				1)
			,[Instance_Name] = CFVX.[Version].[value]('@Instance_Name', 'VARCHAR(100)')
			,[VersionNotes_Text] = CFVX.[Version].[value]('@VersionNotes_Text', 'VARCHAR(500)')
			,[File_Date] = GETUTCDATE()
			,[File_Bytes] = COALESCE(CFVX.[Version].[value]('@File_Bytes', 'VARBINARY(MAX)'), CAST('' AS VARBINARY(MAX)))
			,[CreatedBy_User_Name] = dbo.Session_GetUserName()
			,[CreatedOn_Date] = GETUTCDATE()
			,[Application_Id] = @Application_Id
			,[Release_Number] = CFVX.[Version].[value]('@Release_Number', 'RELEASENUMBER')
	   INTO #NewVersions
	   FROM @ConfigurationFiles_Xml.[nodes]('/ConfigFiles/Version') AS CFVX(Version)

	INSERT INTO [ConfigurationFileVersions]
		([ConfigurationFile_Id],[Version_Number],[Instance_Name],[VersionNotes_Text],[File_Date]
		,[File_Bytes],[CreatedBy_User_Name],[CreatedOn_Date],[Application_Id],[Release_Number])
    SELECT
		[ConfigurationFile_Id],[Version_Number],[Instance_Name],[VersionNotes_Text],[File_Date]
		,[File_Bytes],[CreatedBy_User_Name],[CreatedOn_Date],[Application_Id],[Release_Number]
    FROM #NewVersions

	DECLARE @Instance_Name SQL_VARIANT, @Version_Number SQL_VARIANT, @VER_CUR CURSOR
	SET @VER_CUR = CURSOR FOR (SELECT Instance_Name, Version_Number FROM #NewVersions)

	OPEN @VER_CUR
	FETCH @VER_CUR INTO @Instance_Name, @Version_Number
	WHILE @@FETCH_STATUS=0 BEGIN   
		EXEC [Events_RaiseEvent] 'CFGNWV', @Application_Id, @ConfigurationFile_Id, @Instance_Name, @Version_Number
		FETCH @VER_CUR INTO @Instance_Name, @Version_Number
	END   

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ConfigurationFiles_CreateConfigurationFileVersions] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_DeployConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ConfigurationFileDeployment_Id',
    /* Description_Text        */ 'Saves the result of a deployed configuration file (note this will not physically deploy the configuration file)'
GO

IF OBJECT_ID('[ConfigurationFiles_DeployConfigurationFile]') IS NOT NULL
	DROP PROCEDURE [ConfigurationFiles_DeployConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_DeployConfigurationFile]
(
	@ConfigurationFileDeployment_Id INT = NULL OUT,
	@Execution_Id INT,
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
	@Version_Number INT,
	@Server_Id INT,
	@DeployedTo_Path VARCHAR(500)
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	INSERT INTO [ConfigurationFileDeployments]
		([ConfigurationFile_Id]
		,[Execution_Id]
		,[Instance_Name]
		,[Version_Number]
		,[Server_Id]
		,[DeployedTo_Path]
		,[DeployedBy_User_Name]
		,[DeployedOn_Date])
	VALUES
		(@ConfigurationFile_Id
		,@Execution_Id
		,@Instance_Name
		,@Version_Number
		,@Server_Id
		,@DeployedTo_Path
		,[dbo].[Session_GetUserName]()
		,GETUTCDATE())
	SET @ConfigurationFileDeployment_Id = SCOPE_IDENTITY()

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)

	EXEC [Events_RaiseEvent] 'CFGDPL',@Application_Id,@ConfigurationFileDeployment_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_DeployConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'ConfigurationFiles_Extended,ConfigurationFileInstances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all configuration files and all configuration file instances'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@IncludeInactiveInstances_Indicator YNINDICATOR = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [ConfigurationFiles_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	
	IF (@@ROWCOUNT = 0) BEGIN
		RAISERROR('Configuration file %i does not exist.', 16, 1, @ConfigurationFile_Id)
		RETURN
	END

	SELECT CFI.*
	  FROM [ConfigurationFileInstances_Extended] CFI
	       LEFT  JOIN (
	                   SELECT [Environment_Id], MAX([Workflow_Step_Sequence]) [Workflow_Step_Sequence]
	                     FROM [ApplicationEnvironments_Merged]
	                    GROUP BY [Environment_Id]
	                   ) AEM ON CFI.[Environment_Id] = AEM.[Environment_Id]
	 WHERE CFI.[ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND (CFI.[Active_Indicator] = 'Y' OR @IncludeInactiveInstances_Indicator = 'Y')
	 ORDER BY AEM.[Workflow_Step_Sequence], CFI.[Instance_Name]



END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFile] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileDeployments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileDeployments_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all configuration file deployments based on the specified criteria'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileDeployments]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileDeployments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileDeployments]
(
	@Application_Id INT,
    @ConfigurationFile_Id INT = NULL,
    @Deployable_Id INT = NULL,
    @Instance_Name VARCHAR(100) = NULL,
    @Server_Id INT = NULL,
	@File_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	--Use RowCount to limit results 
	IF (@File_Count IS NOT NULL AND @File_Count > 0)
	BEGIN
		SET ROWCOUNT @File_Count
	END

	SELECT *
	  FROM [ConfigurationFileDeployments_Extended]
     WHERE [Deployable_Id] IN (SELECT [Deployable_Id] 
                                 FROM [Deployables] 
                                WHERE [Application_Id] = @Application_Id)	 
       AND [ConfigurationFile_Id] = COALESCE(@ConfigurationFile_Id,[ConfigurationFile_Id])
       AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])
       AND [Instance_Name] = COALESCE(@Instance_Name,[Instance_Name])
       AND [Server_Id] = COALESCE(@Server_Id,[Server_Id])
	  ORDER BY [DeployedOn_Date] DESC

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileDeployments] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileInstances',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileInstances_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all configuration file instances that match the specified criteria'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileInstances]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileInstances]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileInstances]
(
	@Application_Id INT = NULL,
	@Deployable_Id INT = NULL,
	@Active_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT CFI.*
	  FROM [ConfigurationFileInstances_Extended] CFI
		   INNER JOIN [Environments] E
				 ON E.[Environment_Id] = CFI.[Environment_Id]
     WHERE CFI.[ConfigurationFile_Id] IN (SELECT [ConfigurationFile_Id] 
                                        FROM [ConfigurationFiles_Extended]
                                       WHERE [Application_Id] = @Application_Id
                                         AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])
                                         AND [Active_Indicator] = COALESCE(@Active_Indicator,[Active_Indicator]))
	ORDER BY E.[Display_Sequence], CFI.[Instance_Name]
END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileInstances] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFiles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFiles_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all configuration files that match the specified criteria'
GO


IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFiles]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFiles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFiles]
(
	@Application_Id INT = NULL,
	@Deployable_Id INT = NULL,
	@IncludeInactive_Indicator YNINDICATOR = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	SELECT *
	  FROM [ConfigurationFiles_Extended]
	 WHERE [Application_Id] = ISNULL(@Application_Id, [Application_Id])
	   AND [Deployable_Id] = ISNULL(@Deployable_Id, [Deployable_Id])
	   AND (	[Active_Indicator] = 'Y'
			 OR @IncludeInactive_Indicator = 'Y' )
	ORDER BY [Deployable_Name]
			,[Active_Indicator] DESC
			,[FilePath_Text]

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFiles] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileVersions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of configuration file versions for a specified configuration file'
GO

IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersion]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileVersion]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100) = NULL,
	@Version_Number INT = NULL OUT,
	@Release_Number RELEASENUMBER = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	/* Set defaults */
	SET @Instance_Name = ISNULL(@Instance_Name, 'Default')

	/* Find the app ID based on configuration file ID */
	DECLARE @Application_Id INT
	SELECT @Application_Id = DPL.[Application_Id]
	  FROM [ConfigurationFiles] CF
		   INNER JOIN [Deployables] DPL
				   ON CF.[Deployable_Id] = DPL.[Deployable_Id]
	 WHERE CF.[ConfigurationFile_Id] = @ConfigurationFile_Id
    
   DECLARE @Release_Sequence BIGINT 

   SELECT @Release_Sequence = [Release_Sequence]
     FROM [Releases]
    WHERE [Application_Id] = @Application_Id 
      AND [Release_Number] = @Release_Number

	IF @Version_Number IS NULL
	BEGIN
		/* Find the latest version number appropriate to the release
		   If @Release_Number was supplied, find the max version number for releases less than
		   or equal to the provided @Release_Number. */
		SELECT @Version_Number = MAX([Version_Number])
		  FROM [ConfigurationFileVersions_Extended]
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
		   AND [Instance_Name] = @Instance_Name
		   AND [Application_Id] = @Application_Id
		   AND (@Release_Sequence IS NULL OR [Release_Sequence] <= @Release_Sequence)
	END	

	SELECT *
	  FROM [ConfigurationFileVersions_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name
	   AND [Version_Number] = @Version_Number

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileVersion] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileVersions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileVersions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the configuration file versions that match the specified criteria'
GO

IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersions]') IS NOT NULL 
   DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileVersions]
(
	@ConfigurationFile_Id INT = NULL,

	@Application_Id INT = NULL,
	@Release_Number RELEASENUMBER = NULL,

    @Deployable_Id INT = NULL,
    @Instance_Name VARCHAR(100) = NULL,
	@File_Count INT = NULL
)
AS
BEGIN
	
	SET NOCOUNT ON

	--Use RowCount to limit results 
	IF (@File_Count IS NOT NULL AND @File_Count > 0)
	BEGIN
		SET ROWCOUNT @File_Count
	END

	SELECT *
	  FROM [ConfigurationFileVersions_Extended]
	 WHERE [ConfigurationFile_Id] = COALESCE(@ConfigurationFile_Id,[ConfigurationFile_Id])
	   AND [Application_Id] = COALESCE(@Application_Id,[Application_Id])
	   AND (@Release_Number IS NULL OR ([Application_Id] = @Application_Id AND [Release_Number] = @Release_Number))
	   AND [Deployable_Id] = COALESCE(@Deployable_Id,[Deployable_Id])
	   AND [Instance_Name] = COALESCE(@Instance_Name, [Instance_Name])

	ORDER BY [File_Date] DESC

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileVersions] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_GetConfigurationFileVersionsByRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ConfigurationFileVersions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the configuration file versions for the specified release including all version prior'
GO

IF OBJECT_ID('[ConfigurationFiles_GetConfigurationFileVersionsByRelease]') IS NOT NULL 
   DROP PROCEDURE [ConfigurationFiles_GetConfigurationFileVersionsByRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_GetConfigurationFileVersionsByRelease]
(
	@ConfigurationFile_Id INT,
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
    @Instance_Name VARCHAR(100)
)
AS
BEGIN
	
	SET NOCOUNT ON

    DECLARE @Release_Sequence BIGINT

    SELECT @Release_Sequence = [Release_Sequence]
     FROM [Releases]
    WHERE [Application_Id] = @Application_Id 
      AND [Release_Number] = @Release_Number

	SELECT *
	  FROM [ConfigurationFileVersions_Extended]
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Application_Id] = @Application_Id
	   AND [Release_Sequence] <= @Release_Sequence
	   AND [Instance_Name] = @Instance_Name
	   AND [ConfigurationFile_Active_Indicator] = 'Y'

	ORDER BY [Version_Number] DESC

END
GO

GRANT EXECUTE ON [ConfigurationFiles_GetConfigurationFileVersionsByRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_ReactivateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Reactivates a configuration file that has been marked inactive'
GO

IF OBJECT_ID('[ConfigurationFiles_ReactivateConfigurationFile]') IS NOT NULL
	DROP PROCEDURE [ConfigurationFiles_ReactivateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_ReactivateConfigurationFile]
(
	@ConfigurationFile_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [ConfigurationFiles]
	   SET [Active_Indicator] = 'Y'
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	IF @@ROWCOUNT=0 RAISERROR('ConfigurationFile #%i does not exist.',16,1,@ConfigurationFile_Id)

	UPDATE [ConfigurationFileInstances]
	   SET [Active_Indicator] = 'Y'
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)
	EXEC [Events_RaiseEvent] 'CFGMOD',@Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_ReactivateConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_RenameConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Renames an existing configuration file'
GO


IF OBJECT_ID('[ConfigurationFiles_RenameConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_RenameConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_RenameConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@New_FilePath_Text VARCHAR(500)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	RAISERROR('This procedure is not yet implemented.', 16, 1)
	-- EXEC [Events_RaiseEvent] 'CFGMOD',@Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_RenameConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_UpdateConfigurationFile',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates an existing configuration file'
GO


IF OBJECT_ID('[ConfigurationFiles_UpdateConfigurationFile]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_UpdateConfigurationFile]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_UpdateConfigurationFile]
(
	@ConfigurationFile_Id INT,
	@Active_Indicator YNINDICATOR = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Active_Indicator IS NOT NULL
		UPDATE [ConfigurationFiles]
		   SET [Active_Indicator] = @Active_Indicator
		 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)
	EXEC [Events_RaiseEvent] 'CFGMOD',@Application_Id, @ConfigurationFile_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_UpdateConfigurationFile] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ConfigurationFiles_UpdateConfigurationFileInstance',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates an existing configuration file instance'
GO


IF OBJECT_ID('[ConfigurationFiles_UpdateConfigurationFileInstance]') IS NOT NULL DROP PROCEDURE [ConfigurationFiles_UpdateConfigurationFileInstance]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ConfigurationFiles_UpdateConfigurationFileInstance]
(
	@ConfigurationFile_Id INT,
	@Instance_Name VARCHAR(100),
    @Environment_Id INT,
	@Active_Indicator YNINDICATOR,
	@Template_Indicator YNINDICATOR,
	@Template_Instance_Name VARCHAR(100)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Template_Indicator = 'N'
		UPDATE [ConfigurationFileInstances]
		   SET [Template_Instance_Name] = NULL
		 WHERE [Template_Instance_Name] = @Instance_Name

	UPDATE [ConfigurationFileInstances]
	   SET [Environment_Id] = COALESCE(@Environment_Id,[Environment_Id]) 
          ,[Active_Indicator] = @Active_Indicator
          ,[Template_Indicator] = COALESCE(@Template_Indicator, [Template_Indicator])
          ,[Template_Instance_Name] = @Template_Instance_Name
	 WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id
	   AND [Instance_Name] = @Instance_Name

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT D.[Application_Id] 
	                             FROM [ConfigurationFiles] CF
 	                                  INNER JOIN [Deployables] D
 	                                          ON CF.[Deployable_Id] = D.[Deployable_Id]
 	                            WHERE [ConfigurationFile_Id] = @ConfigurationFile_Id)
	EXEC [Events_RaiseEvent] 'CFGMDI',@Application_Id, @ConfigurationFile_Id, @Instance_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ConfigurationFiles_UpdateConfigurationFileInstance] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Configuration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of BuildMaster configuration settings.'
GO

IF OBJECT_ID('[Configuration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [Configuration_GetConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Configuration_GetConfiguration] AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Configuration]
	 WHERE [Scope_Name] <> 'Internal'
	 ORDER BY [Scope_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [Configuration_GetConfiguration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetInternalValue',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified BuildMaster internal configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_GetInternalValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_GetInternalValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_GetInternalValue]
(
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [dbo].[Configuration]
	 WHERE [Scope_Name] = 'Internal'
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [dbo].[Configuration_GetInternalValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified BuildMaster configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_GetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_GetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_GetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [dbo].[Configuration]
	 WHERE [Scope_Name] = @Scope_Name
	   AND [Key_Name] = @Key_Name
	   AND [Scope_Name] <> 'Internal'

END
GO

GRANT EXECUTE ON [dbo].[Configuration_GetValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a BuildMaster configuration value.'
GO


IF OBJECT_ID('[dbo].[Configuration_SetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_SetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_SetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Scope_Name <> 'Internal'
	BEGIN

		IF EXISTS(SELECT * FROM [Configuration] WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name) BEGIN
			UPDATE [Configuration]
	           SET [Value_Text] = @Value_Text
	         WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name

	    END ELSE BEGIN

			INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
			VALUES
			(@Scope_Name, @Key_Name, @Value_Text)
	    END

	EXEC [Events_RaiseEvent] 'SYSCFG',NULL,@Key_Name

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Configuration_SetValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValueWithoutEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[dbo].[Configuration_SetValueWithoutEvent]') IS NOT NULL
	DROP PROCEDURE [dbo].[Configuration_SetValueWithoutEvent]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Configuration_SetValueWithoutEvent]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [Configuration] WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name) BEGIN
		UPDATE [Configuration]
           SET [Value_Text] = @Value_Text
         WHERE [Scope_Name] = @Scope_Name AND [Key_Name] = @Key_Name

    END ELSE BEGIN

		INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
		VALUES
			(@Scope_Name, @Key_Name, @Value_Text)
    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Configuration_SetValueWithoutEvent] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_CreateDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a dashboard'
GO


IF OBJECT_ID('[Dashboards_CreateDashboard]') IS NOT NULL DROP PROCEDURE [Dashboards_CreateDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_CreateDashboard]
(
	@Application_Id INT,
	@User_Name USERNAME,
	@DashboardScope_Code CHAR(1)	
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	
	INSERT INTO [Dashboards] ([Application_Id], [User_Name], [DashboardScope_Code])
	VALUES (@Application_Id, @User_Name, @DashboardScope_Code)

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_CreateDashboard] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_CreateOrUpdateGadget',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Display_Sequence',
    /* Description_Text        */ 'Creates or updates a gadget in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_CreateOrUpdateGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_CreateOrUpdateGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_CreateOrUpdateGadget]
(
	@Gadget_Id INT = NULL OUT,
	@Dashboard_Id INT,
	@Dashboard_Zone INT,
	@Display_Sequence INT = NULL OUT,
	@DashboardGadget_Name VARCHAR(50),
	@DashboardGadget_Description VARCHAR(MAX),
	@DashboardGadget_Configuration VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		--Create
		IF @Gadget_Id IS NULL
		BEGIN
			SELECT @Display_Sequence = COALESCE(MAX([Display_Sequence]), 0) + 1 FROM [DashboardGadgets]
			
			INSERT INTO [DashboardGadgets]
			(
				[Dashboard_Id],
				[Dashboard_Zone],
				[Display_Sequence],
				[DashboardGadget_Name],
				[DashboardGadget_Description],
				[DashboardGadget_Configuration],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date]
			)
			VALUES
			(
				@Dashboard_Id,
				@Dashboard_Zone,
				@Display_Sequence,
				@DashboardGadget_Name,
				@DashboardGadget_Description,
				@DashboardGadget_Configuration,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE()
			)
			
			SET @Gadget_Id = SCOPE_IDENTITY()
		END
		--Update
		ELSE
		BEGIN
			UPDATE [DashboardGadgets]		
			   SET [DashboardGadget_Name] = @DashboardGadget_Name,
			       [DashboardGadget_Description] = @DashboardGadget_Description,
			       [DashboardGadget_Configuration] = @DashboardGadget_Configuration,
			       [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			       [ModifiedOn_Date] = GETUTCDATE()
			 WHERE [Gadget_Id] = @Gadget_Id
		END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_CreateOrUpdateGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_DeleteGadget',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a gadget in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_DeleteGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_DeleteGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_DeleteGadget]
(
	@Gadget_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [DashboardGadgets]
	 WHERE [Gadget_Id] = @Gadget_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_DeleteGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetDashboard',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Dashboards',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a dashboard.'
GO


IF OBJECT_ID('[Dashboards_GetDashboard]') IS NOT NULL DROP PROCEDURE [Dashboards_GetDashboard]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetDashboard]
(
	@Application_Id INT,
	@DashboardScope_Code CHAR(1)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT TOP 1 *
	  FROM [Dashboards]
	 WHERE [DashboardScope_Code] = @DashboardScope_Code
	  AND ([User_Name] = dbo.Session_GetUserName() OR [User_Name] IS NULL)
	  AND ([Application_Id] = @Application_Id OR [Application_Id] IS NULL)
	ORDER BY CASE WHEN [User_Name] IS NOT NULL THEN 0 ELSE 1 END, /* User Specific Comes First */
	         CASE WHEN [Application_Id] IS NOT NULL THEN 0 ELSE 1 END /* App Specific Comes Second */


END
GO

GRANT EXECUTE ON [Dashboards_GetDashboard] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetGadget',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a gadget in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_GetGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_GetGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetGadget]
(
	@Gadget_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DashboardGadgets_Extended]
	 WHERE [Gadget_Id] = @Gadget_Id

END
GO

GRANT EXECUTE ON [Dashboards_GetGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetGadgets',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the gadgets in an application or system-level dashboard'
GO


IF OBJECT_ID('[Dashboards_GetGadgets]') IS NOT NULL DROP PROCEDURE [Dashboards_GetGadgets]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetGadgets]
(
	@Dashboard_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [DashboardGadgets_Extended]
	 WHERE [Dashboard_Id] = @Dashboard_Id
	ORDER BY [Dashboard_Zone], [Display_Sequence]

END
GO

GRANT EXECUTE ON [Dashboards_GetGadgets] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_ResequenceGadget',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of a gadget within a dashboard'
GO


IF OBJECT_ID('[Dashboards_ResequenceGadget]') IS NOT NULL DROP PROCEDURE [Dashboards_ResequenceGadget]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_ResequenceGadget]
(
	@Gadget_Id INT,
	@New_Display_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @Dashboard_Id INT
	DECLARE @Display_Sequence INT
	SELECT @Dashboard_Id = [Dashboard_Id],
	       @Display_Sequence = [Display_Sequence]
	  FROM [DashboardGadgets]
	 WHERE [Gadget_Id] = @Gadget_Id

	BEGIN TRANSACTION

	UPDATE [DashboardGadgets]
	   SET [Display_Sequence] = CASE
	          WHEN [DashboardGadgets].[Display_Sequence] = CUR.[Display_Sequence] 
	               THEN NEW.[Display_Sequence]
	          WHEN [DashboardGadgets].[Display_Sequence] BETWEEN CUR.[Display_Sequence] AND NEW.[Display_Sequence]
	               THEN [DashboardGadgets].[Display_Sequence] - 1
	          WHEN [DashboardGadgets].[Display_Sequence] BETWEEN NEW.[Display_Sequence] AND CUR.[Display_Sequence]
	               THEN [DashboardGadgets].[Display_Sequence] + 1
	          ELSE [DashboardGadgets].[Display_Sequence]
	          END
	  FROM [DashboardGadgets],
	       [DashboardGadgets] CUR,
	       [DashboardGadgets] NEW
	 WHERE CUR.[Display_Sequence] = @Display_Sequence
	   AND NEW.[Display_Sequence] = @New_Display_Sequence
	   AND @Display_Sequence <> @New_Display_Sequence
	   AND [DashboardGadgets].[Dashboard_Id] = @Dashboard_Id
	   AND CUR.[Dashboard_Id] = @Dashboard_Id
	   AND NEW.[Dashboard_Id] = @Dashboard_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Dashboards_ResequenceGadget] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_AddDatabaseProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a database change script provider if one does not exist, or updates an existing one.'
GO

IF OBJECT_ID('[DatabaseChangeScripts_AddDatabaseProvider]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_AddDatabaseProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_AddDatabaseProvider]
(
	@Provider_Id INT,
	@Application_Id INT,
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF EXISTS(SELECT * FROM [DatabaseChangeScriptProviders]
	           WHERE [Provider_Id] = @Provider_Id)
	BEGIN

		UPDATE [DatabaseChangeScriptProviders]
		   SET [Environment_Id] = @Environment_Id
		 WHERE [Provider_Id] = @Provider_Id
		IF @@ROWCOUNT <> 1 RAISERROR('UPDATE did not update exactly 1 row.', 16, 1)
		
		EXEC [Events_RaiseEvent] 'DBSMOD', @Application_Id, @Provider_Id
	
	END ELSE BEGIN

		INSERT INTO [DatabaseChangeScriptProviders]
			([Provider_Id], [Application_Id], [Environment_Id])
		VALUES
			(@Provider_Id, @Application_Id, @Environment_Id)
			
        EXEC [Events_RaiseEvent] 'DBSNEW', @Application_Id, @Provider_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_AddDatabaseProvider] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_CreateNewChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Script_Id',
    /* Description_Text        */ 'Creates a new database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_CreateNewChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_CreateNewChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_CreateNewChangeScript]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT,
	@Script_Name VARCHAR(50),
	@Script_Text NVARCHAR(MAX),
	@Active_Indicator YNINDICATOR,

	@Script_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [DatabaseChangeScripts]
			   ([Application_Id]
			   ,[Release_Number]
			   ,[Deployable_Id]
			   ,[Script_Name]
			   ,[Script_Text]
			   ,[Active_Indicator]
			   ,[CreatedBy_User_Name]
			   ,[CreatedOn_Date]
			   ,[ModifiedBy_User_Name]
			   ,[ModifiedOn_Date])
		 VALUES
			   (@Application_Id
			   ,@Release_Number
			   ,@Deployable_Id
			   ,@Script_Name
			   ,@Script_Text
			   ,@Active_Indicator
			   ,[dbo].[Session_GetUserName]()
			   ,GETUTCDATE()
			   ,[dbo].[Session_GetUserName]()
			   ,GETUTCDATE())

	    SET @Script_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'DBSNWS', @Application_Id, @Script_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_CreateNewChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_DeleteChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_DeleteChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_DeleteChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_DeleteChangeScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Script_Name SQL_VARIANT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)
	    SET @Script_Name    = (SELECT [Script_Name] 
	                             FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)

	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Script_Id] = @Script_Id

	DELETE [DatabaseChangeScripts]
	 WHERE [Script_Id] = @Script_Id	

	IF @@ROWCOUNT > 0
	EXEC [Events_RaiseEvent] 'DBSDLS', @Application_Id, @Script_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_DeleteChangeScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_DeleteDatabaseProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a database provider'
GO

IF OBJECT_ID('[DatabaseChangeScripts_DeleteDatabaseProvider]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_DeleteDatabaseProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_DeleteDatabaseProvider]
(
	@Provider_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Application_Id INT, @Provider_Name SQL_VARIANT
    SET @Application_Id = (SELECT [Application_Id] 
                             FROM [DatabaseChangeScriptProviders] 
                            WHERE [Provider_Id] = @Provider_Id)
	SET @Provider_Name =  (SELECT [Provider_Name] 
                             FROM [Providers] 
                            WHERE [Provider_Id] = @Provider_Id)

    DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Provider_Id] = @Provider_Id
	 
	DELETE [DatabaseChangeScriptProviders]
	 WHERE [Provider_Id] = @Provider_Id

	DELETE [Providers]
	 WHERE [Provider_Id] = @Provider_Id

	EXEC [Events_RaiseEvent] 'DBSDEL', @Application_Id, @Provider_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_DeleteDatabaseProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_EditChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates an existing database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_EditChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_EditChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_EditChangeScript]
(
	@Application_Id INT,
	@Script_Id INT,
	@Release_Number RELEASENUMBER,
	@Deployable_Id INT,
	@Script_Name VARCHAR(50),
	@Active_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [DatabaseChangeScripts]
	   SET [Application_Id] = @Application_Id
		  ,[Release_Number] = @Release_Number
		  ,[Deployable_Id] = @Deployable_Id
		  ,[Script_Name] = @Script_Name
		  ,[Active_Indicator] = @Active_Indicator
		  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		  ,[ModifiedOn_Date] = GETUTCDATE()
	WHERE [Script_Id] = @Script_Id	
	IF @@ROWCOUNT = 0 RAISERROR('DbChangeScript #%i does not exist.',16,1,@Script_Id)

	EXEC [Events_RaiseEvent] 'DBSMDS', @Application_Id, @Script_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_EditChangeScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetChangeScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database change script'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetChangeScript]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetChangeScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetChangeScript]
(
	@Application_Id INT,
	@Script_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
      FROM [DatabaseChangeScripts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Script_Id] = @Script_Id


END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetChangeScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetChangeScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database change scripts that match the specified criteria'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetChangeScripts]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetChangeScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetChangeScripts]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER = NULL,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@DatabaseChangeScript_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	IF ISNULL(@DatabaseChangeScript_Count, 0) > 0
		SET ROWCOUNT @DatabaseChangeScript_Count

	SELECT * 
      FROM [DatabaseChangeScripts_Extended]
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
       AND (	[Active_Indicator] = 'Y'
			 OR @IncludeInactive_Indicator = 'Y' )
  ORDER BY [Numeric_Release_Number] DESC

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetChangeScripts] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseChangeScriptExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database change script execution details'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseChangeScriptExecution]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecution]
(
	@ScriptExecution_Id INT
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseChangeScriptExecutions_Extended]
	 WHERE [ScriptExecution_Id] = @ScriptExecution_Id
	IF @@ROWCOUNT=0 RAISERROR('ScriptExecution #%i was not found.',16,1,@ScriptExecution_Id)

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseChangeScriptExecution] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseChangeScriptExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database change script executions that have occurred matching the specified criteria'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseChangeScriptExecutions]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseChangeScriptExecutions]
(
	@Script_Id INT,
	@Application_Id INT,
	@IncludeInactive_Indicator YNINDICATOR = NULL,
	@Release_Number RELEASENUMBER,
	@Success_Indicator YNINDICATOR,
	@RowsToReturn_Count INT,
	@Environment_Id INT
) AS
BEGIN

	SET NOCOUNT ON
	
	IF @RowsToReturn_Count IS NOT NULL
		SET ROWCOUNT @RowsToReturn_Count

	SELECT *
	  FROM [DatabaseChangeScriptExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Script_Id] = COALESCE(@Script_Id, [Script_Id])
	   AND ([Script_Active_Indicator] = 'Y' OR @IncludeInactive_Indicator = 'Y')
	   AND [Release_Number] = COALESCE(@Release_Number, [Release_Number])
	   AND [Success_Indicator] = COALESCE(@Success_Indicator, [Success_Indicator])
	   AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])
	ORDER BY [ExecutedOn_Date] DESC

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseChangeScriptExecutions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptProviders_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified database provider'
GO

IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseProvider]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseProvider]
(
	@Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [DatabaseChangeScriptProviders_Extended]
	 WHERE [Provider_Id] = @Provider_Id

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_GetDatabaseProviders',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'DatabaseChangeScriptProviders_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the database providers that match the specified criteria.'
GO


IF OBJECT_ID('[DatabaseChangeScripts_GetDatabaseProviders]') IS NOT NULL DROP PROCEDURE [DatabaseChangeScripts_GetDatabaseProviders]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_GetDatabaseProviders]
(
	@Application_Id INT,
	@Environment_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
      FROM [DatabaseChangeScriptProviders_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_GetDatabaseProviders] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'DatabaseChangeScripts_LogExecution',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Logs the execution of a database change script'
GO

IF OBJECT_ID('[DatabaseChangeScripts_LogExecution]') IS NOT NULL
	DROP PROCEDURE [DatabaseChangeScripts_LogExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [DatabaseChangeScripts_LogExecution]
(
	@Script_Id INT,
	@Success_Indicator YNINDICATOR,
	@Log_Text VARCHAR(MAX),
	@Execution_Id INT,
	@Provider_Id INT,

	@ScriptExecution_Id INT = NULL OUT

) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [DatabaseChangeScriptExecutions]
           ([Script_Id]
           ,[ExecutedOn_Date]
           ,[ExecutedBy_User_Name]
           ,[Success_Indicator]
           ,[Log_Text]
           ,[Execution_Id]
           ,[Provider_Id])
	VALUES
           (@Script_Id
           ,GETUTCDATE()
           ,dbo.[Session_GetUserName]()
           ,@Success_Indicator
           ,@Log_Text
           ,@Execution_Id
           ,@Provider_Id)

	    SET @ScriptExecution_Id = SCOPE_IDENTITY()
	DECLARE  @Application_Id INT
		SET @Application_Id = (SELECT [Application_Id] FROM [DatabaseChangeScripts] 
	                            WHERE [Script_Id] = @Script_Id)

	EXEC [Events_RaiseEvent] 'DBSEXC', @Application_Id, @ScriptExecution_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [DatabaseChangeScripts_LogExecution] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_AddOrRemoveServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Associates or disassociates a server from a specified environment'
GO


IF OBJECT_ID('[Environments_AddOrRemoveServer]') IS NOT NULL DROP PROCEDURE [Environments_AddOrRemoveServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_AddOrRemoveServer]
(
	@Server_Id INT,
	@Environment_Id INT,
	@AddOrRemove_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @AddOrRemove_Code = 'R'
		DELETE [EnvironmentServers]
		 WHERE [Server_Id] = @Server_Id
		   AND [Environment_Id] = @Environment_Id

	ELSE IF @AddOrRemove_Code = 'A' 
		BEGIN
			IF NOT EXISTS
				( SELECT * FROM [EnvironmentServers]
				   WHERE [Server_Id] = @Server_Id
					 AND [Environment_Id] = @Environment_Id )
			INSERT INTO [EnvironmentServers] ([Server_Id], [Environment_Id])
			VALUES (@Server_Id, @Environment_Id)
		END
	ELSE
		RAISERROR('"%s" is an invalid AddOrRemove_Code, must be "A" or "R"', 16, 1, @AddOrRemove_Code)

	EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_AddOrRemoveServer] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_AddOrRemoveServerFromGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds or removes a server from a specified server group'
GO


IF OBJECT_ID('[Environments_AddOrRemoveServerFromGroup]') IS NOT NULL DROP PROCEDURE [Environments_AddOrRemoveServerFromGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_AddOrRemoveServerFromGroup]
(
	@Server_Id INT,
	@ServerGroup_Server_Id INT,
	@AddOrRemove_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @AddOrRemove_Code = 'R'
		DELETE [ServerGroups]
		 WHERE [Server_Id] = @Server_Id
		   AND [ServerGroup_Server_Id] = @ServerGroup_Server_Id

	ELSE IF @AddOrRemove_Code = 'A' 
		BEGIN
			IF NOT EXISTS
				( SELECT * FROM [ServerGroups]
				   WHERE [Server_Id] = @Server_Id
				     AND [ServerGroup_Server_Id] = @ServerGroup_Server_Id)
			INSERT INTO [ServerGroups] ([ServerGroup_Server_Id], [Server_Id])
			VALUES (@ServerGroup_Server_Id, @Server_Id)
		END
	ELSE
		RAISERROR('"%s" is an invalid AddOrRemove_Code, must be "A" or "R"', 16, 1, @AddOrRemove_Code)

	EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @ServerGroup_Server_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_AddOrRemoveServerFromGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Environment_Id',
    /* Description_Text        */ 'Creates a new environment or updates an existing one'
GO


IF OBJECT_ID('[Environments_CreateOrUpdateEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_CreateOrUpdateEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateEnvironment]
(
	@Environment_Id INT = NULL OUT,
	@Environment_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Environment_Id IS NULL) BEGIN
		DECLARE @Display_Sequence INT
		 SELECT @Display_Sequence = COALESCE(MAX([Display_Sequence]),0)+1
		   FROM [Environments]
		BEGIN
			INSERT INTO [Environments] 
			(
				[Environment_Name],
				[Display_Sequence],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date]
			)
			VALUES 
			(
				@Environment_Name,
				@Display_Sequence,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE()
			)
			SET @Environment_Id = SCOPE_IDENTITY()
			INSERT INTO [EnvironmentServers] ([Environment_Id],[Server_Id]) VALUES(@Environment_Id,1)

			EXEC [Events_RaiseEvent] 'ENVNEW', NULL, @Environment_Id
		END

	END ELSE
		BEGIN
			UPDATE [Environments]
			   SET [Environment_Name] = @Environment_Name,
				   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
				   [ModifiedOn_Date] = GETUTCDATE()
			 WHERE [Environment_Id] = @Environment_Id

			EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id
		END



	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_CreateOrUpdateServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Server_Id',
    /* Description_Text        */ 'Creates a new server, or updates an existing one.'
GO


IF OBJECT_ID('[Environments_CreateOrUpdateServer]') IS NOT NULL DROP PROCEDURE [Environments_CreateOrUpdateServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_CreateOrUpdateServer]
(
	@Server_Id INT = NULL OUT,
	@Server_Name VARCHAR(50),
	@ServerGroup_Indicator YNINDICATOR,
	@Agent_Configuration VARCHAR(MAX),
	@Active_Indicator YNINDICATOR,
	@EnvironmentIds_Csv VARCHAR(MAX) = NULL,
	@ServerIdsInGroup_Csv VARCHAR(MAX) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@Server_Id IS NULL)
		BEGIN
			INSERT INTO [Servers] 
				(
					[Server_Name],
					[CreatedBy_User_Name],
					[CreatedOn_Date],
					[ModifiedBy_User_Name],
					[ModifiedOn_Date],
					[ServerGroup_Indicator],
					[Agent_Configuration],
					[ServerStatus_Code],
					[Active_Indicator]
				)
			VALUES
				(
					@Server_Name,
					dbo.Session_GetUserName(),
					GETUTCDATE(),
					dbo.Session_GetUserName(),
					GETUTCDATE(),
					@ServerGroup_Indicator,
					@Agent_Configuration,
					'UNKWN',
					@Active_Indicator
				)
			SET @Server_Id = SCOPE_IDENTITY()

			EXEC [Events_RaiseEvent] 'ENVNWS', NULL, @Server_Id
		END

	ELSE
		BEGIN
			UPDATE [Servers]
			   SET [Server_Name] = @Server_Name,
				   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
				   [ModifiedOn_Date] = GETUTCDATE(),
				   [ServerGroup_Indicator] = @ServerGroup_Indicator,
				   [Agent_Configuration] = @Agent_Configuration,
				   [Active_Indicator] = @Active_Indicator
			 WHERE [Server_Id] = @Server_Id

			EXEC [Events_RaiseEvent] 'ENVMDS', NULL, @Server_Id
		END

	IF (@EnvironmentIds_Csv IS NOT NULL) BEGIN
		DELETE [EnvironmentServers] WHERE [Server_Id] = @Server_Id
		INSERT [EnvironmentServers] ([Server_Id], [Environment_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@EnvironmentIds_Csv, ',')
	END

	IF (@ServerIdsInGroup_Csv IS NOT NULL) BEGIN
		DELETE [ServerGroups] WHERE [ServerGroup_Server_Id] = @Server_Id
		INSERT [ServerGroups] ([ServerGroup_Server_Id], [Server_Id])
		SELECT @Server_Id, [Value_Text] FROM dbo.CsvToTable(@ServerIdsInGroup_Csv, ',')
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_CreateOrUpdateServer] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_DeleteEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Soft deletes an environment'
GO

IF OBJECT_ID('[Environments_DeleteEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_DeleteEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_DeleteEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF EXISTS(SELECT * FROM WorkflowSteps WHERE [Environment_Id] = @Environment_Id)
	BEGIN
		RAISERROR('Environment %i is currently associated with a workflow and cannot be removed.', 16, 1, @Environment_Id)
	END

	UPDATE [Environments] 
	   SET [Active_Indicator] = 'N'
	 WHERE [Environment_Id] = @Environment_Id
	 
	EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_DeleteEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_DeleteServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a server'
GO


IF OBJECT_ID('[Environments_DeleteServer]') IS NOT NULL DROP PROCEDURE [Environments_DeleteServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_DeleteServer]
(
	@Server_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Server_Name SQL_VARIANT
	    SET @Server_Name = (SELECT [Server_Name] FROM [Servers] WHERE [Server_Id] = @Server_Id)
    
    UPDATE [PlanActionGroups]
       SET [Server_Id] = NULL
     WHERE [Server_Id] = @Server_Id

	DELETE [EnvironmentServers]
	 WHERE [Server_Id] = @Server_Id
	 
	DELETE [ServerGroups]
	 WHERE [ServerGroup_Server_Id] = @Server_Id
	    OR [Server_Id] = @Server_Id

	UPDATE [ConfigurationFileDeployments]
	   SET [Server_Id] = NULL
	 WHERE [Server_Id] = @Server_Id

	UPDATE [PlanActions]
	   SET [Server_Id] = 1
	 WHERE [Server_Id] = @Server_Id

    UPDATE [PlanActions]
	   SET [Target_Server_Id] = 1
	 WHERE [Target_Server_Id] = @Server_Id

    UPDATE [BuildExecution_PlanActions]
       SET [Server_Id] = NULL
     WHERE [Server_Id] = @Server_Id

	DELETE [Servers]
	 WHERE [Server_Id] = @Server_Id

	IF @@ROWCOUNT>0
	EXEC [Events_RaiseEvent] 'ENVDLS', NULL, @Server_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_DeleteServer] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Environments,Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an environment'
GO


IF OBJECT_ID('[Environments_GetEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT OFF 

	SELECT * 
	  FROM [Environments]
     WHERE [Environment_Id] = @Environment_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Environment %i does not exist.', 16, 1, @Environment_Id)

	SELECT *
	  FROM [Servers]
	 WHERE [Server_Id] IN (SELECT [Server_Id] FROM [EnvironmentServers]
	                        WHERE [Environment_Id] = @Environment_Id)

END
GO

GRANT EXECUTE ON [Environments_GetEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironments',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Environments',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the environments for a specified application, or all environments if no application is provided'
GO

IF OBJECT_ID('[Environments_GetEnvironments]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironments]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironments]
(
	@Application_Id INT = NULL
)
AS
BEGIN

	IF @Application_Id IS NULL
	BEGIN

		/* All environments */
		SELECT E.*
		  FROM [Environments] E
		 WHERE E.[Active_Indicator] = 'Y'
		 ORDER BY [Display_Sequence]

	END
	ELSE
	BEGIN

		/* Only environments for the specified application */
		SELECT E.*
		  FROM [Environments] E
			   JOIN [ApplicationEnvironments_Merged] AEM
				 ON E.[Environment_Id] = AEM.[Environment_Id]
			    AND [Application_Id] = @Application_Id
		WHERE E.[Active_Indicator] = 'Y'	    
		ORDER BY [Display_Sequence]

	END

END
GO

GRANT EXECUTE ON [Environments_GetEnvironments] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetEnvironmentsAndServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Environments,EnvironmentServers_Extended,Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a dataset containing all environments, all environment/server relations, and servers'
GO


IF OBJECT_ID('[Environments_GetEnvironmentsAndServers]') IS NOT NULL DROP PROCEDURE [Environments_GetEnvironmentsAndServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetEnvironmentsAndServers]
(
	@IncludeInactive_Environments_Indicator YNINDICATOR,
	@IncludeInactive_Servers_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [Environments] 
	 WHERE [Active_Indicator] = 'Y' OR @IncludeInactive_Environments_Indicator = 'Y'
	 ORDER BY [Display_Sequence]

	SELECT * FROM [EnvironmentServers_Extended] 
	 WHERE [Environment_Id] IS NULL OR
	       (([Environment_Active_Indicator] = 'Y' OR @IncludeInactive_Environments_Indicator = 'Y') AND
	       ([Server_Active_Indicator] = 'Y' OR @IncludeInactive_Servers_Indicator = 'Y'))
	 ORDER BY [Server_Name]

	SELECT *
	  FROM [Servers]
     WHERE @IncludeInactive_Servers_Indicator = 'Y' OR [Active_Indicator] = 'Y'
  ORDER BY CASE WHEN [Server_Id] = 1 THEN 1 ELSE 2 END,
	       [Server_Name]

END
GO

GRANT EXECUTE ON [Environments_GetEnvironmentsAndServers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetServer',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Servers,Environments,LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specific server including its environments and any log messages.'
GO


IF OBJECT_ID('[Environments_GetServer]') IS NOT NULL DROP PROCEDURE [Environments_GetServer]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetServer]
(
	@Server_Id INT,
	@LogMessage_Count INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
	 WHERE [Server_Id] = @Server_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Server %i does not exist.', 16, 1, @Server_Id)

	SELECT *
	  FROM [Environments]
	 WHERE [Environment_Id] IN (SELECT [Environment_Id] FROM [EnvironmentServers]
	                             WHERE [Server_Id] = @Server_Id)

	IF @LogMessage_Count IS NOT NULL
		SET ROWCOUNT @LogMessage_Count

	SELECT *
	  FROM [LogMessages_Extended]
	 WHERE [Server_Id] = @Server_Id

END
GO

GRANT EXECUTE ON [Environments_GetServer] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetServers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers in the system.'
GO


IF OBJECT_ID('[Environments_GetServers]') IS NOT NULL DROP PROCEDURE [Environments_GetServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetServers]
(
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Servers]
	 WHERE @IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y'
  ORDER BY CASE WHEN [Server_Id] = 1 THEN 1 ELSE 2 END,
	       [Server_Name] 

END
GO

GRANT EXECUTE ON [Environments_GetServers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_GetServersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the servers in a specified server group.'
GO


IF OBJECT_ID('[Environments_GetServersInGroup]') IS NOT NULL DROP PROCEDURE [Environments_GetServersInGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_GetServersInGroup]
(
	@ServerGroup_Server_Id INT,
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT S.*
	  FROM [Servers] S
	  INNER JOIN [ServerGroups] SG
	          ON SG.[Server_Id] = S.[Server_Id]
	 WHERE (SG.[ServerGroup_Server_Id] = @ServerGroup_Server_Id) AND
	       (@IncludeInactive_Indicator = 'Y' OR S.[Active_Indicator] = 'Y')
	 ORDER BY S.[Server_Name]

END
GO

GRANT EXECUTE ON [Environments_GetServersInGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_ResequenceEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Resequences the specified environment by placing it where another environment is sequenced'
GO


IF OBJECT_ID('[Environments_ResequenceEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_ResequenceEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_ResequenceEnvironment]
(
	@Environment_Id INT,
	@New_Environment_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Environment_Sequence INT
	SELECT @Environment_Sequence = [Display_Sequence]
	  FROM [Environments]
	 WHERE [Environment_Id] = @Environment_Id

	IF @Environment_Sequence IS NULL BEGIN
	   RAISERROR('Environment (ID=%i) was not found.', 16, 1, @Environment_Id)
	   RETURN
    END

	UPDATE [Environments] 
	   SET [Display_Sequence] = CASE
	       WHEN /* id */ [Environment_Id] = @Environment_Id THEN @New_Environment_Sequence

		   WHEN /* up */ @New_Environment_Sequence > @Environment_Sequence THEN CASE
		        WHEN [Display_Sequence] BETWEEN @Environment_Sequence AND @New_Environment_Sequence THEN [Display_Sequence] - 1
				ELSE [Display_Sequence] END
		   WHEN /* dn */ @New_Environment_Sequence < @Environment_Sequence THEN CASE
		        WHEN [Display_Sequence] BETWEEN @New_Environment_Sequence AND @Environment_Sequence THEN [Display_Sequence] + 1
				ELSE [Display_Sequence] END
		   ELSE @New_Environment_Sequence END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_ResequenceEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UndeleteEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Undeletes the specified environment'
GO


IF OBJECT_ID('[Environments_UndeleteEnvironment]') IS NOT NULL DROP PROCEDURE [Environments_UndeleteEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UndeleteEnvironment]
(
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [Environments] 
	   SET [Active_Indicator] = 'Y'
	 WHERE [Environment_Id] = @Environment_Id

    EXEC [Events_RaiseEvent] 'ENVMOD', NULL, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Environments_UndeleteEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Environments_UpdateServerStatus',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Environments_UpdateServerStatus]') IS NOT NULL DROP PROCEDURE [Environments_UpdateServerStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Environments_UpdateServerStatus]
(
	@Server_Id INT,
	@ServerStatus_Code CHAR(5)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @Old_ServerStatus_Code CHAR(5)
        
        SELECT @Old_ServerStatus_Code = [ServerStatus_Code]
	      FROM [Servers]
	     WHERE [Server_Id] = @Server_Id

	    UPDATE [Servers]
	       SET [ServerStatus_Code] = @ServerStatus_Code
	     WHERE [Server_Id] = @Server_Id
	     
	     -- raise server error event if the status changes to ERROR from something else
	     IF (@ServerStatus_Code = 'ERROR' AND @Old_ServerStatus_Code <> 'ERROR')
	        EXEC [Events_RaiseEvent] 'SRVERR', NULL, @Server_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Environments_UpdateServerStatus] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurence',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventOccurences_Extended,EventOccurenceDetails',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Get a specific event occurence along with its event details'
GO

IF OBJECT_ID('[Events_GetEventOccurence]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventOccurence]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventOccurence]
(
	@EventOccurence_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [EventOccurences_Extended]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id
	IF @@ROWCOUNT=0 RAISERROR('Event #%i does not exist.', 16,1,@EventOccurence_Id)

	SELECT *
	  FROM [EventOccurenceDetails]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id

END
GO

GRANT EXECUTE ON [Events_GetEventOccurence] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurences',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'EventOccurences_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the event occurences in the system for a specific application, or all applications, limited to a specified count if provided'
GO

IF OBJECT_ID('[Events_GetEventOccurences]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventOccurences]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetEventOccurences]
(
	@Application_Id INT = NULL,
	@Event_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Event_Count IS NOT NULL
		SET ROWCOUNT @Event_Count

	SELECT *
	  FROM [EventOccurences_Extended]
	 WHERE ([Application_Id] = @Application_Id OR @Application_Id IS NULL)
	 ORDER BY [Occurence_Date] DESC

END
GO

GRANT EXECUTE ON [Events_GetEventOccurences] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetNotifiers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventNotifiers,EventOccurenceDetails',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the notifiers that have arisen since the date specified'
GO

IF OBJECT_ID('[Events_GetNotifiers]') IS NOT NULL
	DROP PROCEDURE [Events_GetNotifiers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Events_GetNotifiers]
(
	@From_Date DATETIME
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  INTO #EN
	  FROM [EventNotifiers]
	 WHERE [Occurence_Date] >= @From_Date


	SELECT * FROM #EN

	SELECT *
	  FROM [EventOccurenceDetails]
	  WHERE [EventOccurence_Id] IN (SELECT [EventOccurence_Id] FROM #EN)

END
GO

GRANT EXECUTE ON [Events_GetNotifiers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_AppendActionLog',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Executions_AppendActionLog]') IS NOT NULL DROP PROCEDURE [Executions_AppendActionLog]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_AppendActionLog]
(
	@BuildExecution_PlanAction_Id INT,
	@LogEntry_Level INT,
	@LogEntry_Text VARCHAR(MAX)
)
AS
BEGIN

	INSERT INTO [BuildExecution_PlanActionLogEntries]
		 ([BuildExecution_PlanAction_Id]
		  ,[LogEntry_Sequence]
		  ,[LogEntry_Level]
		  ,[LogEntry_Text])
	SELECT
	       @BuildExecution_PlanAction_Id
		  ,COALESCE(MAX([LogEntry_Sequence]),0) + 1
		  ,@LogEntry_Level
		  ,@LogEntry_Text
	  FROM [BuildExecution_PlanActionLogEntries] 
	  WITH (TABLOCKX) -- added table lock to resolve issue BM-897
	 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	 IF (@LogEntry_Level BETWEEN 20 AND 29)
	 BEGIN

		DECLARE @Execution_Id INT

		SELECT @Execution_Id = [Execution_Id] 
		  FROM [BuildExecution_PlanActions]
		 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

		 UPDATE [BuildExecutions]
		    SET [WarningLogEntry_Indicator] = 'Y'
		  WHERE [Execution_Id] = @Execution_Id

         UPDATE [BuildExecution_PlanActions]
            SET [WarningLogEntry_Indicator] = 'Y'
		  WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	 END

END
GO

GRANT EXECUTE ON [Executions_AppendActionLog] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CancelExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[dbo].[Executions_CancelExecution]') IS NOT NULL
	DROP PROCEDURE [dbo].[Executions_CancelExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Executions_CancelExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	

	IF NOT EXISTS(SELECT *
					FROM [BuildExecutions]
				   WHERE [Execution_Id] = @Execution_Id
					 AND [ExecutionStatus_Name] IN ('Pending', 'Executing'))
	BEGIN
		RAISERROR('This execution is not currently running; only executions in progress may be canceled.', 16, 1)
		RETURN
	END

	/* Pending executions immediately get failed; ones that are currently
	   executing will be canceled and picked up by the Plan Executer */
	UPDATE [BuildExecutions]
	   SET [ExecutionStatus_Name] = 'Canceled'
		  ,[CanceledBy_User_Name] = dbo.Session_GetUserName()
		  ,[CanceledOn_Date] = GETUTCDATE()
		  ,[ExecutionEnded_Date] = GETUTCDATE()
	 WHERE [Execution_Id] = @Execution_Id

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] FROM [BuildExecutions] WHERE [Execution_Id] = @Execution_Id)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Executions_CancelExecution] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_FailExecutingPlans',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_FailExecutingPlans]') IS NOT NULL DROP PROCEDURE [Executions_FailExecutingPlans]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_FailExecutingPlans]
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	EXEC [Events_RaiseEvent] 'EXEFAL', NULL

	/* Fail all promotions */
	UPDATE [BuildPromotions]
	   SET [PromotionStatus_Name] = 'Failed'
	  FROM [BuildPromotions] BP
	       INNER JOIN [BuildExecutions] BE
	               ON BE.Application_Id = BP.Application_Id
	              AND BE.Release_Number = BP.Release_Number
	              AND BE.Build_Number = BP.Build_Number
	              AND BE.Environment_Id = BP.Environment_Id
	 WHERE BE.[ExecutionStatus_Name] = 'Executing'

	/* Fail all actions */
	UPDATE [BuildExecution_PlanActions]
	   SET [ExecutionStatus_Name] = 'Failed'
	 WHERE [ExecutionStatus_Name] = 'Executing'

	/* Fail all executions */
	DECLARE @CUR CURSOR , @Application_Id INT, @Execution_Id INT
	    SET @CUR = CURSOR FOR (SELECT [Application_Id],[Execution_Id] 
	                             FROM [BuildExecutions] WHERE [ExecutionStatus_Name] = 'Executing')
	   OPEN @CUR FETCH @CUR INTO @Application_Id, @Execution_Id
	WHILE @@FETCH_STATUS=0 BEGIN

		UPDATE [BuildExecutions]
		   SET [ExecutionStatus_Name] = 'Failed'
		 WHERE [Execution_Id] = @Execution_Id
	
		EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

		FETCH @CUR INTO @Application_Id, @Execution_Id
	END


	/* Kill all manual actions */
	DELETE [ManualActionsInProgress]


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH		 


END
GO

GRANT EXECUTE ON [Executions_FailExecutingPlans] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetActualServersUsed',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetActualServersUsed]') IS NOT NULL DROP PROCEDURE [Executions_GetActualServersUsed]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetActualServersUsed]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	-- This proc returns the servers that were actually used in an execution.
	-- It is intended for the executer to determine which servers should skip
	-- initialization in a re-execute from last failed action.
	
	SELECT S.*
	  FROM [Servers] S
	 WHERE S.[Server_Id] IN (SELECT DISTINCT BEPA.[Server_Id]
	                           FROM [BuildExecution_PlanActions] BEPA
	                          WHERE BEPA.[Execution_Id] = @Execution_Id)
END
GO

GRANT EXECUTE ON [Executions_GetActualServersUsed] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetExecutionsToExecute',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetExecutionsToExecute]') IS NOT NULL DROP PROCEDURE [Executions_GetExecutionsToExecute]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetExecutionsToExecute]
(
    @Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT BE.*
      FROM [Applications] A
           INNER JOIN [BuildExecutions_Extended] BE
                   ON BE.[Execution_Id] = (SELECT TOP 1 _BE.[Execution_Id] FROM [BuildExecutions] _BE
                                            WHERE _BE.[ExecutionStatus_Name] IN ('Pending', 'Executing', 'Canceled')
                                              AND _BE.[ExecutionStatus_Name] <> 'Failed'
                                              AND _BE.[ExecutionStart_Date] <= GETUTCDATE()
                                              AND _BE.[Application_Id] = BE.[Application_Id]
                                              ORDER BY _BE.[ExecutionStart_Date])
                  AND BE.[Application_Id] = A.[Application_Id]
     WHERE @Application_Id IS NULL OR BE.[Application_Id] = @Application_Id

END
GO

GRANT EXECUTE ON [Executions_GetExecutionsToExecute] TO [BuildMasterUser_Role]

GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetManualActionsInProgress',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetManualActionsInProgress]') IS NOT NULL DROP PROCEDURE [Executions_GetManualActionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetManualActionsInProgress]
(
	@Application_Id INT = NULL,
	@AssignedTo_Principal_Name VARCHAR(255) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ManualActionsInProgress_Extended]
	 WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@AssignedTo_Principal_Name IS NULL OR [AssignedTo_Principal_Name] = @AssignedTo_Principal_Name)
	   AND [ActionComplete_Indicator] = 'N'
	
END
GO

GRANT EXECUTE ON [Executions_GetManualActionsInProgress] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetManualActionsInProgressForUser',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetManualActionsInProgressForUser]') IS NOT NULL DROP PROCEDURE [Executions_GetManualActionsInProgressForUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetManualActionsInProgressForUser]
(
	@Current_User_Name VARCHAR(255),
	@Current_User_Groups_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [ManualActionsInProgress_Extended]
	 WHERE [ActionComplete_Indicator] = 'N'
	   AND [AssignedTo_Principal_Name] IN 
	        (SELECT [Value_text] FROM [dbo].[CsvToTable](@Current_User_Groups_Csv, ',')
             UNION ALL 
             SELECT @Current_User_Name)
	       
	
END
GO

GRANT EXECUTE ON [Executions_GetManualActionsInProgressForUser] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetOrSetManualActionComplete',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Executions_GetOrSetManualActionComplete]') IS NOT NULL DROP PROCEDURE [Executions_GetOrSetManualActionComplete]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetOrSetManualActionComplete]
(
	@BuildExecution_PlanAction_Id INT,
	@ActionComplete_Indicator YNINDICATOR = NULL OUT,
	@CompletedBy_User_Name USERNAME = NULL OUT,
	@AssignedTo_Principal_Name USERNAME = NULL,
	@Task_Name VARCHAR(50),
	@Task_Description TEXT = NULL
)
AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF NOT EXISTS(SELECT * FROM [BuildExecution_PlanActions]
	               WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	                 AND [ExecutionStatus_Name] = 'Executing')
		RAISERROR('Manual Action Complete Indicator cannot be set unless Action is currently executing. %s', 16, 1)
		

	IF NOT EXISTS(SELECT * FROM [ManualActionsInProgress]
	               WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id)
	BEGIN
		INSERT INTO [ManualActionsInProgress]
		(
			[BuildExecution_PlanAction_Id],
			[ActionComplete_Indicator],
			[AssignedTo_Principal_Name],
			[Task_Name],
			[Task_Description]
		)
		VALUES
		(
			@BuildExecution_PlanAction_Id,
			'N',
			@AssignedTo_Principal_Name,
			@Task_Name,
			@Task_Description
		)
	END

	IF @ActionComplete_Indicator IS NULL
	BEGIN
		SELECT @ActionComplete_Indicator = [ActionComplete_Indicator],
		       @CompletedBy_User_Name = [CompletedBy_User_Name]
		  FROM [ManualActionsInProgress]
		 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	END
	ELSE
	BEGIN
		UPDATE [ManualActionsInProgress]
		   SET [ActionComplete_Indicator] = @ActionComplete_Indicator,
		       [CompletedBy_User_Name] = @CompletedBy_User_Name
	     WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_GetOrSetManualActionComplete] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetServers',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetServers]') IS NOT NULL DROP PROCEDURE [Executions_GetServers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetServers]
(
	@Execution_Id INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	-- This proc returns the servers that are likely to be used for an execution in the order in which
	-- they are first used. It is intended only to allow the executer to initialize servers in the
	-- background to improve performance for large numbers of servers.

    SELECT _S.*
      FROM [Servers] _S
           INNER JOIN (
                       SELECT S.[Server_Id], [Seq1] = MIN(PAG.[PlanActionGroup_Sequence]), [Seq2] = MIN(PA.[Action_Sequence])
                         FROM [BuildExecution_Plans] BEP
                              INNER JOIN [BuildExecutions] BEX
                                      ON BEP.[Execution_Id] = BEX.[Execution_Id]
                              INNER JOIN [Builds] B
                                      ON BEX.[Application_Id] = B.[Application_Id]
                                         AND BEX.[Release_Number] = B.[Release_Number]
                                         AND BEX.[Build_Number] = B.[Build_Number]
                              INNER JOIN [PlanActionGroups] PAG
                                      ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
                              INNER JOIN [Plans] P
                                      ON P.[Plan_Id] = PAG.[ActionGroup_Id]
                              INNER JOIN [PlanActions] PA
                                      ON P.[Plan_Id] = PA.[Plan_Id]
                               LEFT JOIN [PlanActions] PA_T
                                      ON P.[Plan_Id] = PA_T.[Plan_Id]
                               LEFT JOIN [ServerGroups] SG
                                      ON SG.[ServerGroup_Server_Id] = PA.[Server_Id]
                              INNER JOIN [Servers] S
                                      ON COALESCE(PA_T.[Target_Server_Id], SG.[Server_Id], PA.[Server_Id]) = S.[Server_ID]
                        WHERE BEP.[Execution_Id] = @Execution_Id
                              AND PA.[Active_Indicator] = 'Y'
                              AND PAG.[Active_Indicator] = 'Y'
                              AND S.[Active_Indicator] = 'Y'
                        GROUP BY S.[Server_Id]
                      ) _S2
                   ON _S.[Server_Id] = _S2.[Server_Id]
     ORDER BY [Seq1], [Seq2]

END
GO

GRANT EXECUTE ON [Executions_GetServers] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_SkipPlan',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Executions_SkipPlan]') IS NOT NULL
	DROP PROCEDURE [Executions_SkipPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_SkipPlan]
(
	@Execution_Id INT,
	@Original_PlanActionGroup_Id INT
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [BuildExecution_Plans]
	   SET [Skipped_Indicator] = 'Y'
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id

	DELETE [BuildExecution_PlanActionVariableValues]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_PlanAction_Id] FROM [BuildExecution_PlanActions] WHERE [Execution_Id] = @Execution_Id AND [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id)

	DELETE [BuildExecution_PlanActionLogEntries]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_PlanAction_Id] FROM [BuildExecution_PlanActions] WHERE [Execution_Id] = @Execution_Id AND [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id)

	DELETE [BuildExecution_PlanActions]
	 WHERE [Execution_Id] = @Execution_Id
	   AND [Original_PlanActionGroup_Id] = @Original_PlanActionGroup_Id

	/* If there are no actions left to execute */
	IF NOT EXISTS(SELECT * FROM dbo.Executions_GetNextAction(@Execution_Id))
	BEGIN

        EXEC [Executions_CompleteExecution] @Execution_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_SkipPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_ValidateForReexecutionFromLastFailed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Eligible_Indicator,IneligibleReason_Text',
    /* Description_Text        */ 'Indicates whether an execution is eligible to be re-executed from the last failed action'
GO


IF OBJECT_ID('[Executions_ValidateForReexecutionFromLastFailed]') IS NOT NULL DROP PROCEDURE [Executions_ValidateForReexecutionFromLastFailed]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_ValidateForReexecutionFromLastFailed]
(
	@Execution_Id INT,
	@Eligible_Indicator CHAR(1) OUT,
	@IneligibleReason_Text VARCHAR(MAX) OUT
)
AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Application_Id INT
	DECLARE @Release_Number RELEASENUMBER
	DECLARE @Build_Number BUILDNUMBER
	DECLARE @Environment_Id INT
	DECLARE @ExecutionStatus_Name VARCHAR(10)
	DECLARE @ExecutionStart_Date DATETIME
	DECLARE @PlanModifiedSinceLastExecution_Indicator YNINDICATOR

	IF (@Execution_Id IS NULL)
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The specified execution was not found.'
		RETURN
	END

	SELECT 
			@ExecutionStatus_Name = [ExecutionStatus_Name]
		   ,@ExecutionStart_Date = [ExecutionStart_Date]
		   ,@Application_Id = [Application_Id]
		   ,@Release_Number = [Release_Number]
		   ,@Build_Number = [Build_Number]
		   ,@Environment_Id = [Environment_Id]
		   ,@PlanModifiedSinceLastExecution_Indicator = [PlanModifiedSinceLastExecution_Indicator]
      FROM [BuildExecutions] 
	 WHERE [Execution_Id] = @Execution_Id

	IF (@PlanModifiedSinceLastExecution_Indicator = 'Y')
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The specified execution is marked ineligible for re-execution because the deployment plan has been modified since the last execution.'
		RETURN
	END

	DECLARE @LatestExecution_Id INT
	SELECT TOP 1 
		@LatestExecution_Id = [Execution_Id]
      FROM [BuildExecutions] 
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @Environment_Id
  ORDER BY [Execution_Id] DESC

	IF (@Execution_Id <> @LatestExecution_Id)
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The execution must be the latest execution for the specified build and environment.'
		RETURN
	END

	IF (@ExecutionStart_Date > GETUTCDATE())
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The execution cannot be scheduled for a future date and time.'
		RETURN
	END

	IF (@ExecutionStatus_Name <> 'Failed')
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The latest execution''s status is not "Failed".'
		RETURN
	END

	IF (EXISTS(
				SELECT * 
				  FROM [Builds] 
				 WHERE [Application_Id] = @Application_Id 
				   AND [Release_Number] = @Release_Number 
				   AND [Build_Number] = @Build_Number
				   AND [BuildStatus_Name] <> 'Active'))
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The associated build is not active.'
		RETURN
	END

	IF (NOT EXISTS(
				SELECT * 
				  FROM [BuildExecution_PlanActions] BEPA
			INNER JOIN [PlanActionGroups] PAG
					ON PAG.[PlanActionGroup_Id] = BEPA.[Original_PlanActionGroup_Id]
			INNER JOIN [PlanActions] PA
					ON PA.[Plan_Id] = PAG.[ActionGroup_Id]
					AND PA.[Action_Sequence] = BEPA.[Original_Action_Sequence]
				  WHERE BEPA.[Execution_Id] = @Execution_Id
					AND BEPA.[ExecutionStatus_Name] = 'Failed'
					AND PA.[ResumeNextOnFailure_Indicator] = 'N'))
	BEGIN
		SET @Eligible_Indicator = 'N'
		SET @IneligibleReason_Text = 'The only failed actions were marked "Resume Next On Failure".'
		RETURN
	END
		
	/* Validates */
	SET @Eligible_Indicator = 'Y'
	SET @IneligibleReason_Text = ''
	RETURN
	
END
GO

GRANT EXECUTE ON [Executions_ValidateForReexecutionFromLastFailed] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_DeleteConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an extension configuration'
GO

IF OBJECT_ID('[ExtensionConfiguration_DeleteConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_DeleteConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_DeleteConfiguration] 
(
	@ExtensionConfiguration_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DECLARE @QualifiedType_Name VARCHAR(500)
		    SET @QualifiedType_Name = (SELECT [QualifiedType_Name]
		                             FROM [ExtensionConfigurations]
		                            WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id)
		IF @@ROWCOUNT = 0 RETURN

        UPDATE [PlanActions]
           SET [ExtensionConfiguration_Id] = NULL
         WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		DELETE [ExtensionConfigurations]
		 WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

		EXEC [Events_RaiseEvent] 'EXTCFG', NULL, @QualifiedType_Name 

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_DeleteConfiguration] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an extension configurer for extension with the specified name'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfiguration] 
(
	@QualifiedType_Name VARCHAR(500),
	@Profile_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE [QualifiedType_Name] = @QualifiedType_Name
	   AND ([Profile_Name] = @Profile_Name OR [Default_Indicator] = 'Y')


END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfiguration] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfigurationById',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of an extension configurer for extension with the specified ID'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfigurationById]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfigurationById]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfigurationById] 
(
	@ExtensionConfiguration_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE [ExtensionConfiguration_Id]  = @ExtensionConfiguration_Id 

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfigurationById] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_GetConfigurations',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ExtensionConfigurations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details for all extension configurers in the system, or a specific one if specified'
GO

IF OBJECT_ID('[ExtensionConfiguration_GetConfigurations]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_GetConfigurations]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_GetConfigurations] 
(
	@QualifiedType_Name VARCHAR(500) = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [ExtensionConfigurations]
	 WHERE @QualifiedType_Name IS NULL OR [QualifiedType_Name] = @QualifiedType_Name
  ORDER BY [Default_Indicator] DESC, [Profile_Name] ASC


END
GO

GRANT EXECUTE ON [ExtensionConfiguration_GetConfigurations] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'ExtensionConfiguration_SaveConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ExtensionConfiguration_Id',
    /* Description_Text        */ 'Creates an extension configurer in the system, or updates an existing one'
GO

IF OBJECT_ID('[ExtensionConfiguration_SaveConfiguration]') IS NOT NULL DROP PROCEDURE [ExtensionConfiguration_SaveConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ExtensionConfiguration_SaveConfiguration] 
(
	@ExtensionConfiguration_Id INT OUT,

	@QualifiedType_Name VARCHAR(500),
	@Profile_Name VARCHAR(50),

	@Extension_Configuration VARCHAR(MAX),
	@Default_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
		
		IF (@ExtensionConfiguration_Id IS NULL)
		BEGIN

			INSERT INTO [ExtensionConfigurations]
					   ([QualifiedType_Name]
					   ,[Profile_Name]
					   ,[Extension_Configuration]
					   ,[CreatedBy_User_Name]
					   ,[CreatedOn_Date]
					   ,[ModifiedBy_User_Name]
					   ,[ModifiedOn_Date]
					   ,[Default_Indicator])
				 VALUES
					   (@QualifiedType_Name
					   ,@Profile_Name
					   ,@Extension_Configuration
					   ,dbo.Session_GetUserName()
					   ,GETUTCDATE()
					   ,dbo.Session_GetUserName()
					   ,GETUTCDATE()
					   ,@Default_Indicator)
			SET @ExtensionConfiguration_Id = SCOPE_IDENTITY()

		END ELSE BEGIN

			UPDATE [ExtensionConfigurations]
			   SET [QualifiedType_Name] = @QualifiedType_Name
			      ,[Profile_Name] = @Profile_Name
			      ,[Extension_Configuration] = @Extension_Configuration
			      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
			      ,[ModifiedOn_Date] = GETUTCDATE()
			      ,[Default_Indicator] = @Default_Indicator
			 WHERE [ExtensionConfiguration_Id] = @ExtensionConfiguration_Id

             IF (@Default_Indicator = 'Y')
             BEGIN 
                
                UPDATE [ExtensionConfigurations]
                   SET [Default_Indicator] = 'N'
                 WHERE [QualifiedType_Name] = @QualifiedType_Name
                   AND [ExtensionConfiguration_Id] <> @ExtensionConfiguration_Id

             END

		END

		/* Raise Extension Configured event */
		EXEC [Events_RaiseEvent] 'EXTCFG', NULL, @QualifiedType_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ExtensionConfiguration_SaveConfiguration] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Features_GetFeatureLevel',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Features_GetFeatureLevel]') IS NOT NULL DROP PROCEDURE [Features_GetFeatureLevel]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Features_GetFeatureLevel] (
	@Feature_Code CHAR(3),
	@Application_Id INT = NULL,
	@Feature_Level INT OUT
) AS BEGIN

	SELECT TOP 1 @Feature_Level = [Feature_Level]
	  FROM [FeatureLevels]
	 WHERE ([Application_Id] IS NULL OR [Application_Id] = @Application_Id)
	   AND ([Feature_Code] IS NULL OR [Feature_Code] = @Feature_Code)
	 ORDER BY CASE WHEN [Feature_Code] IS NOT NULL AND [Application_Id] IS NOT NULL THEN 3
				   WHEN [Application_Id] IS NOT NULL THEN 2
				   WHEN [Feature_Code] IS NOT NULL THEN 1
				   ELSE 0 END DESC

END
GO

GRANT EXECUTE ON [Features_GetFeatureLevel] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Features_GetFeatureLevels',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Features_GetFeatureLevels]') IS NOT NULL DROP PROCEDURE [Features_GetFeatureLevels]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Features_GetFeatureLevels]
AS BEGIN

	SELECT * FROM [FeatureLevels_Extended]

END
GO

GRANT EXECUTE ON [Features_GetFeatureLevels] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Features_SetFeatureLevel',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Features_SetFeatureLevel]') IS NOT NULL DROP PROCEDURE [Features_SetFeatureLevel]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Features_SetFeatureLevel] (
	@Feature_Code CHAR(3),
	@Application_Id INT,
	@Feature_Level INT
) AS BEGIN

	DELETE [FeatureLevels]
     WHERE (   (@Feature_Code IS NULL AND [Feature_Code] IS NULL)
            OR (@Feature_Code = [Feature_Code]) )
       AND (   (@Application_Id IS NULL AND [Application_Id] IS NULL)
            OR (@Application_Id = [Application_Id]) )
	
	IF @Feature_Level IS NOT NULL
		INSERT INTO [FeatureLevels] ([Feature_Code], [Application_Id], [Feature_Level])
		VALUES (@Feature_Code, @Application_Id, @Feature_Level)

END
GO

GRANT EXECUTE ON [Features_SetFeatureLevel] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_ActivateLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the activation code for a license key.'
GO

IF OBJECT_ID('[LicenseKeys_ActivateLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_ActivateLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_ActivateLicenseKey]
(
	@LicenseKey_Text CHAR(36),
	@ActivationCode_Text VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

IF @ActivationCode_Text IS NULL
BEGIN
	UPDATE [LicenseKeys]
	   SET [ActivationCode_Text] = NULL,
	       [ActivatedBy_User_Name] = NULL,
	       [ActivatedOn_Date] = NULL
	 WHERE [LicenseKey_Text] = @LicenseKey_Text
END
ELSE
BEGIN
	UPDATE [LicenseKeys]
	   SET [ActivationCode_Text] = @ActivationCode_Text,
	       [ActivatedBy_User_Name] = dbo.Session_GetUserName(),
	       [ActivatedOn_Date] = GETUTCDATE()
	 WHERE [LicenseKey_Text] = @LicenseKey_Text
END

END
GO

GRANT EXECUTE ON [LicenseKeys_ActivateLicenseKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_AddLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a new license key.'
GO

IF OBJECT_ID('[LicenseKeys_AddLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_AddLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_AddLicenseKey]
(
	@LicenseKey_Text CHAR(36)
)
AS
BEGIN

SET NOCOUNT ON

INSERT INTO [LicenseKeys]
(
	[LicenseKey_Text],
	[AddedBy_User_Name],
	[AddedOn_Date]
)
VALUES
(
	@LicenseKey_Text,
	dbo.Session_GetUserName(),
	GETUTCDATE()
)

END
GO

GRANT EXECUTE ON [LicenseKeys_AddLicenseKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_DeleteLicenseKey',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a license key.'
GO

IF OBJECT_ID('[LicenseKeys_DeleteLicenseKey]') IS NOT NULL DROP PROCEDURE [LicenseKeys_DeleteLicenseKey]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_DeleteLicenseKey]
(
	@LicenseKey_Text CHAR(36)
)
AS
BEGIN

SET NOCOUNT ON

DELETE [LicenseKeys]
 WHERE [LicenseKey_Text] = @LicenseKey_Text

END
GO

GRANT EXECUTE ON [LicenseKeys_DeleteLicenseKey] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LicenseKeys_GetLicenseKeys',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LicenseKeys',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the license keys.'
GO

IF OBJECT_ID('[LicenseKeys_GetLicenseKeys]') IS NOT NULL DROP PROCEDURE [LicenseKeys_GetLicenseKeys]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LicenseKeys_GetLicenseKeys]
AS
BEGIN

SET NOCOUNT ON
	SELECT *
	  FROM [LicenseKeys]
	 ORDER BY [AddedOn_Date]

END
GO

GRANT EXECUTE ON [LicenseKeys_GetLicenseKeys] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_AddMessage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[LogMessages_AddMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_AddMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_AddMessage]
(
	@Message_Text VARCHAR(MAX),
    @ExceptionStackTrace_Text VARCHAR(MAX) = NULL,
    @Server_Id INT = NULL
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

INSERT INTO [LogMessages]
    ([Read_Indicator] ,[Logged_Date] ,[Message_Text] ,[ExceptionStackTrace_Text] ,[Server_Id])
VALUES
    ('N', GETUTCDATE(), @Message_Text, @ExceptionStackTrace_Text, @Server_Id)

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_AddMessage] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_DeleteMessages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes one or all message from the error log'
GO

IF OBJECT_ID('[LogMessages_DeleteMessages]') IS NOT NULL DROP PROCEDURE [LogMessages_DeleteMessages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_DeleteMessages]
(
	@LogMessage_Id INT = NULL
) AS
BEGIN

    SET NOCOUNT ON

    BEGIN TRY

	    DELETE [LogMessages]
	     WHERE @LogMessage_Id IS NULL
            OR [LogMessage_Id] = @LogMessage_Id

    END TRY BEGIN CATCH
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_DeleteMessages] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetMessage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specified message from the error log'
GO

IF OBJECT_ID('[LogMessages_GetMessage]') IS NOT NULL DROP PROCEDURE [LogMessages_GetMessage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetMessage]
(
	@LogMessage_Id INT
) AS
BEGIN

    SET NOCOUNT ON

    SELECT * FROM [LogMessages_Extended] WHERE @LogMessage_Id = [LogMessage_Id]
	
END
GO

GRANT EXECUTE ON [LogMessages_GetMessage] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_GetMessages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'LogMessages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the messages from the error log that match the specified criteria'
GO

IF OBJECT_ID('[LogMessages_GetMessages]') IS NOT NULL DROP PROCEDURE [LogMessages_GetMessages]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_GetMessages]
(
	@Row_Count INT,
	@From_Date DATETIME = NULL,
    @IncludeReadMessages_Indicator YNINDICATOR = 'N'
) AS
BEGIN

    SET NOCOUNT ON

	IF @Row_Count IS NOT NULL
	    SET ROWCOUNT @Row_Count

    SELECT * FROM [LogMessages_Extended]
     WHERE (@From_Date IS NULL OR [Logged_Date] > @From_Date)
       AND (COALESCE(@IncludeReadMessages_Indicator, 'N') = 'Y' OR [Read_Indicator] = 'N')
     ORDER BY [Logged_Date] ASC

END
GO

GRANT EXECUTE ON [LogMessages_GetMessages] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'LogMessages_SetReadIndicator',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[LogMessages_SetReadIndicator]') IS NOT NULL DROP PROCEDURE [LogMessages_SetReadIndicator]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [LogMessages_SetReadIndicator]
(
	@LogMessage_Id INT,
    @Read_Indicator YNINDICATOR
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

UPDATE [LogMessages] SET [Read_Indicator] = @Read_Indicator
 WHERE [LogMessage_Id] = @LogMessage_Id

END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [LogMessages_SetReadIndicator] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_CreateOrUpdateNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new notifier, or updates an existing one'
GO

IF OBJECT_ID('[Notifiers_CreateOrUpdateNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_CreateOrUpdateNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_CreateOrUpdateNotifier]
(
	@Notifier_Id INT = NULL,
	@Notifier_Name VARCHAR(50),
	@Notifier_Description VARCHAR(MAX),
	@Notifier_Configuration VARCHAR(MAX),
	@Owner_User_Name USERNAME,
	@Active_Indicator YNINDICATOR,
	@EventCodeList_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Notifier_Id IS NULL BEGIN
		INSERT INTO [Notifiers]
				   ([Notifier_Name]
				   ,[Notifier_Description]
				   ,[Notifier_Configuration]
				   ,[Owner_User_Name]
				   ,[Active_Indicator])
			 VALUES
				   (@Notifier_Name
				   ,@Notifier_Description
				   ,@Notifier_Configuration
				   ,@Owner_User_Name
				   ,@Active_Indicator)
		SET @Notifier_Id = SCOPE_IDENTITY()
	END ELSE BEGIN
		UPDATE [Notifiers] SET
		   [Notifier_Name]=@Notifier_Name
		   ,[Notifier_Description] = @Notifier_Description
		   ,[Notifier_Configuration] = @Notifier_Configuration
		   ,[Active_Indicator] = @Active_Indicator
		WHERE [Notifier_Id] = @Notifier_Id
		IF @@ROWCOUNT=0 RAISERROR('Notifier #%i does not exist.',16,1,@Notifier_Id)

		DELETE [NotifierEventCodes] 
		 WHERE [Notifier_Id] = @Notifier_Id
	END

	INSERT INTO [NotifierEventCodes] ([Notifier_Id],[Event_Code])
	SELECT @Notifier_Id, V.[Value_Text]
	  FROM dbo.CsvToTable(@EventCodeList_Csv,',') V

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_CreateOrUpdateNotifier] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_DeleteNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a notifier from the system'
GO

IF OBJECT_ID('[Notifiers_DeleteNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_DeleteNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_DeleteNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NotifierEventCodes] WHERE [Notifier_Id] = @Notifier_Id

	DELETE [Notifiers] WHERE [Notifier_Id] = @Notifier_Id

 

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_DeleteNotifier] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Notifiers,NotifierEventCodes',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a notifier and a list of event codes that the notifier is subscribed to'
GO

IF OBJECT_ID('[Notifiers_GetNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifier]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_GetNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Notifier_Id] = @Notifier_Id
	IF @@ROWCOUNT=0 RAISERROR('Notifier %i does not exist.', 16,1,@Notifier_Id)

	SELECT * FROM [NotifierEventCodes]
	 WHERE [Notifier_Id] = @Notifier_Id

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifier] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifiers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Notifiers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of notifiers for the specified user, or all users if no user is specified'
GO

IF OBJECT_ID('[Notifiers_GetNotifiers]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifiers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Notifiers_GetNotifiers]
(
	@Owner_User_Name USERNAME = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Owner_User_Name] = COALESCE(@Owner_User_Name,[Owner_User_Name])
	   OR ([Owner_User_Name] IS NULL AND @Owner_User_Name IS NULL)

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifiers] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ChangeActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves an action into a different action group'
GO


IF OBJECT_ID('[Plans_ChangeActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_ChangeActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ChangeActionGroup]
(
	@PlanActionGroup_Id INT,
	@Action_Sequence INT,
	@New_PlanActionGroup_Id INT,
	@New_Action_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @Plan_Id INT
	 SELECT @Plan_Id = [ActionGroup_Id]
	   FROM [PlanActionGroups]
	  WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	 
	DECLARE @New_Plan_Id INT
	 SELECT @New_Plan_Id = [ActionGroup_Id]
	   FROM [PlanActionGroups]
	  WHERE [PlanActionGroup_Id] = @New_PlanActionGroup_Id

	BEGIN TRANSACTION
	
	IF (@Plan_Id = @New_Plan_Id)
	    RAISERROR('Actions cannot be moved to the same plan, use Plans_ResequenceAction instead.', 16, 1)

	IF EXISTS(SELECT *
			    FROM [BuildExecution_Plans] BEP
			    JOIN [BuildExecutions] BE
				  ON BEP.[Execution_Id] = BE.[Execution_Id]
		  INNER JOIN [PlanActionGroups] PAG
				  ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
			   WHERE PAG.[ActionGroup_Id] = @Plan_Id
				 AND BE.[ExecutionStatus_Name] IN ('Pending','Executing'))
		RAISERROR('Plan %i is pending or executing and cannot be edited.', 16, 1, @Plan_Id)
	
	
   DECLARE @Last_Action_Sequence INT
	SELECT @Last_Action_Sequence = COALESCE(MAX([Action_Sequence]), 1)
	  FROM [PlanActions]
	 WHERE [Plan_Id] = @Plan_Id
	 
   DECLARE @Target_Last_Action_Sequence INT
	SELECT @Target_Last_Action_Sequence = COALESCE(MAX([Action_Sequence]), 0) + 1
      FROM [PlanActions]
     WHERE [Plan_Id] = @New_Plan_Id
	
	
	-- move action to end of list to appease trigger
	EXEC [Plans_ResequenceAction] @PlanActionGroup_Id, @Action_Sequence, @Last_Action_Sequence
    
    -- move the action to the end of its new action group 
     UPDATE [PlanActions]
        SET [Plan_Id] = @New_Plan_Id,
            [Action_Sequence] = @Target_Last_Action_Sequence
      WHERE [Plan_Id] = @Plan_Id
	    AND [Action_Sequence] = @Last_Action_Sequence
	   
	-- move the action from the end of its action group to its new sequence
	EXEC [Plans_ResequenceAction] @New_PlanActionGroup_Id, @Target_Last_Action_Sequence, @New_Action_Sequence


    DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] FROM [PlanActionGroups] WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id)
	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @PlanActionGroup_Id

	DECLARE @Environment_Id INT
	SELECT @Environment_Id = [Environment_Id]
	  FROM [PlanActionGroups]
	 WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

	 EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ChangeActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ChangeActionGroupEnvironment',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves a plan action group to the final position of a deployment plan in a new environment'
GO


IF OBJECT_ID('[Plans_ChangeActionGroupEnvironment]') IS NOT NULL DROP PROCEDURE [Plans_ChangeActionGroupEnvironment]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ChangeActionGroupEnvironment]
(
	@PlanActionGroup_Id INT,
	@New_Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

    DECLARE @PlanActionGroup_Sequence INT
    DECLARE @Plan_Id INT
	DECLARE @Application_Id INT
	DECLARE @Environment_Id INT
	
	SELECT @Application_Id = [Application_Id],
	       @Environment_Id = [Environment_Id],
	       @Plan_Id = [ActionGroup_Id],
	       @PlanActionGroup_Sequence = [PlanActionGroup_Sequence]
	  FROM [PlanActionGroups]
	 WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
			    FROM [BuildExecution_Plans] BEP
			    JOIN [BuildExecutions] BE
				  ON BEP.[Execution_Id] = BE.[Execution_Id]
		  INNER JOIN [PlanActionGroups] PAG
				  ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
			   WHERE PAG.[ActionGroup_Id] = @Plan_Id
				 AND BE.[ExecutionStatus_Name] IN ('Pending','Executing'))
		RAISERROR('Plan %i is pending or executing and cannot be edited.', 16, 1, @Plan_Id)

    DECLARE @Last_PlanActionGroup_Sequence INT
    SELECT @Last_PlanActionGroup_Sequence = MAX([PlanActionGroup_Sequence])
      FROM [PlanActionGroups]
     WHERE [Application_Id] = @Application_Id
       AND [Environment_Id] = @Environment_Id

    -- move action group to the end of the sequence in the current environment to appease trigger
    EXEC [Plans_ResequenceActionGroup] @PlanActionGroup_Id, @Last_PlanActionGroup_Sequence  

    -- move action group to the end of the sequence of action groups in the new environment
    UPDATE [PlanActionGroups]
	   SET [PlanActionGroup_Sequence] = (SELECT COALESCE(MAX([PlanActionGroup_Sequence]), 0) 
	                                       FROM [PlanActionGroups] 
	                                      WHERE [Application_Id] = @Application_Id 
	                                        AND [Environment_Id] = @New_Environment_Id) + 1,
	       [Environment_Id] = @New_Environment_Id
	 WHERE [PlanActionGroup_Sequence] = @Last_PlanActionGroup_Sequence
	   AND [Application_Id] = @Application_Id
	   AND [Environment_Id] = @Environment_Id

	  
	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @Plan_Id

	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id
	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @New_Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ChangeActionGroupEnvironment] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreateOrUpdateAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Action_Sequence',
    /* Description_Text        */ 'Creates an action in the specified action group, or updates an existing one'
GO


IF OBJECT_ID('[Plans_CreateOrUpdateAction]') IS NOT NULL DROP PROCEDURE [Plans_CreateOrUpdateAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreateOrUpdateAction]
(
	@Plan_Id INT = NULL,
	@Action_Sequence INT = NULL OUT,
	@Server_Id INT = NULL,
	@Server_Variable_Name VARCHAR(50) = NULL,
	@Action_Description VARCHAR(500) = NULL,
	@ResumeNextOnFailure_Indicator YNINDICATOR = NULL,
	@Action_Configuration XML = NULL,
	@ActionType_Name VARCHAR(100) = NULL,
	@Active_Indicator YNINDICATOR = NULL,
	@Retry_Count INT = NULL,
	@LogFailureAsWarning_Indicator YNINDICATOR = NULL,
	@Target_Server_Id INT = NULL,
	@Target_Server_Variable_Name VARCHAR(50) = NULL,
	@ExtensionConfiguration_Id INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @InvalidateLastActionReexecution YNINDICATOR
	SET @InvalidateLastActionReexecution = 'N'

	IF @Plan_Id IS NULL
	BEGIN
		RAISERROR('Plad_Id must be specified.', 16, 1)
	END

	IF EXISTS(SELECT *
			    FROM [PlanActionGroups] PAG
			         INNER JOIN [BuildExecution_Plans] BEP
			                 ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
			         INNER JOIN [BuildExecutions] BE
				  ON BEP.[Execution_Id] = BE.[Execution_Id]
			   WHERE PAG.[ActionGroup_Id] = @Plan_Id
				 AND BE.[ExecutionStatus_Name] IN ('Pending', 'Executing'))
	BEGIN
		RAISERROR('Plan %i is pending or executing and cannot be edited.', 16, 1, @Plan_Id)
	END

	IF (@Action_Sequence IS NULL)
	BEGIN
		SET @Action_Sequence =
			(SELECT COALESCE(MAX([Action_Sequence]), 0) + 1
		       FROM [PlanActions] WHERE [Plan_Id] = @Plan_Id)

		INSERT INTO [PlanActions]
		(
			[Plan_Id],
			[Action_Sequence],
			[Server_Id],
			[Server_Variable_Name],
			[Action_Description],
			[ResumeNextOnFailure_Indicator],
			[Action_Configuration],
			[ActionType_Name],
			[Active_Indicator],
			[Retry_Count],
			[LogFailureAsWarning_Indicator],
			[ExtensionConfiguration_Id],
			[Target_Server_Id],
			[Target_Server_Variable_Name]
		)
		VALUES
		(
			@Plan_Id,
			@Action_Sequence,
			@Server_Id,
			@Server_Variable_Name,
			@Action_Description,
			COALESCE(@ResumeNextOnFailure_Indicator, 'N'),
			@Action_Configuration,
			@ActionType_Name,
			COALESCE(@Active_Indicator, 'Y'),
			COALESCE(@Retry_Count, 0),
			COALESCE(@LogFailureAsWarning_Indicator, 'N'),
			@ExtensionConfiguration_Id,
			@Target_Server_Id,
			@Target_Server_Variable_Name
		)

		SET @InvalidateLastActionReexecution = 'Y'
	END
	ELSE
	BEGIN
		DECLARE @Previous_Active_Indicator YNINDICATOR
		DECLARE @Previous_ResumeNextOnFailure_Indicator YNINDICATOR
		SELECT @Previous_Active_Indicator = [Active_Indicator]
		FROM [PlanActions]
		WHERE [Plan_Id] = @Plan_Id AND [Action_Sequence] = @Action_Sequence

		UPDATE [PlanActions]
		   SET [Server_Id] = @Server_Id
		      ,[Server_Variable_Name] = @Server_Variable_Name
			  ,[Action_Description] = COALESCE(@Action_Description, [Action_Description])
		      ,[Action_Configuration] = COALESCE(@Action_Configuration, [Action_Configuration])
		      ,[ResumeNextOnFailure_Indicator] = COALESCE(@ResumeNextOnFailure_Indicator, [ResumeNextOnFailure_Indicator])
			  ,[ActionType_Name] = COALESCE(@ActionType_Name, [ActionType_Name])
			  ,[Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator])
			  ,[Retry_Count] = COALESCE(@Retry_Count, [Retry_Count])
			  ,[LogFailureAsWarning_Indicator] = COALESCE(@LogFailureAsWarning_Indicator, [LogFailureAsWarning_Indicator])
			  ,[Target_Server_Id] = @Target_Server_Id
			  ,[Target_Server_Variable_Name] = @Target_Server_Variable_Name
			  ,[ExtensionConfiguration_Id] = @ExtensionConfiguration_Id
		 WHERE [Plan_Id] = @Plan_Id AND [Action_Sequence] = @Action_Sequence

		 IF (@Previous_Active_Indicator <> @Active_Indicator
			 OR @Previous_ResumeNextOnFailure_Indicator <> @ResumeNextOnFailure_Indicator)
			SET @InvalidateLastActionReexecution = 'Y'
	END

	DECLARE @PAG TABLE ([Application_Id] INT)
	INSERT INTO @PAG SELECT [Application_Id] FROM [PlanActionGroups] WHERE [ActionGroup_Id] = @Plan_Id
	WHILE ((SELECT COUNT(*) FROM @PAG) > 0) BEGIN
	DECLARE @Application_Id INT
		SET @Application_Id = (SELECT TOP 1 [Application_Id] FROM @PAG)
		DELETE @PAG WHERE [Application_Id] = @Application_Id
		
	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @Plan_Id
	END

	UPDATE [Plans]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [Plan_Id] = @Plan_Id

	IF (@InvalidateLastActionReexecution = 'Y')
	BEGIN
		DECLARE @PAG2 TABLE ([Application_Id] INT, [Environment_Id] INT)
		INSERT INTO @PAG2 SELECT [Application_Id], [Environment_Id] FROM [PlanActionGroups] WHERE [ActionGroup_Id] = @Plan_Id
		WHILE ((SELECT COUNT(*) FROM @PAG2) > 0) BEGIN
			DECLARE @Application_Id2 INT
			DECLARE @Environment_Id INT
			SET @Application_Id2 = (SELECT TOP 1 [Application_Id] FROM @PAG2)
			SET @Environment_Id = (SELECT TOP 1 [Environment_Id] FROM @PAG2)
			DELETE @PAG2 WHERE [Application_Id] = @Application_Id2 AND [Environment_Id] = @Environment_Id
		
			EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id2, @Environment_Id = @Environment_Id
		END
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreateOrUpdateAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_CreatePlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'PlanActionGroup_Id,ActionGroup_Id',
    /* Description_Text        */ 'Creates a new action group for the specified environment of an application'
GO


IF OBJECT_ID('[Plans_CreatePlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_CreatePlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_CreatePlanActionGroup]
(
	@Deployable_Id INT,
	@Environment_Id INT,
	@Application_Id INT,
	@PlanActionGroup_Sequence INT = NULL,
	@Server_Id INT = NULL,
	@Server_Variable_Name VARCHAR(50) = NULL,

	@Active_Indicator YNINDICATOR = NULL,
	@ActionGroup_Name VARCHAR(50) = NULL,
	@ActionGroup_Description VARCHAR(MAX) = NULL,
	@Predicate_Configuration XML = NULL,

	@PlanActionGroup_Id INT = NULL OUT,
	@ActionGroup_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	--If sequence is not specified, create at the end
	IF @PlanActionGroup_Sequence IS NULL BEGIN
		SET @PlanActionGroup_Sequence = COALESCE((SELECT MAX(PAG.[PlanActionGroup_Sequence])
		                                            FROM [PlanActionGroups] PAG
		                                            WHERE PAG.[Application_Id] = @Application_Id
		                                              AND PAG.[Environment_Id] = @Environment_Id), 0) + 1
	END

	IF @ActionGroup_Id IS NULL BEGIN

		INSERT INTO [Plans]
			([CreatedBy_User_Name]
			,[CreatedOn_Date]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[ActionGroup_Name]
			,[Predicate_Configuration]
			,[ActionGroup_Description]
			,[Server_Id]
			,[Server_Variable_Name])
		VALUES
           (dbo.Session_GetUserName()
			,GETUTCDATE()
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@ActionGroup_Name
			,@Predicate_Configuration
			,@ActionGroup_Description
			,@Server_Id
			,NULLIF(@Server_Variable_Name, ''))

		SET @ActionGroup_Id = SCOPE_IDENTITY()

		INSERT INTO [PlanActionGroups]
			([ActionGroup_Id]
			,[Application_Id]
			,[Environment_Id]
			,[PlanActionGroup_Sequence]
			,[Deployable_Id]
			,[Active_Indicator]
			,[ActionGroup_Name]
			,[ActionGroup_Description]
			,[Predicate_Configuration]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Server_Id]
			,[Server_Variable_Name])
	     VALUES
			(@ActionGroup_Id
			,@Application_Id
			,@Environment_Id
			,@PlanActionGroup_Sequence
			,@Deployable_Id
			,COALESCE(@Active_Indicator, 'Y')
			,NULL
			,NULL
			,NULL
			,NULL
			,NULL
			,NULL
			,NULL)

		SET @PlanActionGroup_Id = SCOPE_IDENTITY()
	END
	ELSE BEGIN
		INSERT INTO [PlanActionGroups]
			([ActionGroup_Id]
			,[Application_Id]
			,[Environment_Id]
			,[PlanActionGroup_Sequence]
			,[Deployable_Id]
			,[Active_Indicator]
			,[ActionGroup_Name]
			,[ActionGroup_Description]
			,[Predicate_Configuration]
			,[ModifiedBy_User_Name]
			,[ModifiedOn_Date]
			,[Server_Id]
			,[Server_Variable_Name])
		 VALUES
			(@ActionGroup_Id
			,@Application_Id
			,@Environment_Id
			,@PlanActionGroup_Sequence
			,@Deployable_Id
			,@Active_Indicator
			,NULLIF(@ActionGroup_Name, '')
			,NULLIF(@ActionGroup_Description, '')
			,@Predicate_Configuration
			,dbo.Session_GetUserName()
			,GETUTCDATE()
			,@Server_Id
			,NULLIF(@Server_Variable_Name, ''))

		SET @PlanActionGroup_Id = SCOPE_IDENTITY()
	END

	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @ActionGroup_Id  	  

	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_CreatePlanActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeletePlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified action group.'
GO

IF OBJECT_ID('[Plans_DeletePlanActionGroup]') IS NOT NULL
	DROP PROCEDURE [Plans_DeletePlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeletePlanActionGroup]
(
	@PlanActionGroup_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ActionGroup_Id INT
	DECLARE @Application_Id INT
	DECLARE @PlanActionGroup_Sequence INT
	DECLARE @Environment_Id INT
	 SELECT @Application_Id = [Application_Id],
	        @ActionGroup_Id = [ActionGroup_Id],
	        @PlanActionGroup_Sequence = [PlanActionGroup_Sequence],
	        @Environment_Id = [Environment_Id]
	   FROM [PlanActionGroups]
	  WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

	DECLARE @Inst_Count INT
	SELECT @Inst_Count = COUNT(*)
	  FROM [PlanActionGroups]
	 WHERE [ActionGroup_Id] = @ActionGroup_Id

	-- move plan action group to the end of the sequence before deletion to appease triggers
	DECLARE @New_PlanActionGroup_Sequence INT
	 SELECT @New_PlanActionGroup_Sequence = MAX([PlanActionGroup_Sequence])
	   FROM [PlanActionGroups]
	  WHERE [Application_Id] = @Application_Id
	    AND [Environment_Id] = @Environment_Id

	UPDATE [PlanActionGroups]
	   SET [PlanActionGroup_Sequence] = CASE
	          WHEN [PlanActionGroups].[PlanActionGroup_Sequence] = CUR.[PlanActionGroup_Sequence] 
	               THEN NEW.[PlanActionGroup_Sequence]
	          WHEN [PlanActionGroups].[PlanActionGroup_Sequence] BETWEEN CUR.[PlanActionGroup_Sequence] AND NEW.[PlanActionGroup_Sequence]
	               THEN [PlanActionGroups].[PlanActionGroup_Sequence] - 1
	          WHEN [PlanActionGroups].[PlanActionGroup_Sequence] BETWEEN NEW.[PlanActionGroup_Sequence] AND CUR.[PlanActionGroup_Sequence]
	               THEN [PlanActionGroups].[PlanActionGroup_Sequence] + 1
	          ELSE [PlanActionGroups].[PlanActionGroup_Sequence]
	          END
	  FROM [PlanActionGroups],
	       [PlanActionGroups] CUR,
	       [PlanActionGroups] NEW
	 WHERE CUR.[PlanActionGroup_Sequence] = @PlanActionGroup_Sequence
	   AND NEW.[PlanActionGroup_Sequence] = @New_PlanActionGroup_Sequence
	   AND @PlanActionGroup_Sequence <> @New_PlanActionGroup_Sequence
	   AND [PlanActionGroups].[Application_Id] = @Application_Id
	   AND [PlanActionGroups].[Environment_Id] = @Environment_Id
	   AND CUR.[Application_Id] = @Application_Id
	   AND CUR.[Environment_Id] = @Environment_Id
	   AND NEW.[Application_Id] = @Application_Id
	   AND NEW.[Environment_Id] = @Environment_Id

	-- is this the last PlanActionGroup for the ActionGroup?
	IF @Inst_Count = 1 BEGIN

		DELETE [PlanActions] WHERE [Plan_Id] = @ActionGroup_Id

		-- delete the PlanActionGroup
		DELETE [PlanActionGroups] WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

		-- delete the ActionGroup
		DELETE [Plans] WHERE [Plan_Id] = @ActionGroup_Id

	END ELSE BEGIN
	
		-- if there is only one other instance left, unshare it
		IF @Inst_Count = 2
		BEGIN
			DECLARE @Other_PlanActionGroup_Id INT
			SELECT @Other_PlanActionGroup_Id = [PlanActionGroup_Id]
			  FROM [PlanActionGroups]
			 WHERE [PlanActionGroup_Id] <> @PlanActionGroup_Id
			   AND [ActionGroup_Id] = @ActionGroup_Id

			UPDATE [Plans]
			   SET [ActionGroup_Name] = COALESCE(PAG.[ActionGroup_Name], P.[ActionGroup_Name]),
			       [Predicate_Configuration] = COALESCE(PAG.[Predicate_Configuration], P.[Predicate_Configuration]),
			       [ActionGroup_Description] = COALESCE(PAG.[ActionGroup_Description], P.[ActionGroup_Description]),
			       [Server_Id] = COALESCE(PAG.[Server_Id], P.[Server_Id]),
			       [Server_Variable_Name] = COALESCE(PAG.[Server_Variable_Name], P.[Server_Variable_Name])
			  FROM [Plans] P
			       INNER JOIN [PlanActionGroups] PAG
			               ON PAG.[PlanActionGroup_Id] = @Other_PlanActionGroup_Id
			 WHERE [Plan_Id] = @ActionGroup_Id

			 UPDATE [PlanActionGroups]
			    SET [ActionGroup_Name] = NULL,
			        [ActionGroup_Description] = NULL,
			        [Predicate_Configuration] = NULL,
                    [Server_Id] = NULL,
                    [Server_Variable_Name] = NULL
			  WHERE [PlanActionGroup_Id] = @Other_PlanActionGroup_Id
		END

		-- delete the PlanActionGroup
		DELETE [PlanActionGroups] WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

	END

	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @ActionGroup_Id

	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_DeletePlanActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'PlanActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified action'
GO


IF OBJECT_ID('[Plans_GetAction]') IS NOT NULL DROP PROCEDURE [Plans_GetAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetAction]
(
	@Plan_Id INT,
	@Action_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [PlanActions_Extended]
     WHERE [Plan_Id] = @Plan_Id
	   AND [Action_Sequence] = @Action_Sequence
	IF @@ROWCOUNT=0 RAISERROR('Action %i (Plan #%i) does not exist.',16,1,@Action_Sequence,@Plan_Id)

END
GO

GRANT EXECUTE ON [Plans_GetAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Plans,PlanActions_Extended,PlanUsage_Slim',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets the details of the specified plan, and all the actions associated with the plan.'
GO


IF OBJECT_ID('[Plans_GetActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_GetActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionGroup]
(
	@Plan_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [Plans]
     WHERE [Plan_Id] = @Plan_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Plans %i does not exist.', 16, 1, @Plan_Id)

	SELECT *
	  FROM [PlanActions_Extended]
     WHERE [Plan_Id] = @Plan_Id
	 ORDER BY [Action_Sequence]

	SELECT *
	  FROM [PlanUsage_Slim]
	 WHERE [Plan_Id] = @Plan_Id

END
GO

GRANT EXECUTE ON [Plans_GetActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Plans',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets all action groups.'
GO

IF OBJECT_ID('[Plans_GetActionGroups]') IS NOT NULL DROP PROCEDURE [Plans_GetActionGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionGroups]
AS
BEGIN
	
    SET NOCOUNT ON

    SELECT * FROM [Plans]

END
GO

GRANT EXECUTE ON [Plans_GetActionGroups] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetActionsInProgress',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Plans_GetActionsInProgress]') IS NOT NULL DROP PROCEDURE [Plans_GetActionsInProgress]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetActionsInProgress]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER
)
AS
BEGIN

	SET NOCOUNT ON

    SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

    DECLARE @Execution_Id INT

    SET @Execution_Id =  (SELECT MAX([Execution_Id]) [Execution_Id]
              FROM [BuildExecutions]
             WHERE [Application_Id] = @Application_Id
               AND [Release_Number] = @Release_Number
               AND [Build_Number] = @Build_Number)

	SELECT * 
      FROM [BuildExecutions_ActionsInProgress] 
	 WHERE [Execution_Id] = @Execution_Id
     ORDER BY [PlanExecution_Sequence], [Deployable_Name], [Plan_Id], [Action_Sequence], [Original_Action_Sequence]

END
GO

GRANT EXECUTE ON [Plans_GetActionsInProgress] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetMostCommonActions',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ 'Plans_MostCommonActions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Plans_GetMostCommonActions]') IS NOT NULL DROP PROCEDURE [Plans_GetMostCommonActions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetMostCommonActions]
(
    @MostRecent_Count INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF (@MostRecent_Count IS NOT NULL)
        SET ROWCOUNT @MostRecent_Count

	SELECT *
      FROM [Plans_MostCommonActions]
     ORDER BY [Action_Count] DESC
	 
END
GO

GRANT EXECUTE ON [Plans_GetMostCommonActions] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetPlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'PlanActionGroups_Extended,PlanActions_Extended,PlanUsage_Slim',
    /* OutputPropertyNames_Csv */  NULL,
    /* Description_Text        */ 'Gets the details of the specified plan, and all the actions associated with the plan.'
GO


IF OBJECT_ID('[Plans_GetPlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_GetPlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetPlanActionGroup]
(
	@PlanActionGroup_Id INT
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT *
	  FROM [PlanActionGroups_Extended]
     WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	IF (@@ROWCOUNT = 0) RAISERROR('PlanActionGroup %i does not exist.', 16, 1, @PlanActionGroup_Id)
	
	DECLARE @Plan_Id INT
	SET @Plan_Id = (SELECT [ActionGroup_Id]
	                  FROM [PlanActionGroups]
	                 WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id)

	SELECT *
	  FROM [PlanActions_Extended]
     WHERE [Plan_Id] = @Plan_Id
	 ORDER BY [Action_Sequence]

	SELECT *
	  FROM [PlanUsage_Slim]
	 WHERE [Plan_Id] = @Plan_Id

END
GO

GRANT EXECUTE ON [Plans_GetPlanActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetPlanActionGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'PlanActionGroups_Extended,PlanActions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Returns action groups and actions for an application and optionally an environment.'
GO


IF OBJECT_ID('[Plans_GetPlanActionGroups]') IS NOT NULL DROP PROCEDURE [Plans_GetPlanActionGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetPlanActionGroups]
(
	@Application_Id INT,
	@Environment_Id INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT P.*
	  FROM [PlanActionGroups_Extended] P
	       LEFT JOIN [Deployables] D
	              ON P.[Deployable_Id] = D.[Deployable_Id]
     WHERE COALESCE(P.[Application_Id], D.[Application_Id]) = @Application_Id
           AND (@Environment_Id IS NULL OR P.[Environment_Id] = @Environment_Id)

    SELECT PA.*
      FROM [PlanActions_Extended] PA
           INNER JOIN [Plans] P
                   ON PA.[Plan_Id] = P.[Plan_Id]
     WHERE PA.[Plan_Id] IN (SELECT PAG.[ActionGroup_Id]
                              FROM [PlanActionGroups] PAG
                                   LEFT JOIN [Deployables] D
                                          ON PAG.[Deployable_Id] = D.[Deployable_Id]
                             WHERE COALESCE(PAG.[Application_Id], D.[Application_Id]) = @Application_Id
                                   AND (@Environment_Id IS NULL OR PAG.[Environment_Id] = @Environment_Id))

END
GO

GRANT EXECUTE ON [Plans_GetPlanActionGroups] TO [BuildMasterUser_Role]

GO
--EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
--    /* StoredProc_Name         */ 'Plans_GetPlansInApplications',
--    /* Internal_Indicator      */ 'Y',
--    /* ReturnType_Name         */ NULL,
--    /* DataTableNames_Csv      */ 'Plans_InApplications',
--    /* OutputPropertyNames_Csv */ NULL,
--    /* Description_Text        */ 'Stored procedure for internal use only.'
--GO


IF OBJECT_ID('[Plans_GetPlansInApplications]') IS NOT NULL DROP PROCEDURE [Plans_GetPlansInApplications]
GO

--SET QUOTED_IDENTIFIER ON
--GO

--CREATE PROCEDURE [Plans_GetPlansInApplications]
--(
--	@Application_Id INT,
--	@Environment_Id INT = NULL,
--	@PlanActionGroup_Sequence INT = NULL
--)
--AS
--BEGIN

--SET NOCOUNT ON

--	SELECT *
--	 FROM [Plans_InApplications]
--	 WHERE  (@Application_Id IS NULL OR  [Application_Id] = @Application_Id)
--	   AND (@Environment_Id IS NULL OR [Environment_Id] = @Environment_Id)
--	   AND (@PlanActionGroup_Sequence IS NULL OR [PlanActionGroup_Sequence] = @PlanActionGroup_Sequence)

--  ORDER BY [Workflow_Step_Sequence]
--		  ,[Environment_Name]
--		  ,[PlanActionGroup_Sequence]
--		  ,[Deployable_Name]
--		  ,[PlanActionGroup_Id]
--		  ,[Action_Sequence]
--END
--GO

--GRANT EXECUTE ON [Plans_GetPlansInApplications] TO [BuildMasterUser_Role]
--GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_GetServersForPlan',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Servers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Plans_GetServersForPlan]') IS NOT NULL DROP PROCEDURE [Plans_GetServersForPlan]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_GetServersForPlan]
(
	@Plan_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT S.*
	  FROM [Servers] S
	 WHERE S.[Server_Id] IN (SELECT DISTINCT ES.[Server_Id]
	                                    FROM [EnvironmentServers] ES
	                                         INNER JOIN [PlanActionGroups] PAG
	                                                 ON PAG.[Environment_Id] = ES.[Environment_Id]
	                                         INNER JOIN [Plans] P
	                                                 ON P.[Plan_Id] = PAG.[ActionGroup_Id]
	                                                AND P.[Plan_Id] = @Plan_Id)
END
GO

GRANT EXECUTE ON [Plans_GetServersForPlan] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ResequenceAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of an action within a plan'
GO


IF OBJECT_ID('[Plans_ResequenceAction]') IS NOT NULL DROP PROCEDURE [Plans_ResequenceAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ResequenceAction]
(
	@PlanActionGroup_Id INT,
	@Action_Sequence INT,
	@New_Action_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

	DECLARE @Plan_Id INT
	SELECT @Plan_Id = [ActionGroup_Id]
	  FROM [PlanActionGroups]
	 WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

	BEGIN TRANSACTION

	IF EXISTS(SELECT *
			    FROM [BuildExecution_Plans] BEP
			    JOIN [BuildExecutions] BE
				  ON BEP.[Execution_Id] = BE.[Execution_Id]
		  INNER JOIN [PlanActionGroups] PAG
				  ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
			   WHERE PAG.[ActionGroup_Id] = @Plan_Id
				 AND BE.[ExecutionStatus_Name] IN ('Pending','Executing'))
		RAISERROR('Plan %i is pending or executing and cannot be edited.', 16, 1, @Plan_Id)
		
	UPDATE [PlanActions]
	   SET [Action_Sequence] = CASE
	          WHEN [PlanActions].[Action_Sequence] = CUR.[Action_Sequence] 
	               THEN NEW.[Action_Sequence]
	          WHEN [PlanActions].[Action_Sequence] BETWEEN CUR.[Action_Sequence] AND NEW.[Action_Sequence]
	               THEN [PlanActions].[Action_Sequence] - 1
	          WHEN [PlanActions].[Action_Sequence] BETWEEN NEW.[Action_Sequence] AND CUR.[Action_Sequence]
	               THEN [PlanActions].[Action_Sequence] + 1
	          ELSE [PlanActions].[Action_Sequence]
	          END
	  FROM [PlanActions],
	       [PlanActions] CUR,
	       [PlanActions] NEW
	 WHERE CUR.[Action_Sequence] = @Action_Sequence
	   AND NEW.[Action_Sequence] = @New_Action_Sequence
	   AND @Action_Sequence <> @New_Action_Sequence
	   AND [PlanActions].[Plan_Id] = @Plan_Id
	   AND CUR.[Plan_Id] = @Plan_Id
	   AND NEW.[Plan_Id] = @Plan_Id
	  

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] FROM [PlanActionGroups] WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id)
	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @Plan_Id

	UPDATE [Plans]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [Plan_Id] = @Plan_Id

	DECLARE @Environment_Id INT
	SELECT @Environment_Id = [Environment_Id]
	  FROM [PlanActionGroups]
	 WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ResequenceAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_ResequenceActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of a plan action group'
GO


IF OBJECT_ID('[Plans_ResequenceActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_ResequenceActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_ResequenceActionGroup]
(
	@PlanActionGroup_Id INT,
	@New_PlanActionGroup_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY

    DECLARE @PlanActionGroup_Sequence INT
    DECLARE @ActionGroup_Id INT
	DECLARE @Application_Id INT
	DECLARE @Environment_Id INT
	
	SELECT @Application_Id = [Application_Id],
	       @Environment_Id = [Environment_Id],
	       @ActionGroup_Id = [ActionGroup_Id],
	       @PlanActionGroup_Sequence = [PlanActionGroup_Sequence]
	  FROM [PlanActionGroups]
	 WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
			    FROM [BuildExecution_Plans] BEP
			    JOIN [BuildExecutions] BE
				  ON BEP.[Execution_Id] = BE.[Execution_Id]
		  INNER JOIN [PlanActionGroups] PAG
				  ON PAG.[PlanActionGroup_Id] = BEP.[Original_PlanActionGroup_Id]
			   WHERE PAG.[ActionGroup_Id] = @ActionGroup_Id
				 AND BE.[ExecutionStatus_Name] IN ('Pending','Executing'))
		RAISERROR('Plan %i is pending or executing and cannot be edited.', 16, 1, @ActionGroup_Id)
		
	UPDATE [PlanActionGroups]
	   SET [PlanActionGroup_Sequence] = CASE
	          WHEN [PlanActionGroups].[PlanActionGroup_Sequence] = CUR.[PlanActionGroup_Sequence] 
	               THEN NEW.[PlanActionGroup_Sequence]
	          WHEN [PlanActionGroups].[PlanActionGroup_Sequence] BETWEEN CUR.[PlanActionGroup_Sequence] AND NEW.[PlanActionGroup_Sequence]
	               THEN [PlanActionGroups].[PlanActionGroup_Sequence] - 1
	          WHEN [PlanActionGroups].[PlanActionGroup_Sequence] BETWEEN NEW.[PlanActionGroup_Sequence] AND CUR.[PlanActionGroup_Sequence]
	               THEN [PlanActionGroups].[PlanActionGroup_Sequence] + 1
	          ELSE [PlanActionGroups].[PlanActionGroup_Sequence]
	          END
	  FROM [PlanActionGroups],
	       [PlanActionGroups] CUR,
	       [PlanActionGroups] NEW
	 WHERE CUR.[PlanActionGroup_Sequence] = @PlanActionGroup_Sequence
	   AND NEW.[PlanActionGroup_Sequence] = @New_PlanActionGroup_Sequence
	   AND @PlanActionGroup_Sequence <> @New_PlanActionGroup_Sequence
	   AND [PlanActionGroups].[Application_Id] = @Application_Id
	   AND [PlanActionGroups].[Environment_Id] = @Environment_Id
	   AND CUR.[Application_Id] = @Application_Id
	   AND CUR.[Environment_Id] = @Environment_Id
	   AND NEW.[Application_Id] = @Application_Id
	   AND NEW.[Environment_Id] = @Environment_Id
	  
	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @ActionGroup_Id

	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_ResequenceActionGroup] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdateActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
     /* Description_Text        */ 'Updates the specified action group.'
GO


IF OBJECT_ID('[Plans_UpdateActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_UpdateActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdateActionGroup]
(
	@ActionGroup_Id INT,
	@ActionGroup_Name VARCHAR(50),
	@ActionGroup_Description VARCHAR(MAX),
	@Predicate_Configuration XML,
	@Server_Id INT,
	@Server_Variable_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Old_Predicate_Configuration XML
	SELECT @Old_Predicate_Configuration = [Predicate_Configuration]
	  FROM [Plans]
	 WHERE [Plan_Id] = @ActionGroup_Id

	UPDATE [Plans]
	   SET [ActionGroup_Name] = NULLIF(@ActionGroup_Name, '')
		  ,[ActionGroup_Description] = NULLIF(@ActionGroup_Description, '')
	      ,[Predicate_Configuration] = @Predicate_Configuration
	      ,[ModifiedOn_Date] = GETUTCDATE()
	      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
	      ,[Server_Id] = @Server_Id
	      ,[Server_Variable_Name] = NULLIF(@Server_Variable_Name, '')
	 WHERE [Plan_Id] = @ActionGroup_Id	 

	DECLARE @APP_CUR_TBL TABLE([Application_Id] INT)
	        INSERT INTO @APP_CUR_TBL([Application_Id]) 
	        SELECT DISTINCT [Application_Id] FROM [PlanActionGroups]
	         WHERE [ActionGroup_Id] = @ActionGroup_Id
	WHILE (SELECT COUNT(*) FROM @APP_CUR_TBL) > 0 BEGIN
		DECLARE @Application_Id INT
		SELECT TOP 1 @Application_Id = [Application_Id] FROM @APP_CUR_TBL
		DELETE @APP_CUR_TBL WHERE [Application_Id] = @Application_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @ActionGroup_Id  	  
	END

	-- Invalidate ability of plan action groups to re-execute from last failed action where the plan is not overridden
	DECLARE @PAG TABLE ([PlanActionGroup_Id] INT, [Application_Id] INT, [Environment_Id] INT)
	INSERT INTO @PAG
		SELECT [PlanActionGroup_Id], [Application_Id], [Environment_Id]
		FROM [PlanActionGroups] 
		WHERE [ActionGroup_Id] = @ActionGroup_Id
	WHILE ((SELECT COUNT(*) FROM @PAG) > 0) BEGIN
		DECLARE @PlanActionGroup_Id INT
		DECLARE @Application_Id2 INT
		DECLARE @Environment_Id INT
		SET @PlanActionGroup_Id = (SELECT TOP 1 [PlanActionGroup_Id] FROM @PAG)
		SET @Application_Id2 = (SELECT TOP 1 [Application_Id] FROM @PAG)
		SET @Environment_Id = (SELECT TOP 1 [Environment_Id] FROM @PAG)
		DELETE @PAG WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

		IF (CAST(@Old_Predicate_Configuration AS VARBINARY(MAX)) <> CAST(@Predicate_Configuration AS VARBINARY(MAX)))
			EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id2, @Environment_Id = @Environment_Id
	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdateActionGroup] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_UpdatePlanActionGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
     /* Description_Text        */ 'Updates the specified plan action group.'
GO

IF OBJECT_ID('[Plans_UpdatePlanActionGroup]') IS NOT NULL DROP PROCEDURE [Plans_UpdatePlanActionGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_UpdatePlanActionGroup]
(
	@PlanActionGroup_Id INT,
	@Active_Indicator YNINDICATOR,
	@ActionGroup_Name VARCHAR(50),
	@ActionGroup_Description VARCHAR(MAX),
	@Predicate_Configuration XML,
	@Server_Id INT,
	@Server_Variable_Name VARCHAR(50),
	@Deployable_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    UPDATE [PlanActionGroups]
       SET [Active_Indicator] = @Active_Indicator
          ,[ActionGroup_Name] = NULLIF(@ActionGroup_Name, '')
          ,[ActionGroup_Description] = NULLIF(@ActionGroup_Description, '')
          ,[Predicate_Configuration] = @Predicate_Configuration
          ,[Server_Id] = @Server_Id
          ,[Server_Variable_Name] = NULLIF(@Server_Variable_Name, '')
          ,[Deployable_Id] = @Deployable_Id
          ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
          ,[ModifiedOn_Date] = GETUTCDATE()
     WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id

	DECLARE @Application_Id INT, @ActionGroup_Id INT, @Environment_Id INT
	 SELECT @Application_Id = [Application_Id], @ActionGroup_Id = [ActionGroup_Id], @Environment_Id = [Environment_Id]
	   FROM [PlanActionGroups]
     WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id
	EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @ActionGroup_Id

	EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id, @Environment_Id = @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Plans_UpdatePlanActionGroup] TO [BuildMasterUser_Role]


GO
IF OBJECT_ID('[Plans_UpdatePredicates]') IS NOT NULL DROP PROCEDURE [Plans_UpdatePredicates]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'PromotionRequirements_CreateOrUpdatePromotionRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'PromotionRequirement_Sequence',
    /* Description_Text        */ 'Creates a new promotion requirement, or updates an existing one'
GO

IF OBJECT_ID('[PromotionRequirements_CreateOrUpdatePromotionRequirement]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_CreateOrUpdatePromotionRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [PromotionRequirements_CreateOrUpdatePromotionRequirement]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@PromotionRequirement_Sequence INT = NULL OUT,
	@PromotionRequirement_Name VARCHAR(MAX),
	@PromotionRequirement_Description VARCHAR(MAX),
	@PromotionRequirement_Configuration VARCHAR(MAX)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@PromotionRequirement_Sequence IS NULL) BEGIN

		SET @PromotionRequirement_Sequence =
			(SELECT COALESCE(MAX([PromotionRequirement_Sequence]),0)+1
			   FROM [PromotionRequirements]
			  WHERE [Workflow_Id] = @Workflow_Id
			    AND [Environment_Id] = @Environment_Id)

		INSERT INTO [PromotionRequirements]
				   ([Workflow_Id],[Environment_Id],[PromotionRequirement_Sequence]
				   ,[PromotionRequirement_Name],[PromotionRequirement_Description]
				   ,[PromotionRequirement_Configuration]
				   ,[CreatedBy_User_Name],[CreatedOn_Date]
				   ,[ModifiedBy_User_Name],[ModifiedOn_Date])
			 VALUES
				   (@Workflow_Id,@Environment_Id,@PromotionRequirement_Sequence
				   ,@PromotionRequirement_Name,@PromotionRequirement_Description
				   ,@PromotionRequirement_Configuration
				   ,[dbo].[Session_GetUserName](), GETUTCDATE()
				   ,[dbo].[Session_GetUserName](), GETUTCDATE())

	END ELSE BEGIN

		UPDATE [PromotionRequirements]
		   SET [PromotionRequirement_Name] = @PromotionRequirement_Name
		      ,[PromotionRequirement_Description] = @PromotionRequirement_Description
		      ,[PromotionRequirement_Configuration] = @PromotionRequirement_Configuration
		      ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
		      ,[ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [PromotionRequirement_Sequence] = @PromotionRequirement_Sequence

		IF @@ROWCOUNT=0 RAISERROR('Promotion Requirement %i-%i-%i does not exist.',16,1, @Workflow_Id, @Environment_Id, @PromotionRequirement_Sequence)
		
	END

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [Workflows]
	                            WHERE [Workflow_Id] = @Workflow_Id)
	EXEC [Events_RaiseEvent] 'WKFMDS', @Application_Id, @Workflow_Id, @Environment_Id
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [PromotionRequirements_CreateOrUpdatePromotionRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'PromotionRequirements_DeletePromotionRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a promotion requirement from the system'
GO

IF OBJECT_ID('[PromotionRequirements_DeletePromotionRequirement]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_DeletePromotionRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [PromotionRequirements_DeletePromotionRequirement]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@PromotionRequirement_Sequence INT
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		DELETE [PromotionRequirements]
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [PromotionRequirement_Sequence] = @PromotionRequirement_Sequence

		DECLARE @Application_Id INT
			SET @Application_Id = (SELECT [Application_Id] 
									 FROM [Workflows]
									WHERE [Workflow_Id] = @Workflow_Id)
		EXEC [Events_RaiseEvent] 'WKFMDS', @Application_Id, @Workflow_Id, @Environment_Id
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [PromotionRequirements_DeletePromotionRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'PromotionRequirements_GetPromotionRequirement',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'PromotionRequirements',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified promotion requirement'
GO

IF OBJECT_ID('[PromotionRequirements_GetPromotionRequirement]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_GetPromotionRequirement]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [PromotionRequirements_GetPromotionRequirement]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@PromotionRequirement_Sequence INT
) AS BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [PromotionRequirements]
	 WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id
	   AND [PromotionRequirement_Sequence] = @PromotionRequirement_Sequence
END
GO

GRANT EXECUTE ON [PromotionRequirements_GetPromotionRequirement] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'PromotionRequirements_GetPromotionRequirements',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'PromotionRequirements',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the promotion requirements for the specified environment in a workflow'
GO

IF OBJECT_ID('[PromotionRequirements_GetPromotionRequirements]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_GetPromotionRequirements]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [PromotionRequirements_GetPromotionRequirements]
(
	@Workflow_Id INT,
	@Environment_Id INT
) AS BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [PromotionRequirements]
	 WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id

END
GO

GRANT EXECUTE ON [PromotionRequirements_GetPromotionRequirements] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'PromotionRequirements_GetPromotionRequirementStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'PromotionRequirementStatuses',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the status for the specified state-managed promotion requirement'
GO

IF OBJECT_ID('[PromotionRequirements_GetPromotionRequirementStatus]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_GetPromotionRequirementStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [PromotionRequirements_GetPromotionRequirementStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,

	@Environment_Id INT,
	@PromotionRequirement_Sequence INT
) AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [PromotionRequirementStatuses]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @Environment_Id
	   AND [PromotionRequirement_Sequence] = @PromotionRequirement_Sequence

END
GO

GRANT EXECUTE ON [PromotionRequirements_GetPromotionRequirementStatus] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'PromotionRequirements_SetPromotionRequirementStatus',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the status of a state-managed promotion requirement'
GO

IF OBJECT_ID('[PromotionRequirements_SetPromotionRequirementStatus]') IS NOT NULL 
	DROP PROCEDURE [PromotionRequirements_SetPromotionRequirementStatus]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [PromotionRequirements_SetPromotionRequirementStatus]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,

	@Environment_Id INT,
	@PromotionRequirement_Sequence INT,

	@PromotionRequirementStatus_Code CHAR(1),
	@PromotionRequirementStatus_Description VARCHAR(MAX), 
	@PromotionRequirementState_Configuration VARCHAR(MAX)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [PromotionRequirementStatuses]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @Environment_Id
	   AND [PromotionRequirement_Sequence] = @PromotionRequirement_Sequence

	DECLARE @Workflow_Id INT
	    SET @Workflow_Id = (SELECT [Workflow_Id] FROM [Releases]
	                         WHERE [Application_Id] = @Application_Id
	                           AND [Release_Number] = @Release_Number)

	DECLARE @PromotionRequirement_Name VARCHAR(MAX)
	    SET @PromotionRequirement_Name =
			(SELECT [PromotionRequirement_Name]
			   FROM [PromotionRequirements]
			  WHERE [Workflow_Id] = @Workflow_Id
				AND [Environment_Id] = @Environment_Id
				AND [PromotionRequirement_Sequence] = @PromotionRequirement_Sequence)

	INSERT INTO [PromotionRequirementStatuses]
			   ([Environment_Id],[PromotionRequirement_Sequence]
			   ,[Application_Id],[Release_Number],[Build_Number]
			   ,[PromotionRequirement_Name],[PromotionRequirementStatus_Code]
			   ,[PromotionRequirementStatus_Description],[PromotionRequirementState_Configuration])
	VALUES
			   (@Environment_Id,@PromotionRequirement_Sequence
			   ,@Application_Id,@Release_Number,@Build_Number
			   ,@PromotionRequirement_Name,@PromotionRequirementStatus_Code
			   ,@PromotionRequirementStatus_Description,@PromotionRequirementState_Configuration)
     

	EXEC [Events_RaiseEvent] 'BLDPRU', @Application_Id, @Release_Number, @Build_Number, @Environment_Id, @PromotionRequirement_Sequence

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [PromotionRequirements_SetPromotionRequirementStatus] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_CreateOrUpdateProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Provider_Id',
    /* Description_Text        */ 'Creates a new provider or updates an existing one.'
GO


IF OBJECT_ID('[Providers_CreateOrUpdateProvider]') IS NOT NULL DROP PROCEDURE [Providers_CreateOrUpdateProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_CreateOrUpdateProvider]
(
	@Provider_Id INT = NULL OUT,
	@ProviderType_Code CHAR(1),
	@ExecuteOn_Server_Id INT,
	@Provider_Name VARCHAR(50),
	@Provider_Description VARCHAR(100),
	@Provider_Configuration VARCHAR(MAX),
	@Internal_Indicator YNINDICATOR	 = 'N'
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF (@Provider_Id IS NULL) 
	BEGIN

		IF @ProviderType_Code <> 'B' AND EXISTS(SELECT * FROM [Providers] WHERE [Provider_Name] = @Provider_Name AND [ProviderType_Code] = @ProviderType_Code)
			RAISERROR('A provider with that name already exists.', 16, 1)

		INSERT INTO [Providers]
            (
				[ProviderType_Code],
				[Provider_Name],
				[ExecuteOn_Server_Id],
				[Provider_Description],
				[Provider_Configuration],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
				[Internal_Indicator]
			)
		VALUES
            (
				@ProviderType_Code,
				@Provider_Name,
				@ExecuteOn_Server_Id,
				@Provider_Description,
				@Provider_Configuration,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				@Internal_Indicator
			)
		SET @Provider_Id = SCOPE_IDENTITY()

		IF @ProviderType_Code = 'B' /*DataBase */
			EXEC [Events_RaiseEvent] 'DBSNEW', NULL, @Provider_Id
		ELSE
			EXEC [Events_RaiseEvent] 'PVDNEW', NULL, @Provider_Id
	END
	ELSE
	BEGIN
		IF EXISTS(SELECT * FROM [Providers]
		           WHERE [Provider_Id] = @Provider_Id
		             AND [Internal_Indicator] = 'Y')
		    RAISERROR('A provider marked as internal may not be edited.', 16, 1)

		IF @ProviderType_Code <> 'B' AND EXISTS(SELECT * FROM [Providers] WHERE [Provider_Name] = @Provider_Name AND [Provider_Id] <> @Provider_Id AND [ProviderType_Code] = @ProviderType_Code)
			RAISERROR('A provider with that name already exists.', 16, 1)

		UPDATE [Providers]
		   SET [ProviderType_Code] = @ProviderType_Code,
		       [Provider_Name] = @Provider_Name,
			   [ExecuteOn_Server_Id] = @ExecuteOn_Server_Id,
			   [Provider_Description] = @Provider_Description,
			   [Provider_Configuration] = @Provider_Configuration,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Provider_Id] = @Provider_Id

		IF @ProviderType_Code = 'B' /*DataBase */
			EXEC [Events_RaiseEvent] 'DBSMOD', NULL, @Provider_Id
		ELSE
			EXEC [Events_RaiseEvent] 'PVDMOD', NULL, @Provider_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Providers_CreateOrUpdateProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_DeleteProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified provider'
GO


IF OBJECT_ID('[Providers_DeleteProvider]') IS NOT NULL DROP PROCEDURE [Providers_DeleteProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_DeleteProvider]
(
	@Provider_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ProviderType_Code CHAR(1), @Provider_Name SQL_VARIANT
		SELECT @Provider_Name = [Provider_Name],
			   @ProviderType_Code = [ProviderType_Code]
		  FROM [Providers]
		 WHERE [Provider_Id] = @Provider_Id

	IF @ProviderType_Code = 'B' BEGIN
		RAISERROR('Database providers may not be deleted with this procedure.', 16, 1)
		END

	ELSE IF @ProviderType_Code = 'I' BEGIN
		UPDATE [Applications] 
		   SET [IssueTracking_Provider_Id] = NULL
		 WHERE [IssueTracking_Provider_Id] = @Provider_Id
		END
	
	ELSE IF @ProviderType_Code = 'S' BEGIN
		DELETE [BuildSchedules]
		 WHERE [SourceControl_Provider_Id] = @Provider_Id
		END

	ELSE IF @ProviderType_Code IS NOT NULL BEGIN
		RAISERROR('Deleting Provider type %s is not be supported.', 16, 1, @ProviderType_Code)
		END

	DELETE [Providers]
	 WHERE [Provider_Id] = @Provider_Id

	IF @@ROWCOUNT>0
	EXEC [Events_RaiseEvent] 'PVDDEL', NULL, @Provider_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Providers_DeleteProvider] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_GetProvider',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'Providers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified provider'
GO


IF OBJECT_ID('[Providers_GetProvider]') IS NOT NULL DROP PROCEDURE [Providers_GetProvider]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_GetProvider]
(
	@Provider_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Providers]
	 WHERE [Provider_Id] = @Provider_Id

END
GO

GRANT EXECUTE ON [Providers_GetProvider] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Providers_GetProviders',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Providers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of providers in the system based on the specified criteria'
GO

IF OBJECT_ID('[Providers_GetProviders]') IS NOT NULL DROP PROCEDURE [Providers_GetProviders]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Providers_GetProviders]
(
	@ProviderType_Code CHAR(1) = NULL,
	@Application_Id INT = NULL,
	@Environment_Id INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Providers]
	 WHERE [ProviderType_Code] = COALESCE(@ProviderType_Code, [ProviderType_Code])
	   AND [Internal_Indicator] = 'N'	 
	   AND (
	          @Application_Id IS NULL  
		  OR (@ProviderType_Code = 'B' AND [Provider_Id]
						  IN (SELECT _CSP.[Provider_Id]
								FROM [DatabaseChangeScriptProviders] _CSP
							   WHERE [Application_Id] = COALESCE(@Application_Id, [Application_Id])
								 AND [Environment_Id] = COALESCE(@Environment_Id, [Environment_Id])))
	      OR (@Application_Id IS NOT NULL AND [Provider_Id] 
	                       = (SELECT [IssueTracking_Provider_Id] FROM [Applications]
	                           WHERE [Application_Id] = @Application_Id))
	       )
	 ORDER BY [Provider_Name]
END
GO

GRANT EXECUTE ON [Providers_GetProviders] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CancelRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Cancels the specified release'
GO


IF OBJECT_ID('[Releases_CancelRelease]') IS NOT NULL DROP PROCEDURE [Releases_CancelRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CancelRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@CancelledReason_Text VARCHAR(500)
)
AS
BEGIN

SET NOCOUNT ON

	IF EXISTS(SELECT * 
	            FROM [Releases] 
			   WHERE [Application_Id] = @Application_Id
                 AND [Release_Number] = @Release_Number
			     AND [ReleaseStatus_Name] IN ('Deployed', 'Cancelled'))
	BEGIN
		RETURN
	END



BEGIN TRY
	BEGIN TRANSACTION


    UPDATE [Builds]
       SET [BuildStatus_Name] = 'Rejected'
          ,[RejectedBy_User_Name] = dbo.Session_GetUserName()
          ,[Rejected_Date] = GETUTCDATE()
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
	 
    UPDATE [Releases]
       SET [ReleaseStatus_Name] = 'Cancelled',
		   [CancelledBy_User_Name] = dbo.Session_GetUserName(),
		   [CancelledReason_Text] = @CancelledReason_Text,
           [CancelledOn_Date] = GETUTCDATE()
     WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number

	EXEC [Events_RaiseEvent] 'RELCNL', @Application_Id, @Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CancelRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseNote_Sequence',
    /* Description_Text        */ 'Creates a new release note, or updates an existing one'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_CreateOrUpdateReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateReleaseNote]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT = NULL OUT,
	@Build_Number BUILDNUMBER = NULL,
	@Environment_Id INT= NULL,
	@Notes_Text VARCHAR(MAX)
) AS BEGIN

SET NOCOUNT ON

BEGIN TRY

	BEGIN TRANSACTION

	IF @ReleaseNote_Sequence IS NOT NULL

		UPDATE [ReleaseNotes]
		   SET [Notes_Text] = @Notes_Text
		      ,[Build_Number] = @Build_Number
		      ,[Environment_Id] = @Environment_Id
		      ,[ModifiedBy_User_Name] = dbo.Session_GetUserName()
		      ,[ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Application_ID] = @Application_ID
		   AND [Release_Number] = @Release_Number
		   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence
	
	ELSE BEGIN

		SET @ReleaseNote_Sequence = COALESCE(
		            (SELECT MAX([ReleaseNote_Sequence])+1
		               FROM [ReleaseNotes]
		              WHERE [Application_ID] = @Application_ID
		                AND [Release_Number] = @Release_Number) , 1)
		INSERT INTO [ReleaseNotes]
			 ([Application_Id],[Release_Number],[ReleaseNote_Sequence], [Build_Number],[Environment_Id],[Notes_Text]
			 ,[CreatedBy_User_Name],[CreatedOn_Date],[ModifiedBy_User_Name],[ModifiedOn_Date])
		VALUES
			 (@Application_Id,@Release_Number,@ReleaseNote_Sequence,@Build_Number,@Environment_Id,@Notes_Text
			 ,dbo.Session_GetUserName(),GETUTCDATE(),dbo.Session_GetUserName(),GETUTCDATE())
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0  ROLLBACK
	EXEC [HandleError]
END CATCH
		 

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateReleaseNote] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_DeleteReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a release note from the system'
GO

IF OBJECT_ID('[Releases_DeleteReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_DeleteReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_DeleteReleaseNote]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT
) AS BEGIN

	DELETE [ReleaseNotes]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence

END
GO

GRANT EXECUTE ON [Releases_DeleteReleaseNote] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetBuildExecutions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'BuildExecutions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of build executions for the specified release'
GO

IF OBJECT_ID('[Releases_GetBuildExecutions]') IS NOT NULL
	DROP PROCEDURE [Releases_GetBuildExecutions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetBuildExecutions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [BuildExecutions_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

END
GO

GRANT EXECUTE ON [Releases_GetBuildExecutions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetDependentReleases',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all releases with referenced deployables from a specified release'
GO

IF OBJECT_ID('[Releases_GetDependentReleases]') IS NOT NULL DROP PROCEDURE [Releases_GetDependentReleases]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetDependentReleases]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

 
	SELECT RE.*
	  FROM [Releases_Extended] RE
	 WHERE EXISTS 
           (
	         SELECT * FROM [ReleaseDeployables] RD
	          WHERE RE.[Application_Id] = RD.[Application_Id]
	            AND RE.[Release_Number] = RD.[Release_Number]
	            AND RD.[Referenced_Application_Id] = @Application_Id
	            AND RD.[Referenced_Release_Number] = @Release_Number
	            
	       )
	   AND NOT ([Application_Id] = @Application_Id AND 
	            [Release_Number] = @Release_Number)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_GetDependentReleases] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Releases_Extended,ReleaseDeployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a specified release and the set of deployables associated with the release'
GO


IF OBJECT_ID('[Releases_GetRelease]') IS NOT NULL DROP PROCEDURE [Releases_GetRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [Releases_Extended]
     WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	SELECT *
	  FROM [ReleaseDeployables_Extended]
	 WHERE [Application_Id] = @Application_Id
       AND [Release_Number] = @Release_Number
	ORDER BY [Deployable_Name]
END
GO

GRANT EXECUTE ON [Releases_GetRelease] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseDeployables',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseDeployables_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the set of deployables associated with any release for an application'
GO


IF OBJECT_ID('[Releases_GetReleaseDeployables]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseDeployables]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseDeployables]
(
	@Application_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [ReleaseDeployables_Extended]
	 WHERE @Application_Id IS NULL OR [Application_Id] = @Application_Id
END
GO

GRANT EXECUTE ON [Releases_GetReleaseDeployables] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseNote',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'ReleaseNotes_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a release note from the system'
GO

IF OBJECT_ID('[Releases_GetReleaseNote]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseNote]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseNote]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@ReleaseNote_Sequence INT
) AS BEGIN

	SELECT * 
	  FROM [ReleaseNotes_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND [ReleaseNote_Sequence] = @ReleaseNote_Sequence
	IF @@ROWCOUNT = 0 RAISERROR('Release Note %i not found', 16, 1, @ReleaseNote_Sequence)
END
GO

GRANT EXECUTE ON [Releases_GetReleaseNote] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleaseNotes',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ReleaseNotes_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the release notes that match the specified criteria'
GO

IF OBJECT_ID('[Releases_GetReleaseNotes]') IS NOT NULL DROP PROCEDURE [Releases_GetReleaseNotes]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleaseNotes]
(
	@Application_ID INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER = NULL,
	@Environment_Id INT= NULL
) AS BEGIN

	SELECT * 
	  FROM [ReleaseNotes_Extended]
	 WHERE [Application_ID] = @Application_ID
	   AND [Release_Number] = @Release_Number
	   AND ([Build_Number] = COALESCE(@Build_Number,[Build_Number]) OR COALESCE(@Build_Number,[Build_Number]) IS NULL)
	   AND ([Environment_Id] = COALESCE(@Environment_Id,[Environment_Id]) OR COALESCE(@Environment_Id,[Environment_Id]) IS NULL)

END
GO

GRANT EXECUTE ON [Releases_GetReleaseNotes] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetReleases',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of releases that match the specified criteria'
GO


IF OBJECT_ID('[Releases_GetReleases]') IS NOT NULL DROP PROCEDURE [Releases_GetReleases]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetReleases]
(
	@Application_Id INT,
	@ReleaseStatus_Name VARCHAR(9) = NULL,
	@Release_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	IF @Release_Count IS NOT NULL
		SET ROWCOUNT @Release_Count

	SELECT * 
	  FROM [Releases_Extended]
     WHERE (@Application_Id IS NULL OR [Application_Id] = @Application_Id)
	   AND (@ReleaseStatus_Name IS NULL OR [ReleaseStatus_Name] = @ReleaseStatus_Name)
      ORDER BY [Release_Sequence] DESC
               

END
GO

GRANT EXECUTE ON [Releases_GetReleases] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetRequiredPromotions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Releases_RequiredPromotions',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of the promotions that are required for the specified release'
GO

IF OBJECT_ID('Releases_GetRequiredPromotions') IS NOT NULL DROP PROCEDURE Releases_GetRequiredPromotions
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetRequiredPromotions]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS BEGIN

	SET NOCOUNT ON


	SELECT * FROM [Releases_RequiredPromotions]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	ORDER BY [Step_Sequence]

END
GO

GRANT EXECUTE ON Releases_GetRequiredPromotions TO BuildMasterUser_Role

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_GetSortableReleaseNumber',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Release_Sequence',
    /* Description_Text        */ 'Gets the sortable release number for a release.'
GO


IF OBJECT_ID('[Releases_GetSortableReleaseNumber]') IS NOT NULL DROP PROCEDURE [Releases_GetSortableReleaseNumber]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_GetSortableReleaseNumber]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Release_Sequence BIGINT OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SET @Release_Sequence = COALESCE(
	                               (SELECT [Release_Sequence]
	                                  FROM [Releases]
	                                 WHERE [Release_Number] = @Release_Number AND [Application_Id] = @Application_Id),
	                               dbo.GetSortableReleaseNumber(
	                                                            @Release_Number,
	                                                            (SELECT [ReleaseNumber_Scheme_Name]
	                                                               FROM [Applications]
	                                                              WHERE [Application_Id] = @Application_Id)))
END
GO

GRANT EXECUTE ON [Releases_GetSortableReleaseNumber] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_PurgeReleaseData',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Purges an entire release and all associated data from the system'
GO

IF OBJECT_ID('[Releases_PurgeReleaseData]') IS NOT NULL
	DROP PROCEDURE [Releases_PurgeReleaseData]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_PurgeReleaseData]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS(SELECT * FROM [Applications] WHERE [Application_Id] = @Application_Id)
		RAISERROR('Application %i does not exist.', 16, 1, @Application_Id)
	
	IF NOT EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number)
		RAISERROR('Release %s of application %i does not exist.', 16, 1, @Release_Number, @Application_Id)
		
	DECLARE @Application_Name VARCHAR(50)
	SELECT @Application_Name = [Application_Name]
	  FROM [Applications]
	 WHERE [Application_Id] = @Application_Id

    DECLARE @Release_Sequence INT
    SELECT @Release_Sequence = [Release_Sequence] 
      FROM [Releases] 
     WHERE [Application_Id] = @Application_Id 
       AND [Release_Number] = @Release_Number

	/* 1) Delete data without sub-release dependencies */
	DELETE [ReleaseNotes]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [ReleaseDeployables]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [PromotionRequirementStatuses]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildArtifacts]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildPromotionContingencies]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildApprovals]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

    DELETE [VariableDefinitions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2) Delete data with sub-release dependencies */
	/* Dependencies */
	DECLARE @Builds TABLE ([Build_Number] BUILDNUMBER)
	INSERT INTO @Builds ([Build_Number])
	SELECT [Build_Number]
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DECLARE @Executions TABLE ([Execution_Id] INT NOT NULL)
	INSERT INTO @Executions ([Execution_Id])
	SELECT [Execution_Id]
	  FROM [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DECLARE @BuildExecutionPlanActions TABLE ([BuildExecution_PlanAction_Id] INT NOT NULL)
	INSERT INTO @BuildExecutionPlanActions
	SELECT [BuildExecution_PlanAction_Id]
	  FROM [BuildExecution_PlanActions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DECLARE @DatabaseChangeScripts TABLE ([Script_Id] INT NOT NULL)
	INSERT INTO @DatabaseChangeScripts ([Script_Id])
	SELECT [Script_Id]
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DECLARE @ChangeControls TABLE ([ChangeControl_Id] INT NOT NULL)
	INSERT INTO @ChangeControls ([ChangeControl_Id])
	SELECT [ChangeControl_Id]
	  FROM [ChangeControls]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2a) Builds */
	DELETE
	  FROM [BuildTestResults]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildOutputs]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_PlanActionLogEntries]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )

    DELETE
	  FROM [BuildExecution_PlanActionVariableValues]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )

	DELETE
	  FROM [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN ( SELECT [BuildExecution_PlanAction_Id] FROM @BuildExecutionPlanActions )

	DELETE
	  FROM [BuildExecution_PlanActions]
	 WHERE [Execution_Id] IN  ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildExecution_Plans]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	/* Have to delete change script executions before Build Executions... */
	DELETE [DatabaseChangeScriptExecutions]
	 WHERE [Script_Id] IN ( SELECT [Script_Id] FROM @DatabaseChangeScripts )
	    OR [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE [ConfigurationFileDeployments]
	  FROM [ConfigurationFileDeployments] _CFD
	 WHERE EXISTS (
		SELECT * FROM [ConfigurationFileVersions]
		 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number
		   AND [ConfigurationFile_Id] = _CFD.[ConfigurationFile_Id]
		   AND [Instance_Name] = _CFD.[Instance_Name]
		   AND [Version_Number] = _CFD.[Version_Number] )

	DELETE
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] IN ( SELECT [Execution_Id] FROM @Executions )

	DELETE
	  FROM [BuildPromotions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE
	  FROM [Builds]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2b) Change Scripts */
	DELETE
	  FROM [DatabaseChangeScripts]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	/* 2c) Configuration */
	DELETE [ConfigurationFileVersions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [ChangeControls]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [Builds]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	DELETE [BuildExecutions]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number


	 /* 2d) Change Controls */
	DELETE 
	  FROM [ChangeControlEnvironments]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)

	DELETE
	  FROM [ChangeControlPerformances]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)

	DELETE 
	  FROM [ChangeControls]
	 WHERE [ChangeControl_Id] IN (SELECT [ChangeControl_Id] FROM @ChangeControls)


	 /* 2e) Release Deployables */
	DELETE [ReleaseDeployables]
	 WHERE ([Application_Id] = @Application_Id AND [Release_Number] = @Release_Number)
		OR ([Referenced_Application_Id] = @Application_Id AND [Referenced_Release_Number] = @Release_Number)


	/* 3) Release */
	DELETE [Releases]
	 WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

    UPDATE [Releases]
       SET [Release_Sequence] = [Release_Sequence] - 1
     WHERE [Application_Id] = @Application_Id
       AND [Release_Sequence] > @Release_Sequence


	 /* Raise Event */
	EXEC [Events_RaiseEvent] 'RELDEL',NULL,@Application_Name,@Release_Number

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_PurgeReleaseData] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_ReleaseExists',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ReleaseExists_Indicator,NewerDeployedReleaseExists_Indicator',
    /* Description_Text        */ 'Determines whether a release already exists, and also whether a newer release of an application exists'
GO

IF OBJECT_ID('[Releases_ReleaseExists]') IS NOT NULL DROP PROCEDURE [Releases_ReleaseExists]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_ReleaseExists]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@ReleaseExists_Indicator YNINDICATOR = NULL OUT,
	@NewerDeployedReleaseExists_Indicator YNINDICATOR = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON


	SET @ReleaseExists_Indicator =
		CASE WHEN EXISTS(SELECT *
						   FROM [Releases_Extended]
						  WHERE [Release_Number] = @Release_Number
							AND [Application_Id] = @Application_Id)
			 THEN 'Y'
			 ELSE 'N' END

    DECLARE @Release_Sequence BIGINT

    SELECT @Release_Sequence = [Release_Sequence]
     FROM [Releases]
    WHERE [Application_Id] = @Application_Id 
      AND [Release_Number] = @Release_Number

	SET @NewerDeployedReleaseExists_Indicator =
		CASE WHEN EXISTS(SELECT *
						   FROM [Releases_Extended]
						  WHERE [Release_Sequence] > @Release_Sequence
							AND [Application_Id] = @Application_Id
							AND [ReleaseStatus_Name] = 'Deployed')
			 THEN 'Y'
			 ELSE 'N' END

END
GO

GRANT EXECUTE ON [Releases_ReleaseExists] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_ResequenceRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Resequences the specified release by placing it where another release is sequenced'
GO


IF OBJECT_ID('[Releases_ResequenceRelease]') IS NOT NULL DROP PROCEDURE [Releases_ResequenceRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_ResequenceRelease]
(
	@Application_Id INT,
    @Release_Sequence INT,
	@Replaced_Release_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	UPDATE [Releases]
	   SET [Release_Sequence] = CASE
	          WHEN [Releases].[Release_Sequence] = RTAR.[Release_Sequence] 
	               THEN RPOS.[Release_Sequence]
	          WHEN [Releases].[Release_Sequence] BETWEEN RTAR.[Release_Sequence] AND RPOS.[Release_Sequence]
	               THEN [Releases].[Release_Sequence] - 1
	          WHEN [Releases].[Release_Sequence] BETWEEN RPOS.[Release_Sequence] AND RTAR.[Release_Sequence]
	               THEN [Releases].[Release_Sequence] + 1
	          ELSE [Releases].[Release_Sequence]
	          END
	  FROM [Releases],
	       [Releases] RTAR,
	       [Releases] RPOS
	 WHERE [Releases].[Application_Id] = @Application_Id
       AND RTAR.[Application_Id] = @Application_Id
       AND RTAR.[Release_Sequence] = @Release_Sequence
	   AND RPOS.[Application_Id] = @Application_Id
       AND RPOS.[Release_Sequence] = @Replaced_Release_Sequence
	   AND @Release_Sequence <> @Replaced_Release_Sequence


END
GO

GRANT EXECUTE ON [Releases_ResequenceRelease] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_UnCancelRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Uncancels a previously cancelled release'
GO

IF OBJECT_ID('[Releases_UnCancelRelease]') IS NOT NULL
	DROP PROCEDURE [Releases_UnCancelRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_UnCancelRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER
) AS
BEGIN

	SET NOCOUNT ON

	IF NOT EXISTS ( SELECT *
					  FROM [Releases]
					 WHERE [Application_Id] = @Application_Id
					   AND [Release_Number] = @Release_Number
					   AND [ReleaseStatus_Name] = 'Cancelled' )
	BEGIN
		RAISERROR('Release ''%s'' does not exist or its status is not ''Cancelled''', 16, 1, @Release_Number)
		RETURN
	END


BEGIN TRY
	BEGIN TRANSACTION


		/* Set the release as Active */
		UPDATE [Releases]
		   SET [ReleaseStatus_Name] = 'Active'
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

		/* Cancel other active releases if applicable */
		IF 'N' = ( SELECT [AllowMultipleActiveReleases_Indicator]
					 FROM [Applications]
					WHERE [Application_Id] = @Application_Id )
		BEGIN

			/* Cancel existing active releases */
			UPDATE [Releases]
			   SET [ReleaseStatus_Name] = 'Cancelled'
			 WHERE [Application_Id] = @Application_Id
			   AND [Release_Number] <> @Release_Number
			   AND [ReleaseStatus_Name] = 'Active'
		END

	EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_UnCancelRelease] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateParameter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new script parameter or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateParameter]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateParameter]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateParameter]
(
	@Script_Id INT,
	@Parameter_Name NVARCHAR(50),
	@ParameterType_Code CHAR(1),
	@DefaultValue_Text NVARCHAR(MAX),
	@Description_Text NVARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
	            FROM [ScriptParameters]
	           WHERE [Script_Id] = @Script_Id
	             AND [Parameter_Name] = @Parameter_Name)
	BEGIN

		UPDATE [ScriptParameters]
		   SET [ParameterType_Code] = @ParameterType_Code,
		       [DefaultValue_Text] = @DefaultValue_Text,
		       [Description_Text] = @Description_Text
		 WHERE [Script_Id] = @Script_Id
		   AND [Parameter_Name] = @Parameter_Name

	END
	ELSE
	BEGIN

		INSERT INTO [ScriptParameters]
		(
			[Script_Id],
			[Parameter_Name],
			[ParameterType_Code],
			[DefaultValue_Text],
			[Description_Text]
		)
		VALUES
		(
			@Script_Id,
			@Parameter_Name,
			@ParameterType_Code,
			@DefaultValue_Text,
			@Description_Text
		)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateParameter] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateOrUpdateScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Script_Id',
    /* Description_Text        */ 'Creates a new script or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_CreateOrUpdateScript]') IS NOT NULL DROP PROCEDURE [Scripts_CreateOrUpdateScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateOrUpdateScript]
(
	@Script_Id INT OUT,
	@Script_Name NVARCHAR(50),
	@Description_Text NVARCHAR(MAX),
	@ScriptType_Code VARCHAR(20),
	@Active_Indicator YNINDICATOR,
	@AutoSync_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Script_Id IS NULL
	BEGIN

		INSERT INTO [Scripts]
		(
			[Script_Name],
			[Description_Text],
			[Active_Indicator],
			[AutoSync_Indicator],
			[ScriptType_Code],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
		(
			@Script_Name,
			@Description_Text,
			@Active_Indicator,
			@AutoSync_Indicator,
			@ScriptType_Code,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		SET @Script_Id = SCOPE_IDENTITY()

	END
	ELSE
	BEGIN

		UPDATE [Scripts]
		   SET [Script_Name] = @Script_Name,
		       [Description_Text] = @Description_Text,
		       [ScriptType_Code] = @ScriptType_Code,
		       [Active_Indicator] = @Active_Indicator,
		       [AutoSync_Indicator] = @AutoSync_Indicator,
		       [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
		       [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateOrUpdateScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_CreateVersion',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Version_Number',
    /* Description_Text        */ 'Creates a new version of a script.'
GO


IF OBJECT_ID('[Scripts_CreateVersion]') IS NOT NULL DROP PROCEDURE [Scripts_CreateVersion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_CreateVersion]
(
	@Version_Number INT OUT,
	@Script_Id INT,
	@Script_Bytes VARBINARY(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @Version_Number = (SELECT MAX([Version_Number])
	                         FROM [ScriptVersions]
	                        WHERE [Script_Id] = @Script_Id)

	DECLARE @Latest_Bytes VARBINARY(MAX)
	SET @Latest_Bytes = (SELECT [Script_Bytes]
	                       FROM [ScriptVersions]
	                      WHERE [Script_Id] = @Script_Id
	                        AND [Version_Number] = @Version_Number)

	--Do not insert a copy of the most recent version
	IF @Latest_Bytes IS NULL OR @Latest_Bytes <> @Script_Bytes
	BEGIN

		SET @Version_Number = COALESCE(@Version_Number, 0) + 1

		INSERT INTO [ScriptVersions]
		(
			[Script_Id],
			[Version_Number],
			[Script_Bytes],
			[CreatedBy_User_Name],
			[CreatedOn_Date]
		)
		VALUES
		(
			@Script_Id,
			@Version_Number,
			@Script_Bytes,
			dbo.Session_GetUserName(),
			GETUTCDATE()
		)

		UPDATE [Scripts]
		   SET [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Script_Id] = @Script_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_CreateVersion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteParameter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new script parameter or updates an existing one.'
GO


IF OBJECT_ID('[Scripts_DeleteParameter]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteParameter]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteParameter]
(
	@Script_Id INT,
	@Parameter_Name NVARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id
	   AND [Parameter_Name] = @Parameter_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteParameter] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_DeleteScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Permanently deletes a script.'
GO


IF OBJECT_ID('[Scripts_DeleteScript]') IS NOT NULL DROP PROCEDURE [Scripts_DeleteScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_DeleteScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id

	DELETE [ScriptVersions]
	 WHERE [Script_Id] = @Script_Id

	DELETE [Scripts]
	 WHERE [Script_Id] = @Script_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Scripts_DeleteScript] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScript',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Scripts_Extended,ScriptParameters',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a script and all of its associated information.'
GO


IF OBJECT_ID('[Scripts_GetScript]') IS NOT NULL DROP PROCEDURE [Scripts_GetScript]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScript]
(
	@Script_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [Scripts_Extended]
	 WHERE [Script_Id] = @Script_Id

	SELECT *
	  FROM [ScriptParameters]
	 WHERE [Script_Id] = @Script_Id
	 ORDER BY [Parameter_Name]

END
GO

GRANT EXECUTE ON [Scripts_GetScript] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScripts',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Scripts_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the scripts in the system, optionally filtered by type code.'
GO


IF OBJECT_ID('[Scripts_GetScripts]') IS NOT NULL DROP PROCEDURE [Scripts_GetScripts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScripts]
(
	@ScriptType_Code VARCHAR(20),
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [Scripts_Extended]
	 WHERE (@IncludeInactive_Indicator = 'Y' OR [Active_Indicator] = 'Y')
	   AND (@ScriptType_Code IS NULL OR [ScriptType_Code] = @ScriptType_Code)
	 ORDER BY [Script_Name]

END
GO

GRANT EXECUTE ON [Scripts_GetScripts] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Scripts_GetScriptTypeUsage',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'ScriptTypes_Usage',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets usage data for the script types in the database.'
GO


IF OBJECT_ID('[Scripts_GetScriptTypeUsage]') IS NOT NULL DROP PROCEDURE [Scripts_GetScriptTypeUsage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Scripts_GetScriptTypeUsage]
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [ScriptTypes_Usage]

END
GO

GRANT EXECUTE ON [Scripts_GetScriptTypeUsage] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Assigns the specified user as a named user with the specified user class type'
GO

IF OBJECT_ID('[Security_AddNamedUser]') IS NOT NULL DROP PROCEDURE [Security_AddNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddNamedUser]
(
	@User_Name USERNAME,
	@UserClassType_Code CHAR(1)
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	INSERT INTO [NamedUsers] ([User_Name], [UserClassType_Code])
	VALUES (@User_Name, @UserClassType_Code)

	EXEC [Events_RaiseEvent] 'SECNWU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddNamedUser] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddPrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Privilege_Id',
    /* Description_Text        */ 'Grants or denies the specified principal the ability to perform a certain role'
GO

IF OBJECT_ID('[Security_AddPrivilege]') IS NOT NULL
	DROP PROCEDURE [Security_AddPrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_AddPrivilege]
(
	@Privilege_Id INT = NULL OUT,
	@Directory_Provider_Id INT,
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@Role_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT,
	@PrivilegeType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Catch duplicate data */
	IF EXISTS(SELECT *
				FROM [Privileges]
			   WHERE [Principal_Name] = @Principal_Name
				 AND [Directory_Provider_Id] = @Directory_Provider_Id
				 AND [PrincipalType_Code] = @PrincipalType_Code
				 AND [Role_Id] = @Role_Id
				 AND (	[Application_Id] = @Application_Id
					 OR COALESCE([Application_Id], @Application_Id) IS NULL )
				 AND (	[Environment_Id] = @Environment_Id
					 OR COALESCE([Environment_Id], @Environment_Id) IS NULL )
				 AND (  [ApplicationGroup_Id] = @ApplicationGroup_Id
				     OR COALESCE([ApplicationGroup_Id], @ApplicationGroup_Id) IS NULL )
				 AND [PrivilegeType_Code] = @PrivilegeType_Code )
	BEGIN
	    ROLLBACK
		RETURN
    END

	INSERT INTO [Privileges]
		([Directory_Provider_Id]
		,[Principal_Name]
		,[PrincipalType_Code]
		,[Role_Id]
		,[ApplicationGroup_Id]
		,[Application_Id]
		,[Environment_Id]
		,[PrivilegeType_Code])
	VALUES
		(@Directory_Provider_Id
		,@Principal_Name
		,@PrincipalType_Code
		,@Role_Id
		,@ApplicationGroup_Id
		,@Application_Id
		,@Environment_Id
		,@PrivilegeType_Code)
	SET @Privilege_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'SECNWP', NULL, @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddPrivilege] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_CreateOrUpdateRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Creates a new role defined by the specified tasks'
GO

IF OBJECT_ID('[dbo].[Security_CreateOrUpdateRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_CreateOrUpdateRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_CreateOrUpdateRole]
(
	@Role_Id INT = NULL OUT,
	@Role_Name VARCHAR(50),
	@Tasks_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Role_Id IS NULL
	BEGIN
		/* Add new role */
		INSERT INTO [dbo].[Roles]
			([Role_Name])
		VALUES
			(@Role_Name)
		SET @Role_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'SECNWR', NULL, @Role_Id

	END
	ELSE
	BEGIN
		/* Update existing role */
		UPDATE [Roles]
		   SET [Role_Name] = @Role_Name
		 WHERE [Role_Id] = @Role_Id

		/* Remove existing tasks for the role */
		DELETE [RoleTasks]
		 WHERE [Role_Id] = @Role_Id

		EXEC [Events_RaiseEvent] 'SECMDR', NULL, @Role_Id
	END

	/* Add the appropriate role tasks */
	INSERT INTO [dbo].[RoleTasks]
		([Role_Id]
		,[Task_Id])
	SELECT @Role_Id,[Value_Text]
	  FROM [dbo].[CsvToTable](@Tasks_Csv, ',')

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Security_CreateOrUpdateRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_DeleteRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified role from the system'
GO

IF OBJECT_ID('[dbo].[Security_DeleteRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_DeleteRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_DeleteRole]
(
	@Role_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Role_Name SQL_VARIANT
	    SET @Role_Name = (SELECT [Role_Name] FROM [Roles] WHERE [Role_Id] = @Role_Id)

	DELETE [Privileges]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [RoleTasks]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [Roles]
	 WHERE [Role_Id] = @Role_Id

	IF @@ROWCOUNT>0 EXEC [Events_RaiseEvent] 'SECDLR', NULL, @Role_Name
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [dbo].[Security_DeleteRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'NamedUsers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified named user'
GO

IF OBJECT_ID('[Security_GetNamedUser]') IS NOT NULL DROP PROCEDURE [Security_GetNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetNamedUser]
(
    @User_Name USERNAME
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [NamedUsers]
	WHERE [User_Name] = @User_Name


END
GO

GRANT EXECUTE ON [Security_GetNamedUser] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetNamedUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'NamedUsers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of the named users in the system'
GO

IF OBJECT_ID('[Security_GetNamedUsers]') IS NOT NULL DROP PROCEDURE [Security_GetNamedUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetNamedUsers]  AS
BEGIN

	SET NOCOUNT ON

	SELECT * FROM [NamedUsers]


END
GO

GRANT EXECUTE ON [Security_GetNamedUsers] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of privileges assigned in the system'
GO

IF OBJECT_ID('[dbo].[Security_GetPrivileges]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetPrivileges]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetPrivileges]
(
	@Directory_Provider_Id INT,
	@PrincipalType_Code CHAR(1) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [PrincipalType_Code] = COALESCE(@PrincipalType_Code, [PrincipalType_Code])
	   AND [Directory_Provider_Id] = @Directory_Provider_Id
END
GO

GRANT EXECUTE ON [dbo].[Security_GetPrivileges] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified role in the database, along with all the tasks that comprise the role'
GO

IF OBJECT_ID('[dbo].[Security_GetRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetRole]
(
	@Role_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	 WHERE [Role_Id] = @Role_Id
	IF @@ROWCOUNT=0 RAISERROR('Role #%i does not exist.',16,1,@Role_Id)


	SELECT *
	  FROM [RoleTasks_Extended]
	 WHERE [Role_Id] = @Role_Id

END
GO

GRANT EXECUTE ON [dbo].[Security_GetRole] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRoles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the roles in the system, along with all the tasks comprising each role'
GO

IF OBJECT_ID('[dbo].[Security_GetRoles]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_GetRoles]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_GetRoles]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	ORDER BY [Role_Name]

	SELECT *
	  FROM [RoleTasks_Extended]
	ORDER BY [Task_Name]

END
GO

GRANT EXECUTE ON [dbo].[Security_GetRoles] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetUserClassCounts',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Security_GetUserClassCounts]') IS NOT NULL DROP PROCEDURE [Security_GetUserClassCounts]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetUserClassCounts]  AS
BEGIN

	SET NOCOUNT ON

    DECLARE @FullUsers INT
    DECLARE @LimitedUsers INT
    DECLARE @ApprovalUsers INT
    DECLARE @NamedUsers INT

	SELECT @LimitedUsers = 0 --(SELECT COUNT(*) FROM [NamedUsers] WHERE [UserClassType_Code] = 'L')
	SELECT @ApprovalUsers = (SELECT COUNT(*) FROM [NamedUsers] WHERE [UserClassType_Code] = 'A')
    SELECT @FullUsers = (SELECT COUNT(*) FROM [NamedUsers] WHERE [UserClassType_Code] = 'F')
    SELECT @NamedUsers = (SELECT COUNT(*) FROM [NamedUsers])
    
    SELECT 
        [NamedUsers_Count] = @NamedUsers,
        [FullUsers_Count] = @FullUsers,
        [LimitedUsers_Count] = @LimitedUsers,
        [ApprovalUsers_Count] = @ApprovalUsers

END
GO

GRANT EXECUTE ON [Security_GetUserClassCounts] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetUserPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of privileges specifically assigned to a user or any specified groups'
GO

IF OBJECT_ID('[Security_GetUserPrivileges]') IS NOT NULL
	DROP PROCEDURE [Security_GetUserPrivileges]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_GetUserPrivileges]
(
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	/* Privileges */
	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [Directory_Provider_Id] = @Directory_Provider_Id
	   AND (   ([Principal_Name] = @User_Name AND [PrincipalType_Code] = 'U') 
		    OR ([Principal_Name] IN (SELECT [Value_Text] FROM [CsvToTable](@GroupNames_Csv, ',') )
			    AND [PrincipalType_Code] = 'G' ) )

END
GO

GRANT EXECUTE ON [Security_GetUserPrivileges] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForAnyTask',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Authorized_Indicator',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform a task'
GO


IF OBJECT_ID('Security_IsAuthorizedForAnyTask') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForAnyTask]
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_IsAuthorizedForAnyTask] (
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@Authorized_Indicator YNINDICATOR OUT
) AS
	SET NOCOUNT ON

	IF EXISTS(
		SELECT *
		  FROM [Privileges] P
			   INNER JOIN [RoleTasks] RT
					   ON RT.[Role_Id] = P.[Role_Id]
			   INNER JOIN [Tasks] T
					   ON T.[Task_Id] = RT.[Task_Id]
		 WHERE (( /* User */
				   P.[PrincipalType_Code] = 'U'
				   AND P.[Principal_Name] = @User_Name
				 ) OR (
				   /* Group */ 
				   P.[PrincipalType_Code] = 'G'
				   AND P.[Principal_Name] IN 
					 (SELECT [Value_Text]
						FROM [dbo].[CsvToTable](@GroupNames_Csv, ',') ) 
				))
			AND P.[Directory_Provider_Id] = @Directory_Provider_Id
			AND P.[PrivilegeType_Code] <> 'D'
	) SET @Authorized_Indicator = 'Y'
	  ELSE SET @Authorized_Indicator = 'N'

GO

GRANT EXECUTE ON [Security_IsAuthorizedForAnyTask] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForTask',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Authorized_Indicator',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform a task.'
GO


IF OBJECT_ID('Security_IsAuthorizedForTask') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForTask]
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Security_IsAuthorizedForTask (
	@Directory_Provider_Id INT,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@Task_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Environment_Id INT,
	@Server_Id INT,
	@Authorized_Indicator YNINDICATOR OUT
) AS
	SET NOCOUNT ON

	IF dbo.IsAuthorizedForTask(@Directory_Provider_Id, @User_Name, @GroupNames_Csv, @Task_Id, @ApplicationGroup_Id, @Application_Id, @Environment_Id) = 1
		SET @Authorized_Indicator = 'Y'
	ELSE
		SET @Authorized_Indicator = 'N'

	IF @Server_Id IS NOT NULL AND EXISTS(SELECT * FROM [EnvironmentServers] WHERE [Server_Id] = @Server_Id)
	BEGIN
	
		IF EXISTS(
			SELECT *
			  FROM [EnvironmentServers] ES
			 WHERE ES.[Server_Id] = @Server_Id
			   AND dbo.IsAuthorizedForTask(@Directory_Provider_Id, @User_Name, @GroupNames_Csv, @Task_Id, @ApplicationGroup_Id, @Application_Id, ES.[Environment_Id]) = 1)
		   SET @Authorized_Indicator = 'Y'
		ELSE
		   SET @Authorized_Indicator = 'N'

	END
GO

GRANT EXECUTE ON [Security_IsAuthorizedForTask] TO BuildMasterUser_Role
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForTaskBatch',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'AuthorizedTasks_Xml',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform the specified tasks.'
GO


IF OBJECT_ID('Security_IsAuthorizedForTaskBatch') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForTaskBatch]
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Security_IsAuthorizedForTaskBatch(
	@Directory_Provider_Id INT = NULL,
	@User_Name USERNAME,
	@GroupNames_Csv VARCHAR(MAX),
	@RequiredTasks_Xml XML,
	@AuthorizedTasks_Xml XML OUT
)
AS
	SET NOCOUNT ON

	IF @Directory_Provider_Id IS NULL
	BEGIN
		SET @Directory_Provider_Id = (SELECT [Value_Text]
		                                FROM [Configuration]
		                               WHERE [Scope_Name] = 'CoreEx'
		                                 AND [Key_Name] = 'DirectoryProvider')
	END

SET @AuthorizedTasks_Xml =
	(SELECT [Task] = CASE WHEN dbo.IsAuthorizedForTask(
					       @Directory_Provider_Id,
					       @User_Name,
					       @GroupNames_Csv,
					       T.c.value('(@Id)[1]', 'int'),
					       T.c.value('(@ApplicationGroupId)[1]', 'int'),
					       T.c.value('(@ApplicationId)[1]', 'int'),
					       T.c.value('(@EnvironmentId)[1]', 'int')) = 0
					     THEN 'N' ELSE 'Y' END
	  FROM @RequiredTasks_Xml.nodes('/RequiredTasks/Task') T(c)
	   FOR XML PATH(''),ROOT('AuthorizedTasks'))
GO

GRANT EXECUTE ON [Security_IsAuthorizedForTaskBatch] TO [BuildMasterUser_Role]
GO

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemoveNamedUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Removes the specified user as a named user'
GO

IF OBJECT_ID('[Security_RemoveNamedUser]') IS NOT NULL DROP PROCEDURE [Security_RemoveNamedUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_RemoveNamedUser]
(
	@User_Name USERNAME
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NamedUsers]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'SECDLU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_RemoveNamedUser] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemovePrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a privilege from the system'
GO

IF OBJECT_ID('[dbo].[Security_RemovePrivilege]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_RemovePrivilege]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_RemovePrivilege]
(
	@Privilege_Id INT
)
AS
BEGIN
SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [Privileges]
	 WHERE [Privilege_Id] = @Privilege_Id

	IF @@ROWCOUNT>0 EXEC [Events_RaiseEvent] 'SECDLP', NULL, @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [dbo].[Security_RemovePrivilege] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_UpdateUserClassType',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Updates the user class type of the specified user'
GO

IF OBJECT_ID('[Security_UpdateUserClassType]') IS NOT NULL DROP PROCEDURE [Security_UpdateUserClassType]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_UpdateUserClassType]
(
	@User_Name USERNAME,
	@UserClassType_Code CHAR(1)
) AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	
	UPDATE [NamedUsers]
	   SET [UserClassType_Code] = @UserClassType_Code
	 WHERE [User_Name] = @User_Name
	
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_UpdateUserClassType] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_UserIsNamed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'UserIsNamed_Indicator',
    /* Description_Text        */ 'Determines whether or not a user is a named user in the system'
GO

IF OBJECT_ID('[Security_UserIsNamed]') IS NOT NULL DROP PROCEDURE [Security_UserIsNamed]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Security_UserIsNamed]
(
	@User_Name USERNAME,
	@UserIsNamed_Indicator YNINDICATOR = NULL OUT
) AS
BEGIN

	SET NOCOUNT ON

	SET @UserIsNamed_Indicator = 'N'

	IF EXISTS ( SELECT *
				  FROM [NamedUsers]
				 WHERE [User_Name] = @User_Name )
		SET @UserIsNamed_Indicator = 'Y'

END
GO

GRANT EXECUTE ON [Security_UserIsNamed] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_GetUserConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'UserConfiguration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of user configuration settings'
GO

IF OBJECT_ID('[UserConfiguration_GetUserConfiguration]') IS NOT NULL DROP PROCEDURE [UserConfiguration_GetUserConfiguration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [UserConfiguration_GetUserConfiguration] 
(
	@User_Name USERNAME = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [UserConfiguration]
	 WHERE (@User_Name IS NULL OR [User_Name] = @User_Name)
	ORDER BY [User_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [UserConfiguration_GetUserConfiguration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified user configuration value'
GO


IF OBJECT_ID('[dbo].[UserConfiguration_GetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[UserConfiguration_GetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[UserConfiguration_GetValue]
(
	@User_Name USERNAME,
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [UserConfiguration]
	 WHERE [User_Name] = @User_Name
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [dbo].[UserConfiguration_GetValue] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'UserConfiguration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a user configuration value'
GO


IF OBJECT_ID('[dbo].[UserConfiguration_SetValue]') IS NOT NULL
	DROP PROCEDURE [dbo].[UserConfiguration_SetValue]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[UserConfiguration_SetValue]
(
	@User_Name USERNAME,
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [UserConfiguration]
	 WHERE [User_Name] = @User_Name
	   AND [Key_Name] = @Key_Name

	IF (@Value_Text IS NOT NULL) 
		INSERT INTO [UserConfiguration]
			([User_Name], [Key_Name], [Value_Text])
		VALUES
			(@User_Name, @Key_Name, @Value_Text)

	--EXEC [Events_RaiseEvent] 'SYSCFG',NULL,@Key_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[UserConfiguration_SetValue] TO [BuildMasterUser_Role]
GO


GO
IF OBJECT_ID('[dbo].[Users_CreateGroup]') IS NOT NULL DROP PROCEDURE [dbo].[Users_CreateGroup]
GO


EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user group in the system'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateGroup]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_CreateOrUpdateGroup]
(
	@Group_Name VARCHAR(50),
	@Users_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Users]
			   WHERE [User_Name] = @Group_Name)
		RAISERROR('Cannot create group "%s"; a user of the same name exists', 16, 1)

	IF NOT EXISTS(SELECT * FROM [Groups] WHERE [Group_Name] = @Group_Name)
    BEGIN
        
        INSERT INTO [Groups] ([Group_Name]) VALUES (@Group_Name)
        EXEC [Events_RaiseEvent] 'USRNWG', NULL, @Group_Name

    END

    IF @Users_XML IS NOT NULL 
    BEGIN

	    DELETE [UserGroups] WHERE [Group_Name] = @Group_Name

	    INSERT INTO [UserGroups] ([User_Name] ,[Group_Name])
	    SELECT UX.[value]('.', 'VARCHAR(250)') [User_Name], @Group_Name
	      FROM @Users_Xml.[nodes]('/users/username') AS UsersXml(UX)

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_CreateOrUpdateGroup] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user in the system, or updates an existing one'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateUser]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_CreateOrUpdateUser]
(
	@User_Name USERNAME,
	@Display_Name VARCHAR(100),
	@Email_Address VARCHAR(250) = NULL,
	@Groups_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Groups]
			   WHERE [Group_Name] = @User_Name)
		RAISERROR('Cannot create user "%s"; a group of the same name exists', 16, 1)

	IF EXISTS(SELECT *
				FROM [dbo].[Users]
			   WHERE [User_Name] = @User_Name)
	BEGIN

		/* Existing user; update */
		UPDATE [dbo].[Users]
		   SET [Display_Name] = @Display_Name
			  ,[Email_Address] = @Email_Address
		 WHERE [User_Name] = @User_Name
		EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	END
	ELSE
	BEGIN

		/* New user; create */
		INSERT INTO [dbo].[Users]
			([User_Name]
			,[Display_Name]
			,[Email_Address])
		VALUES
			(@User_Name
			,@Display_Name
			,@Email_Address)
		EXEC [Events_RaiseEvent] 'USRNWU', NULL, @User_Name

	END

	/* Prepare to rebuild group memberships */
	DELETE
	  FROM [dbo].[UserGroups]
	 WHERE [User_Name] = @User_Name

	/* Add in the group memberships */
	INSERT INTO [dbo].[UserGroups]
		([User_Name]
		,[Group_Name])
	SELECT @User_Name
		  ,GX.[value]('.', 'VARCHAR(50)') [Group_Name]
	  FROM @Groups_Xml.[nodes]('/groups/group') AS GroupsXml(GX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_CreateOrUpdateUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user group from the system, but does not delete the users who belong to the group'
GO

IF OBJECT_ID('[dbo].[Users_DeleteGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_DeleteGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_DeleteGroup]
(
	@Group_Name VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Remove group privileges */
	DELETE [Privileges]
	 WHERE [Principal_Name] = @Group_Name
	   AND [PrincipalType_Code] = 'G'

	/* Remove group users */
	DELETE [UserGroups]
	 WHERE [Group_Name] = @Group_Name

	/* Remove group */
	DELETE [Groups]
	 WHERE [Group_Name] = @Group_Name

	EXEC [Events_RaiseEvent] 'USRDLG', NULL, @Group_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_DeleteGroup] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user from the system'
GO

IF OBJECT_ID('[dbo].[Users_DeleteUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_DeleteUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_DeleteUser]
(
	@User_Name VARCHAR(255)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	DELETE [Privileges]
	 WHERE [Principal_Name] = @User_Name
	   AND [PrincipalType_Code] = 'U'

	DELETE [UserGroups]
	 WHERE [User_Name] = @User_Name

	DELETE [Users]
	 WHERE [User_Name] = @User_Name

	DELETE [NamedUsers]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRDLU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Users_DeleteUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of user groups in the system'
GO

IF OBJECT_ID('[dbo].[Users_GetGroups]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_GetGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_GetGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Groups]
	ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [dbo].[Users_GetGroups] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified user, and a list of all the groups the user belongs to'
GO

IF OBJECT_ID('[Users_GetUser]') IS NOT NULL DROP PROCEDURE [Users_GetUser]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_GetUser]
(
	@User_Name USERNAME
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT *
	  FROM [Users]
	 WHERE [User_Name] = @User_Name

	SELECT *
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

END
GO

GRANT EXECUTE ON [Users_GetUser] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the users in the system, along with all the groups in the system'
GO

IF OBJECT_ID('[Users_GetUsers]') IS NOT NULL DROP PROCEDURE [Users_GetUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Users_GetUsers
 AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Users]

	SELECT *
	  FROM [UserGroups]

END
GO

GRANT EXECUTE ON [Users_GetUsers] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of users in the specified group'
GO

IF OBJECT_ID('[dbo].[Users_GetUsersInGroup]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_GetUsersInGroup]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_GetUsersInGroup]
(
    @Group_Name VARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT U.*
	  FROM [UserGroups] UG
	INNER JOIN [Users] U ON UG.[User_Name] = U.[User_Name]
	WHERE [Group_Name] = @Group_Name
	ORDER BY [User_Name]

END
GO

GRANT EXECUTE ON [dbo].[Users_GetUsersInGroup] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SearchGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specific group by name, or if a wildcard search is performed, a group whose name starts with the specified name'
GO

IF OBJECT_ID('[dbo].[Users_SearchGroups]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_SearchGroups]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Users_SearchGroups]
(
    @Group_Name VARCHAR(50),
    @PerformWildcardSearch_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON
    
    IF (@PerformWildcardSearch_Indicator = 'Y')
    BEGIN
    
        SELECT *
          FROM [Groups]
         WHERE [Group_Name] LIKE @Group_Name + '%'
      ORDER BY [Group_Name]
        
    END
    ELSE
    BEGIN
	
	    SELECT *
	      FROM [Groups]
	     WHERE [Group_Name] = @Group_Name
	  ORDER BY [Group_Name]
	
	END

END
GO

GRANT EXECUTE ON [dbo].[Users_SearchGroups] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SearchUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified user, or searchs for a user whose name starts with any of the specified criteria'
GO

IF OBJECT_ID('[Users_SearchUsers]') IS NOT NULL DROP PROCEDURE [Users_SearchUsers]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_SearchUsers]
(
    @User_Name USERNAME,
    @Display_Name VARCHAR(100),
    @Email_Address VARCHAR(250),
    @PerformWildcardSearch_Indicator YNINDICATOR
)
 AS BEGIN

	SET NOCOUNT ON

    IF (@PerformWildcardSearch_Indicator = 'Y')
    BEGIN
    
        -- Add wildcards if specified and the arguments are not null
        IF (@User_Name IS NOT NULL)
            SET @User_Name = @User_Name + '%'
        IF (@Display_Name IS NOT NULL)
            SET @Display_Name = @Display_Name + '%'
        IF (@Email_Address IS NOT NULL)
            SET @Email_Address = @Email_Address + '%'
    
        SELECT *
	      FROM [Users]
	      WHERE [User_Name] LIKE @User_Name
	         OR [Display_Name] LIKE @Display_Name
	         OR [Email_Address] LIKE @Email_Address
    END
    ELSE
    BEGIN
        -- Perform an exact match
         SELECT *
	      FROM [Users]
	      WHERE [User_Name] = @User_Name
	        OR [Display_Name] = @Display_Name
	        OR [Email_Address] = @Email_Address
    END

	

END
GO

GRANT EXECUTE ON [Users_SearchUsers] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SetPassword',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Users_SetPassword]') IS NOT NULL
	DROP PROCEDURE [Users_SetPassword]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_SetPassword]
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @Salt_Bytes BINARY(10)
	DECLARE @Password_Bytes BINARY(20)
	
	SET @Salt_Bytes = CAST(NEWID() AS BINARY(10))
	SET @Password_Bytes = HASHBYTES('SHA1', @Salt_Bytes + CAST(@Password_Text AS VARBINARY))
	
	UPDATE [Users]
	   SET [Password_Bytes] = @Password_Bytes
	      ,[Salt_Bytes] = @Salt_Bytes
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_SetPassword] TO [BuildMasterUser_Role]
GO


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_ValidateLogin',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Validated_Indicator',
    /* Description_Text        */ 'Validates the password of a specified user'
GO

IF OBJECT_ID('Users_ValidateLogin') IS NOT NULL DROP PROCEDURE Users_ValidateLogin
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE Users_ValidateLogin
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20),
	@Validated_Indicator YNINDICATOR = NULL OUT
) AS BEGIN

	SET NOCOUNT ON

	/* Set defaults */
	SET @Validated_Indicator = 'N'

	IF EXISTS(SELECT * FROM [Users]
	           WHERE [User_Name] = @User_Name
	             AND HASHBYTES('SHA1', [Salt_Bytes] + CAST(@Password_Text AS VARBINARY)) = [Password_Bytes])
		SET @Validated_Indicator = 'Y'


END
GO

GRANT EXECUTE ON Users_ValidateLogin TO BuildMasterUser_Role

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'VariableDeclaration_Id',
    /* Description_Text        */ 'Creates a new variable declaration or updates an existing one'
GO


IF OBJECT_ID('[Variables_CreateOrUpdateVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariableDeclaration]
(
	@VariableDeclaration_Id INT OUT,
	@Variable_Name VARCHAR(50),
	@Scope_Code CHAR(1),
	@Scoped_Application_Id INT,
	@Scoped_Environment_Id INT,
	@Required_Indicator YNINDICATOR,
	@Variable_Configuration VARCHAR(MAX),
	@DefaultValue_Text VARCHAR(MAX),
	@Sensitive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	IF @VariableDeclaration_Id IS NULL
	BEGIN
		INSERT INTO [VariableDeclarations]
		(
			[Variable_Name],
			[Scope_Code],
			[Scoped_Application_Id],
			[Scoped_Environment_Id],
			[Required_Indicator],
			[Variable_Configuration],
			[DefaultValue_Text],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
			[Sensitive_Indicator]
		)
		VALUES
		(
			@Variable_Name,
			@Scope_Code,
			@Scoped_Application_Id,
			@Scoped_Environment_Id,
			@Required_Indicator,
			@Variable_Configuration,
			@DefaultValue_Text,
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@Sensitive_Indicator
		)

		SET @VariableDeclaration_Id = SCOPE_IDENTITY()
	END
	ELSE
	BEGIN
		UPDATE [VariableDeclarations]
		   SET [Required_Indicator] = COALESCE(@Required_Indicator, [Required_Indicator]),
			   [Variable_Configuration] = COALESCE(@Variable_Configuration, [Variable_Configuration]),
			   [Scoped_Environment_Id] = @Scoped_Environment_Id,
			   [DefaultValue_Text] = @DefaultValue_Text,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Sensitive_Indicator] = @Sensitive_Indicator
		 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariableDeclaration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_CreateOrUpdateVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new variable definition or updates an existing one'
GO


IF OBJECT_ID('[Variables_CreateOrUpdateVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_CreateOrUpdateVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_CreateOrUpdateVariableDefinition]
(
	@Variable_Name VARCHAR(50),
	@Environment_Id INT,
	@Server_Id INT,
	@ApplicationGroup_Id INT,
	@Application_Id INT,
	@Deployable_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Execution_Id INT,
	@Value_Text VARCHAR(MAX),
	@Sensitive_Indicator YNINDICATOR
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Inferred_Scope_Code CHAR(1)
        SET @Inferred_Scope_Code = CASE
            WHEN @Server_Id IS NOT NULL THEN 'V'
            WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
            WHEN @Deployable_Id IS NOT NULL THEN 'D'
            WHEN @Environment_Id IS NOT NULL AND @Build_Number IS NULL THEN 'E'
            WHEN @Application_Id IS NOT NULL AND @Release_Number IS NULL THEN 'A'
            WHEN @Release_Number IS NOT NULL AND @Build_Number IS NULL THEN 'R'
            WHEN @Build_Number IS NOT NULL AND @Environment_Id IS NULL THEN 'B'
            WHEN @Environment_Id IS NOT NULL AND @Execution_Id IS NULL THEN 'P'
            WHEN @Execution_Id IS NOT NULL THEN 'X'              
            ELSE 'S' END
            
    DECLARE @VariableDeclaration_Id INT
		SET @VariableDeclaration_Id =
              (SELECT [VariableDeclaration_Id] 
                 FROM [VariableDeclarations]
                WHERE [Variable_Name] = @Variable_Name
                  AND [Scope_Code] = @Inferred_Scope_Code
                  AND ([Scope_Code] NOT IN ('R','B','P','X') OR [Scoped_Application_Id] = @Application_Id)
                  AND ([Scope_Code] NOT IN ('P','X') OR [Scoped_Environment_Id] = @Environment_Id))

    IF @VariableDeclaration_Id IS NULL AND @Inferred_Scope_Code IN ('R','B','P')
        RAISERROR('Variable %s is not a System- or Execution-scoped variable, and thus requires a declaration to be set.', 16, 1, @Variable_Name)

    DECLARE @Proper_Variable_Name VARCHAR(50)
    DECLARE @Declaration_Sensitive_Indicator YNINDICATOR

    SELECT @Proper_Variable_Name = [Variable_Name],
           @Declaration_Sensitive_Indicator = [Sensitive_Indicator]
      FROM [VariableDeclarations]
     WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

    SET @Variable_Name = COALESCE(@Proper_Variable_Name, @Variable_Name)
    SET @Sensitive_Indicator = COALESCE(@Declaration_Sensitive_Indicator, @Sensitive_Indicator)

    IF @Inferred_Scope_Code = 'R' AND 
       EXISTS(SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [ReleaseStatus_Name] <> 'Active')
       RAISERROR('Cannot set Release-Scoped Variable %s because Release %s of application id %d is not Active.', 16, 1, @Variable_Name, @Release_Number, @Application_Id)

    IF @Inferred_Scope_Code = 'B' AND 
       EXISTS(SELECT * FROM [Builds] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number AND [Build_Number] = @Build_Number AND [BuildStatus_Name] <> 'Active')
       RAISERROR('Cannot set Build-Scoped Variable %s Build %s of release %s in application id %d is not Active.', 16, 1, @Variable_Name, @Build_Number, @Release_Number, @Application_Id)

    IF @Inferred_Scope_Code = 'P' AND 
       EXISTS(SELECT * FROM [BuildPromotions] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number 
                                                AND [Build_Number] = @Build_Number AND [Environment_Id] = @Environment_Id AND [PromotionStatus_Name] ='Completed')
       RAISERROR('Cannot set Promotion-Scoped Variable %s Promotion to (EnvId=%d) for Build %s of release %s in application id %d is not Active.', 16, 1, @Variable_Name, @Environment_Id, @Build_Number, @Release_Number, @Application_Id)

    IF @Inferred_Scope_Code = 'X' AND 
       EXISTS(SELECT * FROM [BuildExecutions] WHERE [Execution_Id] = @Execution_Id AND [ExecutionStatus_Name] NOT IN ('Pending', 'Executing'))
       RAISERROR('Cannot set Execution-Scoped Variable %s Execution id %d is not executing or pending execution', 16, 1, @Variable_Name, @Execution_Id)

    DECLARE @VariableDefinition_Id INT
        SET @VariableDefinition_Id = (SELECT [VariableDefinition_Id] FROM [VariableDefinitions]
             WHERE [Variable_Name] = @Variable_Name
               AND ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
               AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
               AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
               AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))

               AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
               AND ([Release_Number] = @Release_Number OR ([Release_Number] IS NULL AND @Release_Number IS NULL))
               AND ([Build_Number] = @Build_Number OR ([Build_Number] IS NULL AND @Build_Number IS NULL))

               AND ([Execution_Id] = @Execution_Id OR ([Execution_Id] IS NULL AND @Execution_Id IS NULL)) )

    IF @VariableDefinition_Id IS NULL
    BEGIN

		INSERT INTO [VariableDefinitions] 
        (
			[Variable_Name],
			[Environment_Id],
			[Server_Id],
			[ApplicationGroup_Id],
			[Application_Id],
			[Deployable_Id], 
			[Release_Number],
			[Build_Number],
			[Execution_Id],
			[Value_Text],
			[Sensitive_Indicator]
		)
		VALUES
		(
			@Variable_Name,
			@Environment_Id,
			@Server_Id,
			@ApplicationGroup_Id,
			@Application_Id,
			@Deployable_Id,
			@Release_Number,
			@Build_Number,
			@Execution_Id,
			@Value_Text,
			@Sensitive_Indicator
		)

	END
	ELSE
	BEGIN

		UPDATE [VariableDefinitions]
		   SET [Value_Text] = @Value_Text,
		       [Sensitive_Indicator] = @Sensitive_Indicator
		 WHERE [VariableDefinition_Id] = @VariableDefinition_Id

	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_CreateOrUpdateVariableDefinition] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable declaration'
GO


IF OBJECT_ID('[Variables_DeleteVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariableDeclaration]
(
	@VariableDeclaration_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

    DELETE [VariableDefinitions]
      FROM [VariableDefinitions] VDEF
           INNER JOIN [VariableDeclarations] VDEC
                   ON VDEF.[Variable_Name] = VDEC.[Variable_Name]
                  AND (
                        (VDEC.[Scope_Code] = 'S' AND VDEF.[Environment_Id] IS NULL
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'E' AND VDEF.[Environment_Id] IS NOT NULL
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'V' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NOT NULL
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'G' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NOT NULL
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'A' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NOT NULL
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'D' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NULL 
                                                 AND VDEF.[Deployable_Id] IS NOT NULL
                                                 AND VDEF.[Release_Number] IS NULL 
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                        OR 
                        (VDEC.[Scope_Code] = 'R' AND VDEF.[Environment_Id] IS NULL 
                                                 AND VDEF.[Server_Id] IS NULL 
                                                 AND VDEF.[ApplicationGroup_Id] IS NULL 
                                                 AND VDEF.[Application_Id] IS NOT NULL
                                                 AND VDEF.[Deployable_Id] IS NULL 
                                                 AND VDEF.[Release_Number] IS NOT NULL
                                                 AND VDEF.[Build_Number] IS NULL
                                                 AND VDEF.[Execution_Id] IS NULL)
                      )               
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id
	
    DELETE [BuildScheduleVariableDefinitions]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

    DELETE [VariableDeclarations]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_DeleteVariableDeclaration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_DeleteVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a variable definition'
GO


IF OBJECT_ID('[Variables_DeleteVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_DeleteVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_DeleteVariableDefinition]
(
	@VariableDefinition_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DELETE [VariableDefinitions]
	 WHERE [VariableDefinition_Id] = @VariableDefinition_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Variables_DeleteVariableDefinition] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetSystemVariableDefinitions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_System',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable definitions for system-scoped variables.'
GO


IF OBJECT_ID('[Variables_GetSystemVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetSystemVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetSystemVariableDefinitions]
AS
BEGIN

    SET NOCOUNT ON

    SELECT * FROM [VariableDefinitions_System]


END
GO

GRANT EXECUTE ON [Variables_GetSystemVariableDefinitions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDeclaration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'VariableDeclarations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable declaration in the system that has the specified ID.'
GO


IF OBJECT_ID('[Variables_GetVariableDeclaration]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDeclaration]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDeclaration]
(
	@VariableDeclaration_Id INT
)
AS
BEGIN

	SET NOCOUNT ON
	
	SELECT *
	  FROM [VariableDeclarations]
	 WHERE [VariableDeclaration_Id] = @VariableDeclaration_Id

END
GO

GRANT EXECUTE ON [Variables_GetVariableDeclaration] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDeclarations',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDeclarations',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable declarations in the system that are defined in the specified scope'
GO


IF OBJECT_ID('[Variables_GetVariableDeclarations]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDeclarations]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDeclarations]
(
	@Scope_Code CHAR(1),
	@Scoped_Application_Id INT,
    @Scoped_Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

    SELECT * 
      FROM [VariableDeclarations]
     WHERE (@Scope_Code IS NULL OR [Scope_Code] = @Scope_Code)
       AND ([Scope_Code] NOT IN ('R','B','P','X') OR @Scoped_Application_Id IS NULL OR [Scoped_Application_Id] = @Scoped_Application_Id)
       AND ([Scope_Code] NOT IN ('P','X') OR @Scoped_Environment_Id IS NULL OR [Scoped_Environment_Id] = @Scoped_Environment_Id)
     ORDER BY [Variable_Name]

END
GO

GRANT EXECUTE ON [Variables_GetVariableDeclarations] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDefinition',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataRow',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified variable definition.'
GO


IF OBJECT_ID('[Variables_GetVariableDefinition]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDefinition]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDefinition]
(
    @VariableDefinition_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
	 WHERE [VariableDefinition_Id] = @VariableDefinition_Id


END
GO

GRANT EXECUTE ON [Variables_GetVariableDefinition] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableDefinitions',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'VariableDefinitions_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable definitions in the system that are defined in the specified scope'
GO


IF OBJECT_ID('[Variables_GetVariableDefinitions]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableDefinitions]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableDefinitions]
(
    @Environment_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    SELECT *
      FROM [VariableDefinitions_Extended]
     WHERE ([Environment_Id] = @Environment_Id OR ([Environment_Id] IS NULL AND @Environment_Id IS NULL))
       AND ([Server_Id] = @Server_Id OR ([Server_Id] IS NULL AND @Server_Id IS NULL))
       AND ([ApplicationGroup_Id] = @ApplicationGroup_Id OR ([ApplicationGroup_Id] IS NULL AND @ApplicationGroup_Id IS NULL))
       AND ([Deployable_Id] = @Deployable_Id OR ([Deployable_Id] IS NULL AND @Deployable_Id IS NULL))
       
       AND ([Application_Id] = @Application_Id OR ([Application_Id] IS NULL AND @Application_Id IS NULL))
       AND ([Release_Number] = @Release_Number OR ([Release_Number] IS NULL AND @Release_Number IS NULL))
       AND ([Build_Number] = @Build_Number OR ([Build_Number] IS NULL AND @Build_Number IS NULL))
       
       AND ([Execution_Id] = @Execution_Id OR ([Execution_Id] IS NULL AND @Execution_Id IS NULL))


END
GO

GRANT EXECUTE ON [Variables_GetVariableDefinitions] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableValues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variable_Values',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable values for the specified scope.'
GO

IF OBJECT_ID('[Variables_GetVariableValues]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableValues]
(
    @Environment_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF @Execution_Id IS NOT NULL
    BEGIN
		-- Use standard function for execution-scoped variable cascading
		SELECT *
		  FROM dbo.Variables_GetValuesForExecution(@Execution_Id, @Server_Id, @Deployable_Id)
    END
    ELSE
    BEGIN

		-- If deployable is specified but not application, find the application
		IF @Deployable_Id IS NOT NULL AND @Application_Id IS NULL
		BEGIN
			SET @Application_Id = (SELECT [Application_Id]
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)
		END

		-- If application is specified but not application group find the group
		IF @Application_Id IS NOT NULL AND @ApplicationGroup_Id IS NULL
		BEGIN
			SET @ApplicationGroup_Id = (SELECT [ApplicationGroup_Id]
			                              FROM [Applications]
			                             WHERE [Application_Id] = @Application_Id)
		END

		-- Make sure irrelevant release number is not considered
		IF @Release_Number IS NOT NULL AND @Application_Id IS NULL
			SET @Release_Number = NULL

		-- Make sure irrelevant build number is not considered
		IF @Build_Number IS NOT NULL AND @Release_Number IS NULL
			SET @Build_Number = NULL

		-- Determine the narrowest scope that will be considered			
		DECLARE @DesiredScope_Code CHAR(1)
		    SET @DesiredScope_Code = CASE
				WHEN @Environment_Id IS NOT NULL AND @Build_Number IS NOT NULL THEN 'P'
				WHEN @Build_Number IS NOT NULL THEN 'B'
				WHEN @Release_Number IS NOT NULL THEN 'R'
				WHEN @Deployable_Id IS NOT NULL THEN 'D'
				WHEN @Application_Id IS NOT NULL THEN 'A'
				WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
				WHEN @Server_Id IS NOT NULL THEN 'V'
				WHEN @Environment_Id IS NOT NULL THEN 'E'
				ELSE 'S'
			END

		DECLARE @DesiredScope_Index INT
		    SET @DesiredScope_Index = CHARINDEX(@DesiredScope_Code, 'SEVGADRBPX')

		--Return the results by joining against all variable names in the system and filtering based on input criteria
		--The inner SELECT DISTINCTS required that the VDEF and VDEC tables need an index on the Variable_Name columns for decent performance
        SELECT [Variable_Name],
               [Value_Text],
               [Scope_Code],
               [Sensitive_Indicator]

          FROM (SELECT VN.[Variable_Name],
                       [Value_Text] = COALESCE(VDEF.[Value_Text], VDEC.[DefaultValue_Text]),
                       [Scope_Code] = COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]),
                       [Sensitive_Indicator] = COALESCE(VDEF.[Sensitive_Indicator], VDEC.[Sensitive_Indicator]),
                       [Rank] = RANK() OVER (PARTITION BY VN.[Variable_Name] ORDER BY CHARINDEX(COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]), 'SEVGADRBPX') DESC)
                  FROM (SELECT DISTINCT [Variable_Name]
						  FROM [VariableDeclarations]
                               UNION
                        SELECT DISTINCT [Variable_Name]
                          FROM [VariableDefinitions]) VN

                       LEFT JOIN [VariableDefinitions] VDEF
                              ON VN.[Variable_Name] = VDEF.[Variable_Name]
                             AND CHARINDEX(VDEF.[Scope_Code], 'SEVGADRBPX') <= @DesiredScope_Index
                             AND (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = @Application_Id)
                             AND (VDEF.[Release_Number] IS NULL OR (VDEF.[Application_Id] = @Application_Id AND VDEF.[Release_Number] = @Release_Number))
                             AND (VDEF.[Build_Number] IS NULL OR (VDEF.[Application_Id] = @Application_Id AND VDEF.[Release_Number] = @Release_Number AND VDEF.[Build_Number] = @Build_Number))
                             AND (VDEF.[ApplicationGroup_Id] IS NULL OR VDEF.[ApplicationGroup_Id] = @ApplicationGroup_Id)
                             AND (VDEF.[Deployable_Id] IS NULL OR VDEF.[Deployable_Id] = @Deployable_Id)
                             AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = @Environment_Id)
                             AND (VDEF.[Server_Id] IS NULL OR VDEF.[Server_Id] = @Server_Id)
                       LEFT JOIN [VariableDeclarations] VDEC
                              ON VN.[Variable_Name] = VDEC.[Variable_Name]
                             AND CHARINDEX(VDEC.[Scope_Code], 'SEVGADRBPX') <= @DesiredScope_Index
                             AND (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = @Application_Id)
                             AND (VDEC.[Scoped_Environment_Id] IS NULL OR (VDEC.[Scoped_Application_Id] = @Application_Id AND VDEC.[Scoped_Environment_Id] = @Environment_Id))
                 WHERE VDEF.[Scope_Code] = VDEC.[Scope_Code]
                    OR (VDEF.[Scope_Code] IS NULL AND VDEC.[Scope_Code] IS NOT NULL)
                    OR (VDEC.[Scope_Code] IS NULL AND VDEF.[Scope_Code] IS NOT NULL)) _
         WHERE [Rank] = 1

    END

END
GO

GRANT EXECUTE ON [Variables_GetVariableValues] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_AddOrRemoveApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Approval_Sequence,AlreadyExists_Indicator',
    /* Description_Text        */ 'Adds or removes an approval to the specified workflow step'
GO


IF OBJECT_ID('[Workflows_AddOrRemoveApproval]') IS NOT NULL DROP PROCEDURE [Workflows_AddOrRemoveApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_AddOrRemoveApproval]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@Approval_Sequence INT = NULL OUT,
	@User_Name VARCHAR(255) = NULL,
    @Approval_Description VARCHAR(100) = NULL,
	@ApprovalType_Code CHAR(1) = NULL,
	@Or_Indicator YNINDICATOR = NULL,
	@RequiredInGroup_Count INT = NULL,
	@AddRemove_Indicator CHAR(1),
	@AlreadyExists_Indicator YNINDICATOR = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SET @AlreadyExists_Indicator = 'N'

	IF (@AddRemove_Indicator = 'A')
	BEGIN

		IF  @Approval_Sequence IS NULL
		BEGIN
			IF EXISTS(SELECT *
						FROM [PromotionApprovalRequirements]
					   WHERE [Workflow_Id] = @Workflow_Id
						 AND [Environment_Id] = @Environment_Id
						 AND [User_Name] = @User_Name
						 AND [ApprovalType_Code] = @ApprovalType_Code
                         AND [Approval_Description] = @Approval_Description)
			BEGIN
				/* Approval requirement already exists */
				SET @AlreadyExists_Indicator = 'Y'
				ROLLBACK RETURN
			END

			/* Create new approval requirement */
			SELECT @Approval_Sequence = COALESCE(MAX([Approval_Sequence]), 0) + 1
			  FROM [PromotionApprovalRequirements]
			 WHERE [Workflow_Id] = @Workflow_Id
			   AND [Environment_Id] = @Environment_Id

			INSERT INTO [PromotionApprovalRequirements]
	           ([Workflow_Id]
	           ,[Environment_Id]
	           ,[User_Name]
               ,[Approval_Description]
			   ,[ApprovalType_Code]
			   ,[Or_Indicator]
			   ,[RequiredInGroup_Count]
			   ,[Approval_Sequence]
			   ,[CreatedBy_User_Name]
			   ,[CreatedOn_Date]
			   ,[ModifiedBy_User_Name]
			   ,[ModifiedOn_Date])
			VALUES
	           (@Workflow_Id
	           ,@Environment_Id
	           ,@User_Name
               ,@Approval_Description
			   ,@ApprovalType_Code
			   ,@Or_Indicator
			   ,@RequiredInGroup_Count
			   ,@Approval_Sequence
			   ,dbo.[Session_GetUserName]()
			   ,GETUTCDATE()
			   ,dbo.[Session_GetUserName]()
			   ,GETUTCDATE())
		END
		ELSE
		BEGIN
			/* Existing approval; edit */
			IF EXISTS(SELECT *
						FROM [PromotionApprovalRequirements]
					   WHERE [Workflow_Id] = @Workflow_Id
						 AND [Environment_Id] = @Environment_Id
						 AND [Approval_Sequence] <> @Approval_Sequence
						 AND [User_Name] = @User_Name
                         AND [Approval_Description] = @Approval_Description)
			BEGIN
				SET @AlreadyExists_Indicator = 'Y'
				ROLLBACK RETURN
			END

			UPDATE [PromotionApprovalRequirements]
			   SET [User_Name] = @User_Name
                  ,[Approval_Description] = @Approval_Description
				  ,[ModifiedBy_User_Name] = dbo.[Session_GetUserName]()
				  ,[ModifiedOn_Date] = GETUTCDATE()
				  ,[ApprovalType_Code] = @ApprovalType_Code
				  ,[Or_Indicator] = @Or_Indicator
				  ,[RequiredInGroup_Count] = @RequiredInGroup_Count
			 WHERE [Workflow_Id] = @Workflow_Id
			   AND [Environment_Id] = @Environment_Id
			   AND [Approval_Sequence] = @Approval_Sequence
		END
	END
	
	IF (@AddRemove_Indicator = 'R')
	BEGIN
		IF @Approval_Sequence IS NULL
		BEGIN
			RAISERROR('@Approval_Sequence is required to remove an approval.', 16, 1)
		END


		DELETE
		  FROM [PromotionApprovalRequirements]
		 WHERE [Workflow_Id] = @Workflow_Id
           AND [Environment_Id] = @Environment_Id
           AND [Approval_Sequence] = @Approval_Sequence

		/* Move subsequent approvals up */
		UPDATE [PromotionApprovalRequirements]
		   SET [Approval_Sequence] = [Approval_Sequence] - 1
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [Approval_Sequence] > @Approval_Sequence

	END

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [Workflows]
	                            WHERE [Workflow_Id] = @Workflow_Id)
	EXEC [Events_RaiseEvent] 'WKFMDS', @Application_Id, @Workflow_Id, @Environment_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_AddOrRemoveApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_CreateOrUpdateWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Workflow_Id',
    /* Description_Text        */ 'Adds a new workflow to an application, or updates an existing one'
GO

IF OBJECT_ID('[Workflows_CreateOrUpdateWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_CreateOrUpdateWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_CreateOrUpdateWorkflow]
(
	@Workflow_Id INT = NULL OUT,
	@Application_Id INT,
	@Workflow_Name VARCHAR(20),
	@Active_Indicator YNINDICATOR = NULL,
    @AutoIncrementRelease_Indicator YNINDICATOR = NULL,
    @AutoDeployRelease_Indicator YNINDICATOR = NULL,
    @AllowOutOfSequencePromotions_Indicator YNINDICATOR = NULL,
    @AutoCancelReleases_Indicator YNINDICATOR = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF @Workflow_Id IS NULL
	BEGIN
		SET @Active_Indicator = COALESCE(@Active_Indicator, 'Y')

		/* Create new workflow */
		INSERT INTO [Workflows]
			(
				[Application_Id], 
				[Workflow_Name], 
				[Active_Indicator],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
                [AutoIncrementRelease_Indicator],
				[AutoDeployRelease_Indicator],
                [AllowOutOfSequencePromotions_Indicator],
                [AutoCancelReleases_Indicator]
			)
		VALUES
			(
				@Application_Id, 
				@Workflow_Name, 
				@Active_Indicator,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
                @AutoIncrementRelease_Indicator,
				@AutoDeployRelease_Indicator,
                @AllowOutOfSequencePromotions_Indicator,
                @AutoCancelReleases_Indicator
			)
		SET @Workflow_Id = SCOPE_IDENTITY()
		EXEC [Events_RaiseEvent] 'WKFNEW', @Application_Id, @Workflow_Id	
	END
	ELSE
	BEGIN

		/* Update Workflow */
		UPDATE [Workflows]
		   SET [Workflow_Name] = @Workflow_Name,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
			   [Active_Indicator] = COALESCE(@Active_Indicator, [Active_Indicator]),
               [AutoIncrementRelease_Indicator] = @AutoIncrementRelease_Indicator,
               [AutoDeployRelease_Indicator] = @AutoDeployRelease_Indicator,
               [AllowOutOfSequencePromotions_Indicator] = @AllowOutOfSequencePromotions_Indicator,
               [AutoCancelReleases_Indicator] = @AutoCancelReleases_Indicator
		 WHERE [Workflow_Id] = @Workflow_Id
		EXEC [Events_RaiseEvent] 'WKFMOD', @Application_Id, @Workflow_Id
	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_CreateOrUpdateWorkflow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_DeleteStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a workflow step'
GO


IF OBJECT_ID('[Workflows_DeleteStep]') IS NOT NULL DROP PROCEDURE [Workflows_DeleteStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_DeleteStep]
(
	@Workflow_Id INT,
	@Step_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Environment_Id INT, @Application_Id INT
	    SET @Environment_Id = (SELECT [Environment_Id] 
	                             FROM [WorkflowSteps]
	                            WHERE [Workflow_Id] = @Workflow_Id
	                              AND [Step_Sequence] = @Step_Sequence)
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [Workflows]
	                            WHERE [Workflow_Id] = @Workflow_Id)
	DELETE FROM [PromotionApprovalRequirements]
		  WHERE [Workflow_Id] = @Workflow_Id
			AND [Environment_Id] = @Environment_Id
			
	DELETE [PromotionRequirements]
	 WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id
	   
	-- move the workflow step to the end of the list to appease the sequencing trigger before deletion
	DECLARE @Last_Step_Sequence INT
	SELECT @Last_Step_Sequence = MAX([Step_Sequence])
	  FROM [WorkflowSteps]
	 WHERE [Workflow_Id] = @Workflow_Id
	  
	UPDATE [WorkflowSteps] SET [Step_Sequence] = CASE
	  WHEN [Step_Sequence] = @Step_Sequence THEN @Last_Step_Sequence  -- Move Target Item
      WHEN (@Last_Step_Sequence > @Step_Sequence) AND [Step_Sequence] > @Step_Sequence AND [Step_Sequence] <= @Last_Step_Sequence THEN [Step_Sequence] - 1  --If "Move Up (@New_Step_Sequence > @Step_Sequence)"
      WHEN (@Last_Step_Sequence < @Step_Sequence) AND [Step_Sequence] < @Step_Sequence AND [Step_Sequence] >= @Last_Step_Sequence THEN [Step_Sequence] + 1  --If "Move Down (@New_Step_Sequence < @Step_Sequence)"
      ELSE [Step_Sequence] END
	 WHERE [Workflow_Id] = @Workflow_Id   
	
	DELETE FROM [WorkflowSteps]
		  WHERE [Workflow_Id] = @Workflow_Id
			AND [Step_Sequence] = @Last_Step_Sequence

    EXEC [Events_RaiseEvent] 'WKFDLS', @Application_Id, @Workflow_Id, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_DeleteStep] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetPromotionApprovalRequirements',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'PromotionApprovalRequirements',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of approvals required for the specified environment in the workflow'
GO


IF OBJECT_ID('[Workflows_GetPromotionApprovalRequirements]') IS NOT NULL DROP PROCEDURE [Workflows_GetPromotionApprovalRequirements]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetPromotionApprovalRequirements]
(
	@Workflow_Id INT,
	@Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [PromotionApprovalRequirements]
     WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id
	ORDER BY [Approval_Sequence]

END
GO

GRANT EXECUTE ON [Workflows_GetPromotionApprovalRequirements] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetWorkflow',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Workflows_Extended,WorkflowSteps_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a specific workflow and all of its workflow steps'
GO


IF OBJECT_ID('[Workflows_GetWorkflow]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflow]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetWorkflow]
(
	@Workflow_Id INT
)
AS
BEGIN

	SET NOCOUNT ON 

	SELECT * 
	  FROM [Workflows_Extended]
     WHERE [Workflow_Id] = @Workflow_Id
	IF (@@ROWCOUNT = 0) RAISERROR('Workflow %i does not exist.', 16, 1, @Workflow_Id)

	SELECT * 
	  FROM [WorkflowSteps_Extended]
     WHERE [Workflow_Id] = @Workflow_Id
	ORDER BY [Step_Sequence]

END
GO

GRANT EXECUTE ON [Workflows_GetWorkflow] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetWorkflows',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Workflows_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of all workflows for an application'
GO


IF OBJECT_ID('[Workflows_GetWorkflows]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflows]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetWorkflows]
(
	@Application_Id INT,
	@IncludeInactive_Indicator YNINDICATOR = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SET @IncludeInactive_Indicator = ISNULL(@IncludeInactive_Indicator, 'N')

	SELECT *
	  FROM [Workflows_Extended]
     WHERE [Application_Id] = @Application_Id
	   AND (	@IncludeInactive_Indicator = 'Y'
			 OR [Active_Indicator] = 'Y' )
	ORDER BY CASE [Workflow_Name] WHEN 'Standard' THEN 1 ELSE 2 END
	         ,CASE [Active_Indicator] WHEN 'Y' THEN 1 ELSE 2 END
			 ,[Workflow_Name]

END
GO

GRANT EXECUTE ON [Workflows_GetWorkflows] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_GetWorkflowStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'WorkflowSteps_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified workflow step'
GO


IF OBJECT_ID('[Workflows_GetWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_GetWorkflowStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_GetWorkflowStep]
(
	@Workflow_Id INT,
	@Environment_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT * 
	  FROM [WorkflowSteps_Extended]
     WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id

END
GO

GRANT EXECUTE ON [Workflows_GetWorkflowStep] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_MoveApproval',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves an approval up or down in sequence'
GO

IF OBJECT_ID('[Workflows_MoveApproval]') IS NOT NULL
	DROP PROCEDURE [Workflows_MoveApproval]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_MoveApproval]
(
	@Workflow_Id INT,
	@Environment_Id INT,
	@Approval_Sequence INT,
	@Direction_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF @Direction_Code NOT IN ('U', 'D') RETURN

	DECLARE @Old_Sequence INT,
			@New_Sequence INT

	/* Figure out the old and new step sequences */
	SET @Old_Sequence = @Approval_Sequence
	IF @Direction_Code = 'U'
	BEGIN
		/* Moving up */
		SELECT @New_Sequence = COALESCE(MAX([Approval_Sequence]), 1)
		  FROM [PromotionApprovalRequirements]
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [Approval_Sequence] < @Approval_Sequence
	END
	ELSE
	BEGIN
		/* Moving down */
		SELECT @New_Sequence = COALESCE(MIN([Approval_Sequence]), @Approval_Sequence)
		  FROM [PromotionApprovalRequirements]
		 WHERE [Workflow_Id] = @Workflow_Id
		   AND [Environment_Id] = @Environment_Id
		   AND [Approval_Sequence] > @Approval_Sequence
	END

	IF @Old_Sequence = @New_Sequence ROLLBACK RETURN

	/* Swap sequences */
	UPDATE [PromotionApprovalRequirements]
	   SET [Approval_Sequence] = CASE [Approval_Sequence]
								 WHEN @Old_Sequence THEN @New_Sequence
								 WHEN @New_Sequence THEN @Old_Sequence
								 ELSE [Approval_Sequence] END
	 WHERE [Workflow_Id] = @Workflow_Id
	   AND [Environment_Id] = @Environment_Id


	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [Workflows]
	                            WHERE [Workflow_Id] = @Workflow_Id)
	EXEC [Events_RaiseEvent] 'WKFMDS', @Application_Id, @Workflow_Id, @Environment_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_MoveApproval] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_MoveWorkflowStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Moves a workflow step to a new sequence'
GO


IF OBJECT_ID('[Workflows_MoveWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_MoveWorkflowStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_MoveWorkflowStep]
(
	@Workflow_Id INT,
	@Step_Sequence INT,
	@New_Step_Sequence INT
)
AS
BEGIN

    DECLARE @Environment_Id INT, @Application_Id INT
	    SET @Environment_Id = (SELECT [Environment_Id] 
	                             FROM [WorkflowSteps]
	                            WHERE [Workflow_Id] = @Workflow_Id
	                              AND [Step_Sequence] = @Step_Sequence)
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [Workflows]
	                            WHERE [Workflow_Id] = @Workflow_Id)

	IF NOT EXISTS(SELECT * FROM [WorkflowSteps] WHERE [Workflow_Id] = @Workflow_Id AND [Step_Sequence] = @New_Step_Sequence)
	BEGIN
		/* Silently Return or throw an error*/
		Return;
	END
	
	IF EXISTS(SELECT *
			  FROM [Builds] B
				   INNER JOIN [Releases] R
				         ON B.[Application_Id] = @Application_Id
						AND B.[Release_Number] = R.[Release_Number]
			  WHERE R.[Workflow_Id] = @Workflow_Id AND [BuildStatus_Name] = 'Active')
	BEGIN
		/* Silently Return or throw an error*/
		Return;
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	UPDATE [WorkflowSteps] SET [Step_Sequence] = CASE
	  WHEN [Step_Sequence] = @Step_Sequence THEN @New_Step_Sequence  -- Move Target Item
      WHEN (@New_Step_Sequence > @Step_Sequence) AND [Step_Sequence] > @Step_Sequence AND [Step_Sequence] <= @New_Step_Sequence THEN [Step_Sequence] - 1  --If "Move Up (@New_Step_Sequence > @Step_Sequence)"
      WHEN (@New_Step_Sequence < @Step_Sequence) AND [Step_Sequence] < @Step_Sequence AND [Step_Sequence] >= @New_Step_Sequence THEN [Step_Sequence] + 1  --If "Move Down (@New_Step_Sequence < @Step_Sequence)"
      ELSE [Step_Sequence] END
	 WHERE [Workflow_Id] = @Workflow_Id


	EXEC [Events_RaiseEvent] 'WKFMDS', @Application_Id, @Workflow_Id, @Environment_Id


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Workflows_MoveWorkflowStep] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_InitiateBuildExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Execution_Id',
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Builds_InitiateBuildExecution]') IS NOT NULL DROP PROCEDURE [Builds_InitiateBuildExecution]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_InitiateBuildExecution]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@Start_Date DATETIME = NULL,
	@ContinueFromLastFailed_Indicator YNINDICATOR = 'N',
    @Variables_Xml XML = NULL,
	@Execution_Id INT = NULL OUT
)
AS
BEGIN
	SET @Start_Date = COALESCE(@Start_Date, GETUTCDATE())

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @LatestExecution_Id INT
	SELECT TOP 1 
		@LatestExecution_Id = [Execution_Id]
      FROM [BuildExecutions] 
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	   AND [Environment_Id] = @Environment_Id
  ORDER BY [Execution_Id] DESC

	IF (@ContinueFromLastFailed_Indicator = 'Y')
	BEGIN
		
		DECLARE @Eligible_Indicator CHAR(1)
		DECLARE @IneligibleReason_Text VARCHAR(MAX)
		EXEC [Executions_ValidateForReexecutionFromLastFailed] 
			@Execution_Id = @LatestExecution_Id, 
			@Eligible_Indicator = @Eligible_Indicator OUT, 
			@IneligibleReason_Text = @IneligibleReason_Text OUT
		SET @IneligibleReason_Text = 'Cannot re-execute from last failed action because: ' + @IneligibleReason_Text
		
		IF (@Eligible_Indicator = 'N')
			RAISERROR(@IneligibleReason_Text, 16, 1)

	END

	INSERT INTO [BuildExecutions]
			   ([Application_Id],
				[Release_Number],
				[Build_Number],
				[Environment_Id],
				[ExecutionStart_Date],
				[ExecutionStatus_Name],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[PlanModifiedSinceLastExecution_Indicator],
				[LastActionReexecutionOf_Execution_Id],
				[WarningLogEntry_Indicator])
		 VALUES
			   (@Application_Id,
				@Release_Number,
				@Build_Number,
				@Environment_Id,
				@Start_Date,
				'Pending',
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				'N',
				CASE WHEN @ContinueFromLastFailed_Indicator = 'Y' THEN @LatestExecution_Id
					 ELSE NULL 
					END,
				'N')
	SET @Execution_Id = SCOPE_IDENTITY()

	IF @Variables_Xml IS NOT NULL BEGIN

		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Environment_Id],
			[Application_Id],
			[Release_Number],
			[Build_Number],
            [Execution_Id],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)'),
		       [Environment_Id] = @Environment_Id,
		       [Application_Id] = @Application_Id,
		       [Release_Number] = @Release_Number,
		       [Build_Number] = @Build_Number,
               [Execution_Id] = @Execution_Id,
			   [Value_Text] = LIST.[Variables].[value]('@Value', 'VARCHAR(MAX)'),
			   [Sensitive_Indicator] = COALESCE(VDEC.[Sensitive_Indicator], 'N')
		  FROM @Variables_Xml.[nodes]('/Variables/Variable') AS LIST([Variables])
		       LEFT JOIN [VariableDeclarations] VDEC
		              ON VDEC.[Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)')
		             AND VDEC.[Scoped_Application_Id] = @Application_Id
		             AND VDEC.[Scoped_Environment_Id] = @Environment_Id
		             AND VDEC.[Scope_Code] = 'X'

	END

	INSERT INTO [BuildExecution_Plans] ([Execution_Id], [PlanExecution_Sequence], [Deployable_Name], [ActionGroup_Name], [Original_PlanActionGroup_Id])
	SELECT DISTINCT
		BE.[Execution_Id],
		PAG.[PlanActionGroup_Sequence],
		D.[Deployable_Name],
		COALESCE(PAG.[ActionGroup_Name], P.[ActionGroup_Name]),
		PAG.[PlanActionGroup_Id]

	  FROM [BuildExecutions] BE
		   INNER JOIN [Releases] R
		           ON R.[Application_Id] = BE.[Application_Id]
		          AND R.[Release_Number] = BE.[Release_Number]
		   INNER JOIN [WorkflowSteps] WS
				   ON WS.[Workflow_Id] = R.[Workflow_Id]
				  AND WS.[Environment_Id] = BE.[Environment_Id]
		   INNER JOIN [PlanActionGroups] PAG
				   ON PAG.[Environment_Id] = WS.[Environment_Id]
				      AND PAG.[Application_Id] = BE.[Application_Id]
		   INNER JOIN [Plans] P
				   ON PAG.[ActionGroup_Id] = P.[Plan_Id]
		   LEFT JOIN [Deployables] D
				   ON D.[Deployable_Id] = PAG.[Deployable_Id] 
	 WHERE [Execution_Id] = @Execution_Id
	   AND (PAG.[Active_Indicator] = 'Y')
		/* If a plan's Deployable_Id is null, always execute;
		   If not null, make sure it's in the ReleaseDeployables table. */
	   AND (	PAG.[Deployable_Id] IS NULL
			 OR PAG.[Deployable_Id] IN ( SELECT _RS.[Deployable_Id]
									   FROM [ReleaseDeployables] _RS
									  WHERE _RS.[Application_Id] = @Application_Id
										AND _RS.[Release_Number] = @Release_Number ) )

	IF (@ContinueFromLastFailed_Indicator = 'Y')
	BEGIN
        -- copy variables
        INSERT INTO [VariableDefinitions] ([Variable_Name]
                   ,[Environment_Id],[Server_Id],[ApplicationGroup_Id]
                   ,[Application_Id],[Deployable_Id]
                   ,[Release_Number],[Build_Number],[Value_Text],[Sensitive_Indicator],[Execution_Id])
        SELECT      [Variable_Name]
                   ,[Environment_Id],[Server_Id],[ApplicationGroup_Id]
                   ,[Application_Id],[Deployable_Id]
                   ,[Release_Number],[Build_Number],[Value_Text],[Sensitive_Indicator],@Execution_Id
        FROM        [VariableDefinitions] 
        WHERE       [Execution_Id] = @LatestExecution_Id

		-- copy actions (and their logs) that have already run without failure from the previous execution into the current execution
		DECLARE @LastFailed_BuildExecution_PlanAction_Id INT
	    SELECT TOP 1 
	       @LastFailed_BuildExecution_PlanAction_Id = MIN([BuildExecution_PlanAction_Id])
			  FROM [BuildExecution_PlanActions]
			 WHERE Execution_Id = @LatestExecution_Id
		  GROUP BY [Original_PlanActionGroup_Id], [Original_Action_Sequence]
		  ORDER BY MIN([BuildExecution_PlanAction_Id]) DESC

		DECLARE @ExecutionStarted_Date DATETIME
		DECLARE @ExecutionEnded_Date DATETIME
		DECLARE @ExecutionStatus_Name VARCHAR(10)
		DECLARE @ExecutionAction_Description VARCHAR(500)
		DECLARE @Original_Action_Sequence INT 
		DECLARE @Server_Id INT
		DECLARE @Original_Server_Name VARCHAR(50)
		DECLARE @Original_PlanActionGroup_Id INT
		DECLARE @Attempt_Sequence INT
        DECLARE @WarningLogEntry_Indicator YNINDICATOR
		DECLARE @PreviousBuildExecution_PlanAction_Id INT
		DECLARE @BuildExecution_PlanAction_Id INT
		DECLARE @BEP_Cursor CURSOR 
			
		SET @BEP_Cursor = CURSOR FOR
			(SELECT  [BuildExecution_PlanAction_Id]
					,[ExecutionStarted_Date]
					,[ExecutionEnded_Date]
					,[ExecutionStatus_Name]
					,[ExecutionAction_Description]
					,[Original_Action_Sequence]
					,[Server_Id]
					,[Original_Server_Name]
					,[Original_PlanActionGroup_Id] 
					,[Attempt_Sequence]
                    ,[WarningLogEntry_Indicator]
				FROM [BuildExecution_PlanActions]
				WHERE [Execution_Id] = @LatestExecution_Id
				AND [BuildExecution_PlanAction_Id] < @LastFailed_BuildExecution_PlanAction_Id)

		OPEN @BEP_Cursor
		FETCH NEXT FROM @BEP_Cursor
		INTO @PreviousBuildExecution_PlanAction_Id, @ExecutionStarted_Date, @ExecutionEnded_Date, @ExecutionStatus_Name, @ExecutionAction_Description, @Original_Action_Sequence, @Server_Id, @Original_Server_Name, @Original_PlanActionGroup_Id, @Attempt_Sequence, @WarningLogEntry_Indicator

		WHILE @@FETCH_STATUS <> -1
		BEGIN

			INSERT INTO [BuildExecution_PlanActions]  
			(
				 [Execution_Id]
				,[ExecutionStarted_Date]
				,[ExecutionEnded_Date]
				,[ExecutionStatus_Name]
				,[ExecutionAction_Description]
				,[Original_Action_Sequence]
				,[Server_Id]
				,[Original_Server_Name]
				,[Original_PlanActionGroup_Id]
				,[Attempt_Sequence]
                ,[WarningLogEntry_Indicator]
			)
			VALUES 
			(
				 @Execution_Id
				,@ExecutionStarted_Date
				,@ExecutionEnded_Date
				,@ExecutionStatus_Name
				,@ExecutionAction_Description
				,@Original_Action_Sequence
				,@Server_Id
				,@Original_Server_Name
				,@Original_PlanActionGroup_Id
				,@Attempt_Sequence
                ,@WarningLogEntry_Indicator
			)
				
			SET @BuildExecution_PlanAction_Id = SCOPE_IDENTITY()

			INSERT INTO [BuildExecution_PlanActionLogEntries]
			(
				 [LogEntry_Sequence]
				,[LogEntry_Level]
				,[LogEntry_Text]
				,[BuildExecution_PlanAction_Id]
			)
			SELECT
				 [LogEntry_Sequence]
				,[LogEntry_Level]
				,[LogEntry_Text]
				,@BuildExecution_PlanAction_Id
			FROM [BuildExecution_PlanActionLogEntries]
			WHERE [BuildExecution_PlanAction_Id] = @PreviousBuildExecution_PlanAction_Id

            INSERT INTO [BuildExecution_PlanActionVariableValues]
            (
                [BuildExecution_PlanAction_Id]
               ,[Variable_Name]
               ,[Value_Text]
            )
            SELECT
                @BuildExecution_PlanAction_Id
               ,[Variable_Name]
               ,[Value_Text]
            FROM [BuildExecution_PlanActionVariableValues]
            WHERE [BuildExecution_PlanAction_Id] = @PreviousBuildExecution_PlanAction_Id

			FETCH NEXT FROM @BEP_Cursor
			INTO @PreviousBuildExecution_PlanAction_Id, @ExecutionStarted_Date, @ExecutionEnded_Date, @ExecutionStatus_Name, @ExecutionAction_Description, @Original_Action_Sequence, @Server_Id, @Original_Server_Name, @Original_PlanActionGroup_Id, @Attempt_Sequence, @WarningLogEntry_Indicator

		END
		CLOSE @BEP_Cursor
		DEALLOCATE @BEP_Cursor

	END

	EXEC [Events_RaiseEvent] 'EXEINI', @Application_Id, @Execution_Id

	/* If no plans or actions exist, it will never get picked up by the executor, so complete it now  */
	IF NOT EXISTS(SELECT *
	                FROM [PlanActions] PA
	               WHERE PA.[Active_Indicator] = 'Y'
	                 AND PA.[Plan_Id] IN (SELECT P.[Plan_Id]
											FROM [PlanActionGroups] PAG
											 INNER JOIN [Plans] P
												     ON PAG.[ActionGroup_Id] = P.[Plan_Id]
										   WHERE (PAG.[Active_Indicator] = 'Y')
										     AND [PlanActionGroup_Id] IN (SELECT [Original_PlanActionGroup_Id]
											FROM [BuildExecution_Plans]
											WHERE [Execution_Id] = @Execution_Id)))
	BEGIN
		UPDATE [BuildExecutions] SET [ExecutionStatus_Name]='Succeeded'
		 WHERE [Execution_Id] = @Execution_Id
		EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

		EXEC [Builds_CompletePromotion]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number,
			@Build_Number = @Build_Number,
			@Environment_Id = @Environment_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Builds_InitiateBuildExecution] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PromoteBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Promotes a build to its next environment in its workflow'
GO


IF OBJECT_ID('[Builds_PromoteBuild]') IS NOT NULL DROP PROCEDURE [Builds_PromoteBuild]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PromoteBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Comments_Text VARCHAR(255) = NULL,
	@ExecutionStart_Date DATETIME = NULL,
	@ForcePromotion_Indicator YNINDICATOR = 'N',
	@DependencyContingent_Indicator YNINDICATOR = 'N',
	@PromoteTo_Environment_Id INT,
	@Promotion_Variables_Xml XML = NULL,
    @Execution_Variables_Xml XML = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Eligible_Indicator CHAR(1)
		   ,@IneligibleReason_Text VARCHAR(100)
		   ,@Workflow_Id INT
		   ,@Next_Step_Sequence INT

	SELECT @Next_Step_Sequence = [Next_Step_Sequence],
	       @Workflow_Id = [Workflow_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number

	EXEC [Builds_ValidateForPromotion]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number, 
			@Build_Number = @Build_Number,
			@Eligible_Indicator = @Eligible_Indicator OUT,
			@IneligibleReason_Text = @IneligibleReason_Text OUT,
			@ForcePromotion_Indicator = @ForcePromotion_Indicator,
			@DependencyContingent_Indicator = @DependencyContingent_Indicator,
			@PromoteTo_Environment_Id = @PromoteTo_Environment_Id
		
	IF (@Eligible_Indicator = 'N')
		RAISERROR('Cannot promote build. %s', 16, 1, @IneligibleReason_Text)

	/* Reject existing builds in environment if multiple active builds are disabled */
	IF NOT EXISTS(SELECT *
					FROM [dbo].[Applications]
				   WHERE [Application_Id] = @Application_Id
					 AND [AllowMultipleActiveBuilds_Indicator] = 'Y')
	BEGIN
		DECLARE Builds_Cursor CURSOR FOR
		SELECT [Build_Number]
		  FROM [dbo].[Builds_Extended]
		 WHERE [Current_Environment_Id] = @PromoteTo_Environment_Id
		   AND [BuildStatus_Name] = 'Active'
		   AND [Release_Number] = @Release_Number
		   AND [Application_Id] = @Application_Id

		DECLARE @tmp_Build_Number BUILDNUMBER

		OPEN Builds_Cursor
		FETCH NEXT FROM Builds_Cursor INTO @tmp_Build_Number
		WHILE @@FETCH_STATUS <> -1
		BEGIN
			EXEC [dbo].[Builds_RejectBuild]
				@Application_Id = @Application_Id,
				@Release_Number = @Release_Number,
				@Build_Number = @tmp_Build_Number,
				@Rejection_Notes = 'Build rejected due to promotion of another build in the same environment.'

			IF @@ERROR<>0
			BEGIN
				ROLLBACK
				CLOSE Builds_Cursor
				DEALLOCATE Builds_Cursor
				RETURN
			END

			FETCH NEXT FROM Builds_Cursor INTO @tmp_Build_Number
		END

		CLOSE Builds_Cursor
		DEALLOCATE Builds_Cursor
	END
	
	INSERT INTO [BuildPromotions]
		([Application_Id], [Release_Number], [Build_Number], [Environment_Id],
		 [PromotedBy_Name], [Comments_Text], [PromotionStatus_Name], [Forced_Indicator], [Promoted_Date])
	VALUES
		(@Application_Id, @Release_Number, @Build_Number, @PromoteTo_Environment_Id,
		 dbo.Session_GetUserName(), @Comments_Text, 'AwaitingExecution', @ForcePromotion_Indicator, GETUTCDATE())
         
    IF @Promotion_Variables_Xml IS NOT NULL BEGIN
	
		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Environment_Id],
			[Application_Id],
			[Release_Number],
			[Build_Number],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)'),
		       [Environment_Id] = @PromoteTo_Environment_Id,
		       [Application_Id] = @Application_Id,
		       [Release_Number] = @Release_Number,
		       [Build_Number] = @Build_Number,
			   [Value_Text] = LIST.[Variables].[value]('@Value', 'VARCHAR(MAX)'),
			   [Sensitive_Indicator] = COALESCE(VDEC.[Sensitive_Indicator], 'N')
		  FROM @Promotion_Variables_Xml.[nodes]('/Variables/Variable') AS LIST([Variables])
		       LEFT JOIN [VariableDeclarations] VDEC
		              ON VDEC.[Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)')
		             AND VDEC.[Scoped_Application_Id] = @Application_Id
		             AND VDEC.[Scoped_Environment_Id] = @PromoteTo_Environment_Id
		             AND VDEC.[Scope_Code] = 'P'

	END

	EXEC [Events_RaiseEvent] 'BLDPMT',@Application_Id,@Release_Number,@Build_Number,@PromoteTo_Environment_Id

	/* Initiate execution  */
	IF (@ExecutionStart_Date IS NOT NULL) BEGIN
		EXEC [Builds_InitiateBuildExecution]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number,
			@Build_Number = @Build_Number,
			@Environment_Id = @PromoteTo_Environment_Id,
			@Start_Date = @ExecutionStart_Date,
            @Variables_Xml = @Execution_Variables_Xml
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
		   
END
GO

GRANT EXECUTE ON [Builds_PromoteBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CompleteAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_CompleteAction]') IS NOT NULL DROP PROCEDURE [Executions_CompleteAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CompleteAction]
(
	@BuildExecution_PlanAction_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @ResumeNextOnFailure_Indicator YNINDICATOR,
	        @Retry_Count INT, 
			@Attempt_Sequence INT
	 SELECT @ResumeNextOnFailure_Indicator = [ResumeNextOnFailure_Indicator],
	        @Retry_Count = COALESCE([Retry_Count],0),
	        @Attempt_Sequence = [Attempt_Sequence]
	   FROM [BuildExecution_PlanActions_Extended]
	  WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	DECLARE @ExecutionStatus_Name VARCHAR(10)
		SET @ExecutionStatus_Name = CASE 
		       WHEN EXISTS(SELECT * FROM [BuildExecution_PlanActionLogEntries]
		                    WHERE [LogEntry_Level] >= 30 
		                      AND [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id) THEN 'Failed'
		       ELSE 'Succeeded' END

	DECLARE @Execution_Id INT
	 SELECT @Execution_Id = [Execution_Id]
	   FROM [BuildExecution_PlanActions]
	  WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	DECLARE 
		@Application_Id INT,
		@Application_Name VARCHAR(50),
		@Release_Number RELEASENUMBER,
		@Build_Number BUILDNUMBER,
		@Environment_Id INT,
		@NotificationTitle_Text VARCHAR(100),
		@NotificationDetails_Text VARCHAR(MAX)

	SELECT @Application_Id = [Application_Id],
	       @Release_Number = [Release_Number],
	       @Build_Number = [Build_Number],
	       @Environment_Id = [Environment_Id]
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] = @Execution_Id

	--Clear Any ManualActions
	DELETE [ManualActionsInProgress]
	 WHERE [BuildExecution_PlanAction_Id] IN (SELECT [BuildExecution_PlanAction_Id]
											    FROM [BuildExecution_PlanActions]
											   WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id)

	--Marks the Action Execution status as Success or Failure and sets the end date
	UPDATE [BuildExecution_PlanActions]
	   SET [ExecutionStatus_Name] = @ExecutionStatus_Name,
		   [ExecutionEnded_Date] = GETUTCDATE()
	 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	
	IF ( -- the execution shouldb be failed (no retry, no resume next)
         (@ExecutionStatus_Name = 'Failed' AND @ResumeNextOnFailure_Indicator = 'N' AND @Retry_Count < @Attempt_Sequence)
  	     -- or if there is nothing left to execute
	     OR NOT EXISTS (SELECT * FROM dbo.Executions_GetNextAction(@Execution_Id)) )
	BEGIN
        EXEC [Executions_CompleteExecution] @Execution_Id

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CompleteAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_GetAndExecuteNextAction',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_GetAndExecuteNextAction]') IS NOT NULL DROP PROCEDURE [Executions_GetAndExecuteNextAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_GetAndExecuteNextAction]
(
	@Execution_Id INT
)
AS
BEGIN

DECLARE @Next_Action_Sequence INT,
		@Next_PlanActionGroup_Id INT,
		@Next_PlanExecution_Sequence INT,
		@Next_Server_Id INT,
		@Next_Attempt_Sequence INT,
		@Application_Id INT,
		@Release_Number RELEASENUMBER,
		@Build_Number BUILDNUMBER,
		@Environment_Id INT,
		@BuildExecution_PlanAction_Id INT

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	SELECT @Application_Id = [Application_Id],
		   @Release_Number = [Release_Number],
		   @Build_Number = [Build_Number],
		   @Environment_Id = [Environment_Id]
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] = @Execution_Id

	/* If there are any Executing actions, but NOT in a Canceled exec return nothing */
	IF EXISTS (SELECT *
				 FROM [BuildExecution_PlanActions_Extended] BEPA
				      INNER JOIN [BuildExecutions] BE
				              ON BE.[Execution_Id] = BEPA.[Execution_Id]
				WHERE BEPA.[Execution_Id] = @Execution_Id
				  AND BEPA.[ExecutionStatus_Name] = 'Executing'
				  AND BE.[ExecutionStatus_Name] <> 'Canceled')
	BEGIN
		GOTO DORETURN
	END

	/* If the execution was canceled, fail the execution and return nothing */
	IF EXISTS (SELECT *
				 FROM [dbo].[BuildExecutions]
				WHERE [Execution_Id] = @Execution_Id
				  AND [ExecutionStatus_Name] = 'Canceled')
	BEGIN
		UPDATE [dbo].[BuildExecutions]
		   SET [ExecutionStatus_Name] = 'Failed'
		 WHERE [Execution_Id] = @Execution_Id
		 
 		 UPDATE [dbo].[BuildPromotions]
		    SET [PromotionStatus_Name] = 'Failed'
		  WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number
		    AND [Build_Number] = @Build_Number
		    AND [Environment_Id] = @Environment_Id

		EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

		GOTO DORETURN
	END

	/* Find the Next Plan to Execute */
	SELECT TOP 1
	       @Next_PlanActionGroup_Id = [Next_PlanActionGroup_Id],
	       @Next_PlanExecution_Sequence = [Next_PlanExecution_Sequence],
	       @Next_Action_Sequence = [Next_Action_Sequence],
	       @Next_Server_Id = [Next_Server_Id],
		   @Next_Attempt_Sequence = [Next_Attempt_Sequence]
	 FROM dbo.Executions_GetNextAction(@Execution_Id)

	/* Abort if un-inclused */
	IF @Next_Action_Sequence IS NULL GOTO DORETURN 

	/* Add the next action */
	INSERT INTO [BuildExecution_PlanActions]
	  ([Execution_Id]
      ,[Original_PlanActionGroup_Id]
      ,[Original_Action_Sequence]
      ,[ExecutionStarted_Date]
      ,[ExecutionStatus_Name]
      ,[ExecutionAction_Description]
      ,[Server_Id]
      ,[Original_Server_Name]
	  ,[Attempt_Sequence]
      ,[WarningLogEntry_Indicator])
	SELECT
	   @Execution_Id,
	   @Next_PlanActionGroup_Id,
	   @Next_Action_Sequence,
	   GETUTCDATE(),
	   'Executing',
	   LEFT(PA.[ActionType_Name] + COALESCE('; ' + NULLIF(PA.[Action_Description],''), ''), 500),
	   @Next_Server_Id,
	   (SELECT [Server_Name]
	      FROM [Servers]
	     WHERE [Server_Id] = @Next_Server_Id),
	   @Next_Attempt_Sequence,
       'N'
	 FROM [PlanActions] PA
	      INNER JOIN [PlanActionGroups] PAG
	              ON PA.[Plan_Id] = PAG.[ActionGroup_Id]
	WHERE PAG.[PlanActionGroup_Id] = @Next_PlanActionGroup_Id
	  AND PA.[Action_Sequence] = @Next_Action_Sequence

	/* Capture BEPA_Id */
	SET @BuildExecution_PlanAction_Id = SCOPE_IDENTITY()

 	/* Initialize variable values */
	INSERT INTO [BuildExecution_PlanActionVariableValues] ([BuildExecution_PlanAction_Id], [Variable_Name], [Value_Text])
	SELECT @BuildExecution_PlanAction_Id, CV.[Variable_Name], CV.[Value_Text]
	  FROM dbo.[Variables_GetValuesForExecution](
	          @Execution_Id, 
			  @Next_Server_Id, 
			  (SELECT [Deployable_Id] FROM [PlanActionGroups] WHERE [PlanActionGroup_Id] = @Next_PlanActionGroup_Id)) CV

	/* Update Execution to Executing */
	UPDATE [BuildExecutions]
	   SET [ExecutionStatus_Name] = 'Executing'
	 WHERE [Execution_Id] = @Execution_Id
	   AND [ExecutionStatus_Name] = 'Pending'


DORETURN:

	/* Return the action */
	SELECT *
	  FROM [BuildExecution_PlanActions_Extended]
	 WHERE [BuildExecution_PlanAction_Id] = @BuildExecution_PlanAction_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_GetAndExecuteNextAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Plans_DeleteAction',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes an action from an action group'
GO


IF OBJECT_ID('[Plans_DeleteAction]') IS NOT NULL DROP PROCEDURE [Plans_DeleteAction]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Plans_DeleteAction]
(
	@PlanActionGroup_Id INT,
	@Action_Sequence INT
)
AS
BEGIN

	DECLARE @ActionGroup_Id INT
	    SET @ActionGroup_Id = (SELECT [ActionGroup_Id] FROM [PlanActionGroups]
	                            WHERE [PlanActionGroup_Id] = @PlanActionGroup_Id)

	IF NOT EXISTS(SELECT * FROM [PlanActions] WHERE [Plan_id] = @ActionGroup_Id AND [Action_Sequence] = @Action_Sequence)
	BEGIN
		/* Silently Return */
		Return;
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	-- move action to the end of the sequence before deletion to appease trigger
    DECLARE @New_Action_Sequence INT
    SELECT @New_Action_Sequence = MAX([Action_Sequence])
      FROM [PlanActions]
     WHERE [Plan_Id] = @ActionGroup_Id
    
	UPDATE [PlanActions]
	   SET [Action_Sequence] = CASE
	          WHEN [PlanActions].[Action_Sequence] = CUR.[Action_Sequence] 
	               THEN NEW.[Action_Sequence]
	          WHEN [PlanActions].[Action_Sequence] BETWEEN CUR.[Action_Sequence] AND NEW.[Action_Sequence]
	               THEN [PlanActions].[Action_Sequence] - 1
	          WHEN [PlanActions].[Action_Sequence] BETWEEN NEW.[Action_Sequence] AND CUR.[Action_Sequence]
	               THEN [PlanActions].[Action_Sequence] + 1
	          ELSE [PlanActions].[Action_Sequence]
	          END
	  FROM [PlanActions],
	       [PlanActions] CUR,
	       [PlanActions] NEW
	 WHERE CUR.[Action_Sequence] = @Action_Sequence
	   AND NEW.[Action_Sequence] = @New_Action_Sequence
	   AND @Action_Sequence <> @New_Action_Sequence
	   AND [PlanActions].[Plan_Id] = @ActionGroup_Id
	   AND CUR.[Plan_Id] = @ActionGroup_Id
	   AND NEW.[Plan_Id] = @ActionGroup_Id
	   
	-- delete the action now that it's at the end
	DELETE [PlanActions]
	WHERE [Plan_Id] = @ActionGroup_Id
	  AND [Action_Sequence] = @New_Action_Sequence

	-- event
	DECLARE @APP_CUR_TBL TABLE([Application_Id] INT)
	        INSERT INTO @APP_CUR_TBL([Application_Id]) 
	        SELECT DISTINCT [Application_Id] FROM [PlanActionGroups]
	         WHERE [ActionGroup_Id] = @ActionGroup_Id
	WHILE (SELECT COUNT(*) FROM @APP_CUR_TBL) > 0 BEGIN
	DECLARE @Application_Id INT
		SELECT TOP 1 @Application_Id = [Application_Id] FROM @APP_CUR_TBL
		DELETE @APP_CUR_TBL WHERE [Application_Id] = @Application_Id

		EXEC [Events_RaiseEvent] 'PLNMOD', @Application_Id, @ActionGroup_Id  	  
	END

	UPDATE [Plans]
	   SET [ModifiedOn_Date] = GETUTCDATE(),
	       [ModifiedBy_User_Name] = dbo.Session_GetUserName()
	 WHERE [Plan_Id] = @ActionGroup_Id

	DECLARE @PAG2 TABLE ([Application_Id] INT, [Environment_Id] INT)
	INSERT INTO @PAG2 SELECT [Application_Id], [Environment_Id] FROM [PlanActionGroups] WHERE [ActionGroup_Id] = @ActionGroup_Id
	WHILE ((SELECT COUNT(*) FROM @PAG2) > 0) BEGIN
		DECLARE @Application_Id2 INT
		DECLARE @Environment_Id INT
		SET @Application_Id2 = (SELECT TOP 1 [Application_Id] FROM @PAG2)
		SET @Environment_Id = (SELECT TOP 1 [Environment_Id] FROM @PAG2)
		DELETE @PAG2 WHERE [Application_Id] = @Application_Id2 AND [Environment_Id] = @Environment_Id
		
		EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id2, @Environment_Id = @Environment_Id
	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
END
GO

GRANT EXECUTE ON [Plans_DeleteAction] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new release, or updates an existing one'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateRelease]') IS NOT NULL
	DROP PROCEDURE [Releases_CreateOrUpdateRelease]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Workflow_Id INT = NULL,
	@Target_Date DATETIME,
	@Release_Name VARCHAR(50),
	@Notes_Text VARCHAR(MAX),
	@DeployableIds_Xml XML
	/* XML format example
		<?xml version="1.0" encoding="utf-16"?>
		<ReleaseDeployables>
			<ReleaseDeployable
				Deployable_Id="4"
				InclusionType_Code="R"
				Referenced_Release_Number="3.4"
				Referenced_Application_Id="1" (or not included to indicate NULL)
			/>
		</ReleaseDeployables>
	*/
) AS
BEGIN

SET @Release_Name = NULLIF(REPLACE(@Release_Name,' ',''),'')

BEGIN TRY
	BEGIN TRANSACTION

	IF NOT EXISTS ( SELECT * FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number )
	BEGIN

		/* Create the new release */
		DECLARE @ReleaseExists_Indicator YNINDICATOR
			   ,@NewerDeployedReleaseExists_Indicator YNINDICATOR
		EXEC [Releases_ReleaseExists]
				 @Application_Id = @Application_Id
				,@Release_Number = @Release_Number
				,@ReleaseExists_Indicator = @ReleaseExists_Indicator OUT
				,@NewerDeployedReleaseExists_Indicator = @NewerDeployedReleaseExists_Indicator OUT

		IF @ReleaseExists_Indicator = 'Y'
		BEGIN
			RAISERROR ('Release number ''%s'' already exists.', 16, 1, @Release_Number)
		END

		IF @NewerDeployedReleaseExists_Indicator = 'Y'
		BEGIN
			RAISERROR ('Release number ''%s'' cannot be created because a newer deployed release already exists.', 16, 1, @Release_Number)
		END

        DECLARE @ReleaseNumber_Scheme_Name VARCHAR(50)
            SET @ReleaseNumber_Scheme_Name = (SELECT [ReleaseNumber_Scheme_Name] FROM [Applications] WHERE [Application_Id] = @Application_Id)
        
        DECLARE @Release_Sequence INT
		 SELECT @Release_Sequence = COALESCE(MAX([Release_Sequence]), 0) + 1
		   FROM [Releases]
		  WHERE [Application_Id] = @Application_Id

		INSERT INTO [Releases]
			(
				[Application_Id], 
				[Release_Number], 
				[Workflow_Id], 
				[ReleaseStatus_Name], 
				[Target_Date], 
				[Release_Name], 
				[Notes_Text],
				[CreatedBy_User_Name],
				[CreatedOn_Date],
				[ModifiedBy_User_Name],
				[ModifiedOn_Date],
                [Release_Sequence]
			)
		VALUES
			(
				@Application_Id, 
				@Release_Number, 
				@Workflow_Id, 
				'Active', 
				@Target_Date, 
				@Release_Name, 
				@Notes_Text,
				dbo.Session_GetUserName(),
				GETUTCDATE(),
				dbo.Session_GetUserName(),
				GETUTCDATE(),
                @Release_Sequence
			)

		INSERT INTO [ReleaseDeployables]
			(
				[Application_Id],
				[Release_Number],
				[Deployable_Id],
				[InclusionType_Code],
				[Referenced_Release_Number],
				[Referenced_Application_Id]
			)
		SELECT
				@Application_Id,
				@Release_Number,
				[Deployable_Id] = RDX.[ReleaseDeployable].[value]('@Deployable_Id', 'INT'),
				[InclusionType_Code] = RDX.[ReleaseDeployable].[value]('@InclusionType_Code', 'CHAR(1)'),
				[Referenced_Release_Number] = RDX.[ReleaseDeployable].[value]('@Referenced_Release_Number', 'RELEASENUMBER'),
				[Referenced_Application_Id] = RDX.[ReleaseDeployable].[value]('@Referenced_Application_Id', 'INT')
		FROM @DeployableIds_Xml.[nodes]('/ReleaseDeployables/ReleaseDeployable') AS RDX(ReleaseDeployable)

		EXEC [Events_RaiseEvent] 'RELNEW', @Application_Id, @Release_Number

	END
	ELSE
	BEGIN
	
		/* verify that there are no active builds for this release if we're including a workflow id */
		DECLARE @Old_Workflow_Id INT
		
		SELECT @Old_Workflow_Id = [Workflow_Id] FROM [Releases] 
		WHERE [Application_Id] = @Application_Id 
		AND [Release_Number] = @Release_Number
		
		EXEC [dbo].[Builds_GetBuilds] @Application_Id, @Release_Number, 'Active'
		
		IF (@Workflow_Id IS NOT NULL 
			AND @Workflow_Id <> @Old_Workflow_Id 
			AND @@ROWCOUNT > 0)
        BEGIN
			RAISERROR('A release''s workflow may not be changed if the release has any active builds', 16, 1)
			ROLLBACK
        END

		/* Update the existing release */
		UPDATE [Releases]
		   SET [Target_Date] = @Target_Date
			  ,[Release_Name] = @Release_Name
			  ,[Workflow_Id] = @Workflow_Id
			  ,[ModifiedBy_User_Name] = [dbo].[Session_GetUserName]()
			  ,[ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

		DELETE
		  FROM [ReleaseDeployables]
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number

		INSERT INTO [ReleaseDeployables]
			(
				[Application_Id],
				[Release_Number],
				[Deployable_Id],
				[InclusionType_Code],
				[Referenced_Release_Number],
				[Referenced_Application_Id]
			)
		SELECT
				@Application_Id,
				@Release_Number,
				[Deployable_Id] = RDX.[ReleaseDeployable].[value]('@Deployable_Id', 'INT'),
				[InclusionType_Code] = RDX.[ReleaseDeployable].[value]('@InclusionType_Code', 'CHAR(1)'),
				[Referenced_Release_Number] = RDX.[ReleaseDeployable].[value]('@Referenced_Release_Number', 'RELEASENUMBER'),
				[Referenced_Application_Id] = RDX.[ReleaseDeployable].[value]('@Referenced_Application_Id', 'INT')
		FROM @DeployableIds_Xml.[nodes]('/ReleaseDeployables/ReleaseDeployable') AS RDX(ReleaseDeployable)

		EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number

		EXEC [InvalidateLastActionReexecution] @Application_Id = @Application_Id
		
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateRelease] TO [BuildMasterUser_Role]



GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Workflows_CreateOrUpdateWorkflowStep',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new workflow step'
GO


IF OBJECT_ID('[Workflows_CreateOrUpdateWorkflowStep]') IS NOT NULL DROP PROCEDURE [Workflows_CreateOrUpdateWorkflowStep]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Workflows_CreateOrUpdateWorkflowStep]
(
	@Workflow_Id INT,
	@Environment_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	
	DECLARE @Old_Step_Sequence INT

	IF NOT EXISTS(SELECT * FROM [WorkflowSteps] WHERE [Workflow_Id] = @Workflow_Id AND [Environment_Id] = @Environment_Id)
		BEGIN

			INSERT INTO [WorkflowSteps]
				([Workflow_Id], [Step_Sequence], [Environment_Id])
			SELECT @Workflow_Id, COALESCE(MAX([Step_Sequence]) + 1, 1), @Environment_Id
			  FROM [WorkflowSteps]
			 WHERE [Workflow_Id] = @Workflow_Id
		END
	ELSE
		RAISERROR('Updating not supported',16,1)

	DECLARE @Application_Id INT
	    SET @Application_Id = (SELECT [Application_Id] 
	                             FROM [Workflows]
	                            WHERE [Workflow_Id] = @Workflow_Id)
	EXEC [Events_RaiseEvent] 'WKFMDS', @Application_Id, @Workflow_Id, @Environment_Id
		

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Workflows_CreateOrUpdateWorkflowStep] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CompletePromotion',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Builds_CompletePromotion]') IS NOT NULL DROP PROCEDURE [Builds_CompletePromotion]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CompletePromotion]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
	@Environment_Id INT,
	@StatusOverriddenReason_Text VARCHAR(100) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Workflow_Id INT
	DECLARE @Step_Sequence INT
	DECLARE @Application_Name VARCHAR(50)
	DECLARE @NotificationTitle_Text VARCHAR(100)
	DECLARE @NotificationDetails_Text VARCHAR(MAX)
    DECLARE @AutoIncrementRelease_Indicator YNINDICATOR
    DECLARE @AutoDeployRelease_Indicator YNINDICATOR
    DECLARE @AutoCancelReleases_Indicator YNINDICATOR

	SELECT @Workflow_Id = [Workflow_Id], 
           @AutoIncrementRelease_Indicator = [AutoIncrementRelease_Indicator],
           @AutoDeployRelease_Indicator = [AutoDeployRelease_Indicator],
           @AutoCancelReleases_Indicator = [AutoCancelReleases_Indicator]
      FROM [Releases_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number

	/* Skip if the promotion is Already Completed */
	IF EXISTS(SELECT * FROM [BuildPromotions]
	           WHERE [Application_Id] = @Application_Id
	             AND [Release_Number] = @Release_Number
	             AND [Build_Number] = @Build_Number
	             AND [Environment_Id] = @Environment_Id
	             AND [PromotionStatus_Name] = 'Completed')
		BEGIN COMMIT RETURN END

	/* Validate Status */
	IF NOT EXISTS(SELECT * FROM [BuildPromotions]
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [Build_Number] = @Build_Number
	                 AND [Environment_Id] = @Environment_Id
	                 AND [PromotionStatus_Name] IN ('AwaitingExecution','Failed'))
			RAISERROR('Build Promotion must be AwaitingExecution or Failed', 16, 1)

	/* if there are no Promotion Contingiencies that aren't Completed... */
	IF NOT EXISTS(
		SELECT *
		  FROM [BuildPromotionContingencies] BPC
		       INNER JOIN [BuildPromotions] BP
		               ON BPC.[Contingent_Application_Id] = BP.[Application_Id]
		              AND BPC.[Contingent_Release_Number] = BP.[Release_Number]
		              AND BPC.[Contingent_Build_Number] = BP.[Build_Number]
		              AND BPC.[Contingent_Environment_Id] = BP.[Environment_Id]
		 WHERE BPC.[Application_Id] = @Application_Id
		   AND BPC.[Release_Number] = @Release_Number
		   AND BPC.[Build_Number] = @Build_Number
		   AND BPC.[Environment_Id] = @Environment_Id
		   AND BP.[PromotionStatus_Name] <> 'Completed')
	BEGIN


		/* Do the update */
		/* Special CASE logic for when build is being overridden (@StatusOverriddenReason_Text is not null) */
		UPDATE [BuildPromotions]
		   SET [PromotionStatus_Name] = 'Completed'
			  ,[StatusOverriddenBy_User_Name] = CASE WHEN @StatusOverriddenReason_Text IS NULL THEN NULL ELSE dbo.Session_GetUserName() END
			  ,[StatusOverriddenReason_Text] = @StatusOverriddenReason_Text
			  ,[StatusOverridden_Date] = CASE WHEN @StatusOverriddenReason_Text IS NULL THEN NULL ELSE GETUTCDATE() END
		 WHERE [Application_Id] = @Application_Id
		   AND [Release_Number] = @Release_Number
		   AND [Build_Number] = @Build_Number
		   AND [Environment_Id] = @Environment_Id

		/* Find the current Sequence */
		SET @Step_Sequence = 
			(SELECT [Step_Sequence] FROM [WorkflowSteps]
			  WHERE [Workflow_Id] = @Workflow_Id 
				AND [Environment_Id] = @Environment_Id)
				
		/* if there is no next step (i.e. deployed to Production) */
		IF EXISTS(SELECT * FROM [Workflows]
		           WHERE [Workflow_Id] = @Workflow_Id
		             AND [AutoDeployRelease_Indicator]='Y')
		 AND NOT EXISTS(SELECT * FROM [WorkflowSteps]
					     WHERE [Workflow_Id] = @Workflow_Id 
						   AND [Step_Sequence] > @Step_Sequence)
		BEGIN
			
            EXEC [Builds_DeployBuild] @Application_Id, @Release_Number, @Build_Number, @Workflow_Id

		END
		
		EXEC [Events_RaiseEvent] 'BLDCPP',@Application_Id,@Release_Number,@Build_Number,@Environment_Id
	
	END


	/* for each Promotion Contingent Upon this one */
    DECLARE @ContingentPromotionsCursor CURSOR,
			@CUR_Application_Id INT,
			@CUR_Release_Number RELEASENUMBER, 
			@CUR_Build_Number BUILDNUMBER, 
			@CUR_Environment_Id INT

	/* Promote (with no execute) and Insert Contingencies */
	SET @ContingentPromotionsCursor = CURSOR FOR
		SELECT BPC.[Application_Id], BPC.[Release_Number], BPC.[Build_Number], BPC.[Environment_Id]
			FROM [BuildPromotionContingencies] BPC
			    INNER JOIN [BuildPromotions_Extended] BP
			            ON BP.[Application_Id] = BPC.[Application_Id]
			            AND BP.[Release_Number] = BPC.[Release_Number]
			            AND BP.[Build_Number] = BPC.[Build_Number] 
			            AND BP.[Environment_Id] = BPC.[Environment_Id]
			WHERE BP.[Last_ExecutionStatus_Name] = 'Succeeded'
			AND BP.[PromotionStatus_Name] = 'AwaitingExecution'
			AND BPC.[Contingent_Application_Id] = @Application_Id
			AND BPC.[Contingent_Release_Number] = @Release_Number
			AND BPC.[Contingent_Build_Number] = @Build_Number
			AND BPC.[Contingent_Environment_Id] = @Environment_Id
	OPEN @ContingentPromotionsCursor
	FETCH NEXT FROM @ContingentPromotionsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN
			
			EXEC [Builds_CompletePromotion]
				@Application_Id = @CUR_Application_Id,
				@Release_Number = @CUR_Release_Number,
				@Build_Number = @CUR_Build_Number,
				@Environment_Id = @CUR_Environment_Id				

			FETCH NEXT FROM @ContingentPromotionsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Environment_Id
		END
	CLOSE @ContingentPromotionsCursor

	
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Builds_CompletePromotion] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CreateBuild',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Build_Number',
    /* Description_Text        */ 'Creates a new build of an application and optionally promotes it to the first environment.',
    /* Remarks_Text            */ 'To specify build variable values, the Variables_Xml argument should be a string in the format:
	<Variables>
		<Variable Name="VARIABLE_NAME" Value="VARIABLE_VALUE" />
	</Variables>'
GO

IF OBJECT_ID('[Builds_CreateBuild]') IS NOT NULL DROP PROCEDURE [Builds_CreateBuild]
GO


SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CreateBuild]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@PromoteBuild_Indicator YNINDICATOR = NULL,
	@StartExecution_Indicator YNINDICATOR = NULL,
	@ExecutionStart_Date DATETIME = NULL,
	@Requested_Build_Number BUILDNUMBER = NULL,
	@BuildVariables_Xml XML = NULL,
	@PromotionVariables_Xml XML = NULL,
    @ExecutionVariables_Xml XML = NULL,
	@Build_Number BUILDNUMBER OUT
)
AS
BEGIN

	/* Set defaults */
	SET @PromoteBuild_Indicator = COALESCE(@PromoteBuild_Indicator, 'N')
	SET @StartExecution_Indicator = COALESCE(@StartExecution_Indicator, @PromoteBuild_Indicator)
	
	IF @PromoteBuild_Indicator = 'Y' AND @StartExecution_Indicator = 'Y' BEGIN
		SET @ExecutionStart_Date = CASE
									WHEN @ExecutionStart_Date >= GETUTCDATE() THEN @ExecutionStart_Date
									ELSE GETUTCDATE()
								   END
	END
	ELSE BEGIN
		SET @ExecutionStart_Date = NULL
	END

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	IF NOT EXISTS(SELECT * FROM [Releases] 
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [ReleaseStatus_Name] = 'Active')
		RAISERROR('Release "%s" for Application #%i is not an active release or does not exist.', 16, 1, @Release_Number, @Application_Id)
		
		IF @Requested_Build_Number IS NOT NULL BEGIN
			IF EXISTS (SELECT *
						 FROM [Builds]
						WHERE [Application_Id] = @Application_Id
						  AND [Release_Number] = @Release_Number
						  AND [Build_Number] = @Requested_Build_Number)
				RAISERROR('Build "%s" of release "%s" for Application #%i already exists.', 16, 1, @Requested_Build_Number, @Release_Number, @Application_Id)
				
			IF EXISTS (SELECT *
						 FROM [Builds]
						WHERE [Application_Id] = @Application_Id
						  AND [Release_Number] = @Release_Number
						  AND dbo.SortableBuildNumber(@Application_Id, [Build_Number]) >= dbo.SortableBuildNUmber(@Application_Id, @Requested_Build_Number))
				RAISERROR('Build numbers must increase. A greater build number than "%s" of release "%s" in Application #%i already exists.', 16, 1, @Requested_Build_Number, @Release_Number, @Application_Id)
				
			SET @Build_Number = @Requested_Build_Number
		END
		ELSE BEGIN
		    SET @Build_Number = dbo.GetNextBuildNumber(
		            @Application_Id, 
			        ( SELECT MAX(dbo.SortableBuildNumber(@Application_Id, [Build_Number])) FROM [Builds] 
				       WHERE [Application_Id] = @Application_Id
					     AND [Release_Number] = @Release_Number ) )
		END

	INSERT INTO [Builds]
	(	
		[Application_Id], 
		[Release_Number], 
		[Build_Number], 
		[BuildStatus_Name],
		[CreatedBy_User_Name],
		[CreatedOn_Date],
		[ModifiedBy_User_Name],
		[ModifiedOn_Date]
	)
	VALUES
	(
		@Application_Id, 
		@Release_Number, 
		@Build_Number, 
		'Active',
		dbo.Session_GetUserName(),
		GETUTCDATE(),
		dbo.Session_GetUserName(),
		GETUTCDATE()
	)

	IF @BuildVariables_Xml IS NOT NULL BEGIN

		INSERT INTO [VariableDefinitions]
		(
			[Variable_Name],
			[Application_Id],
			[Release_Number],
			[Build_Number],
			[Value_Text],
			[Sensitive_Indicator]
		)
		SELECT [Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)'),
		       [Application_Id] = @Application_Id,
		       [Release_Number] = @Release_Number,
		       [Build_Number] = @Build_Number,
			   [Value_Text] = LIST.[Variables].[value]('@Value', 'VARCHAR(MAX)'),
			   [Sensitive_Indicator] = COALESCE(VDEC.[Sensitive_Indicator], 'N')
		  FROM @BuildVariables_Xml.[nodes]('/Variables/Variable') AS LIST([Variables])
		       LEFT JOIN [VariableDeclarations] VDEC
		               ON VDEC.[Variable_Name] = LIST.[Variables].[value]('@Name', 'VARCHAR(50)')
		              AND VDEC.[Scoped_Application_Id] = @Application_Id
		              AND VDEC.[Scope_Code] = 'B'

	END

	EXEC [Events_RaiseEvent] 'BLDNEW', @Application_Id, @Release_Number, @Build_Number

	/* Promote the Build */
	IF @PromoteBuild_Indicator = 'Y' BEGIN
		DECLARE @Next_Environment_Id INT
		SET @Next_Environment_Id = (
			SELECT [Next_Environment_Id] FROM [Builds_Extended]
			 WHERE [Application_Id] = @Application_Id
			   AND [Release_Number] = @Release_Number
			   AND [Build_Number] = @Build_Number)

		EXEC [Builds_PromoteBuild]
			@Application_Id = @Application_Id,
			@Release_Number = @Release_Number,
			@Build_Number = @Build_Number,
			@Comments_Text = NULL,
			@ExecutionStart_Date = @ExecutionStart_Date,
			@PromoteTo_Environment_Id = @Next_Environment_Id,
			@Promotion_Variables_Xml = @PromotionVariables_Xml,
            @Execution_Variables_Xml = @ExecutionVariables_Xml
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_CreateBuild] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_CreateDependentBuilds',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates new builds in all dependent applications a build.',
    /* Remarks_Text            */ 'To specify build variable values, the Variables_Xml argument should be a string in the format:
	<Variables>
		<Variable Name="VARIABLE_NAME" Value="VARIABLE_VALUE" />
	</Variables>'
GO

IF OBJECT_ID('[Builds_CreateDependentBuilds]') IS NOT NULL DROP PROCEDURE [Builds_CreateDependentBuilds]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_CreateDependentBuilds]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,

	@ExecutionStart_Date DATETIME = NULL,
	@OverrideApprovals_Indicator YNINDICATOR = 'N',
	@Variables_Xml XML = NULL,

	@PromoteFor_Release_Number RELEASENUMBER = NULL,
	@ContinueOnFail_Indicator YNINDICATOR = 'N'	
)
AS
BEGIN


SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	/* Verify Build */
	IF NOT EXISTS(SELECT * FROM [Builds]
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [Build_Number] = @Build_Number
	                 AND [BuildStatus_Name] <> 'Rejected')
		RAISERROR('Build %s does not exist or has been rejected', 16, 1, @Build_Number)

	/* Verify Current Environment Id */		
	DECLARE @Current_Environment_Id INT
	SELECT @Current_Environment_Id = [Current_Environment_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF @Current_Environment_Id IS NULL
		RAISERROR('Build %s does not have a current environment.', 16, 1, @Build_Number)

	
	/* Find All Dependent Releases */
	SELECT RE.*
	  INTO #DependentReleases
	  FROM [Releases_Extended] RE
	 WHERE EXISTS 
           (
	         SELECT * FROM [ReleaseDeployables] RD
	          WHERE RE.[Application_Id] = RD.[Application_Id]
	            AND RE.[Release_Number] = RD.[Release_Number]
	            AND RD.[Referenced_Application_Id] = @Application_Id
	            AND RD.[Referenced_Release_Number] = @Release_Number
	            
	       )
	   AND NOT ([Application_Id] = @Application_Id AND 
	            [Release_Number] = @Release_Number)
	   AND (@PromoteFor_Release_Number IS NULL OR [Release_Number] = @PromoteFor_Release_Number)
	   AND RE.[ReleaseStatus_Name] = 'Active'
	   
	/* "Declare" a temp table */
	SELECT * INTO #CreatedBuilds FROM [Builds_Extended] WHERE 1=0
	DECLARE @DependentReleasesCursor CURSOR,
	        @CUR_Application_Id INT,
	        @CUR_Release_Number RELEASENUMBER, 
	        @CUR_Build_Number BUILDNUMBER, 
	        @CUR_Next_Environment_Id INT,
	        @DeferredError VARCHAR(MAX),
	        @Eligible_Indicator CHAR(1),
	        @IneligibleReason_Text VARCHAR(MAX)

	/* Create Builds, Promote, and Insert Contingencies */
    SET @DependentReleasesCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number]
          FROM #DependentReleases
          
	OPEN @DependentReleasesCursor
	FETCH NEXT FROM @DependentReleasesCursor INTO @CUR_Application_Id, @CUR_Release_Number
		WHILE @@FETCH_STATUS <> -1
		BEGIN
			
			SET @CUR_Build_Number = NULL
			
			EXEC [Builds_CreateBuild] 
			   @Application_Id = @CUR_Application_Id
			  ,@Release_Number = @CUR_Release_Number
			  ,@Requested_Build_Number = NULL
			  ,@PromoteBuild_Indicator = 'N'
			  ,@StartExecution_Indicator = 'N'
			  ,@ExecutionStart_Date = NULL
			  ,@BuildVariables_Xml = @Variables_Xml
              ,@PromotionVariables_Xml = NULL
			  ,@Build_Number  = @CUR_Build_Number OUTPUT
			  
			INSERT INTO #CreatedBuilds
			SELECT * FROM [Builds_Extended] 
			 WHERE [Application_Id] = @CUR_Application_Id
			   AND [Release_Number] = @CUR_Release_Number
			   AND [Build_Number] = @CUR_Build_Number

			/* Get Build's @CUR_Next_Environment_Id for Promote */
			SELECT @CUR_Next_Environment_Id = [Next_Environment_Id] 
			  FROM #CreatedBuilds
			 WHERE [Application_Id] = @CUR_Application_Id
			   AND [Release_Number] = @CUR_Release_Number
			   AND [Build_Number] = @CUR_Build_Number
			   
			/* Do the Promotion */
			EXECUTE [Builds_PromoteBuild] 
			   @Application_Id = @CUR_Application_Id
			  ,@Release_Number = @CUR_Release_Number
			  ,@Build_Number = @CUR_Build_Number
			  ,@Comments_Text = 'Dependency Promotion'
			  ,@ExecutionStart_Date = NULL
			  ,@ForcePromotion_Indicator = @OverrideApprovals_Indicator
			  ,@DependencyContingent_Indicator = 'Y'
			  ,@PromoteTo_Environment_Id = @CUR_Next_Environment_Id

			/* Insert Contingencies */
			INSERT INTO [BuildPromotionContingencies]
				([Application_Id],[Release_Number],[Build_Number],[Environment_Id]
				,[Contingent_Application_Id],[Contingent_Release_Number],[Contingent_Build_Number],[Contingent_Environment_Id])
			VALUES
				(@CUR_Application_Id,@CUR_Release_Number,@CUR_Build_Number,@CUR_Next_Environment_Id
				,@Application_Id,@Release_Number,@Build_Number,@Current_Environment_Id)			

			FETCH NEXT FROM @DependentReleasesCursor INTO @CUR_Application_Id, @CUR_Release_Number
		END
	CLOSE @DependentReleasesCursor
	
	/* Execute Dependents whose Contingencies match Dependencies */
	DECLARE @CreatedBuildsCursor CURSOR
    SET @CreatedBuildsCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number], [Build_Number], [Next_Environment_Id]
          FROM #CreatedBuilds CB
         WHERE [Application_Id] IN
               (
                 SELECT _RD.[Application_Id]
                   FROM [ReleaseDeployables] _RD
                        LEFT JOIN [BuildPromotionContingencies] _BPC
                               ON _RD.[Referenced_Application_Id] = _BPC.[Contingent_Application_Id]
                              AND _RD.[Referenced_Release_Number] = _BPC.[Contingent_Release_Number]
                  WHERE _RD.[InclusionType_Code] = 'R'
                    AND _RD.[Application_Id] = CB.[Application_Id]
                    AND _RD.[Release_Number] = CB.[Release_Number]
                    AND _BPC.[Contingent_Application_Id] IS NOT NULL
               )
    OPEN @CreatedBuildsCursor
	FETCH NEXT FROM @CreatedBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN	

			/* Initiate execution  */
			IF (@ExecutionStart_Date IS NOT NULL) BEGIN
				EXEC [Builds_InitiateBuildExecution]
					@Application_Id = @CUR_Application_Id,
					@Release_Number = @CUR_Release_Number,
					@Build_Number = @CUR_Build_Number,
					@Environment_Id = @CUR_Next_Environment_Id,
					@Start_Date = @ExecutionStart_Date
			END

			FETCH NEXT FROM @CreatedBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		END
	CLOSE @CreatedBuildsCursor	


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_CreateDependentBuilds] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Builds_PromoteDependents',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Promotes all dependents for a given promotion'
GO


IF OBJECT_ID('[Builds_PromoteDependents]') IS NOT NULL DROP PROCEDURE [Builds_PromoteDependents]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Builds_PromoteDependents]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Build_Number BUILDNUMBER,
--	@Comments_Text VARCHAR(255) = NULL,
	@ExecutionStart_Date DATETIME = NULL,
	@OverrideApprovals_Indicator YNINDICATOR = 'N',
	
	@From_Environment_Id INT = NULL,
	@To_Environment_Id INT = NULL,
	
	@PromoteFor_Release_Number RELEASENUMBER = NULL,

	@ContinueOnFailedPromote_Indicator YNINDICATOR = 'N'
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	/* Verify Build */
	IF NOT EXISTS(SELECT * FROM [Builds]
	               WHERE [Application_Id] = @Application_Id
	                 AND [Release_Number] = @Release_Number
	                 AND [Build_Number] = @Build_Number
	                 AND [BuildStatus_Name] <> 'Rejected')
		RAISERROR('Build %s does not exist or has been rejected', 16, 1, @Build_Number)

	/* Verify Current Environment Id */		
	DECLARE @Current_Environment_Id INT
	SELECT @Current_Environment_Id = [Current_Environment_Id]
	  FROM [Builds_Extended]
	 WHERE [Application_Id] = @Application_Id
	   AND [Release_Number] = @Release_Number
	   AND [Build_Number] = @Build_Number
	IF @Current_Environment_Id IS NULL
		RAISERROR('Build %s does not have a current environment.', 16, 1, @Build_Number)
	
	/* Find All Dependent Builds */
	SELECT DISTINCT B.* -- distinct to filter out multiple dependencies against same deployable
	  INTO #DependentBuilds
	  FROM [Builds_Extended] B
	       INNER JOIN [ReleaseDeployables] RD
	               ON B.[Application_Id] = RD.[Application_Id]
	              AND B.[Release_Number] = RD.[Release_Number]
	              AND RD.[InclusionType_Code] = 'R'
	 WHERE [BuildStatus_Name] = 'Active'
	   AND RD.[Referenced_Application_Id] = @Application_Id
	   AND RD.[Referenced_Release_Number] = @Release_Number
	   AND (@From_Environment_Id IS NULL OR B.[Current_Environment_Id] = @From_Environment_Id)
	   AND (@To_Environment_Id IS NULL OR B.[Next_Environment_Id] = @To_Environment_Id)
	   AND (@PromoteFor_Release_Number IS NULL OR B.[Release_Number] = @PromoteFor_Release_Number)

	DECLARE @DependentBuildsCursor CURSOR,
	        @CUR_Application_Id INT,
	        @CUR_Release_Number RELEASENUMBER, 
	        @CUR_Build_Number BUILDNUMBER, 
	        @CUR_Next_Environment_Id INT,
	        @DeferredError VARCHAR(MAX),
	        @Eligible_Indicator CHAR(1),
	        @IneligibleReason_Text VARCHAR(MAX)

	/* Promote (with no execute) and Insert Contingencies */
    SET @DependentBuildsCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number], [Build_Number], [Next_Environment_Id]
          FROM #DependentBuilds
	OPEN @DependentBuildsCursor
	FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN
		
			EXEC [Builds_ValidateForPromotion]
			   @Application_Id = @CUR_Application_Id
			  ,@Release_Number = @CUR_Release_Number
			  ,@Build_Number = @CUR_Build_Number
			  ,@Eligible_Indicator = @Eligible_Indicator OUT
			  ,@IneligibleReason_Text = @IneligibleReason_Text OUT
			  ,@ForcePromotion_Indicator = @OverrideApprovals_Indicator
			  ,@DependencyContingent_Indicator = 'Y'
			  ,@PromoteTo_Environment_Id = @CUR_Next_Environment_Id

			IF @Eligible_Indicator = 'Y' BEGIN

				EXECUTE [Builds_PromoteBuild] 
				   @Application_Id = @CUR_Application_Id
				  ,@Release_Number = @CUR_Release_Number
				  ,@Build_Number = @CUR_Build_Number
				  ,@Comments_Text = 'Dependency Promotion'
				  ,@ExecutionStart_Date = NULL
				  ,@ForcePromotion_Indicator = @OverrideApprovals_Indicator
				  ,@DependencyContingent_Indicator = 'Y'
				  ,@PromoteTo_Environment_Id = @CUR_Next_Environment_Id
				  
				INSERT INTO [BuildPromotionContingencies]
					([Application_Id],[Release_Number],[Build_Number],[Environment_Id]
					,[Contingent_Application_Id],[Contingent_Release_Number],[Contingent_Build_Number],[Contingent_Environment_Id])
				VALUES
					(@CUR_Application_Id,@CUR_Release_Number,@CUR_Build_Number,@CUR_Next_Environment_Id
					,@Application_Id,@Release_Number,@Build_Number,@Current_Environment_Id)

			END ELSE BEGIN
				IF @ContinueOnFailedPromote_Indicator='N' BEGIN
					RAISERROR('Cannot Promote Build. %s', 16, 1, @IneligibleReason_Text)
				END ELSE BEGIN
					SET @DeferredError = @DeferredError 
						+ 'Build #' + @CUR_Build_Number + ' ' + ' for Application #' + @CUR_Application_Id + ' '
						+ 'cannot be be promoted: ' + @IneligibleReason_Text
					DELETE #DependentBuilds
					 WHERE CURRENT OF @DependentBuildsCursor
				END
			END

			FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		END
	CLOSE @DependentBuildsCursor
	
	/* Execute Dependents whose Contingencies match Dependencies */
    SET @DependentBuildsCursor = CURSOR FOR
        SELECT [Application_Id], [Release_Number], [Build_Number], [Next_Environment_Id]
          FROM #DependentBuilds DB
         WHERE [Application_Id] IN
               (
                 SELECT _BPC.[Application_Id]
                   FROM [ReleaseDeployables] _RD
                        LEFT JOIN [BuildPromotionContingencies] _BPC
                               ON _RD.[Referenced_Application_Id] = _BPC.[Contingent_Application_Id]
                              AND _RD.[Referenced_Release_Number] = _BPC.[Contingent_Release_Number]
                  WHERE _RD.[InclusionType_Code] = 'R'
                    AND _RD.[Application_Id] = DB.[Application_Id]
                    AND _RD.[Release_Number] = DB.[Release_Number]
                    AND _BPC.[Contingent_Application_Id] IS NOT NULL
               )
    OPEN @DependentBuildsCursor
	FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		WHILE @@FETCH_STATUS <> -1
		BEGIN	

			/* Initiate execution  */
			IF (@ExecutionStart_Date IS NOT NULL) BEGIN
				EXEC [Builds_InitiateBuildExecution]
					@Application_Id = @CUR_Application_Id,
					@Release_Number = @CUR_Release_Number,
					@Build_Number = @CUR_Build_Number,
					@Environment_Id = @CUR_Next_Environment_Id,
					@Start_Date = @ExecutionStart_Date
			END

			FETCH NEXT FROM @DependentBuildsCursor INTO @CUR_Application_Id, @CUR_Release_Number, @CUR_Build_Number, @CUR_Next_Environment_Id
		END
	CLOSE @DependentBuildsCursor
	
	COMMIT
	
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH
 
BEGIN TRY
	IF @DeferredError IS NOT NULL
		RAISERROR('Promotion error(s) occured. %s', 16, 1, @DeferredError)
END TRY BEGIN CATCH
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Builds_PromoteDependents] TO [BuildMasterUser_Role]

GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Dashboards_GetApplications',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Applications_Extended,Environments_LatestDeployedReleases',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets an application summary for all applications in BuildMaster or a specific group of applications'
GO


IF OBJECT_ID('[Dashboards_GetApplications]') IS NOT NULL DROP PROCEDURE [Dashboards_GetApplications]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Dashboards_GetApplications]
(
    @ApplicationGroup_Id INT
)
AS
BEGIN

	SELECT *
	  FROM [Applications_Extended]
	 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	    OR @ApplicationGroup_Id IS NULL
	 ORDER BY   
	          CASE WHEN [ApplicationGroup_Id] IS NULL THEN 1 ELSE 0 END
	         ,[ApplicationGroup_Name]
	         ,[Application_Name]

	SELECT *
	  FROM [Environments_LatestDeployedReleases]
	 WHERE [ApplicationGroup_Id] = @ApplicationGroup_Id
	    OR @ApplicationGroup_Id IS NULL
	 ORDER BY [Application_Id], [Display_Sequence]

END
GO

GRANT EXECUTE ON [Dashboards_GetApplications] TO [BuildMasterUser_Role]


GO
EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_CompleteExecution',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO


IF OBJECT_ID('[Executions_CompleteExecution]') IS NOT NULL DROP PROCEDURE [Executions_CompleteExecution]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Executions_CompleteExecution]
(
	@Execution_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE 
		@Application_Id INT,
		@Application_Name VARCHAR(50),
		@Release_Number RELEASENUMBER,
		@Build_Number BUILDNUMBER,
		@Environment_Id INT,
		@NotificationTitle_Text VARCHAR(100),
		@NotificationDetails_Text VARCHAR(MAX)

	SELECT @Application_Id = [Application_Id],
	       @Release_Number = [Release_Number],
	       @Build_Number = [Build_Number],
	       @Environment_Id = [Environment_Id]
	  FROM [BuildExecutions]
	 WHERE [Execution_Id] = @Execution_Id

	-- if there is a failed action execution ...
	IF EXISTS (SELECT * 
				FROM [BuildExecution_PlanActions] BEPA
				      INNER JOIN [PlanActionGroups] PAG
				              ON PAG.[PlanActionGroup_Id] = BEPA.[Original_PlanActionGroup_Id]
				      INNER JOIN [Plans] P
                              ON P.[Plan_Id] = PAG.[ActionGroup_Id]
				      INNER JOIN [PlanActions] PA
                              ON PA.[Plan_Id] = P.[Plan_Id]
                             AND PA.[Action_Sequence] = BEPA.[Original_Action_Sequence]
				WHERE [Execution_Id] = @Execution_Id
				  AND [Attempt_Sequence] - 1 >= COALESCE([Retry_Count], 0)
				  AND [ExecutionStatus_Name] = 'Failed')
	BEGIN
		-- Update the status to Failed
		UPDATE [BuildExecutions]
			SET [ExecutionStatus_Name] = 'Failed',
				[ExecutionEnded_Date] = GETUTCDATE()
			WHERE [Execution_Id] = @Execution_Id

		--fail the promotion
		UPDATE [BuildPromotions]
			SET [PromotionStatus_Name] = 'Failed' 
			WHERE [Application_Id] = @Application_Id
			AND [Release_Number] = @Release_Number
			AND [Build_Number] = @Build_Number
			AND [Environment_Id] = @Environment_Id

		EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

	END ELSE BEGIN

		-- Update the execution's status
		UPDATE [BuildExecutions]
			SET [ExecutionStatus_Name] = 'Succeeded',
				[ExecutionEnded_Date] = GETUTCDATE()
			WHERE [Execution_Id] = @Execution_Id

		EXEC [Events_RaiseEvent] 'EXECPT', @Application_Id, @Execution_Id

		/* if there are no promotion contingencies, then...  */
		IF NOT EXISTS(
				SELECT *
				FROM [BuildPromotionContingencies] BPC
					INNER JOIN [BuildPromotions] BP
							ON BPC.[Contingent_Application_Id] = BP.[Application_Id]
							AND BPC.[Contingent_Release_Number] = BP.[Release_Number]
							AND BPC.[Contingent_Build_Number] = BP.[Build_Number]
							AND BPC.[Contingent_Environment_Id] = BP.[Environment_Id]
				WHERE BPC.[Application_Id] = @Application_Id
				AND BPC.[Release_Number] = @Release_Number
				AND BPC.[Build_Number] = @Build_Number
				AND BPC.[Environment_Id] = @Environment_Id
				AND BP.[PromotionStatus_Name] <> 'Completed' )
		BEGIN
			-- complete the promotion
			EXEC [Builds_CompletePromotion]
				@Application_Id = @Application_Id,
				@Release_Number = @Release_Number,
				@Build_Number = @Build_Number,
				@Environment_Id = @Environment_Id
		END
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_CompleteExecution] TO [BuildMasterUser_Role]


GO
